@@ -34,7 +34,6 @@ import org.gudy.azureus2.core3.util.AEThread;
 import org.gudy.azureus2.core3.util.HashWrapper;
 import org.gudy.azureus2.core3.util.SimpleTimer;
 import org.gudy.azureus2.core3.util.SystemTime;
-import org.gudy.azureus2.core3.util.Timer;
 import org.gudy.azureus2.core3.util.TimerEvent;
 import org.gudy.azureus2.core3.util.TimerEventPerformer;
 
@@ -1846,5 +1845,20 @@ DHTDBImpl
 		{
 			return( delegate.getKeyForKeyBlock( request ));
 		}
+		
+		public void
+		setStorageForKey(
+			String	key,
+			byte[]	data )
+		{
+			delegate.setStorageForKey( key, data );
+		}
+		
+		public byte[]
+		getStorageForKey(
+			String	key )
+		{
+			return( delegate.getStorageForKey(key));
+		}
 	}
 }
