@@ -1,7 +1,7 @@
 /*
  * Created on 03-Aug-2004
  * Created by Paul Gardner
- * Copyright (C) 2004 Aelitis, All Rights Reserved.
+ * Copyright (C) 2004, 2005, 2006 Aelitis, All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  * 
- * AELITIS, SARL au capital de 30,000 euros
+ * AELITIS, SAS au capital de 46,603.30 euros
  * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
  *
  */
@@ -37,11 +37,13 @@ import org.gudy.azureus2.core3.logging.*;
 
 import com.aelitis.azureus.core.diskmanager.cache.*;
 import com.aelitis.azureus.core.diskmanager.file.*;
+import com.aelitis.azureus.core.util.CaseSensitiveFileMap;
 
 public class 
 CacheFileManagerImpl 
-	implements CacheFileManager
+	implements CacheFileManager, AEDiagnosticsEvidenceGenerator
 {
+  private static final LogIDs LOGID = LogIDs.CACHE;
 	public static final boolean	DEBUG	= false;
 	
 	public static final int	CACHE_CLEANER_TICKS		= 60;	// every 60 seconds
@@ -65,7 +67,9 @@ CacheFileManagerImpl
 	
 	protected long		cache_minimum_free_size;
 	protected long		cache_space_free;
-		
+
+	private long	cache_file_id_next	= 0;
+	
 	protected FMFileManager		file_manager;
 	
 		// copy on update semantics
@@ -95,9 +99,15 @@ CacheFileManagerImpl
 	
 	protected AEMonitor			this_mon	= new AEMonitor( "CacheFileManager" );
 	
+	private long	cleaner_ticks	= CACHE_CLEANER_TICKS;
+	
+	
+	
 	public
 	CacheFileManagerImpl()
 	{
+		AEDiagnostics.addEvidenceGenerator( this );
+		
 		file_manager	= FMFileManagerFactory.getSingleton();
 		
 		boolean	enabled	= COConfigurationManager.getBooleanParameter( "diskmanager.perf.cache.enable" );
@@ -148,21 +158,14 @@ CacheFileManagerImpl
 		
 		stats = new CacheFileManagerStatsImpl( this );
 		
-		AEThread	t = 
-			new AEThread( "CacheStatsAndCleaner")
-			{
-				public void
-				runSupport()
-				{
-					cacheStatsAndCleaner();
-				}
-			};
-				
-		t.setDaemon(true);
-			
-		t.start();
-			
-		LGLogger.log( "DiskCache: enabled = " + cache_enabled + ", read = " + cache_read_enabled + ", write = " + cache_write_enabled + ", size = " + cache_size + " MB" );
+
+		cacheStatsAndCleaner();
+		
+
+		if (Logger.isEnabled())
+			Logger.log(new LogEvent(LOGID, "DiskCache: enabled = " + cache_enabled
+					+ ", read = " + cache_read_enabled + ", write = "
+					+ cache_write_enabled + ", size = " + cache_size + " B"));
 	}
 	
 	protected boolean
@@ -180,10 +183,24 @@ CacheFileManagerImpl
 	public CacheFile
 	createFile(
 		final CacheFileOwner	owner,
-		File					file )
+		File					file,
+		int						type )
 	
 		throws CacheFileManagerException
 	{
+		final long	my_id;
+		
+			// we differentiate the 
+		try{
+			this_mon.enter();
+			
+			my_id = cache_file_id_next++;
+			
+		}finally{
+			
+			this_mon.exit();
+		}
+		
 		try{
 			FMFile	fm_file	= 
 				file_manager.createFile(
@@ -192,20 +209,27 @@ CacheFileManagerImpl
 						public String
 						getName()
 						{
-							return( owner.getCacheFileOwnerName());
+							return( owner.getCacheFileOwnerName() + "[" + my_id + "]" );
 						}
 						public TOTorrentFile
 						getTorrentFile()
 						{
 							return( owner.getCacheFileTorrentFile());
 						}
-					}, file );
+						public File
+						getControlFile(
+							String	name )
+						{
+							return( owner.getCacheFileControlFile( name ));
+						}
+					}, file,
+					type==CacheFile.CT_LINEAR?FMFile.FT_LINEAR:FMFile.FT_COMPACT );
 				
 			TOTorrentFile	tf = owner.getCacheFileTorrentFile();
 			
 			CacheFile	cf;
 			
-			if ( tf != null && tf.getLength() < cache_files_not_smaller_than  ){
+			if (( tf != null && tf.getLength() < cache_files_not_smaller_than  ) || !cache_enabled ){ 
 				
 				cf = new CacheFileWithoutCache( this, fm_file, tf );
 				
@@ -243,7 +267,7 @@ CacheFileManagerImpl
 			
 		}catch( FMFileManagerException e ){
 			
-			rethrow( e );
+			rethrow( null, e );
 			
 			return( null );
 		}
@@ -278,6 +302,7 @@ CacheFileManagerImpl
 		 * @return
 		 * @throws CacheFileManagerException
 		 */
+	
 	protected CacheEntry
 	allocateCacheSpace(
 		int					entry_type,
@@ -322,11 +347,35 @@ CacheFileManagerImpl
 			
 				CacheFileWithCache	oldest_file = oldest_entry.getFile();
 				
-				oldest_file.flushCache( oldest_entry.getFilePosition(), true, cache_minimum_free_size );
+				try{
+					
+					oldest_file.flushCache( oldest_entry.getFilePosition(), true, cache_minimum_free_size );
+					
+				}catch( CacheFileManagerException e ){
+					
+						// if the flush failed on a file other than this one then we don't report the error here,
+						// rather we tag the existing file as failed so that when it is next accessed the error
+						// will be reported
+					
+					if ( oldest_file != file ){
+												
+						oldest_file.setPendingException( e );
+						
+					}else{
+						
+						throw( e );
+					}
+				}
 				
 				long	flushed = cache_space_free - old_free;
 				
-				LGLogger.log( "DiskCache: cache full, flushed " + ( flushed ) + " from " + oldest_file.getName());
+				if (Logger.isEnabled()) {
+					TOTorrentFile tf = file.getTorrentFile();
+					TOTorrent torrent = tf == null ? null : tf.getTorrent();
+					Logger.log(new LogEvent(torrent, LOGID,
+							"DiskCache: cache full, flushed " + flushed + " from "
+									+ oldest_file.getName()));
+				}
 				
 				if ( flushed == 0 ){
 				
@@ -339,7 +388,7 @@ CacheFileManagerImpl
 								// hmm, something wrong with cache as the flush should have got rid
 								// of at least the oldest entry
 							
-							throw( new CacheFileManagerException( "Cache inconsistent: 0 flushed"));
+							throw( new CacheFileManagerException( null, "Cache inconsistent: 0 flushed"));
 						}
 					}finally{
 						
@@ -351,11 +400,13 @@ CacheFileManagerImpl
 					
 		CacheEntry	entry = new CacheEntry( entry_type, file, buffer, file_position, length );
 			
-		if ( log ){
-				
-			LGLogger.log( 
-					"DiskCache: cr=" + cache_bytes_read + ",cw=" + cache_bytes_written+
-					",fr=" + file_bytes_read + ",fw=" + file_bytes_written ); 
+		if (log && Logger.isEnabled()) {
+			TOTorrentFile tf = file.getTorrentFile();
+			TOTorrent torrent = tf == null ? null : tf.getTorrent();
+
+			Logger.log(new LogEvent(torrent, LOGID, "DiskCache: cr="
+					+ cache_bytes_read + ",cw=" + cache_bytes_written + ",fr="
+					+ file_bytes_read + ",fw=" + file_bytes_written));
 		}
 			
 		return( entry );
@@ -364,106 +415,106 @@ CacheFileManagerImpl
 	protected void
 	cacheStatsAndCleaner()
 	{
-		long	cleaner_ticks	= CACHE_CLEANER_TICKS;
-		
-		while( true ){
 			
-			try{
-			
-				Thread.sleep( STATS_UPDATE_FREQUENCY );
-				
-			}catch( InterruptedException e ){
-				
-				Debug.printStackTrace( e );
-				
-				break;
-			}
-			
-			stats.update();
-			
-			
-			// System.out.println( "cache file count = " + cache_files.size());
-								
-			Iterator	cf_it = cache_files.keySet().iterator();
-			
-			while(cf_it.hasNext()){
-				
-				((CacheFileWithCache)cf_it.next()).updateStats();
-			}
-			
-			if ( --cleaner_ticks == 0 ){
-				
-				cleaner_ticks	= CACHE_CLEANER_TICKS;
-				
-				Set		dirty_files	= new HashSet();
-	
-				long	now 	= SystemTime.getCurrentTime();
-							
-				long	oldest	= now - DIRTY_CACHE_WRITE_MAX_AGE;
-				
-				try{
-					this_mon.enter();
-			
-					if ( updated_cache_files != null ){
-						
-						cache_files	= updated_cache_files;
-							
-						updated_cache_files	= null;
-					}
+		SimpleTimer.addPeriodicEvent(
+				"CacehFile:stats",
+				STATS_UPDATE_FREQUENCY,
+        new TimerEventPerformer() {
+          public void perform( TimerEvent ev ) {
+      			
+      			stats.update();      			
+      			
+      			// System.out.println( "cache file count = " + cache_files.size());
+      								
+      			Iterator	cf_it = cache_files.keySet().iterator();
+      			
+      			while(cf_it.hasNext()){
+      				
+      				((CacheFileWithCache)cf_it.next()).updateStats();
+      			}
+      			
+      			if ( --cleaner_ticks == 0 ){
+      				
+      				cleaner_ticks	= CACHE_CLEANER_TICKS;
+      				
+      				final Set	dirty_files	= new HashSet();
+      	
+      				final long	oldest	=SystemTime.getCurrentTime() - DIRTY_CACHE_WRITE_MAX_AGE;
+      				
+      				try{
+      					this_mon.enter();
+      			
+      					if ( updated_cache_files != null ){
+      						
+      						cache_files	= updated_cache_files;
+      							
+      						updated_cache_files	= null;
+      					}
 
-					if ( cache_entries.size() > 0 ){
-						
-						Iterator it = cache_entries.keySet().iterator();
-						
-						while( it.hasNext()){
-							
-							CacheEntry	entry = (CacheEntry)it.next();
-								
-							// System.out.println( "oldest entry = " + ( now - entry.getLastUsed()));
-							
-							if ( entry.isDirty()){
-								
-								dirty_files.add( entry.getFile());
-							}
-						}
-					}
-					
-					// System.out.println( "cache file = " + cache_files.size() + ", torrent map = " + torrent_to_cache_file_map.size());
-					
-				}finally{
-					
-					this_mon.exit();
-				}
-				
-				Iterator	it = dirty_files.iterator();
-				
-				while( it.hasNext()){
-					
-					try{
-						CacheFileWithCache	file = (CacheFileWithCache)it.next();
-						
-						TOTorrentFile	tf = file.getTorrentFile();
-						
-						long	min_flush_size	= -1;
-						
-						if ( tf != null ){
-							
-							min_flush_size	= tf.getTorrent().getPieceLength();
-							
-						}
-						
-						file.flushOldDirtyData( oldest, min_flush_size );
-						
-					}catch( Throwable e ){
-						
-							// if this fails then the error should reoccur on a "proper"
-							// flush later and be reported
-						
-						Debug.printStackTrace( e );
-					}
-				}
-			}
-		}
+      					if ( cache_entries.size() > 0 ){
+      						
+      						Iterator it = cache_entries.keySet().iterator();
+      						
+      						while( it.hasNext()){
+      							
+      							CacheEntry	entry = (CacheEntry)it.next();
+      								
+      							// System.out.println( "oldest entry = " + ( now - entry.getLastUsed()));
+      							
+      							if ( entry.isDirty()){
+      								
+      								dirty_files.add( entry.getFile());
+      							}
+      						}
+      					}
+      					
+      					// System.out.println( "cache file = " + cache_files.size() + ", torrent map = " + torrent_to_cache_file_map.size());
+      					
+      				}finally{
+      					
+      					this_mon.exit();
+      				}
+      				
+      				Iterator	it = dirty_files.iterator();
+      				
+      				while( it.hasNext()){
+      					
+      					CacheFileWithCache	file = (CacheFileWithCache)it.next();
+
+      					try{
+      						
+      						TOTorrentFile	tf = file.getTorrentFile();
+      						
+      						long	min_flush_size	= -1;
+      						
+      						if ( tf != null ){
+      							
+      							min_flush_size	= tf.getTorrent().getPieceLength();
+      							
+      						}
+      						
+      						file.flushOldDirtyData( oldest, min_flush_size );
+      						
+      					}catch( CacheFileManagerException e ){
+      						
+      						file.setPendingException( e );
+      						
+      							// if this fails then the error should reoccur on a "proper"
+      							// flush later and be reported
+      						
+      						Debug.printStackTrace( e );
+      						
+      					}catch( Throwable e ){
+      						
+      						Debug.printStackTrace( e );
+      					}
+      				}
+      			}
+      			
+          }
+        }
+     );
+		
 	}
 	
 		// must be called when the cachefileimpl is synchronised to ensure that the file's
@@ -510,7 +561,7 @@ CacheFileManagerImpl
 					
 					Debug.out( "Cache inconsistency: my count = " + my_count + ", file = " + file.cache.size());
 					
-					throw( new CacheFileManagerException( "Cache inconsistency: counts differ"));
+					throw( new CacheFileManagerException( null, "Cache inconsistency: counts differ"));
 					
 				}else{
 					
@@ -521,7 +572,7 @@ CacheFileManagerImpl
 					
 					Debug.out( "Cache inconsistency: used_size = " + total_cache_size + ", free = " + cache_space_free + ", size = " + cache_size );
 					
-					throw( new CacheFileManagerException( "Cache inconsistency: sizes differ"));
+					throw( new CacheFileManagerException( null, "Cache inconsistency: sizes differ"));
 					
 				}else{
 					
@@ -549,7 +600,7 @@ CacheFileManagerImpl
 				
 				Debug.out( "Cache inconsistency: entry missing on usage" );
 				
-				throw( new CacheFileManagerException( "Cache inconsistency: entry missing on usage"));
+				throw( new CacheFileManagerException( null, "Cache inconsistency: entry missing on usage"));
 				
 			}else{
 				
@@ -578,7 +629,7 @@ CacheFileManagerImpl
 				
 				Debug.out( "Cache inconsistency: entry missing on removal" );
 
-				throw( new CacheFileManagerException( "Cache inconsistency: entry missing on removal"));
+				throw( new CacheFileManagerException( null, "Cache inconsistency: entry missing on removal"));
 			}
 
 			/*
@@ -820,7 +871,8 @@ CacheFileManagerImpl
 	
 	protected void
 	rethrow(
-		FMFileManagerException e )
+		CacheFile				file,
+		FMFileManagerException 	e )
 	
 		throws CacheFileManagerException
 	{
@@ -828,9 +880,80 @@ CacheFileManagerImpl
 		
 		if ( cause != null ){
 			
-			throw( new CacheFileManagerException( e.getMessage(), cause ));
+			throw( new CacheFileManagerException( file, e.getMessage(), cause ));
 		}
 		
-		throw( new CacheFileManagerException( e.getMessage(), e ));
+		throw( new CacheFileManagerException( file, e.getMessage(), e ));
+	}
+	
+	public void
+	generate(
+		IndentWriter		writer )
+	{
+		writer.println( "Cache Manager" );
+			
+		try{
+			writer.indent();
+			
+			try{
+				this_mon.enter();
+
+				writer.println( "Entries = " + cache_entries.size());
+				
+				Iterator it = cache_entries.keySet().iterator();
+				
+				Set	files = new HashSet();
+				
+				while( it.hasNext()){
+					
+					CacheEntry	entry = (CacheEntry)it.next();
+					
+					CacheFileWithCache file = entry.getFile();
+					
+					if( !files.contains( file )){
+						
+						files.add( file );
+						
+						TOTorrentFile torrentFile = file.getTorrentFile();
+						String fileLength = "";
+						try {
+							fileLength = "" + file.getLength();
+						} catch (Exception e) {
+							if (torrentFile != null)
+								fileLength = "" + torrentFile.getLength();
+						}
+						String	hash = "<unknown>";
+						
+						try{
+							if (torrentFile != null)
+								hash = ByteFormatter.encodeString( torrentFile.getTorrent().getHash());
+							
+						}catch( Throwable e ){
+						}
+						
+						String name = file.getName();
+
+						writer.println("File: " + Debug.secretFileName(name) + ", size "
+								+ fileLength + ", torrent " + hash + ", access = "
+								+ file.getAccessMode());
+					}
+				}
+				
+			}finally{
+				
+				this_mon.exit();
+			}
+		}finally{
+			
+			writer.exdent();
+		}
+	}
+	
+	public void
+	setFileLinks(
+		TOTorrent				torrent,
+		CaseSensitiveFileMap	links )
+	{
+		file_manager.setFileLinks( torrent, links );
 	}
 }
