@@ -1,7 +1,7 @@
 /*
  * Created on 03-Aug-2004
  * Created by Paul Gardner
- * Copyright (C) 2004 Aelitis, All Rights Reserved.
+ * Copyright (C) 2004, 2005, 2006 Aelitis, All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  * 
- * AELITIS, SARL au capital de 30,000 euros
+ * AELITIS, SAS au capital de 46,603.30 euros
  * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
  *
  */
@@ -42,7 +42,11 @@ public class
 CacheFileWithCache 
 	implements CacheFile
 {
-	protected static Comparator comparator = new
+  // Make code prettier by bringing over SS_CACHE from DirectByteBuffer
+  private static final byte SS_CACHE = DirectByteBuffer.SS_CACHE;
+  private static final LogIDs LOGID = LogIDs.CACHE;
+
+  protected static Comparator comparator = new
 		Comparator()
 		{
 			public int 
@@ -93,7 +97,9 @@ CacheFileWithCache
 	
 	protected CacheFileManagerImpl		manager;
 	protected FMFile					file;
+	protected int						access_mode	= CF_READ;
 	protected TOTorrentFile				torrent_file;
+	protected TOTorrent      			torrent;
 	protected long						file_offset_in_torrent;
 	
 	protected long[]					read_history		= new long[ READAHEAD_HISTORY ];
@@ -117,11 +123,12 @@ CacheFileWithCache
 	
 	protected int piece_size						= 0;
 	protected int piece_offset						= 0;
-	
-	protected int file_offset						= 0;
-	
+		
 	protected AEMonitor				this_mon		= new AEMonitor( "CacheFile" );
 	
+	protected volatile CacheFileManagerException	pending_exception;
+	
+	
 	protected
 	CacheFileWithCache(
 		CacheFileManagerImpl	_manager,
@@ -137,7 +144,7 @@ CacheFileWithCache
 			
 			torrent_file	= _torrent_file;
 			
-			TOTorrent	torrent = torrent_file.getTorrent();
+			torrent = torrent_file.getTorrent();
 					
 			piece_size	= (int)torrent.getPieceLength();
 									
@@ -164,6 +171,12 @@ CacheFileWithCache
 		}
 	}
 	
+	public TOTorrentFile
+	getTorrentFile()
+	{
+		return( torrent_file );
+	}
+	
 	protected void
 	updateStats()
 	{
@@ -228,23 +241,26 @@ CacheFileWithCache
 	readCache(
 		final DirectByteBuffer	file_buffer,
 		final long				file_position,
-		final boolean			recursive )
+		final boolean			recursive,
+		final boolean			disable_read_cache )
 	
 		throws CacheFileManagerException
 	{
-		final int	file_buffer_position	= file_buffer.position(DirectByteBuffer.SS_CACHE);
-		final int	file_buffer_limit		= file_buffer.limit(DirectByteBuffer.SS_CACHE);
+		checkPendingException();
+		
+		final int	file_buffer_position	= file_buffer.position(SS_CACHE);
+		final int	file_buffer_limit		= file_buffer.limit(SS_CACHE);
 		
 		final int	read_length	= file_buffer_limit - file_buffer_position;
 	
 		try{
 			if ( manager.isCacheEnabled()){
 			
-				if ( TRACE ){
-					LGLogger.log( 
-							"readCache: " + getName() + ", " + file_position + " - " + (file_position + read_length - 1 ) + 
-							":" + file_buffer_position + "/" + file_buffer_limit );
-				}
+				if (TRACE)
+					Logger.log(new LogEvent(torrent, LOGID, "readCache: " + getName()
+							+ ", " + file_position + " - "
+							+ (file_position + read_length - 1) + ":" + file_buffer_position
+							+ "/" + file_buffer_limit));
 				
 				if ( read_length == 0 ){
 					
@@ -311,33 +327,34 @@ CacheFileWithCache
 							
 							DirectByteBuffer	entry_buffer = entry.getBuffer();
 							
-							int					entry_buffer_position 	= entry_buffer.position(DirectByteBuffer.SS_CACHE);
-							int					entry_buffer_limit		= entry_buffer.limit(DirectByteBuffer.SS_CACHE);
+							int					entry_buffer_position 	= entry_buffer.position(SS_CACHE);
+							int					entry_buffer_limit		= entry_buffer.limit(SS_CACHE);
 							
 							try{
 															
-								entry_buffer.limit( DirectByteBuffer.SS_CACHE, entry_buffer_position + skip + available );
+								entry_buffer.limit( SS_CACHE, entry_buffer_position + skip + available );
 								
-								entry_buffer.position( DirectByteBuffer.SS_CACHE, entry_buffer_position + skip );
+								entry_buffer.position( SS_CACHE, entry_buffer_position + skip );
 								
-								if ( TRACE ){
-									LGLogger.log( 
-											"cacheRead: using " + entry.getString() + 
-											"[" + entry_buffer.position(DirectByteBuffer.SS_CACHE)+"/"+entry_buffer.limit(DirectByteBuffer.SS_CACHE)+ "]" +
-											"to write to [" + file_buffer.position(DirectByteBuffer.SS_CACHE) + "/" + file_buffer.limit(DirectByteBuffer.SS_CACHE) + "]" );
-								}
+								if (TRACE)
+									Logger.log(new LogEvent(torrent, LOGID, "cacheRead: using "
+											+ entry.getString() + "["
+											+ entry_buffer.position(SS_CACHE) + "/"
+											+ entry_buffer.limit(SS_CACHE) + "]" + "to write to ["
+											+ file_buffer.position(SS_CACHE) + "/"
+											+ file_buffer.limit(SS_CACHE) + "]"));
 								
 								used_entries++;
 								
-								file_buffer.put( DirectByteBuffer.SS_CACHE, entry_buffer );
+								file_buffer.put( SS_CACHE, entry_buffer );
 									
 								manager.cacheEntryUsed( entry );
 								
 							}finally{
 								
-								entry_buffer.limit( DirectByteBuffer.SS_CACHE, entry_buffer_limit );
+								entry_buffer.limit( SS_CACHE, entry_buffer_limit );
 								
-								entry_buffer.position( DirectByteBuffer.SS_CACHE, entry_buffer_position );						
+								entry_buffer.position( SS_CACHE, entry_buffer_position );						
 							}
 							
 							writing_file_position	+= available;
@@ -370,21 +387,19 @@ CacheFileWithCache
 						manager.cacheBytesRead( read_length );
 					}
 						
-					if ( TRACE ){
-							
-						LGLogger.log( "cacheRead: cache use ok [entries = " + used_entries + "]" );
-					}
+					if (TRACE)
+						Logger.log(new LogEvent(torrent, LOGID,
+								"cacheRead: cache use ok [entries = " + used_entries + "]"));
 										
 				}else{
 						
-					if ( TRACE ){
-							
-						LGLogger.log( "cacheRead: cache use fails, reverting to plain read" );
-					}
+					if (TRACE)
+						Logger.log(new LogEvent(torrent, LOGID,
+								"cacheRead: cache use fails, reverting to plain read"));
 								
 						// reset in case we've done some partial reads
 						
-					file_buffer.position( DirectByteBuffer.SS_CACHE, file_buffer_position );
+					file_buffer.position( SS_CACHE, file_buffer_position );
 					
 						// If read-ahead fails then we resort to a straight read
 						// Read-ahead can fail if a cache-flush fails (e.g. out of disk space
@@ -397,6 +412,7 @@ CacheFileWithCache
 							boolean	do_read_ahead	= 
 										i == 0 &&		// first time round
 										!recursive &&
+										!disable_read_cache && 
 										manager.isReadCacheEnabled() &&
 										read_length <  current_read_ahead_size &&
 										file_position + current_read_ahead_size <= file.getLength();
@@ -424,7 +440,7 @@ CacheFileWithCache
 							
 									// don't read ahead over the end of a piece
 								
-								int	request_piece_offset = (int)((file_position - ( piece_offset + file_offset )) % piece_size);
+								int	request_piece_offset = (int)((file_position - piece_offset ) % piece_size);
 								
 								if ( request_piece_offset < 0 ){
 									
@@ -452,10 +468,9 @@ CacheFileWithCache
 							
 							if ( do_read_ahead ){
 									
-								if ( TRACE ){
-										
-									LGLogger.log( "\tperforming read-ahead" );
-								}
+								if (TRACE)
+									Logger.log(new LogEvent(torrent, LOGID,
+											"\tperforming read-ahead"));
 									
 								DirectByteBuffer	cache_buffer = 
 										DirectByteBufferPool.getBuffer( DirectByteBuffer.AL_CACHE_READ, actual_read_ahead );
@@ -488,7 +503,7 @@ CacheFileWithCache
 										
 										manager.fileBytesRead( actual_read_ahead );
 											
-										cache_buffer.position( DirectByteBuffer.SS_CACHE, 0 );
+										cache_buffer.position( SS_CACHE, 0 );
 										
 										cache.add( entry );
 										
@@ -516,14 +531,13 @@ CacheFileWithCache
 									// there is the possibility that it could be flushed before then - hence the
 									// recursion flag that will avoid this happening next time around
 								
-								readCache( file_buffer, file_position, true );
+								readCache( file_buffer, file_position, true, disable_read_cache );
 							
 							}else{
 									
-								if ( TRACE ){
-										
-									LGLogger.log( "\tnot performing read-ahead" );
-								}
+								if (TRACE)
+									Logger.log(new LogEvent(torrent, LOGID,
+											"\tnot performing read-ahead"));
 									
 								try{
 									
@@ -554,7 +568,7 @@ CacheFileWithCache
 							
 							if ( i == 1 ){
 								
-								manager.rethrow(e);
+								manager.rethrow(this,e);
 							}
 						}
 					}				
@@ -568,7 +582,7 @@ CacheFileWithCache
 		
 				}catch( FMFileManagerException e ){
 						
-					manager.rethrow(e);
+					manager.rethrow(this,e);
 				}
 			}
 		}finally{
@@ -577,18 +591,18 @@ CacheFileWithCache
 				
 				long	temp_position = file_position + file_offset_in_torrent;
 			
-				file_buffer.position( DirectByteBuffer.SS_CACHE, file_buffer_position );
+				file_buffer.position( SS_CACHE, file_buffer_position );
 				
-				while( file_buffer.hasRemaining( DirectByteBuffer.SS_CACHE )){
+				while( file_buffer.hasRemaining( SS_CACHE )){
 						
-					byte	v = file_buffer.get( DirectByteBuffer.SS_CACHE );
+					byte	v = file_buffer.get( SS_CACHE );
 						
 					if ((byte)temp_position != v ){
 							
 						System.out.println( "readCache: read is bad at " + temp_position +
 											": expected = " + (byte)temp_position + ", actual = " + v );
 				
-						file_buffer.position( DirectByteBuffer.SS_CACHE, file_buffer_limit );
+						file_buffer.position( SS_CACHE, file_buffer_limit );
 						
 						break;
 					}
@@ -607,12 +621,14 @@ CacheFileWithCache
 	
 		throws CacheFileManagerException
 	{
+		checkPendingException();
+		
 		boolean	buffer_cached	= false;
 		boolean	failed			= false;
 		
 		try{
-			int	file_buffer_position	= file_buffer.position(DirectByteBuffer.SS_CACHE);
-			int file_buffer_limit		= file_buffer.limit(DirectByteBuffer.SS_CACHE);
+			int	file_buffer_position	= file_buffer.position(SS_CACHE);
+			int file_buffer_limit		= file_buffer.limit(SS_CACHE);
 			
 			int	write_length = file_buffer_limit - file_buffer_position;
 			
@@ -625,9 +641,9 @@ CacheFileWithCache
 			
 				long	temp_position = file_position + file_offset_in_torrent;
 				
-				while( file_buffer.hasRemaining( DirectByteBuffer.SS_CACHE )){
+				while( file_buffer.hasRemaining( SS_CACHE )){
 					
-					byte	v = file_buffer.get( DirectByteBuffer.SS_CACHE );
+					byte	v = file_buffer.get( SS_CACHE );
 					
 					if ((byte)temp_position != v ){
 						
@@ -640,35 +656,35 @@ CacheFileWithCache
 					temp_position++;
 				}
 				
-				file_buffer.position( DirectByteBuffer.SS_CACHE, file_buffer_position );
+				file_buffer.position( SS_CACHE, file_buffer_position );
 			}
 			
 			if ( manager.isWriteCacheEnabled() ){
 				
-				if ( TRACE ){
-					
-					LGLogger.log( 
-							"writeCache: " + getName() + ", " + file_position + " - " + (file_position + write_length - 1 ) + 
-							":" + file_buffer_position + "/" + file_buffer_limit );
-				}
+				if (TRACE)
+					Logger.log(new LogEvent(torrent, LOGID, "writeCache: " + getName()
+							+ ", " + file_position + " - "
+							+ (file_position + write_length - 1) + ":" + file_buffer_position
+							+ "/" + file_buffer_limit));
 				
-					// if the data is smaller than a piece and not handed over then it is most
-					// likely apart of a piece at the start or end of a file. If so, copy it
+					// if the data is smaller than a piece and not handed over
+                    // then it is most
+					// likely apart of a piece at the start or end of a file. If
+                    // so, copy it
 					// and insert the copy into cache
 							
 				if ( 	( !buffer_handed_over ) &&
 						write_length < piece_size ){
 				
-					if ( TRACE ){
-						
-						LGLogger.log( "    making copy of non-handedover buffer" );
-					}
+					if (TRACE)
+						Logger.log(new LogEvent(torrent, LOGID,
+								"    making copy of non-handedover buffer"));
 					
 					DirectByteBuffer	cache_buffer = DirectByteBufferPool.getBuffer( DirectByteBuffer.AL_CACHE_WRITE, write_length );
 										
-					cache_buffer.put( DirectByteBuffer.SS_CACHE, file_buffer );
+					cache_buffer.put( SS_CACHE, file_buffer );
 					
-					cache_buffer.position( DirectByteBuffer.SS_CACHE, 0 );
+					cache_buffer.position( SS_CACHE, 0 );
 					
 						// make it look like this buffer has been handed over
 					
@@ -693,9 +709,13 @@ CacheFileWithCache
 								write_length );
 					
 					try{
-
 						this_mon.enter();
 						
+						if ( access_mode != CF_WRITE ){
+							
+							throw( new CacheFileManagerException( this,"Write failed - cache file is read only" ));
+						}
+						
 							// if we are overwriting stuff already in the cache then force-write overlapped
 							// data (easiest solution as this should only occur on hash-fails)
 
@@ -755,7 +775,7 @@ CacheFileWithCache
 			
 			failed	= true;
 			
-			manager.rethrow(e);
+			manager.rethrow(this,e);
 			
 		}finally{
 			
@@ -782,8 +802,37 @@ CacheFileWithCache
 		throws CacheFileManagerException
 	{
 		try{
-			this_mon.enter();
+			flushCacheSupport( file_position, length, release_entries, minimum_to_release, oldest_dirty_time, min_chunk_size );
 			
+		}catch( CacheFileManagerException	e ){
+			
+			if ( !release_entries ){
+			
+					// make sure we release the offending buffer entries otherwise they'll hang around
+					// in memory causing grief when the next attempt it made to flush them...
+				
+				flushCacheSupport( 0, -1, true, -1, 0, -1 );
+			}
+			
+			throw( e );
+		}
+	}
+	
+	protected void
+	flushCacheSupport(
+		long				file_position,
+		long				length,					// -1 -> do all from position onwards
+		boolean				release_entries,
+		long				minimum_to_release,		// -1 -> all
+		long				oldest_dirty_time, 		// dirty entries newer than this won't be flushed
+													// 0 -> now
+		long				min_chunk_size )		// minimum contiguous size for flushing, -1 -> no limit
+	
+		throws CacheFileManagerException
+	{
+		try{
+			this_mon.enter();	
+
 			if ( cache.size() == 0 ){
 				
 				return;
@@ -883,11 +932,11 @@ CacheFileWithCache
 							multi_block_entries.add( entry );
 							
 							if ( skip_chunk ){
-								
-								if ( TRACE ){
-									
-									LGLogger.log( "flushCache: skipping " + multi_block_entries.size() + " entries, [" + multi_block_start + "," + multi_block_next + "] as too small" );			
-								}
+								if (TRACE)
+									Logger.log(new LogEvent(torrent, LOGID,
+											"flushCache: skipping " + multi_block_entries.size()
+													+ " entries, [" + multi_block_start + ","
+													+ multi_block_next + "] as too small"));			
 							}else{
 								
 								multiBlockFlush(
@@ -899,9 +948,7 @@ CacheFileWithCache
 						}
 					}
 				}catch( Throwable e ){
-					
-					Debug.out( "cacheFlush fails: " + e.getMessage());
-					
+										
 					last_failure	= e;
 					
 				}finally{
@@ -946,10 +993,12 @@ CacheFileWithCache
 
 				if ( skip_chunk ){
 					
-					if ( TRACE ){
-						
-						LGLogger.log( "flushCache: skipping " + multi_block_entries.size() + " entries, [" + multi_block_start + "," + multi_block_next + "] as too small" );			
-					}
+					if (TRACE)
+						Logger
+								.log(new LogEvent(torrent, LOGID, "flushCache: skipping "
+										+ multi_block_entries.size() + " entries, ["
+										+ multi_block_start + "," + multi_block_next
+										+ "] as too small"));			
 					
 				}else{
 					
@@ -968,7 +1017,7 @@ CacheFileWithCache
 					throw((CacheFileManagerException)last_failure );
 				}
 				
-				throw( new CacheFileManagerException( "cache flush failed", last_failure ));
+				throw( new CacheFileManagerException( this,"cache flush failed", last_failure ));
 			}
 		}finally{
 			
@@ -988,10 +1037,10 @@ CacheFileWithCache
 		boolean	write_ok	= false;
 		
 		try{
-			if ( TRACE ){
-				
-				LGLogger.log( "multiBlockFlush: writing " + multi_block_entries.size() + " entries, [" + multi_block_start + "," + multi_block_next + "," + release_entries + "]" );			
-			}
+			if (TRACE)
+				Logger.log(new LogEvent(torrent, LOGID, "multiBlockFlush: writing "
+						+ multi_block_entries.size() + " entries, [" + multi_block_start
+						+ "," + multi_block_next + "," + release_entries + "]"));			
 			
 			DirectByteBuffer[]	buffers = new DirectByteBuffer[ multi_block_entries.size()];
 			
@@ -1005,9 +1054,9 @@ CacheFileWithCache
 			
 				DirectByteBuffer	buffer = entry.getBuffer();
 				
-				if ( buffer.limit(DirectByteBuffer.SS_CACHE) - buffer.position(DirectByteBuffer.SS_CACHE) != entry.getLength()){
+				if ( buffer.limit(SS_CACHE) - buffer.position(SS_CACHE) != entry.getLength()){
 					
-					throw( new CacheFileManagerException( "flush: inconsistent entry length, position wrong" ));
+					throw( new CacheFileManagerException( this,"flush: inconsistent entry length, position wrong" ));
 				}
 				
 				expected_per_entry_write	+= entry.getLength();
@@ -1019,7 +1068,7 @@ CacheFileWithCache
 
 			if ( expected_per_entry_write != expected_overall_write ){
 		
-				throw( new CacheFileManagerException( "flush: inconsistent write length, entrys = " + expected_per_entry_write + " overall = " + expected_overall_write ));
+				throw( new CacheFileManagerException( this,"flush: inconsistent write length, entrys = " + expected_per_entry_write + " overall = " + expected_overall_write ));
 				
 			}
 			
@@ -1031,7 +1080,7 @@ CacheFileWithCache
 			
 		}catch( FMFileManagerException e ){
 			
-			throw( new CacheFileManagerException( "flush fails", e ));
+			throw( new CacheFileManagerException( this,"flush fails", e ));
 			
 		}finally{			
 			
@@ -1066,22 +1115,28 @@ CacheFileWithCache
 	{
 		if ( manager.isCacheEnabled()){
 							
-			if ( TRACE ){
-					
-				LGLogger.log( "flushCache: " + getName() + ", rel = " + release_entries + ", min = " + minumum_to_release );
-			}
+			if (TRACE)
+				Logger.log(new LogEvent(torrent, LOGID, "flushCache: " + getName()
+						+ ", rel = " + release_entries + ", min = " + minumum_to_release));
 			
 			flushCache( file_start_position, -1, release_entries, minumum_to_release, 0, -1 );
 		}
 	}
 	
+		// this is the flush method used by the public methods directly (as opposed to those use when reading, writing etc)
+		// and it is the place that pending exceptions are checked for. We don't want to check for this in the internal
+		// logic for flushing as we need to be able to flush from files that have a pending error to clear the cache
+		// state
+	
 	protected void
-	flushCache(
+	flushCachePublic(
 		boolean				release_entries,
 		long				minumum_to_release )
 	
 		throws CacheFileManagerException
 	{
+		checkPendingException();
+		
 		flushCache(0, release_entries, minumum_to_release );
 	}
 	
@@ -1094,10 +1149,9 @@ CacheFileWithCache
 	{
 		if ( manager.isCacheEnabled()){
 			
-			if ( TRACE ){
-	
-				LGLogger.log( "flushOldDirtyData: " + getName());
-			}
+			if (TRACE)
+				Logger.log(new LogEvent(torrent, LOGID, "flushOldDirtyData: "
+						+ getName()));
 
 			flushCache( 0, -1, false, -1, oldest_dirty_time, min_chunk_size );
 		}
@@ -1162,11 +1216,28 @@ CacheFileWithCache
 	
 		// support methods
 	
+	protected void
+	checkPendingException()
 	
+		throws CacheFileManagerException
+	{
+		if ( pending_exception != null ){
+			
+			throw( pending_exception );
+		}
+	}
+	
+	protected void
+	setPendingException(
+		CacheFileManagerException	e )
+	{
+		pending_exception	= e;
+	}
+
 	protected String
 	getName()
 	{
-		return( file.getFile().toString());
+		return( file.getName());
 	}
 	
 	protected FMFile
@@ -1175,20 +1246,14 @@ CacheFileWithCache
 		return( file );
 	}
 	
-	protected TOTorrentFile
-	getTorrentFile()
-	{
-		return( torrent_file );
-	}
-	
 		// public methods
 	
-	public File
-	getFile()
+	public boolean
+	exists()
 	{
-		return( file.getFile());
+		return( file.exists());
 	}
-
+	
 	public void
 	moveFile(
 		File		new_file )
@@ -1196,13 +1261,13 @@ CacheFileWithCache
 		throws CacheFileManagerException
 	{
 		try{
-			flushCache( true, -1 );
+			flushCachePublic( true, -1 );
 			
 			file.moveFile( new_file );
 			
 		}catch( FMFileManagerException e ){
 			
-			manager.rethrow(e);
+			manager.rethrow(this,e);
 		}	
 	}
 	
@@ -1213,38 +1278,79 @@ CacheFileWithCache
 		throws CacheFileManagerException
 	{
 		try{
-			if ( getAccessMode() != mode ){
-				
-				flushCache( false, -1 );
-			}
+			this_mon.enter();
 			
+			if ( access_mode != mode ){
+				
+				flushCachePublic( false, -1 );
+			}
+							
 			file.setAccessMode( mode==CF_READ?FMFile.FM_READ:FMFile.FM_WRITE );
 			
+			access_mode	= mode;
+
 		}catch( FMFileManagerException e ){
 			
-			manager.rethrow(e);
-		}	
+			manager.rethrow(this,e);
+			
+		}finally{
+			
+			this_mon.exit();
+		}
 	}
 	
 	public int
 	getAccessMode()
 	{
-		return( file.getAccessMode()==FMFile.FM_READ?CF_READ:CF_WRITE );
+		return( access_mode );
 	}
 	
 	public void
-	ensureOpen()
+	setStorageType(
+		int		type )
+	
+		throws CacheFileManagerException
+	{
+		try{
+			this_mon.enter();
+			
+			if ( getStorageType() != type ){
+				
+				flushCachePublic( false, -1 );
+			}
+			
+			file.setStorageType( type==CT_COMPACT?FMFile.FT_COMPACT:FMFile.FT_LINEAR );
+			
+		}catch( FMFileManagerException e ){
+			
+			manager.rethrow(this,e);
+			
+		}finally{
+			
+			this_mon.exit();
+		}		
+	}
+	
+	public int
+	getStorageType()
+	{
+		return( file.getStorageType()==FMFile.FT_COMPACT?CT_COMPACT:CT_LINEAR );
+	}
+	
+	public void
+	ensureOpen(
+		String	reason )
 
 		throws CacheFileManagerException
 	{
 		try{
 				// no cache flush required here
 			
-			file.ensureOpen();
+			file.ensureOpen( reason );
 			
 		}catch( FMFileManagerException e ){
 			
-			manager.rethrow(e);
+			manager.rethrow(this,e);
 		}	
 	}
 
@@ -1253,17 +1359,88 @@ CacheFileWithCache
 	
 		throws CacheFileManagerException
 	{
+		
+			// not sure of the difference between "size" and "length" here. Old code
+			// used to use "size" so I'm going to carry on for the moment in case
+			// there is some weirdness here
+		
 		try{
+				// bug found here with "incremental creation" failing with lots of hash
+				// fails. Caused by the reported length not taking into account the cache
+				// entries that have yet to be flushed.			
 			
-				// not sure of the difference between "size" and "length" here. Old code
-				// used to use "size" so I'm going to carry on for the moment in case
-				// there is some weirdness here
+			if ( manager.isCacheEnabled()){
+				
+				try{
+					this_mon.enter();
 			
-			return( file.getSize());
+					long	physical_size = file.getLength();
+
+					Iterator	it = cache.iterator();
+					
+						// last entry is furthest down the file
+					
+					while( it.hasNext()){
+					
+						CacheEntry	entry = (CacheEntry)it.next();
+					
+						if ( !it.hasNext()){
+							
+							long	entry_file_position 	= entry.getFilePosition();
+							int		entry_length			= entry.getLength();
+
+							long	logical_size = entry_file_position + entry_length;
+							
+							if ( logical_size > physical_size ){
+								
+								physical_size	= logical_size;								
+							}
+						}
+					}
+					
+					return( physical_size );
+
+				}finally{
+					
+					this_mon.exit();
+				}
+			}else{
+				
+				return( file.getLength());
+			}
+						
+		}catch( FMFileManagerException e ){
+			
+			manager.rethrow(this,e);
+			
+			return( 0 );
+		}
+	}
+	
+	public long
+	compareLength(
+		long	compare_to )
+	
+		throws CacheFileManagerException
+	{
+		try{
+				// we can optimise this if the file's already big enough as cache entries can
+				// only make it bigger
+			
+			long	physical_length = file.getLength();
+			
+			long	res = physical_length - compare_to;
+			
+			if ( res >= 0 ){
+				
+				return( res );
+			}
+			
+			return( getLength() - compare_to );
 			
 		}catch( FMFileManagerException e ){
 			
-			manager.rethrow(e);
+			manager.rethrow(this,e);
 			
 			return( 0 );
 		}
@@ -1279,24 +1456,64 @@ CacheFileWithCache
 			
 				// flush in case length change will invalidate cache data (unlikely but possible) 
 			
-			flushCache( true, -1 );
+			flushCachePublic( true, -1 );
 			
 			file.setLength( length );
 			
 		}catch( FMFileManagerException e ){
 			
-			manager.rethrow(e);
+			manager.rethrow(this,e);
+		}
+	}
+	
+	public void
+	read(
+		DirectByteBuffer[]	buffers,
+		long				position,
+		short				policy )
+	
+		throws CacheFileManagerException
+	{
+		for (int i=0;i<buffers.length;i++){
+			
+			DirectByteBuffer	buffer = buffers[i];
+			
+			int	len = buffer.remaining( DirectByteBuffer.SS_CACHE );
+			
+			try{
+				read( buffer, position, policy );
+				
+				position += len;
+				
+			}catch( CacheFileManagerException e ){
+				
+				throw( new CacheFileManagerException( this, e.getMessage(), e, i ));
+			}
 		}
 	}
 	
 	public void
 	read(
 		DirectByteBuffer	buffer,
-		long				position )
+		long				position,
+		short				policy )
 	
 		throws CacheFileManagerException
 	{
-		readCache( buffer, position, false );
+		boolean	read_cache 	= ( policy & CP_READ_CACHE ) != 0;
+		boolean	flush		= ( policy & CP_FLUSH ) != 0;
+				
+		if ( flush ){
+			
+			int	file_buffer_position	= buffer.position(DirectByteBuffer.SS_CACHE);
+			int	file_buffer_limit		= buffer.limit(DirectByteBuffer.SS_CACHE);
+			
+			int	read_length	= file_buffer_limit - file_buffer_position;
+					
+			flushCache( position, read_length, false, -1, 0, -1 );
+		}
+		
+		readCache( buffer, position, false, !read_cache );
 	}
 		
 	public void
@@ -1309,6 +1526,31 @@ CacheFileWithCache
 		writeCache( buffer, position, false );
 	}
 	
+	public void
+	write(
+		DirectByteBuffer[]	buffers,
+		long				position )
+	
+		throws CacheFileManagerException
+	{
+		for (int i=0;i<buffers.length;i++){
+			
+			DirectByteBuffer	buffer = buffers[i];
+			
+			int	len = buffer.remaining( DirectByteBuffer.SS_CACHE );
+			
+			try{
+				write( buffer, position );
+			
+				position += len;
+				
+			}catch( CacheFileManagerException e ){
+				
+				throw( new CacheFileManagerException( this, e.getMessage(), e, i ));
+			}
+		}
+	}
+	
 	public void
 	writeAndHandoverBuffer(
 		DirectByteBuffer	buffer,
@@ -1319,12 +1561,45 @@ CacheFileWithCache
 		writeCache( buffer, position, true );
 	}
 	
+	public void
+	writeAndHandoverBuffers(
+		DirectByteBuffer[]	buffers,
+		long				position )
+	
+		throws CacheFileManagerException
+	{
+		for (int i=0;i<buffers.length;i++){
+			
+			DirectByteBuffer	buffer = buffers[i];
+			
+			int	len = buffer.remaining( DirectByteBuffer.SS_CACHE );
+			
+			try{
+				writeAndHandoverBuffer( buffer, position );
+			
+				position += len;
+				
+			}catch( CacheFileManagerException e ){
+				
+				throw( new CacheFileManagerException( this, e.getMessage(), e, i ));
+			}
+		}
+	}
+	
 	public void
 	flushCache()
 	
 		throws CacheFileManagerException
 	{
-		flushCache( false, -1 );
+		try{
+			flushCachePublic( false, -1 );
+			
+			file.flush();
+			
+		}catch( FMFileManagerException e ){
+			
+			manager.rethrow(this,e);
+		}
 	}
 	
 	public void
@@ -1332,7 +1607,7 @@ CacheFileWithCache
 	
 		throws CacheFileManagerException
 	{
-		flushCache(true, -1);
+		flushCachePublic(true, -1);
 	}
 	
 	public void
@@ -1345,7 +1620,7 @@ CacheFileWithCache
 		boolean	fm_file_closed = false;
 		
 		try{
-			flushCache( true, -1 );
+			flushCachePublic( true, -1 );
 			
 			file.close();
 			
@@ -1353,7 +1628,7 @@ CacheFileWithCache
 			
 		}catch( FMFileManagerException e ){
 			
-			manager.rethrow(e);
+			manager.rethrow(this,e);
 			
 		}finally{
 			
@@ -1366,18 +1641,26 @@ CacheFileWithCache
 					
 						// we're already on our way out via exception, no need to
 						// throw a new one
-					
-					Debug.printStackTrace( e );
+
 				}
 			}
+			
 			manager.closeFile( this );
 		}
 	}
 	
 	public void
-	setFileOffset(
-		int		_file_offset )
+	delete()
+	
+		throws CacheFileManagerException
 	{
-		file_offset		= _file_offset;
+		try{
+			
+			file.delete();
+			
+		}catch( FMFileManagerException e ){
+			
+			manager.rethrow(this,e);			
+		}
 	}
 }
