@@ -52,6 +52,7 @@ import com.aelitis.azureus.core.instancemanager.AZInstanceManager;
 import com.aelitis.azureus.core.instancemanager.AZInstanceManagerFactory;
 import com.aelitis.azureus.core.nat.NATTraverser;
 import com.aelitis.azureus.core.networkmanager.NetworkManager;
+import com.aelitis.azureus.core.networkmanager.admin.NetworkAdmin;
 import com.aelitis.azureus.core.peermanager.PeerManager;
 import com.aelitis.azureus.core.peermanager.download.session.TorrentSessionManager;
 import com.aelitis.azureus.core.peermanager.nat.PeerNATTraverser;
@@ -152,6 +153,10 @@ AzureusCoreImpl
     
 		AEThread.setOurThread();
 		
+			// set up a backwards pointer from config -> app dir so we can derive one from the other. It'll get saved on closedown, no need to do so now
+				
+		COConfigurationManager.setParameter( "azureus.application.directory", SystemProperties.getApplicationPath());
+		
 		crypto_manager = CryptoManagerFactory.getSingleton();
 		
 		PlatformManagerFactory.getPlatformManager().addListener(
@@ -189,6 +194,8 @@ AzureusCoreImpl
 					this,
 					new SpeedManagerAdapter()
 					{
+						private boolean setting_limits;
+						
 						public int
 						getCurrentProtocolUploadSpeed()
 						{
@@ -230,6 +237,7 @@ AzureusCoreImpl
 							if ( k_per_second == 0 ){
 								
 								bytes_per_second = Integer.MAX_VALUE;
+								
 							}else{
 								
 								bytes_per_second = k_per_second*1024;
@@ -261,17 +269,90 @@ AzureusCoreImpl
 							}
 						}
 						
+						public int
+						getCurrentDownloadLimit()
+						{
+							return( COConfigurationManager.getIntParameter( TransferSpeedValidator.getDownloadParameter()) * 1024 );
+						}
+						
 						public void
 						setCurrentDownloadLimit(
 							int		bytes_per_second )
 						{
 							COConfigurationManager.setParameter( TransferSpeedValidator.getDownloadParameter(), bytes_per_second/1024 );
 						}
+						
+						public Object
+						getLimits()
+						{
+							String up_key 	= TransferSpeedValidator.getActiveUploadParameter( global_manager );
+							String down_key	= TransferSpeedValidator.getDownloadParameter();
+							
+							return( 
+								new Object[]{
+									up_key,
+									new Integer( COConfigurationManager.getIntParameter( up_key )),
+									down_key,
+									new Integer( COConfigurationManager.getIntParameter( down_key )),
+								});
+						}
+						
+						public void
+						setLimits(
+							Object		limits,
+							boolean		do_up,
+							boolean		do_down )
+						{
+							if ( limits == null ){
+								
+								return;
+							}
+							try{
+								if ( setting_limits ){
+									
+									return;
+								}
+							
+								setting_limits	= true;
+							
+								Object[]	bits = (Object[])limits;
+								
+								if ( do_up ){
+									
+									COConfigurationManager.setParameter((String)bits[0], ((Integer)bits[1]).intValue());
+								}
+								
+								if ( do_down ){
+									
+									COConfigurationManager.setParameter((String)bits[2], ((Integer)bits[3]).intValue());
+								}
+								
+							}finally{
+								
+								setting_limits	= false;
+								
+							}
+						}
 					});
 		
 		nat_traverser = new NATTraverser( this );
 		
 		PeerNATTraverser.initialise( this );
+		
+			// one off explicit GC to clear up initialisation mem
+		
+		SimpleTimer.addEvent(
+				"AzureusCore:gc",
+				SystemTime.getOffsetTime(60*1000),
+				new TimerEventPerformer()
+				{
+					public void 
+					perform(
+						TimerEvent event) 
+					{
+						System.gc();
+					}
+				});
 	}
 	
 	public LocaleUtil
@@ -324,6 +405,7 @@ AzureusCoreImpl
 		// always publish a complete rss feed
 		
 		global_manager = GlobalManagerFactory.create(
+				this,
 				new GlobalMangerProgressListener()
 				{
 					public void 
@@ -432,7 +514,12 @@ AzureusCoreImpl
 	        		}
 	        	}
 	       }.start();
-           
+       
+	   if ( COConfigurationManager.getBooleanParameter( "Resume Downloads On Start" )){
+	   
+		   global_manager.resumeDownloads();
+	   }
+	   
        //late inits
 	   NetworkManager.getSingleton().initialize(); 
           
@@ -447,6 +534,8 @@ AzureusCoreImpl
 	   });	
 	   
 	   checkBadNatives();
+	   
+	   NetworkAdmin.getSingleton().runInitialChecks();
 	}
 	
 	public void triggerLifeCycleComponentCreated(AzureusCoreComponent component) {
@@ -472,7 +561,19 @@ AzureusCoreImpl
 				p_man.hasCapability( PlatformManagerCapabilities.TestNativeAvailability )){
 		
 	
-			String[]	dlls = { "niphk", "nvappfilter", "netdog", "vlsp", "imon", "sarah" };
+			String[]	dlls = { 	"niphk", 
+									"nvappfilter", 
+									"netdog", 
+									"vlsp", 
+									"imon", 
+									"sarah", 
+									"MxAVLsp", 
+									"mclsp", 
+									"radhslib", 
+									"winsflt",
+									"nl_lsp",
+									"AxShlex"
+						};
 			
 			for (int i=0;i<dlls.length;i++){
 				
@@ -601,6 +702,13 @@ AzureusCoreImpl
 				
 				Logger.log(new LogEvent(LOGID, "Core not started"));
 				
+					// might have been marked dirty due to core being created to allow functions to be used but never started...
+				
+				if ( AEDiagnostics.isDirty()){
+					
+					AEDiagnostics.markClean();
+				}
+				
 				stopping_sem.releaseForever();
 				
 				return;
