@@ -42,10 +42,15 @@ public class VirtualChannelSelector {
   public static final int OP_READ   = SelectionKey.OP_READ;
   public static final int OP_WRITE  = SelectionKey.OP_WRITE;
 
-  private boolean SAFE_SELECTOR_MODE_ENABLED = COConfigurationManager.getBooleanParameter( "network.tcp.enable_safe_selector_mode" );
+  private boolean SAFE_SELECTOR_MODE_ENABLED = TEST_SAFE_MODE || COConfigurationManager.getBooleanParameter( "network.tcp.enable_safe_selector_mode" );
   
-  private static final int MAX_SAFEMODE_SELECTORS = 100;
+  private static final boolean TEST_SAFE_MODE	= false;
   
+  private static final int MAX_CHANNELS_PER_SAFE_SELECTOR	= COConfigurationManager.getIntParameter( "network.tcp.safe_selector_mode.chunk_size" );
+  
+  private static final int MAX_SAFEMODE_SELECTORS = 20000 / MAX_CHANNELS_PER_SAFE_SELECTOR;
+  
+  private String		name;
 
   private VirtualChannelSelectorImpl selector_impl;
   
@@ -66,7 +71,8 @@ public class VirtualChannelSelector {
    * @param interest_op operation set of OP_CONNECT, OP_ACCEPT, OP_READ, or OP_WRITE
    * @param pause_after_select whether or not to auto-disable interest op after select  
    */
-  public VirtualChannelSelector( int interest_op, boolean pause_after_select ) { 
+  public VirtualChannelSelector( String name, int interest_op, boolean pause_after_select ) { 
+	this.name = name;
     this.op = interest_op;
     this.pause = pause_after_select;
     
@@ -81,7 +87,11 @@ public class VirtualChannelSelector {
     }
   }
 
-  
+  public String
+  getName()
+  {
+	  return( name );
+  }
   
   private void initSafeMode() {
     if (Logger.isEnabled())
@@ -117,14 +127,38 @@ public class VirtualChannelSelector {
   protected void registerSupport( AbstractSelectableChannel channel, VirtualAbstractSelectorListener listener, Object attachment ) {
     if( SAFE_SELECTOR_MODE_ENABLED ) {
       try{  selectors_mon.enter();
+      	//System.out.println( "register - " + channel.hashCode()  + " - " + Debug.getCompressedStackTrace());
         for( Iterator it = selectors.entrySet().iterator(); it.hasNext(); ) {
           Map.Entry entry = (Map.Entry)it.next();          
           VirtualChannelSelectorImpl sel = (VirtualChannelSelectorImpl)entry.getKey();
           ArrayList channels = (ArrayList)entry.getValue();
           
-          if( channels.size() < 60 ) {  //there's room in the current selector
+          if( channels.size() >= ( TEST_SAFE_MODE?0:MAX_CHANNELS_PER_SAFE_SELECTOR )) { 
+        	  
+         	  // it seems that we have a bug somewhere where a selector is being registered
+        	  // but not cancelled on close. As an interim fix scan channels and remove any
+        	  // closed ones
+        	  
+        	  Iterator	chan_it = channels.iterator();
+        	  
+        	  while( chan_it.hasNext()){
+        		  
+        		  AbstractSelectableChannel	chan = (AbstractSelectableChannel)chan_it.next();
+        		  
+        		  if ( !chan.isOpen()){
+        			  
+        			  Debug.out( "Selector '" + getName() + "' - removing orphaned safe channel registration" );
+        			  
+        			  chan_it.remove();
+        		  }
+        	  }
+          }
+          
+          if( channels.size() < MAX_CHANNELS_PER_SAFE_SELECTOR ) {  //there's room in the current selector  
+
             sel.register( channel, listener, attachment );
             channels.add( channel );
+            
             return;
           }
         }
@@ -169,6 +203,7 @@ public class VirtualChannelSelector {
   public void pauseSelects( AbstractSelectableChannel channel ) {
     if( SAFE_SELECTOR_MODE_ENABLED ) {
       try{  selectors_mon.enter();
+      	//System.out.println( "pause - " + channel.hashCode() + " - " + Debug.getCompressedStackTrace());
         for( Iterator it = selectors.entrySet().iterator(); it.hasNext(); ) {
           Map.Entry entry = (Map.Entry)it.next();          
           VirtualChannelSelectorImpl sel = (VirtualChannelSelectorImpl)entry.getKey();
@@ -180,7 +215,7 @@ public class VirtualChannelSelector {
           }
         }
         
-        System.out.println( "pauseSelects():: channel not found!" );
+        Debug.out( "pauseSelects():: channel not found!" );
       }
       finally{ selectors_mon.exit();  }
     }
@@ -198,6 +233,7 @@ public class VirtualChannelSelector {
   public void resumeSelects( AbstractSelectableChannel channel ) {
     if( SAFE_SELECTOR_MODE_ENABLED ) {
       try{  selectors_mon.enter();
+      	//System.out.println( "resume - " + channel.hashCode() + " - " + Debug.getCompressedStackTrace());
         for( Iterator it = selectors.entrySet().iterator(); it.hasNext(); ) {
           Map.Entry entry = (Map.Entry)it.next();          
           VirtualChannelSelectorImpl sel = (VirtualChannelSelectorImpl)entry.getKey();
@@ -209,7 +245,7 @@ public class VirtualChannelSelector {
           }
         }
         
-        System.out.println( "resumeSelects():: channel not found!" );
+        Debug.out( "resumeSelects():: channel not found!" );
       }
       finally{ selectors_mon.exit();  }
     }
@@ -227,6 +263,7 @@ public class VirtualChannelSelector {
   public void cancel( AbstractSelectableChannel channel ) {
     if( SAFE_SELECTOR_MODE_ENABLED ) {
       try{  selectors_mon.enter();
+      	//System.out.println( "cancel - " + channel.hashCode()  + " - " + Debug.getCompressedStackTrace());
         for( Iterator it = selectors.entrySet().iterator(); it.hasNext(); ) {
           Map.Entry entry = (Map.Entry)it.next();          
           VirtualChannelSelectorImpl sel = (VirtualChannelSelectorImpl)entry.getKey();
@@ -256,17 +293,37 @@ public class VirtualChannelSelector {
    */
   public int select(long timeout) {
     if( SAFE_SELECTOR_MODE_ENABLED ) {
-      HashSet sels = selectors_keyset_cow;
+      boolean	was_destroyed = destroyed;
+      
+      try{	
+	      int count = 0;
+	      
+	      for( Iterator it = selectors_keyset_cow.iterator(); it.hasNext(); ){
+	    	  
+	        VirtualChannelSelectorImpl sel = (VirtualChannelSelectorImpl)it.next();
+	        
+	        count += sel.select( timeout );
+	      }
+	      
+	      return count;
+	      
+      }finally{
+    	  
+    	  if ( was_destroyed ){
+    		  
+    		  // destruction process requires select op after destroy...
+    		  
+   			 try{  
+   				 selectors_mon.enter();
 
-      int count = 0;
-      
-      for( Iterator it = sels.iterator(); it.hasNext(); ) {
-        VirtualChannelSelectorImpl sel = (VirtualChannelSelectorImpl)it.next();
-        
-        count += sel.select( timeout );
+	 		     selectors.clear();
+			     selectors_keyset_cow = new HashSet();
+			     
+   			 }finally{
+   				 selectors_mon.exit();
+   			 }
+    	  }
       }
-      
-      return count;
     }
    
     return selector_impl.select( timeout );
@@ -276,20 +333,14 @@ public class VirtualChannelSelector {
   {
 	  destroyed	= true;
 	  
-	  if( SAFE_SELECTOR_MODE_ENABLED ) {
+	  if ( SAFE_SELECTOR_MODE_ENABLED ){
 				      
-		 try{  
-			 selectors_mon.enter();
-		      
-		     for( Iterator it = selectors.keySet().iterator(); it.hasNext(); ) {
-		    	 
-		        VirtualChannelSelectorImpl sel = (VirtualChannelSelectorImpl)it.next();
-		        
-		       sel.destroy();
-		     }
-		 }finally{
-			 selectors_mon.exit();
-		 }
+	     for( Iterator it = selectors_keyset_cow.iterator(); it.hasNext(); ) {
+	    	 
+	        VirtualChannelSelectorImpl sel = (VirtualChannelSelectorImpl)it.next();
+	        
+	        sel.destroy();
+	     }
 	  }else{
 		  selector_impl.destroy();
 	  }
