@@ -38,6 +38,8 @@ import com.aelitis.azureus.core.networkmanager.VirtualChannelSelector;
  */
 public class VirtualChannelSelectorImpl {
 	
+	private static final LogIDs LOGID = LogIDs.NWMAN;
+
 	/*
 	static boolean	rm_trace 	= false;
 	static boolean	rm_test_fix = false;
@@ -82,6 +84,11 @@ public class VirtualChannelSelectorImpl {
     private volatile boolean	destroyed;
     
     
+    private static final int WRITE_SELECTOR_DEBUG_CHECK_PERIOD	= 10000;
+    private static final int WRITE_SELECTOR_DEBUG_MAX_TIME		= 20000;
+    
+    private long last_write_select_debug;
+    
     public VirtualChannelSelectorImpl( VirtualChannelSelector _parent, int _interest_op, boolean _pause_after_select ) {	
       this.parent = _parent;
       INTEREST_OP = _interest_op;
@@ -128,6 +135,8 @@ public class VirtualChannelSelectorImpl {
       
       try {
         sel = Selector.open();
+        
+        AEDiagnostics.logWithStack( "seltrace", "Selector created for '" + parent.getName() + "'," + selector_guard.getType());
       }
       catch (Throwable t) {
         Debug.out( "ERROR: caught exception on Selector.open()", t );
@@ -139,6 +148,9 @@ public class VirtualChannelSelectorImpl {
         while( fail_count < 10 ) {
           try {
             sel = Selector.open();
+            
+            AEDiagnostics.logWithStack( "seltrace", "Selector created for '" + parent.getName() + "'," + selector_guard.getType());
+            
             break;
           }
           catch( Throwable f ) {
@@ -204,6 +216,14 @@ public class VirtualChannelSelectorImpl {
       SelectionKey key = channel.keyFor( selector );
       
       if( key != null && key.isValid() ) {
+    	  	// if we're resuming a non-interested key then reset the metrics
+    	  
+    	if (( key.interestOps() & INTEREST_OP ) == 0 ){
+     	   RegistrationData data = (RegistrationData)key.attachment();
+
+     	   data.last_select_success_time 	= SystemTime.getCurrentTime();
+     	   data.non_progress_count			= 0;
+    	}
         key.interestOps( key.interestOps() | INTEREST_OP );
       }
       else {  //channel not (yet?) registered
@@ -233,6 +253,13 @@ public class VirtualChannelSelectorImpl {
     		// don't worry too much about cancels
     	}
     	
+    	if ( channel == null ){
+    		
+    		Debug.out( "Attempt to cancel selects for null channel" );
+    		
+    		return;
+    	}
+    	
     	try{
     		register_cancel_list_mon.enter();
       	   		
@@ -278,6 +305,13 @@ public class VirtualChannelSelectorImpl {
    			Debug.out( "register called after selector destroyed" );
     	}
     	
+    	if ( channel == null ){
+    		
+    		Debug.out( "Attempt to register selects for null channel" );
+    		
+    		return;
+    	}
+    	
     	try{
     		register_cancel_list_mon.enter();
       	   		
@@ -486,7 +520,25 @@ public class VirtualChannelSelectorImpl {
       int	progress_made_key_count	= 0;
       int	total_key_count			= 0;
       
-      //notification of ready keys via listener callback
+      long	now = SystemTime.getCurrentTime();
+      
+      	//notification of ready keys via listener callback
+      
+      	// debug handling for channels stuck pending write select for long periods
+      
+      Set	non_selected_keys = null;
+      
+      if ( INTEREST_OP == VirtualChannelSelector.OP_WRITE ){
+    	  
+    	  if ( 	now < last_write_select_debug ||
+    			now - last_write_select_debug > WRITE_SELECTOR_DEBUG_CHECK_PERIOD ){
+    		  
+    		  last_write_select_debug = now;
+    		  
+    		  non_selected_keys = new HashSet( selector.keys());
+    	  }
+      }
+      
       for( Iterator i = selector.selectedKeys().iterator(); i.hasNext(); ) {
     	  
     	total_key_count++;
@@ -495,6 +547,12 @@ public class VirtualChannelSelectorImpl {
         i.remove();
         RegistrationData data = (RegistrationData)key.attachment();
 
+        if ( non_selected_keys != null ){
+        	
+        	non_selected_keys.remove( key );
+        }
+        
+        data.last_select_success_time = now;
         // int	rm_type;
         
         if( key.isValid() ) {
@@ -573,6 +631,57 @@ public class VirtualChannelSelectorImpl {
           }
           */
       }
+      
+      if ( non_selected_keys != null ){
+    	  
+    	  for( Iterator i = non_selected_keys.iterator(); i.hasNext(); ) {
+    	    	     	    	
+    		  SelectionKey key = (SelectionKey)i.next();
+    	    
+    	      RegistrationData data = (RegistrationData)key.attachment();
+ 
+        	  if (( key.interestOps() & INTEREST_OP) == 0 ) { 
+
+        		  continue;
+        	  }
+        	  
+    	      long	stall_time = now - data.last_select_success_time;
+    	      
+    	      if ( stall_time < 0 ){
+    	    	  
+    	    	  data.last_select_success_time	= now;
+    	    	  
+    	      }else{
+    	    	  
+	    	      if ( stall_time > WRITE_SELECTOR_DEBUG_MAX_TIME ){
+	    	    	
+	    	    	  Logger.log(
+	    	    		new LogEvent(LOGID,LogEvent.LT_WARNING,"Write select for " + key.channel() + " stalled for " + stall_time ));	    	  
+	    	    	  
+	    	    	  	// hack - trigger a dummy write select to see if things are still OK
+	    	    	  
+	    	          if( key.isValid() ) {
+    	        	  
+    	        		  if( pause_after_select ) { 
+
+    	        			  key.interestOps( key.interestOps() & ~INTEREST_OP );
+    	        		  }
+    	        		  
+    	        		  if ( parent.selectSuccess( data.listener, data.channel, data.attachment )){
+
+    	        			  data.non_progress_count = 0;
+    	        		  }
+	    	          }else{
+
+	    	        	  key.cancel();
+
+	    	        	  parent.selectFailure( data.listener, data.channel, data.attachment, new Throwable( "key is invalid" ) );
+	    	          }
+	    	      }
+	    	  }
+    	  }
+      }
+    	  
         
       	// if any of the ready keys hasn't made any progress then enforce minimum sleep period to avoid
       	// spinning
@@ -651,6 +760,8 @@ public class VirtualChannelSelectorImpl {
       
       try{
         selector.close();
+        
+        AEDiagnostics.log( "seltrace", "Selector destroyed for '" + parent.getName() + "'," + selector_guard.getType());
       }
       catch( Throwable t ) { t.printStackTrace(); }
     }
@@ -663,12 +774,15 @@ public class VirtualChannelSelectorImpl {
         protected final VirtualChannelSelector.VirtualAbstractSelectorListener listener;
         protected final Object attachment;
         
-        protected int non_progress_count;
+        protected int 	non_progress_count;
+        protected long	last_select_success_time;
         
       	private RegistrationData( AbstractSelectableChannel _channel, VirtualChannelSelector.VirtualAbstractSelectorListener _listener, Object _attachment ) {
       		channel 		= _channel;
       		listener		= _listener;
       		attachment 		= _attachment;
+      		
+      		last_select_success_time	= SystemTime.getCurrentTime();
       	}
       }
           
