@@ -41,18 +41,21 @@ import org.gudy.azureus2.core3.util.SystemTime;
 
 import com.aelitis.azureus.core.networkmanager.*;
 import com.aelitis.azureus.core.networkmanager.impl.IncomingConnectionManager;
+import com.aelitis.azureus.core.networkmanager.impl.TransportHelper;
 import com.aelitis.azureus.core.peermanager.download.TorrentDownload;
 import com.aelitis.azureus.core.peermanager.download.TorrentDownloadFactory;
 import com.aelitis.azureus.core.peermanager.messaging.*;
 import com.aelitis.azureus.core.peermanager.messaging.bittorrent.*;
 import com.aelitis.azureus.core.peermanager.piecepicker.util.BitFlags;
+import com.aelitis.azureus.core.stats.AzureusCoreStats;
+import com.aelitis.azureus.core.stats.AzureusCoreStatsProvider;
 import com.aelitis.azureus.core.util.bloom.BloomFilter;
 import com.aelitis.azureus.core.util.bloom.BloomFilterFactory;
 
 /**
  *
  */
-public class PeerManager {
+public class PeerManager implements AzureusCoreStatsProvider{
   private static final LogIDs LOGID = LogIDs.PEER;
 	
   private static final PeerManager instance = new PeerManager();
@@ -151,25 +154,106 @@ public class PeerManager {
   
 
   private PeerManager() {
-    legacy_handshake_header = ByteBuffer.allocate( 20 );
-    legacy_handshake_header.put( (byte)BTHandshake.PROTOCOL.length() );
-    legacy_handshake_header.put( BTHandshake.PROTOCOL.getBytes() );
-    legacy_handshake_header.flip();
-    
-    MessageManager.getSingleton().initialize();  //ensure it gets initialized
+	  legacy_handshake_header = ByteBuffer.allocate( 20 );
+	  legacy_handshake_header.put( (byte)BTHandshake.PROTOCOL.length() );
+	  legacy_handshake_header.put( BTHandshake.PROTOCOL.getBytes() );
+	  legacy_handshake_header.flip();
+	    
+	  Set	types = new HashSet();
+
+	  types.add( AzureusCoreStats.ST_PEER_MANAGER_COUNT );
+	  types.add( AzureusCoreStats.ST_PEER_MANAGER_PEER_COUNT );
+	  types.add( AzureusCoreStats.ST_PEER_MANAGER_PEER_SNUBBED_COUNT );
+	  types.add( AzureusCoreStats.ST_PEER_MANAGER_PEER_STALLED_DISK_COUNT );
+
+	  AzureusCoreStats.registerProvider( types, this );
+	  
+	  init();
+  }
+  
+  public void
+  updateStats(
+	  Set		types,
+	  Map		values )
+  {
+	  if ( types.contains( AzureusCoreStats.ST_PEER_MANAGER_COUNT )){
+
+		  values.put( AzureusCoreStats.ST_PEER_MANAGER_COUNT, new Long( registered_legacy_managers.size()));
+	  }
+
+	  if ( 	types.contains( AzureusCoreStats.ST_PEER_MANAGER_PEER_COUNT ) ||
+			types.contains( AzureusCoreStats.ST_PEER_MANAGER_PEER_SNUBBED_COUNT ) ||
+			types.contains( AzureusCoreStats.ST_PEER_MANAGER_PEER_STALLED_DISK_COUNT )){
+
+		  long	total_peers 				= 0;
+		  long	total_snubbed_peers			= 0;
+		  long	total_stalled_pending_load	= 0;
+
+		  try{
+
+			  managers_mon.enter();
+
+			  Iterator	it = registered_legacy_managers.values().iterator();
+
+			  while( it.hasNext()){
+
+				  List	registrations = (List)it.next();
+
+				  Iterator	it2 = registrations.iterator();
+
+				  while( it2.hasNext()){
+
+					  PeerManagerRegistrationImpl reg = (PeerManagerRegistrationImpl)it2.next();
+
+					  PEPeerControl control = reg.getActiveControl();
+
+					  if ( control != null ){
+
+						  total_peers 				+= control.getNbPeers();
+						  total_snubbed_peers			+= control.getNbPeersSnubbed();
+						  total_stalled_pending_load	+= control.getNbPeersStalledPendingLoad();
+					  }
+				  }
+			  }
+		  }finally{
+
+			  managers_mon.exit();
+		  }
+		  if ( types.contains( AzureusCoreStats.ST_PEER_MANAGER_PEER_COUNT )){
+
+			  values.put( AzureusCoreStats.ST_PEER_MANAGER_PEER_COUNT, new Long( total_peers ));
+		  }
+		  if ( types.contains( AzureusCoreStats.ST_PEER_MANAGER_PEER_SNUBBED_COUNT )){
+
+			  values.put( AzureusCoreStats.ST_PEER_MANAGER_PEER_SNUBBED_COUNT, new Long( total_snubbed_peers ));
+		  }
+		  if ( types.contains( AzureusCoreStats.ST_PEER_MANAGER_PEER_STALLED_DISK_COUNT )){
+
+			  values.put( AzureusCoreStats.ST_PEER_MANAGER_PEER_STALLED_DISK_COUNT, new Long( total_stalled_pending_load ));
+		  }
+	  }
+  }
+  
+  protected void
+  init()
+  {
+     MessageManager.getSingleton().initialize();  //ensure it gets initialized
     
     NetworkManager.ByteMatcher matcher =
     	new NetworkManager.ByteMatcher() 
     {
-    	public int size() {  return 48;  }
+		public int matchThisSizeOrBigger(){	return( 48 );}
+    	public int maxSize() {  return 48;  }
     	public int minSize() { return 20; }
 
     	public Object
     	matches( 
-    		InetSocketAddress	address,
+    		TransportHelper		transport,
     		ByteBuffer 			to_compare, 
     		int 				port ) 
     	{ 
+    		InetSocketAddress	address = transport.getAddress();
+    		
     		int old_limit = to_compare.limit();
     		int old_position = to_compare.position();
 
@@ -235,7 +319,7 @@ public class PeerManager {
     	
     	public Object 
     	minMatches( 
-    		InetSocketAddress	address,
+    		TransportHelper		transport,
     		ByteBuffer 			to_compare, 
     		int 				port ) 
     	{ 
@@ -258,11 +342,17 @@ public class PeerManager {
     		return matches?"":null;
     	}
 
-    	public byte[] 
-    	getSharedSecret()
+    	public byte[][] 
+    	getSharedSecrets()
     	{
     		return( null );	// registered manually above
     	}
+    	
+	   	public int 
+		getSpecificPort()
+		{
+			return( -1 );
+		}
     };
     
     // register for incoming connection routing
@@ -277,7 +367,7 @@ public class PeerManager {
         	{
         		PeerManagerRegistrationImpl	registration = (PeerManagerRegistrationImpl)routing_data;
         		
-        		registration.route( connection );
+        		registration.route( connection, null );
         	}
         	
         	public boolean
@@ -292,7 +382,68 @@ public class PeerManager {
         });
   }
   
+  	public PeerManagerRegistration
+  	manualMatchHash(
+  		InetSocketAddress	address,
+  		byte[]				hash )
+  	{			
+  		PeerManagerRegistrationImpl	routing_data = null;
+	  
+  		try{
+  			
+  			managers_mon.enter();
+				  		
+  			List	registrations = (List)registered_legacy_managers.get( new HashWrapper( hash ));
+   				
+  			if ( registrations != null ){
+   					
+  				routing_data = (PeerManagerRegistrationImpl)registrations.get(0);
+   			}
+		}finally{
+			
+			managers_mon.exit();
+		}
+		
+		if ( routing_data != null ){
+			
+			if ( !routing_data.isActive()){
+			
+				if ( routing_data.isKnownSeed( address )){
+					
+					if (Logger.isEnabled()){
+						Logger.log(new LogEvent(LOGID, "Activation request from " + address + " denied as known seed" ));
+					}
+					
+					routing_data = null;
+					
+				}else{
+					
+					if ( !routing_data.getAdapter().activateRequest( address )){
+
+	  					if (Logger.isEnabled()){
+    						Logger.log(new LogEvent(LOGID, "Activation request from " + address + " denied by rules" ));
+    					}
+ 
+						routing_data = null;	
+					}
+				}
+			}
+		}
+		
+		return routing_data;
+  	}	
      
+  	public void
+  	manualRoute(
+  		PeerManagerRegistration		_registration,
+  		NetworkConnection			_connection,
+  		PeerManagerRoutingListener	_listener )
+  	{
+   		PeerManagerRegistrationImpl	registration = (PeerManagerRegistrationImpl)_registration;
+		
+		registration.route( _connection, _listener );
+  	}
+  	
   public PeerManagerRegistration
   registerLegacyManager(
 	HashWrapper						hash,
@@ -300,19 +451,21 @@ public class PeerManager {
   {
 	  try{
 		  managers_mon.enter();
-		  		
+		  		 
 		  	// normally we only get a max of 1 of these. However, due to DownloadManager crazyness
 		  	// we can get an extra one when adding a download that already exists...
 		  
 		  List	registrations = (List)registered_legacy_managers.get( hash );
 			  
+		  byte[][]	secrets = adapter.getSecrets();
+		  
 		  if ( registrations == null ){
 			  
 			  registrations = new ArrayList(1);
 			  
 			  registered_legacy_managers.put( hash, registrations );
 		  		  
-			  IncomingConnectionManager.getSingleton().addSharedSecret( hash.getBytes());
+			  IncomingConnectionManager.getSingleton().addSharedSecrets( secrets );
 		  }
 		  
 		  PeerManagerRegistration	registration = new PeerManagerRegistrationImpl( hash, adapter );
@@ -320,6 +473,7 @@ public class PeerManager {
 		  registrations.add( registration );
 		  
 		  return( registration );
+		  
 	  }finally{
 		  
 		  managers_mon.exit();
@@ -352,12 +506,6 @@ public class PeerManager {
 		adapter	= _adapter;
 	}
 	
-	protected HashWrapper
-	getHash()
-	{
-		return( hash );
-	}
-	
 	protected PeerManagerRegistrationAdapter
 	getAdapter()
 	{
@@ -405,7 +553,9 @@ public class PeerManager {
 								
 				NetworkConnection	nc = (NetworkConnection)entry[0];
 				
-				route( _active_control, nc, true );
+				PeerManagerRoutingListener	listener = (PeerManagerRoutingListener)entry[2];
+				
+				route( _active_control, nc, true, listener );
 			}
 		}
 	}
@@ -479,7 +629,7 @@ public class PeerManager {
 				
 				  if ( registrations.size() == 0 ){
 					  
-					  IncomingConnectionManager.getSingleton().removeSharedSecret( hash.getBytes());
+					  IncomingConnectionManager.getSingleton().removeSharedSecrets( adapter.getSecrets());
 					  
 					  registered_legacy_managers.remove( hash );
 				  }
@@ -537,9 +687,16 @@ public class PeerManager {
 		}
 	}
 	
+	protected PEPeerControl
+	getActiveControl()
+	{
+		return( active_control );
+	}
+	
 	protected void
 	route(
-		NetworkConnection 	connection )
+		NetworkConnection 			connection,
+		PeerManagerRoutingListener	listener )
 	{	
 		PEPeerControl	control;
 		
@@ -569,7 +726,7 @@ public class PeerManager {
 						pending_connections = new ArrayList();
 					}
 										
-					pending_connections.add( new Object[]{ connection, new Long( SystemTime.getCurrentTime())});
+					pending_connections.add( new Object[]{ connection, new Long( SystemTime.getCurrentTime()), listener });
 					
 					if ( pending_connections.size() == 1 ){
 						
@@ -592,7 +749,7 @@ public class PeerManager {
    		
 		if ( control != null ){
     		
-			route( control, connection, false );
+			route( control, connection, false, listener );
    		}
 	}
 	
@@ -653,7 +810,8 @@ public class PeerManager {
 	route(
 		PEPeerControl				control,	
 		final NetworkConnection 	connection,
-		boolean						is_activation )
+		boolean						is_activation,
+		PeerManagerRoutingListener	listener )
 	{
 	        // make sure not already connected to the same IP address; allow
 	        // loopback connects for co-located proxy-based connections and
@@ -683,6 +841,31 @@ public class PeerManager {
         
         PEPeerTransport	pt = PEPeerTransportFactory.createTransport( control, PEPeerSource.PS_INCOMING, connection );
         
+        if ( listener != null ){
+        	
+        	boolean	ok = false;
+        	
+        	try{
+        		if ( listener.routed( pt )){
+        			
+        			ok	= true;
+        		}
+        		
+        	}catch( Throwable e ){
+        		
+        		Debug.printStackTrace(e);
+        	}
+        	
+        	if ( !ok ){
+        		     
+        		connection.close();
+        		
+        		return;
+        	}
+        }
+       
+        pt.start();
+        
         if ( is_activation ){
         	
         	pt.addListener( 
