@@ -38,7 +38,19 @@ SpeedManagerAdapter
 	setCurrentUploadLimit(
 		int		bytes_per_second );
 	
+	public int
+	getCurrentDownloadLimit();
+
 	public void
 	setCurrentDownloadLimit(
 		int		bytes_per_second );
+	
+	public Object
+	getLimits();
+	
+	public void
+	setLimits(
+		Object		limits,
+		boolean		do_up,
+		boolean		do_down );
 }
