@@ -22,7 +22,9 @@
 
 package com.aelitis.net.magneturi.impl;
 
+import java.awt.image.BufferedImage;
 import java.io.BufferedReader;
+import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStreamReader;
 import java.io.OutputStream;
@@ -38,6 +40,8 @@ import java.net.URL;
 import java.net.URLDecoder;
 import java.util.*;
 
+import javax.imageio.ImageIO;
+
 import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.logging.*;
 import org.gudy.azureus2.core3.util.*;
@@ -89,6 +93,8 @@ MagnetURIHandlerImpl
 	
 	private List	listeners	= new ArrayList();
 	
+	private Map		info_map 	= new HashMap();
+	
 	protected
 	MagnetURIHandlerImpl()
 	{
@@ -334,6 +340,12 @@ MagnetURIHandlerImpl
 			
 			return( true );
 			
+		}else if ( get.startsWith( "/azversion" )){
+
+			writeReply( os, "text/plain", Constants.AZUREUS_VERSION );
+			
+			return( true );
+			
 		}else if ( 	get.startsWith( "/magnet10/options.js?" ) ||
 					get.startsWith( "/magnet10/default.js?" )){
 		
@@ -382,11 +394,20 @@ MagnetURIHandlerImpl
 					
 				try{
 				
-					URL	magnet = new URL( "magnet:?xt=" + urn );
-				
+					URL	url;
+					
+					if ( urn.startsWith( "http:") || urn.startsWith( "https:" )){
+						
+						url = new URL( urn );
+						
+					}else{
+						
+						url = new URL( "magnet:?xt=" + urn );
+					}
+					
 					for (int i=0;i<listeners.size();i++){
 						
-						if (((MagnetURIHandlerListener)listeners.get(i)).download( magnet )){
+						if (((MagnetURIHandlerListener)listeners.get(i)).download( url )){
 							
 							ok = true;
 							
@@ -572,6 +593,46 @@ MagnetURIHandlerImpl
 				
 				return( !params.containsKey( "pause_on_error" ));
 			}
+		}else if ( get.startsWith( "/getinfo?" )){
+
+			String name = (String)params.get( "name" );
+
+			if ( name != null ){
+				
+				Integer	info = (Integer)info_map.get( name );
+				
+				if ( info != null ){					
+					
+					ByteArrayOutputStream	baos = new ByteArrayOutputStream();
+					
+					int	width 	= info.intValue();
+					int	height	= 1;
+					
+					BufferedImage	image = new BufferedImage( width, height, BufferedImage.TYPE_INT_RGB );
+					
+					for (int i=0;i<width;i++){
+						for (int j=0;j<height;j++){
+							int	red 	= i+j;
+							int	green 	= i*255/width;
+							int	blue	= j*255/height;
+							
+							image.setRGB(i,j,((red<<16)&0xff0000)|((green<<8)&0xff00)|((blue)&0xff));
+						}
+					}
+					ImageIO.write( image, "JPG", baos );
+
+					byte[]	data = baos.toByteArray();
+											
+					writeReply( os, "image/jpeg", data );
+						
+					return( true );
+				}
+			}
+			
+			writeNotFound( os );
+			
+			return( true );
+			
 		}
 		
 		return( true );
@@ -658,6 +719,16 @@ MagnetURIHandlerImpl
 		return( port );
 	}
 	
+	public void
+	addInfo(
+		String		name,
+		int			info )
+	{
+		info_map.put( name, new Integer(info));
+		
+		Logger.log(new LogEvent(LOGID, LogEvent.LT_INFORMATION,"MagnetURIHandler: global info registered: " + name + " -> " + info ));
+	}
+	
 	public void
 	addListener(
 		MagnetURIHandlerListener	l )
