@@ -30,10 +30,11 @@ import java.util.*;
 import java.io.*;
 import java.net.*;
 
-import org.gudy.azureus2.core3.config.*;
 import org.gudy.azureus2.core3.logging.*;
 import org.gudy.azureus2.core3.util.*;
 
+import com.aelitis.azureus.core.networkmanager.admin.NetworkAdmin;
+import com.aelitis.azureus.core.networkmanager.admin.NetworkAdminPropertyChangeListener;
 import com.aelitis.net.udp.uc.PRUDPPacket;
 import com.aelitis.net.udp.uc.PRUDPPacketHandler;
 import com.aelitis.net.udp.uc.PRUDPPacketHandlerException;
@@ -91,13 +92,33 @@ PRUDPPacketHandlerImpl
 	private long		total_replies;
 	private long		last_error_report;
 	
+	private AEMonitor	bind_address_mon	= new AEMonitor( "PRUDPPH:bind" );
+
+	private InetAddress				default_bind_ip;
+	private InetAddress				explicit_bind_ip;
+	
+	private volatile InetAddress				current_bind_ip;
+	private volatile InetAddress				target_bind_ip;
+	
+	private volatile boolean		failed;
+	private volatile boolean		destroyed;
+	private AESemaphore destroy_sem = new AESemaphore("PRUDPPacketHandler:destroy");
+
+	private Throwable 	init_error;
+	
 	protected
 	PRUDPPacketHandlerImpl(
-		int		_port )
+		int				_port,
+		InetAddress		_bind_ip )
 	{
-		port		= _port;
+		port				= _port;
+		explicit_bind_ip	= _bind_ip;
 		
-		final AESemaphore init_sem = new AESemaphore("PRUDPPacketHandler");
+		default_bind_ip = NetworkAdmin.getSingleton().getDefaultBindAddress();
+		
+		calcBind();
+		
+		final AESemaphore init_sem = new AESemaphore("PRUDPPacketHandler:init");
 		
 		Thread t = new AEThread( "PRUDPPacketReciever:".concat(String.valueOf(port)))
 			{
@@ -112,7 +133,10 @@ PRUDPPacketHandlerImpl
 		
 		t.start();
 		
-		SimpleTimer.addPeriodicEvent(
+		final TimerEventPeriodic[]	f_ev = {null};
+		
+		TimerEventPeriodic ev = 
+			SimpleTimer.addPeriodicEvent(
 				"PRUDP:timeouts",
 				5000,
 				new TimerEventPerformer()
@@ -121,10 +145,16 @@ PRUDPPacketHandlerImpl
 					perform(
 						TimerEvent	event )
 					{
+						if ( destroyed && f_ev[0] != null ){
+							
+							f_ev[0].cancel();
+						}
 						checkTimeouts();
 					}
 				});
 		
+		f_ev[0] = ev;
+		
 		init_sem.reserve();
 	}
 	
@@ -171,114 +201,233 @@ PRUDPPacketHandlerImpl
 		return( port );
 	}
 	
+	protected void
+	setDefaultBindAddress(
+		InetAddress	address )
+	{
+		try{
+			bind_address_mon.enter();
+			
+			default_bind_ip	= address;
+			
+			calcBind();
+			
+		}finally{
+			
+			bind_address_mon.exit();
+		}
+	}
+	
+	public void
+	setExplicitBindAddress(
+		InetAddress	address )
+	{
+		try{
+			bind_address_mon.enter();
+			
+			explicit_bind_ip	= address;
+			
+			calcBind();
+			
+		}finally{
+			
+			bind_address_mon.exit();
+		}
+		
+		int	loops = 0;
+		
+		while( current_bind_ip != target_bind_ip && !(failed || destroyed)){
+			
+			if ( loops >= 100 ){
+				
+				Debug.out( "Giving up on wait for bind ip change to take effect" );
+				
+				break;
+			}
+			
+			try{
+				Thread.sleep(50);
+				
+				loops++;
+				
+			}catch( Throwable e ){
+				
+				break;
+			}
+		}
+	}
+	
+	protected void
+	calcBind()
+	{
+		if ( explicit_bind_ip != null ){
+			
+			target_bind_ip = explicit_bind_ip;
+			
+		}else{
+			
+			target_bind_ip = default_bind_ip;
+		}
+	}
+	
 	protected void
 	receiveLoop(
 		AESemaphore	init_sem )
 	{
+		NetworkAdminPropertyChangeListener prop_listener = 
+			new NetworkAdminPropertyChangeListener()
+	    	{
+	    		public void
+	    		propertyChanged(
+	    			String		property )
+	    		{
+	    			if ( property == NetworkAdmin.PR_DEFAULT_BIND_ADDRESS ){
+	    				
+	    				setDefaultBindAddress( NetworkAdmin.getSingleton().getDefaultBindAddress());
+	    			}
+	    		}
+	    	};
+    	
+	    NetworkAdmin.getSingleton().addPropertyChangeListener( prop_listener );
+
 		try{
-			String bind_ip = COConfigurationManager.getStringParameter("Bind IP", "");
+				// outter loop picks up bind-ip changes
 			
-			InetSocketAddress	address;
-			
-			if ( bind_ip.length() == 0 ){
+			while( !( failed || destroyed )){
 				
-				address = new InetSocketAddress("127.0.0.1",port);
-				
-				socket = new DatagramSocket( port );
-				
-			}else{
-				
-				address = new InetSocketAddress(InetAddress.getByName(bind_ip), port);
-				
-				socket = new DatagramSocket( address );		
-			}
+				if ( socket != null ){
 					
-			socket.setReuseAddress(true);
-			
-			socket.setSoTimeout( PRUDPPacket.DEFAULT_UDP_TIMEOUT );
-			
-			init_sem.release();
-			
-			if (Logger.isEnabled())
-				Logger.log(new LogEvent(LOGID,
-						"PRUDPPacketReceiver: receiver established on port " + port)); 
-	
-			byte[] buffer = null;
-			
-			long	successful_accepts 	= 0;
-			long	failed_accepts		= 0;
-			
-			while(true){
-				
-				try{
-					
-					if ( buffer == null ){
+					try{
+						socket.close();
 						
-						buffer = new byte[PRUDPPacket.MAX_PACKET_SIZE];
-					}
-
-					DatagramPacket packet = new DatagramPacket( buffer, buffer.length, address );
-					
-					socket.receive( packet );
-					
-					long	receive_time = SystemTime.getCurrentTime();
-					
-					successful_accepts++;
-					
-					failed_accepts = 0;
-					
-					PRUDPPrimordialHandler prim_hand = primordial_handler;
-					
-					if ( prim_hand != null ){
+					}catch( Throwable e ){
 						
-						if ( prim_hand.packetReceived( packet )){
-					
-								// primordial handlers get their own buffer as we can't guarantee
-								// that they don't need to hang onto the data
-							
-							buffer	= null;
-							
-							stats.primordialPacketReceived( packet.getLength());
-						}
-					}
-					
-					if ( buffer != null ){
-						
-						process( packet, receive_time );
+						Debug.printStackTrace(e);
 					}
+				}
+								
+				InetSocketAddress	address;
 				
-				}catch( SocketTimeoutException e ){
-										
-				}catch( Throwable e ){
-						
-					failed_accepts++;
+				DatagramSocket	new_socket;
+				
+				if ( target_bind_ip == null ){
 					
-					if (Logger.isEnabled())
-						Logger.log(new LogEvent(LOGID,
-								"PRUDPPacketReceiver: receive failed on port " + port, e)); 
-
-					if (( failed_accepts > 100 && successful_accepts == 0 ) || failed_accepts > 1000 ){						
+					address = new InetSocketAddress("127.0.0.1",port);
+					
+					new_socket = new DatagramSocket( port );
+					
+				}else{
+					
+					address = new InetSocketAddress( target_bind_ip, port );
+					
+					new_socket = new DatagramSocket( address );		
+				}
+						
+				new_socket.setReuseAddress(true);
+				
+					// short timeout on receive so that we can interrupt a receive fairly quickly
+				
+				new_socket.setSoTimeout( 1000 );
+				
+					// only make the socket public once fully configured
+								
+				socket = new_socket;
+				
+				current_bind_ip	= target_bind_ip;
+								
+				init_sem.release();
+				
+				if (Logger.isEnabled())
+					Logger.log(new LogEvent(LOGID,
+							"PRUDPPacketReceiver: receiver established on port " + port + (current_bind_ip==null?"":(", bound to " + current_bind_ip )))); 
 		
-						Logger.logTextResource(new LogAlert(LogAlert.UNREPEATABLE,
-								LogAlert.AT_ERROR, "Network.alert.acceptfail"), new String[] {
-								"" + port, "UDP" });
-										
-							// break, sometimes get a screaming loop. e.g.
-						/*
-						[2:01:55]  DEBUG::Tue Dec 07 02:01:55 EST 2004
-						[2:01:55]    java.net.SocketException: Socket operation on nonsocket: timeout in datagram socket peek
-						[2:01:55]  	at java.net.PlainDatagramSocketImpl.peekData(Native Method)
-						[2:01:55]  	at java.net.DatagramSocket.receive(Unknown Source)
-						[2:01:55]  	at org.gudy.azureus2.core3.tracker.server.impl.udp.TRTrackerServerUDP.recvLoop(TRTrackerServerUDP.java:118)
-						[2:01:55]  	at org.gudy.azureus2.core3.tracker.server.impl.udp.TRTrackerServerUDP$1.runSupport(TRTrackerServerUDP.java:90)
-						[2:01:55]  	at org.gudy.azureus2.core3.util.AEThread.run(AEThread.java:45)
-						*/
+				byte[] buffer = null;
+				
+				long	successful_accepts 	= 0;
+				long	failed_accepts		= 0;
+				
+				while( !( failed || destroyed )){
+					
+					if ( current_bind_ip != target_bind_ip ){
 						
 						break;
-					}					
+					}
+					
+					try{
+						
+						if ( buffer == null ){
+							
+							buffer = new byte[PRUDPPacket.MAX_PACKET_SIZE];
+						}
+	
+						DatagramPacket packet = new DatagramPacket( buffer, buffer.length, address );
+						
+						socket.receive( packet );
+						
+						long	receive_time = SystemTime.getCurrentTime();
+						
+						successful_accepts++;
+						
+						failed_accepts = 0;
+						
+						PRUDPPrimordialHandler prim_hand = primordial_handler;
+						
+						if ( prim_hand != null ){
+							
+							if ( prim_hand.packetReceived( packet )){
+						
+									// primordial handlers get their own buffer as we can't guarantee
+									// that they don't need to hang onto the data
+								
+								buffer	= null;
+								
+								stats.primordialPacketReceived( packet.getLength());
+							}
+						}
+						
+						if ( buffer != null ){
+							
+							process( packet, receive_time );
+						}
+					
+					}catch( SocketTimeoutException e ){
+											
+					}catch( Throwable e ){
+							
+						failed_accepts++;
+						
+						if (Logger.isEnabled())
+							Logger.log(new LogEvent(LOGID,
+									"PRUDPPacketReceiver: receive failed on port " + port, e)); 
+	
+						if (( failed_accepts > 100 && successful_accepts == 0 ) || failed_accepts > 1000 ){						
+			
+							Logger.logTextResource(new LogAlert(LogAlert.UNREPEATABLE,
+									LogAlert.AT_ERROR, "Network.alert.acceptfail"), new String[] {
+									"" + port, "UDP" });
+											
+								// break, sometimes get a screaming loop. e.g.
+							/*
+							[2:01:55]  DEBUG::Tue Dec 07 02:01:55 EST 2004
+							[2:01:55]    java.net.SocketException: Socket operation on nonsocket: timeout in datagram socket peek
+							[2:01:55]  	at java.net.PlainDatagramSocketImpl.peekData(Native Method)
+							[2:01:55]  	at java.net.DatagramSocket.receive(Unknown Source)
+							[2:01:55]  	at org.gudy.azureus2.core3.tracker.server.impl.udp.TRTrackerServerUDP.recvLoop(TRTrackerServerUDP.java:118)
+							[2:01:55]  	at org.gudy.azureus2.core3.tracker.server.impl.udp.TRTrackerServerUDP$1.runSupport(TRTrackerServerUDP.java:90)
+							[2:01:55]  	at org.gudy.azureus2.core3.util.AEThread.run(AEThread.java:45)
+							*/
+							
+							init_error	= e;
+							
+							failed	= true;
+						}					
+					}
 				}
 			}
 		}catch( Throwable e ){
+			
+			init_error	= e;
+			
 			Logger.logTextResource(new LogAlert(LogAlert.UNREPEATABLE,
 					LogAlert.AT_ERROR, "Tracker.alert.listenfail"), new String[] { "UDP:"
 					+ port });
@@ -289,6 +438,21 @@ PRUDPPacketHandlerImpl
 		}finally{
 			
 			init_sem.release();
+			
+			destroy_sem.releaseForever();
+			
+			if ( socket != null ){
+				
+				try{
+					socket.close();
+					
+				}catch( Throwable e ){
+					
+					Debug.printStackTrace(e);
+				}
+			}
+			
+			NetworkAdmin.getSingleton().removePropertyChangeListener( prop_listener );
 		}
 	}
 	
@@ -590,10 +754,22 @@ PRUDPPacketHandlerImpl
 		PRUDPPacket				request_packet,
 		InetSocketAddress		destination_address )
 	
+		throws PRUDPPacketHandlerException
+	{
+		return( sendAndReceive( auth, request_packet, destination_address, PRUDPPacket.DEFAULT_UDP_TIMEOUT ));
+	}
+	
+	public PRUDPPacket
+	sendAndReceive(
+		PasswordAuthentication	auth,
+		PRUDPPacket				request_packet,
+		InetSocketAddress		destination_address,
+		long					timeout )
+	
 		throws PRUDPPacketHandlerException
 	{
 		PRUDPPacketHandlerRequestImpl	request = 
-			sendAndReceive( auth, request_packet,destination_address, null, PRUDPPacket.DEFAULT_UDP_TIMEOUT, PRUDPPacketHandler.PRIORITY_MEDIUM );
+			sendAndReceive( auth, request_packet, destination_address, null, timeout, PRUDPPacketHandler.PRIORITY_MEDIUM );
 		
 		return( request.getReply());
 	}
@@ -624,10 +800,17 @@ PRUDPPacketHandlerImpl
 	{
 		if ( socket == null ){
 			
+			if ( init_error != null ){
+				
+				throw( new PRUDPPacketHandlerException( "Transport unavailable", init_error ));
+			}
+		
 			throw( new PRUDPPacketHandlerException( "Transport unavailable" ));
 		}
 		
 		try{
+			checkTargetAddress( destination_address );
+			
 			ByteArrayOutputStream	baos = new ByteArrayOutputStream();
 			
 			DataOutputStream os = new DataOutputStream( baos );
@@ -914,10 +1097,17 @@ PRUDPPacketHandlerImpl
 	{
 		if ( socket == null ){
 			
+			if ( init_error != null ){
+				
+				throw( new PRUDPPacketHandlerException( "Transport unavailable", init_error ));
+			}
+			
 			throw( new PRUDPPacketHandlerException( "Transport unavailable" ));
 		}
 		
 		try{
+			checkTargetAddress( destination_address );
+			
 			ByteArrayOutputStream	baos = new ByteArrayOutputStream();
 			
 			DataOutputStream os = new DataOutputStream( baos );
@@ -944,14 +1134,32 @@ PRUDPPacketHandlerImpl
 			
 				// this is a reply to a request, no time delays considered here 
 			
+		}catch( PRUDPPacketHandlerException e ){
+			
+			throw( e );
+			
 		}catch( Throwable e ){
 			
+			e.printStackTrace();
+			
 			Logger.log(new LogEvent(LOGID, LogEvent.LT_ERROR, "PRUDPPacketHandler: send to " + destination_address + " failed: " + Debug.getNestedExceptionMessage(e)));
 			
 			throw( new PRUDPPacketHandlerException( "PRUDPPacketHandler:send failed", e ));
 		}
 	}
 	
+	protected void
+	checkTargetAddress(
+		InetSocketAddress	address )
+	
+		throws PRUDPPacketHandlerException
+	{
+		if ( address.getPort() == 0 ){
+			
+			throw( new PRUDPPacketHandlerException( "Invalid port - 0" ));
+		}
+	}
+	
 	public void
 	setDelays(
 		int		_send_delay,
@@ -995,6 +1203,8 @@ PRUDPPacketHandlerImpl
 		throws PRUDPPacketHandlerException
 	{
 		try{
+			checkTargetAddress( target );
+			
 			DatagramPacket dg_packet = new DatagramPacket(buffer, buffer.length, target );
 			
 			// System.out.println( "Outgoing to " + dg_packet.getAddress());	
@@ -1019,4 +1229,12 @@ PRUDPPacketHandlerImpl
 	{
 		return( stats );
 	}
+	
+	protected void
+	destroy()
+	{
+		destroyed	= true;
+		
+		destroy_sem.reserve();
+	}
 }
