@@ -24,21 +24,16 @@
 
 package org.gudy.azureus2.core3.config.impl;
 
+import java.io.File;
+import java.util.*;
+
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.stats.StatsWriterPeriodic;
 import org.gudy.azureus2.core3.torrent.TOTorrent;
 import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncer;
 import org.gudy.azureus2.core3.tracker.host.TRHost;
 import org.gudy.azureus2.core3.tracker.server.TRTrackerServer;
-import org.gudy.azureus2.core3.util.AEMonitor;
-import org.gudy.azureus2.core3.util.SystemProperties;
-
-import com.aelitis.azureus.plugins.startstoprules.defaultplugin.DefaultRankCalculator;
-
-import java.util.HashMap;
-import java.util.Locale;
-import java.util.Map;
-import java.util.Set;
+import org.gudy.azureus2.core3.util.*;
 
 
 /**
@@ -65,20 +60,27 @@ import java.util.Set;
  */
 
 public class ConfigurationDefaults {
-  private static final Long FALSE	= new Long(0);
-  private static final Long TRUE	= new Long(1);
   
+  private static final Long ZERO	= new Long(0);
+  private static final Long ONE		= new Long(1);
+
+  private static final Long FALSE	= ZERO;
+  private static final Long TRUE	= ONE;
+
   private static ConfigurationDefaults configdefaults;
   private static AEMonitor				class_mon	= new AEMonitor( "ConfigDef");
   
   private HashMap def = null;
   
-  public int def_int = 0;
-  public float def_float = 0;
-  public int def_boolean = 0;
-  public String def_String = "";
-  public byte[] def_bytes = null;
+  public static final int def_int = 0;
+  public static final long def_long = 0;
+  public static final float def_float = 0;
+  public static final int def_boolean = 0;
+  public static final String def_String = "";
+  public static final byte[] def_bytes = null;
   
+  private Hashtable parameter_verifiers	= new Hashtable();
+
   public static ConfigurationDefaults 
   getInstance() 
   {
@@ -109,18 +111,29 @@ public class ConfigurationDefaults {
 
     def.put("Override Ip", "");
     def.put("Enable incremental file creation", FALSE);
+    
     def.put("TCP.Listen.Port", new Long( 6881 ));
     def.put("TCP.Listen.Port.Enable", TRUE );
     def.put("UDP.Listen.Port", new Long( 6881 ));
     def.put("UDP.Listen.Port.Enable", TRUE );
     def.put("UDP.NonData.Listen.Port", new Long( 6881 ));	// two effective enablers for this, dht + tracker udp client
     def.put("UDP.NonData.Listen.Port.Same", TRUE );			// control over whether non-data and data udp port are the same
+    def.put("HTTP.Data.Listen.Port", new Long( Constants.isWindows?80:8080 ));
+    def.put("HTTP.Data.Listen.Port.Override", ZERO);
+    def.put("HTTP.Data.Listen.Port.Enable", FALSE );
+    
     def.put("max active torrents", new Long(4));
     def.put("max downloads", new Long(4));
+    def.put("min downloads", ONE);
     def.put("Newly Seeding Torrents Get First Priority", TRUE);
+    
     def.put("Max.Peer.Connections.Per.Torrent", new Long(COConfigurationManager.CONFIG_DEFAULT_MAX_CONNECTIONS_PER_TORRENT));
+    def.put("Max.Peer.Connections.Per.Torrent.When.Seeding", new Long(COConfigurationManager.CONFIG_DEFAULT_MAX_CONNECTIONS_PER_TORRENT));
+    def.put("Max.Peer.Connections.Per.Torrent.When.Seeding.Enable", FALSE );
     def.put("Max.Peer.Connections.Total", new Long(COConfigurationManager.CONFIG_DEFAULT_MAX_CONNECTIONS_GLOBAL));
 
+    def.put( "Peer.Fast.Initial.Unchoke.Enabled", FALSE );
+    
     def.put( "File Max Open", new Long(50));
     def.put( "Use Config File Backups", TRUE);
     
@@ -128,35 +141,39 @@ public class ConfigurationDefaults {
     def.put( "Max Uploads Seeding", new Long(4));
     def.put( "enable.seedingonly.maxuploads", FALSE );
     def.put( "max.uploads.when.busy.inc.min.secs", new Long( 30 ));
-    def.put( "Max Download Speed KBs", new Long(0) );
-    def.put( "Max Upload Speed KBs", new Long(0));
-    def.put( "Max Upload Speed Seeding KBs", new Long(0) );
+    def.put( "Max Download Speed KBs", ZERO );
+    def.put( "Max Upload Speed KBs", ZERO);
+    def.put( "Max Upload Speed Seeding KBs", ZERO );
     def.put( "enable.seedingonly.upload.rate", FALSE );
     
     def.put( "Auto Upload Speed Enabled", FALSE );
     def.put( "Auto Upload Speed Seeding Enabled", FALSE );
     def.put( "AutoSpeed Available", FALSE );	// informative read-only parameter
-    def.put( "AutoSpeed Min Upload KBs", new Long(0) );
-    def.put( "AutoSpeed Max Upload KBs", new Long(0) );
-    def.put( "AutoSpeed Max Increment KBs", new Long(1));
+    def.put( "AutoSpeed Min Upload KBs", ZERO );
+    def.put( "AutoSpeed Max Upload KBs", ZERO );
+    def.put( "AutoSpeed Max Increment KBs", ONE);
     def.put( "AutoSpeed Max Decrement KBs", new Long(4));
-    def.put( "AutoSpeed Choking Ping Millis", new Long(500) );
+    def.put( "AutoSpeed Choking Ping Millis", new Long(200) );
     def.put( "AutoSpeed Download Adj Enable", FALSE );
     def.put( "AutoSpeed Download Adj Ratio", "1.0" );
     def.put( "AutoSpeed Latency Factor", new Long(50));
     def.put( "Auto Upload Speed Debug Enabled", FALSE );
     
+    def.put( "ASN Autocheck Performed Time", ZERO );
+
+    
     def.put( "LAN Speed Enabled", TRUE );
-    def.put( "Max LAN Download Speed KBs", new Long(0) );
-    def.put( "Max LAN Upload Speed KBs", new Long(0) );
+    def.put( "Max LAN Download Speed KBs", ZERO );
+    def.put( "Max LAN Upload Speed KBs", ZERO );
     
     def.put("Use Resume", TRUE);
     def.put("On Resume Recheck All", FALSE);
     def.put("Save Resume Interval", new Long(5));
     def.put("Check Pieces on Completion", TRUE);
     def.put("Stop Ratio", new Float(0));
-    def.put("Stop Peers Ratio", new Long(0));
+    def.put("Stop Peers Ratio", ZERO);
     def.put("Disconnect Seed", TRUE);
+    def.put("Seeding Piece Check Recheck Enable", TRUE );
     def.put("priorityExtensions", "");
     def.put("priorityExtensionsIgnoreCase", FALSE);
     def.put("Ip Filter Enabled", TRUE);
@@ -167,38 +184,25 @@ public class ConfigurationDefaults {
     def.put("Ip Filter Enable Description Cache", TRUE);
     def.put("Allow Same IP Peers",FALSE);
     def.put("Use Super Seeding",FALSE);
-
+    def.put("Pause Downloads On Exit", FALSE );
+    def.put("Resume Downloads On Start", FALSE );
         
-    /** SWT GUI Settings **/
+    // SWT GUI Settings
     
-    def.put("User Mode", new Long(0));
-    
-    def.put("useCustomTab",TRUE);    
-    def.put("GUI Refresh",new Long(1000));
-    def.put("Graphics Update",new Long(4));
-    def.put("ReOrder Delay",new Long(0));
-    def.put("Send Version Info",TRUE);
-    def.put("Show Download Basket",FALSE);
-    def.put("config.style.refreshMT",new Long(0));
-    def.put("Open Details", FALSE);
-    def.put("IconBar.enabled", TRUE);
+    def.put("User Mode", ZERO);
     
     //default data location options
     def.put("Use default data dir", FALSE);	
-    def.put("Default save path", "" );
-    def.put("DefaultDir.BestGuess", TRUE);
-    def.put("DefaultDir.AutoUpdate", TRUE);
+		String docPath =  SystemProperties.getDocPath();
+		File f = new File(docPath, "Azureus Downloads");
+		def.put("Default save path", f.getAbsolutePath());
     
-    def.put("GUI_SWT_bFancyTab", TRUE);
-    def.put("GUI_SWT_bAlternateTablePainting", FALSE);
     def.put("update.start",TRUE);
     def.put("update.periodic",TRUE);
     def.put("update.opendialog",TRUE);
-    def.put("Colors.progressBar.override", FALSE);
-    def.put("GUI_SWT_DisableAlertSliding", FALSE);
-    def.put("NameColumn.showProgramIcon", TRUE);
     
     
+    // Logging
     def.put("Logger.Enabled", FALSE);  //logging in general
     def.put("Logging Enable", FALSE);  //file logging
     def.put("Logging Dir", "");
@@ -212,37 +216,6 @@ public class ConfigurationDefaults {
 
     
     
-    // Start/Stop Automation Stuff
-    def.put("StartStopManager_iNumPeersAsFullCopy", new Long(0));
-    def.put("StartStopManager_iFakeFullCopySeedStart", new Long(1));
-    def.put("StartStopManager_iMinPeersToBoostNoSeeds", new Long(1));
-    def.put("StartStopManager_iMinSpeedForActiveDL", new Long(512));
-    def.put("StartStopManager_iMinSpeedForActiveSeeding", new Long(512));
-    def.put("StartStopManager_iRankType", new Long(com.aelitis.azureus.plugins.startstoprules.defaultplugin.StartStopRulesDefaultPlugin.RANK_SPRATIO));
-    def.put("StartStopManager_iRankTypeSeedFallback", new Long(0));
-    def.put("StartStopManager_bAutoReposition", FALSE);
-    def.put("StartStopManager_iMinSeedingTime", new Long(60*3));
-    def.put("StartStopManager_bIgnore0Peers", TRUE);
-    def.put("StartStopManager_iIgnoreSeedCount", new Long(0));
-    def.put("StartStopManager_bPreferLargerSwarms", TRUE);
-    def.put("StartStopManager_bDebugLog", FALSE);
-    def.put("StartStopManager_iFirstPriority_Type", new Long(DefaultRankCalculator.FIRSTPRIORITY_ANY));
-    def.put("StartStopManager_iFirstPriority_ShareRatio", new Long(500));
-    def.put("StartStopManager_iFirstPriority_SeedingMinutes", new Long(0));
-    def.put("StartStopManager_iFirstPriority_DLMinutes", new Long(0));
-    def.put("StartStopManager_bAutoStart0Peers", FALSE);
-	// for ignore FP rules
-	def.put("StartStopManager_iFirstPriority_ignoreSPRatio", new Long(0));
-	def.put("StartStopManager_bFirstPriority_ignore0Peer", FALSE);
-    // for "Stop Peers Ratio" ignore rule
-    def.put("StartStopManager_iIgnoreRatioPeersSeedStart", new Long(0));
-    // for "Stop Ratio" ignore rule
-    def.put("StartStopManager_iIgnoreShareRatioSeedStart", new Long(0));
-    def.put("StartStopManager_bNewSeedsMoveTop", TRUE);
-    def.put("StartStopManager_iAddForSeedingDLCopyCount", new Long(1));
-    def.put("StartStopManager_iMaxActiveTorrentsWhenSeeding", new Long(0));
-    def.put("StartStopManager_bMaxActiveTorrentsWhenSeedingEnabled", FALSE);
-    
     	//tracker proxy defaults
     def.put( "Enable.Proxy", FALSE );
     def.put( "Enable.SOCKS", FALSE );
@@ -250,6 +223,7 @@ public class ConfigurationDefaults {
     def.put( "Proxy.Port", "" );
     def.put( "Proxy.Username", "<none>" );	// default is explicit "none", as opposed to "not defined"
     def.put( "Proxy.Password", "" );
+    def.put( "Proxy.Check.On.Start", TRUE );
     
     	// data proxy defaults
     def.put( "Proxy.Data.Enable", FALSE);
@@ -273,25 +247,22 @@ public class ConfigurationDefaults {
     def.put( "ID", "" );
     def.put( "Play Download Finished", FALSE );
     def.put( "Play Download Finished File", "" );
-    def.put( "Close To Tray", TRUE );
-    def.put( "Minimize To Tray", FALSE );
     def.put( "Watch Torrent Folder", FALSE );
-    def.put( "Watch Torrent Folder Interval", new Long(1) );
+    def.put( "Watch Torrent Folder Interval", ONE );
     def.put( "Start Watched Torrents Stopped", FALSE );
     def.put( "Watch Torrent Folder Path", "" );
     def.put( "Prioritize First Piece", FALSE );
+    def.put( "Prioritize Most Completed Files", FALSE );
     def.put( "Use Lazy Bitfield", FALSE );
-    def.put( "Move Completed When Done", FALSE );
-    def.put( "Completed Files Directory", "" );
     def.put( "Zero New", FALSE );
-    def.put( "Move Only When In Default Save Dir", TRUE );
+    def.put( "XFS Allocation", FALSE );
     def.put( "Copy And Delete Data Rather Than Move", FALSE);
-    def.put( "Move Torrent When Done", TRUE );
     def.put( "File.save.peers.enable", TRUE );
     def.put( "File.strict.locking", TRUE );
     def.put( "Move Deleted Data To Recycle Bin", TRUE);
     def.put( "Popup Download Finished", FALSE);
     def.put( "Popup File Finished", FALSE);
+    def.put( "Popup Download Added", FALSE);
     def.put( "Show Timestamp For Alerts", FALSE);
     
     //default torrent directory option
@@ -330,7 +301,8 @@ public class ConfigurationDefaults {
     def.put( "Tracker Port SSL Backups", "" );
     def.put( "Tracker Port Force External", FALSE );
     def.put( "Tracker Host Add Our Announce URLs", TRUE );
-    def.put( "Tracker IP", "" );
+    def_put( "Tracker IP", "", new IPVerifier());
+    
     def.put( "Tracker Port UDP Enable", FALSE );
     def.put( "Tracker Port UDP Version", new Long(2) );
     def.put( "Tracker Send Peer IDs", TRUE );
@@ -350,10 +322,10 @@ public class ConfigurationDefaults {
     def.put( "Tracker Poll Inc Per", new Long( TRTrackerServer.DEFAULT_INC_PER ) );
     def.put( "Tracker NAT Check Enable", TRUE);
     def.put( "Tracker NAT Check Timeout", new Long(TRTrackerServer.DEFAULT_NAT_CHECK_SECS));
-    def.put( "Tracker Max Seeds Retained", new Long( 0 ) );
-    def.put( "Tracker Max Seeds", new Long( 0 ) );
+    def.put( "Tracker Max Seeds Retained", ZERO );
+    def.put( "Tracker Max Seeds", ZERO );
     def.put( "Tracker Max GET Time", new Long(20));
-    def.put( "Tracker Max POST Time Multiplier", new Long(1));
+    def.put( "Tracker Max POST Time Multiplier", ONE);
     def.put( "Tracker Max Threads", new Long( 48 ));
     def.put( "Tracker TCP NonBlocking", FALSE);
     def.put( "Tracker TCP NonBlocking Conc Max", new Long(2048));
@@ -361,7 +333,8 @@ public class ConfigurationDefaults {
     def.put( "Tracker Client Scrape Stopped Enable", TRUE);
     def.put( "Tracker Client Scrape Single Only", FALSE);
     def.put( "Tracker Server Full Scrape Enable", TRUE );
-    
+    def.put( "Tracker Server Not Found Redirect", "" );
+    def.put( "Tracker Server Support Experimental Extensions", FALSE );
     
     def.put( "Network Selection Prompt", TRUE);
     def.put( "Network Selection Default.Public", TRUE);
@@ -398,33 +371,14 @@ public class ConfigurationDefaults {
     def.put( "File.Decoder.ShowAll", FALSE);
     def.put( "Password enabled", FALSE );
     def.put( "Password", null );
-    def.put( "Save detail views column widths", FALSE );
     def.put( "config.interface.checkassoc", TRUE );
-    def.put( "Wizard Completed", FALSE );
-    def.put( "donations.donated", FALSE );
-    def.put( "donations.lastVersion", "" );
-    def.put( "donations.nextAskTime", new Long(0) );
-    def.put( "Color Scheme.red", new Long(0) );
-    def.put( "Color Scheme.green", new Long(128) );
-    def.put( "Color Scheme.blue", new Long(255) );
-    def.put( "Show Splash", TRUE );
-    def.put( "window.maximized", TRUE );
-    def.put( "window.rectangle", "" );
-    def.put( "Open Console", FALSE );
-    def.put( "Open Config", FALSE );
-    def.put( "Open Stats On Start", FALSE);
-    def.put( "Start Minimized", FALSE );
-    def.put( "Open Bar", FALSE );
     def.put( "confirmationOnExit", FALSE );
     def.put( "locale", Locale.getDefault().toString() );
-    def.put( "locale.set.complete.count", new Long(0));
-    def.put( "Add URL Silently", FALSE );
-    def.put( "config.style.dropdiraction", "1" );
-    def.put( "MyTorrents.SplitAt", new Long(30) );
+    def.put( "locale.set.complete.count", ZERO);
     def.put( "Confirm Data Delete", TRUE );
     def.put( "Password Confirm", null );
     def.put( "Auto Update", TRUE );
-    def.put( "Alert on close", TRUE );
+    def.put( "Alert on close", FALSE );
     def.put( "diskmanager.friendly.hashchecking", FALSE );
     def.put( "diskmanager.hashchecking.smallestfirst", TRUE );    
     def.put( "Default Start Torrents Stopped", FALSE);
@@ -440,21 +394,35 @@ public class ConfigurationDefaults {
     def.put( "diskmanager.perf.write.maxmb", new Long(5));
     def.put( "diskmanager.perf.cache.trace", FALSE);
     def.put( "diskmanager.perf.cache.flushpieces", TRUE);
+    def.put( "diskmanager.perf.read.aggregate.enable", FALSE );
+    def.put( "diskmanager.perf.read.aggregate.request.limit", ZERO );
+    def.put( "diskmanager.perf.read.aggregate.byte.limit", ZERO );
+    def.put( "diskmanager.perf.write.aggregate.enable", FALSE );
+    def.put( "diskmanager.perf.write.aggregate.request.limit", ZERO );
+    def.put( "diskmanager.perf.write.aggregate.byte.limit", ZERO );
+    def.put( "diskmanager.perf.checking.read.priority", FALSE );
+    def.put( "diskmanager.perf.queue.torrent.bias", TRUE );
+    
     def.put( "File.truncate.if.too.large", FALSE);
     def.put( "Enable System Tray", TRUE);
-    def.put( "config.style.table.defaultSortOrder", new Long(0));
+    def.put( "config.style.table.defaultSortOrder", ZERO);
     def.put( "Ignore.peer.ports", "0" );
     def.put( "Security.JAR.tools.dir", "" );
     def.put( "network.max.simultaneous.connect.attempts", new Long( 8 ));
     def.put( "network.tcp.mtu.size", new Long(1500) );
     def.put( "network.udp.mtu.size", new Long(1500) );
-    def.put( "network.tcp.socket.SO_SNDBUF", new Long(0) );
-    def.put( "network.tcp.socket.SO_RCVBUF", new Long(0) );
+    def.put( "network.tcp.socket.SO_SNDBUF", ZERO );
+    def.put( "network.tcp.socket.SO_RCVBUF", ZERO );
     def.put( "network.tcp.socket.IPTOS", "" );
+    def.put( "network.tcp.read.select.time", new Long(25));
+    def.put( "network.tcp.write.select.time", new Long(25));
+    def.put( "network.control.write.idle.time", new Long(50));
+    def.put( "network.control.write.aggressive", FALSE );
+    def.put( "network.control.read.idle.time", new Long(50));
+    def.put( "peermanager.schedule.time", new Long(100));
     def.put( "confirm_torrent_removal", FALSE );
     def.put( "add_torrents_silently", FALSE );
     def.put( "enable_small_osx_fonts", TRUE );
-    def.put( "Message Popup Autoclose in Seconds", new Long(15));
     def.put( "Play Download Finished Announcement", FALSE);
     def.put( "Play Download Finished Announcement Text", "Download Complete");
     def.put( "Play File Finished", FALSE );
@@ -464,6 +432,7 @@ public class ConfigurationDefaults {
     
     def.put( "BT Request Max Block Size", new Long(65536));
     def.put( "network.tcp.enable_safe_selector_mode", FALSE );
+    def.put( "network.tcp.safe_selector_mode.chunk_size", new Long( 60 ));
     
     def.put( "network.transport.encrypted.require", FALSE );
     def.put( "network.transport.encrypted.min_level", "RC4" );
@@ -471,33 +440,93 @@ public class ConfigurationDefaults {
     def.put( "network.transport.encrypted.fallback.incoming", FALSE );
     def.put( "network.transport.encrypted.use.crypto.port", FALSE );
     
-    def.put( "network.bind.local.port", new Long(0) );
-   
+    def.put( "network.bind.local.port", ZERO );
+    
+    // Memory settings
+    
+    def.put( "memory.slice.limit.multiplier", new Long(1));
+    
+    // Move on completion settings.
+    def.put( "Move Completed When Done", FALSE );
+    def.put( "Completed Files Directory", "" );
+    def.put( "Move Only When In Default Save Dir", TRUE );
+    def.put( "Move Torrent When Done", TRUE );
+    def.put( "File.move.subdir_is_default", FALSE );
+    
+    // This gets permanently set by GlobalManagerImpl to false once Azureus has initialised.
+    // This is intended to be used once as part of an upgrade.
+    def.put( "Set Completion Flag For Completed Downloads On Start", TRUE );
+    
+    // Move on removal settings.
+    def.put( "File.move.download.removed.enabled", FALSE );
+    def.put( "File.move.download.removed.path", "" );
+    def.put( "File.move.download.removed.only_in_default", TRUE );
+    def.put( "File.move.download.removed.move_torrent", TRUE );
+    def.put( "File.move.download.removed.move_partial", FALSE );
+    
+    def.put("FilesView.separate_rename_and_retarget", FALSE);
+
+  }
+  
+  protected void
+  def_put(
+	String										key,
+	String										key_def,
+	COConfigurationManager.ParameterVerifier	verifier )
+  {
+	  def.put( key, key_def );
+	  
+	  List	l = (List)parameter_verifiers.get( key );
+	  
+	  if ( l == null ){
+		  
+		  l = new ArrayList(1);
+		  
+		  parameter_verifiers.put( key, l );
+	  }
+	  
+	  l.add( verifier );
+  }
+  
+  private void checkParameterExists(String p) throws ConfigurationParameterNotFoundException {
+	  if (!def.containsKey(p)) {
+		  ConfigurationParameterNotFoundException cpnfe = new ConfigurationParameterNotFoundException(p);
+		  //cpnfe.fillInStackTrace();
+		  //Debug.out(cpnfe);
+		  throw cpnfe;
+	  }
   }
   
   public String getStringParameter(String p) throws ConfigurationParameterNotFoundException {
-    if (def.containsKey(p)) {
-      Object o = def.get(p);
-      if (o instanceof Number)
-        return ((Number)o).toString();
-
-      return (String)o;
-    } else
-      throw new ConfigurationParameterNotFoundException(p);
+    checkParameterExists(p);
+    Object o = def.get(p);
+    if (o instanceof Number)
+      return ((Number)o).toString();
+    return (String)o;
   }
   
   public int getIntParameter(String p) throws ConfigurationParameterNotFoundException {
-    if (def.containsKey(p))
-      return ((Long) def.get(p)).intValue();
-    else
-      throw new ConfigurationParameterNotFoundException(p);
+	    checkParameterExists(p);
+	    return ((Long) def.get(p)).intValue();
   }
-
+  
+  public long getLongParameter(String p) throws ConfigurationParameterNotFoundException {
+	    checkParameterExists(p);
+	    return ((Long) def.get(p)).longValue();
+  }
+  
   public float getFloatParameter(String p) throws ConfigurationParameterNotFoundException {
-    if (def.containsKey(p))
-      return ((Float) def.get(p)).floatValue();
-    else
-      throw new ConfigurationParameterNotFoundException(p);
+	    checkParameterExists(p);
+	    return ((Float) def.get(p)).floatValue();
+  }
+  
+  public byte[] getByteParameter(String p) throws ConfigurationParameterNotFoundException {
+	  checkParameterExists(p);
+	  return (byte[])def.get(p);
+  }
+  
+  public boolean hasParameter(String p) {
+	  return def.containsKey(p);
   }
   
   /**
@@ -530,6 +559,10 @@ public class ConfigurationDefaults {
     def.put(sKey, lParameter);
   }
   
+  public void addParameter(String sKey, float fParameter) {
+	  def.put(sKey, new Float(fParameter));
+  }
+  
   public void registerExternalDefaults(Map addmap) {
   	def.putAll(addmap);
   }
@@ -544,4 +577,90 @@ public class ConfigurationDefaults {
   {
 	return( def.get( key ));  
   }
+  
+  public List
+  getVerifiers(
+	 String	key )
+  {
+	  return((List)parameter_verifiers.get( key ));
+  }
+  
+  protected void
+  runVerifiers()
+  {
+	  Iterator	it = parameter_verifiers.entrySet().iterator();
+	  
+	  while( it.hasNext()){
+		  
+		  Map.Entry	entry =(Map.Entry)it.next();
+		  
+		  String	key 		= (String)entry.getKey();
+		  List		verifiers 	= (List)entry.getValue();
+		  
+		  for (int i=0;i<verifiers.size();i++){
+			  
+			  COConfigurationManager.ParameterVerifier	verifier = (COConfigurationManager.ParameterVerifier)verifiers.get(i);
+			  
+			  Object	val_def = getDefaultValueAsObject( key );
+			  Object	val;
+			  
+			  if ( val_def == null ){
+				  
+				  continue;
+			  }
+			  
+			  if ( val_def instanceof String ){
+				
+				  val = COConfigurationManager.getStringParameter( key );
+				  
+			  }else{
+				  
+				  Debug.out( "Unsupported verifier type for parameter '" + key + "' - " + val_def );
+				  
+				  continue;
+			  }
+			  
+			  if ( val == null ){
+				  
+				  continue;
+			  }
+			  
+			  if ( !verifier.verify( key, val )){
+				  
+				  Debug.out( "Parameter '" + key + "', value '" + val +"' failed verification - setting back to default '" + val_def + "'" );
+				  
+				  COConfigurationManager.removeParameter( key );
+			  }
+		  }
+	  }
+  }
+  
+  protected class
+  IPVerifier
+  	implements COConfigurationManager.ParameterVerifier
+  {
+	  public boolean
+	  verify(
+		String	parameter,
+		Object	_value )
+	  {
+		  String	value = (String)_value;
+		  
+		  	// see http://www.faqs.org/rfcs/rfc3696.html
+		  
+		  for (int i=0;i<value.length();i++){
+			  
+			  char	c = value.charAt(i);
+			  
+			  if ( Character.isLetterOrDigit( c ) || c == '.' || c == '-' ){
+				  
+			  }else{
+				  
+				  return( false );
+			  }
+		  }
+		  
+		  return( true );
+	  }
+  }
 }
