@@ -1,7 +1,7 @@
 /*
  * Created on Jul 19, 2004
  * Created by Alon Rohter
- * Copyright (C) 2004 Aelitis, All Rights Reserved.
+ * Copyright (C) 2004, 2005, 2006 Aelitis, All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  * 
- * AELITIS, SARL au capital de 30,000 euros
+ * AELITIS, SAS au capital de 46,603.30 euros
  * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
  *
  */
@@ -30,16 +30,24 @@ import org.gudy.azureus2.core3.util.DirectByteBuffer;
 public interface 
 DiskManagerReadRequestListener 
 {
-  
-  /**
-   * Notification that the read request has completed,
-   * along with its accompanying buffer data.
-   * @param request
-   * @param data
-   */
-	
-  public void 
-  readCompleted( 
-  		DiskManagerReadRequest 	request, 
-		DirectByteBuffer 	data );
+
+	/**
+	 * Notification that the read request has completed,
+	 * along with its accompanying buffer data.
+	 * @param request
+	 * @param data
+	 */
+
+	public void 
+	readCompleted( 
+			DiskManagerReadRequest 	request, 
+			DirectByteBuffer 		data );
+
+	public void 
+	readFailed( 
+			DiskManagerReadRequest 	request, 
+			Throwable		 		cause );
+
+	public int
+	getPriority();
 }
