@@ -50,10 +50,7 @@ DiskManagerFileInfoImpl
   private CacheFile		cache_file;
   
   private String 		extension;
-  private long 			length;
   private long 			downloaded;
-  private int 			firstPieceNumber = -1;
-  private int 			nbPieces = 0;
   
   private DiskManagerHelper 	diskManager;
   private TOTorrentFile			torrent_file;
@@ -290,19 +287,19 @@ DiskManagerFileInfoImpl
    * @return
    */
   public int getFirstPieceNumber() {
-    return firstPieceNumber;
+    return torrent_file.getFirstPieceNumber();
   }
   
   
   public int getLastPieceNumber() {
-    return firstPieceNumber + nbPieces - 1;
+    return torrent_file.getLastPieceNumber();
   }
 
   /**
    * @return
    */
   public long getLength() {
-	return length;
+	return torrent_file.getLength();
   }
 
 	public int	
@@ -314,7 +311,7 @@ DiskManagerFileInfoImpl
    * @return
    */
   public int getNbPieces() {
-	return nbPieces;
+	return torrent_file.getNumberOfPieces();
   }
 
 
@@ -332,29 +329,6 @@ DiskManagerFileInfoImpl
 	extension = string;
   }
 
-
-  /**
-   * @param i
-   */
-  public void setFirstPieceNumber(int i) {
-	firstPieceNumber = i;
-  }
-
-  /**
-   * @param l
-   */
-  public void setLength(long l) {
-	length = l >= 0L ? l : 0L;
-  }
-
-
-  /**
-   * @param i
-   */
-  public void setNbPieces(int i) {
-	nbPieces = i;
-  }
-
   /**
    * @return
    */
@@ -478,6 +452,7 @@ DiskManagerFileInfoImpl
 	public void
 	close()
 	{
+		// this doesn't need to do anything as overall closure is handled by the disk manager closing
 	}
 	
 	public void
