@@ -185,6 +185,12 @@ DMReaderImpl
 					  {
 						  sem.release();
 					  }
+					  
+					  public int
+					  getPriority()
+					  {
+						  return( -1 );
+					  }
 				});
 		
 		sem.reserve();
@@ -221,6 +227,12 @@ DMReaderImpl
 
 				  _listener.readFailed( request, cause );
 			  }
+			  
+			  public int
+			  getPriority()
+			  {
+				  return( _listener.getPriority());
+			  }
 			};
 			
 		DirectByteBuffer buffer	= null;
@@ -340,6 +352,12 @@ DMReaderImpl
 						  listener.readFailed( request, cause );
 					  }
 					  
+					  public int
+					  getPriority()
+					  {
+						  return( _listener.getPriority());
+					  }
+					  
 					  protected void
 					  complete()
 					  {
@@ -490,6 +508,12 @@ DMReaderImpl
 										
 										sem.release();
 									}
+									
+									public int
+									getPriority()
+									{
+										return( listener.getPriority());
+									}							
 								});
 							
 							sem.reserve();
@@ -564,6 +588,12 @@ DMReaderImpl
 			failed( cause );
 		}
 		
+		public int
+		getPriority()
+		{
+			return( listener.getPriority());
+		}
+		
 		protected void
 		failed(
 			Throwable			cause )
