@@ -28,6 +28,7 @@ package org.gudy.azureus2.core3.download.impl;
 import org.gudy.azureus2.core3.download.*;
 import org.gudy.azureus2.core3.peer.*;
 import org.gudy.azureus2.core3.torrent.TOTorrent;
+import org.gudy.azureus2.core3.util.IndentWriter;
 import org.gudy.azureus2.core3.util.SystemTime;
 import org.gudy.azureus2.core3.disk.*;
 
@@ -55,6 +56,9 @@ DownloadManagerStatsImpl
 	private long saved_SecondsDownloading = 0;
 	private long saved_SecondsOnlySeeding = 0;
 	
+	private int saved_SecondsSinceDownload	= 0;
+	private int saved_SecondsSinceUpload	= 0;
+	
 	private int max_upload_rate_bps = 0;  //0 for unlimited
 	private int max_download_rate_bps = 0;  //0 for unlimited
   
@@ -314,6 +318,22 @@ DownloadManagerStatsImpl
 		return( saved_protocol_bytes_uploaded );
 	}
  
+	public long 
+	getRemaining()
+	{
+		DiskManager disk_manager = download_manager.getDiskManager();
+		
+	    if ( disk_manager == null ){
+	    	
+	    	return download_manager.getSize() - 
+		             ((long)getCompleted() * download_manager.getSize() / 1000L);
+		
+	    }else{
+		     
+	    	return disk_manager.getRemainingExcludingDND();
+		}
+	}
+	
 	public long 
 	getDiscarded()
 	{
@@ -468,6 +488,88 @@ DownloadManagerStatsImpl
 		max_download_rate_bps = max_rate_bps;  
 	}
     
+	public int 
+	getTimeSinceLastDataReceivedInSeconds()
+	{
+		PEPeerManager	pm = download_manager.getPeerManager();
+		
+		int	res = saved_SecondsSinceDownload;
+		
+		if ( pm != null ){
+			
+			int	current = pm.getStats().getTimeSinceLastDataReceivedInSeconds();
+			
+			if ( current >= 0 ){
+			
+					// activity this session, use this value
+				
+				res = current;
+				
+			}else{
+				
+					// no activity this session. If ever has been activity add in session
+					// time
+				
+				if ( res >= 0 ){
+					
+					long	now = SystemTime.getCurrentTime();
+					
+					long	elapsed = now - pm.getTimeStarted();
+					
+					if ( elapsed < 0 ){
+						
+						elapsed = 0;
+					}
+					
+					res += elapsed/1000;
+				}
+			}
+		}
+		
+		return( res );	
+	}
+	
+	public int 
+	getTimeSinceLastDataSentInSeconds()
+	{
+		PEPeerManager	pm = download_manager.getPeerManager();
+		
+		int	res = saved_SecondsSinceUpload;
+		
+		if ( pm != null ){
+			
+			int	current = pm.getStats().getTimeSinceLastDataSentInSeconds();
+			
+			if ( current >= 0 ){
+			
+					// activity this session, use this value
+				
+				res = current;
+				
+			}else{
+				
+					// no activity this session. If ever has been activity add in session
+					// time
+				
+				if ( res >= 0 ){
+					
+					long	now = SystemTime.getCurrentTime();
+					
+					long	elapsed = now - pm.getTimeStarted();
+					
+					if ( elapsed < 0 ){
+						
+						elapsed = 0;
+					}
+					
+					res += elapsed/1000;
+				}
+			}
+		}
+		
+		return( res );	
+	}
+	
 	protected void
 	saveSessionTotals()
 	{
@@ -481,6 +583,15 @@ DownloadManagerStatsImpl
 	
 		saved_SecondsDownloading 		= getSecondsDownloading();
 		saved_SecondsOnlySeeding		= getSecondsOnlySeeding();
+		
+		saved_SecondsSinceDownload		= getTimeSinceLastDataReceivedInSeconds();
+		saved_SecondsSinceUpload		= getTimeSinceLastDataSentInSeconds();
+		
+		DownloadManagerState state = download_manager.getDownloadState();
+
+		state.setIntAttribute( DownloadManagerState.AT_TIME_SINCE_DOWNLOAD, saved_SecondsSinceDownload );
+		state.setIntAttribute( DownloadManagerState.AT_TIME_SINCE_UPLOAD, saved_SecondsSinceUpload );
+
 	}
 	
  	protected void
@@ -507,5 +618,31 @@ DownloadManagerStatsImpl
 		saved_hashfails				= _saved_hashfails;
 		saved_SecondsDownloading	= _saved_SecondsDownloading;
 		saved_SecondsOnlySeeding	= _saved_SecondsOnlySeeding;
+		
+		DownloadManagerState state = download_manager.getDownloadState();
+		
+		saved_SecondsSinceDownload	= state.getIntAttribute( DownloadManagerState.AT_TIME_SINCE_DOWNLOAD );
+		saved_SecondsSinceUpload	= state.getIntAttribute( DownloadManagerState.AT_TIME_SINCE_UPLOAD );
+	}
+	
+	protected void 
+	generateEvidence(
+		IndentWriter writer) 
+	{
+		writer.println( "DownloadManagerStats" );
+		
+		try{
+			writer.indent();
+			
+			writer.println( 
+				"recv_d=" + getTotalDataBytesReceived() + ",recv_p=" + getTotalProtocolBytesReceived() + ",recv_g=" + getTotalGoodDataBytesReceived() + 
+				",sent_d=" + getTotalDataBytesSent() + ",sent_p=" + getTotalProtocolBytesSent() + 
+				",discard=" + getDiscarded() + ",hash_fails=" + getHashFailCount() + "/" + getHashFailBytes() +
+				",comp=" + getCompleted() + "[live:" + getDownloadCompleted(true) + "/" + getDownloadCompleted( false)  + "],dl_comp=" + downloadCompleted );
+	
+		}finally{
+			
+			writer.exdent();
+		}
 	}
 }
