@@ -28,27 +28,35 @@ package org.gudy.azureus2.core3.global;
 public interface 
 GlobalManagerStats 
 {
-	public int 
-	getDownloadAverage();
-	
-	public int 
-	getUploadAverage();
+	public int getDataReceiveRate();
   
-  public long
-  getTotalReceivedRaw();
+	public int getProtocolReceiveRate();
 	
-  public long
-  getTotalSentRaw();
+	public int getDataSendRate();
+	public int getDataSendRateNoLAN();
+	  
+	public int getProtocolSendRate();
+	public int getProtocolSendRateNoLAN();
+  
+	public int getDataSendRateAtClose();
+  
+	public long getTotalDataBytesReceived();
+  
+	public long getTotalProtocolBytesReceived();
+	
+	public long getTotalDataBytesSent();
+  
+	public long getTotalProtocolBytesSent();
+  
+  	public long getTotalSwarmsPeerRate( boolean downloading, boolean seeding );
   
 		// set methods
 		
-	public void
-	sent(
-		int		bytes );
+	public void	dataBytesSent( int bytes, boolean LAN );
+	public void protocolBytesSent( int bytes, boolean LAN );
 		
-	public void
-	received(
-		int		bytes );
+	public void dataBytesReceived( int bytes );
+	public void protocolBytesReceived( int bytes );
 		
 	public void
 	discarded(
