@@ -51,8 +51,8 @@ import org.gudy.azureus2.core3.tracker.util.TRTrackerUtils;
 import org.gudy.azureus2.core3.tracker.util.TRTrackerUtilsListener;
 import org.gudy.azureus2.core3.util.*;
 
+import com.aelitis.azureus.core.AzureusCore;
 import com.aelitis.azureus.core.helpers.TorrentFolderWatcher;
-import com.aelitis.azureus.core.peermanager.uploadslots.UploadSlotManager;
 import com.aelitis.azureus.core.util.CopyOnWriteList;
 
 import org.gudy.azureus2.plugins.network.ConnectionManager;
@@ -142,6 +142,8 @@ public class GlobalManagerImpl
 	
 	private Map		manager_map			= new HashMap();
 		
+	private GlobalMangerProgressListener	progress_listener;
+	
 	private Checker checker;
 	private GlobalManagerStatsImpl		stats;
     private long last_swarm_stats_calc_time		= 0;
@@ -154,6 +156,8 @@ public class GlobalManagerImpl
   
 	private Map							saved_download_manager_state	= new HashMap();
 	
+	private int							next_seed_piece_recheck_index;
+	
 	private TorrentFolderWatcher torrent_folder_watcher;
   
 	private ArrayList paused_list = new ArrayList();
@@ -197,6 +201,7 @@ public class GlobalManagerImpl
     private int saveResumeLoopCount = 5*60*1000 / waitTime;
     private int netCheckLoopCount	= 60*1000 / waitTime;
     private int natCheckLoopCount	= 30*1000 / waitTime;
+    private int seedPieceCheckCount	= 30*1000 / waitTime;
            
     private AESemaphore	run_sem = new AESemaphore( "GM:Checker:run");
     
@@ -245,6 +250,11 @@ public class GlobalManagerImpl
 		        checkSeedingOnlyState();
 	        }
 	        	
+	        if ((loopFactor % seedPieceCheckCount == 0)) {
+
+	        	seedPieceRecheck();
+	        }
+	        
 	        for (Iterator it=managers_cow.iterator();it.hasNext();) {
           	
 	        	DownloadManager manager = (DownloadManager)it.next();
@@ -292,18 +302,21 @@ public class GlobalManagerImpl
   }
 
   public 
-  GlobalManagerImpl(final
-  		GlobalMangerProgressListener listener,
-  		long existingTorrentLoadDelay)
+  GlobalManagerImpl(
+	AzureusCore						core,
+	GlobalMangerProgressListener 	listener,
+  	long 							existingTorrentLoadDelay)
   {
     //Debug.dumpThreadsLoop("Active threads");
   	
+	progress_listener = listener;
+	
   	AEDiagnostics.addEvidenceGenerator( this );
 	
     stats = new GlobalManagerStatsImpl( this );
        
     try{
-    	stats_writer = new GlobalManagerStatsWriter( this );
+    	stats_writer = new GlobalManagerStatsWriter( core );
     	
     }catch( Throwable e ){
     	
@@ -317,17 +330,18 @@ public class GlobalManagerImpl
 			loadTorrentsDelay = new DelayedEvent("GM:tld", existingTorrentLoadDelay,
 					new AERunnable() {
 						public void runSupport() {
-							loadExistingTorrentsNow(listener, false); // already async
+							loadExistingTorrentsNow(false); // already async
 						}
 					});
 		} else {
 			// run sync
-			loadDownloads(listener);
+			loadDownloads();
 		}
 
-    if (listener != null)
-      listener.reportCurrentTask(MessageText.getString("splash.initializeGM"));
-
+    if (progress_listener != null){
+    	progress_listener.reportCurrentTask(MessageText.getString("splash.initializeGM"));
+    }
+    
     // Initialize scraper after loadDownloads so that we can merge scrapes
     // into one request per tracker
     trackerScraper = TRTrackerScraperFactory.getSingleton();
@@ -479,11 +493,6 @@ public class GlobalManagerImpl
        	
     checker.start();
     
-    if ( stats_writer != null ){
-    	
-    	stats_writer.initialisationComplete();
-    }
-    
     torrent_folder_watcher = new TorrentFolderWatcher( this );
     
     
@@ -501,8 +510,7 @@ public class GlobalManagerImpl
     	});
   }
   
-  public void loadExistingTorrentsNow(final GlobalMangerProgressListener listener,
-			boolean async)
+  public void loadExistingTorrentsNow(boolean async)
 	{
 		if (loadTorrentsDelay == null) {
 			return;
@@ -513,13 +521,13 @@ public class GlobalManagerImpl
 		if (async) {
 			AEThread thread = new AEThread("load torrents", true) {
 				public void runSupport() {
-					loadDownloads(listener);
+					loadDownloads();
 				}
 			};
 			thread.setPriority(3);
 			thread.start();
 		} else {
-			loadDownloads(listener);
+			loadDownloads();
 		}
 	}
 
@@ -677,7 +685,7 @@ public class GlobalManagerImpl
    {
     if (!isStopping) {
     	// make sure we have existing ones loaded so that existing check works
-    	loadExistingTorrentsNow(null, false);
+    	loadExistingTorrentsNow(false);
 
       try{
       	managers_mon.enter();
@@ -907,6 +915,17 @@ public class GlobalManagerImpl
         		Debug.printStackTrace( e );
         	}
         }
+        
+        // Old completed downloads should have their "considered for move on completion"
+        // flag set, to prevent them being moved.
+        if (COConfigurationManager.getBooleanParameter("Set Completion Flag For Completed Downloads On Start")) {
+
+        	// We only want to know about truly complete downloads, since we aren't able to move partially complete
+        	// ones yet.
+        	if (download_manager.isDownloadComplete(true)) {
+        		download_manager.getDownloadState().setFlag(DownloadManagerState.FLAG_MOVE_ON_COMPLETION_DONE, true);
+        	}
+        }
 
         if (notifyListeners) {
         	listeners.dispatch( LDT_MANAGER_ADDED, download_manager );
@@ -1023,6 +1042,16 @@ public class GlobalManagerImpl
     	managers_mon.exit();
     }
 	
+    	// when we remove a download manager from Azureus this is the time to remove it from the record of
+    	// created torrents if present
+    
+    TOTorrent	torrent = manager.getTorrent();
+    
+    if ( torrent != null ){
+    	
+    	TorrentUtils.removeCreatedTorrent( torrent );
+    }
+    
 	manager.destroy( false );
 	
     fixUpDownloadManagerPositions();
@@ -1076,6 +1105,8 @@ public class GlobalManagerImpl
   		managers_mon.exit();
   	}
   		
+  	stats.save();
+  	
 	informDestroyInitiated();
 	
 	if ( host_support != null ){
@@ -1104,15 +1135,30 @@ public class GlobalManagerImpl
 	
   checker.stopIt();
   
-  saveDownloads( true );
-  
-  stopAllDownloads( true );
+  if ( COConfigurationManager.getBooleanParameter("Pause Downloads On Exit" )){
+	  
+	  pauseDownloads( true );
+	  
+	  	// do this before save-downloads so paused state gets saved
+	  
+	  stopAllDownloads( true );
+	  
+	  saveDownloads( true );
 
+  }else{
+  
+	  saveDownloads( true );
+  
+	  stopAllDownloads( true );
+  }
+ 
   if ( stats_writer != null ){
   	
   	stats_writer.destroy();
   }
   
+  DownloadManagerStateFactory.saveGlobalStateCache();
+  
   managers_cow	= new ArrayList();
   
   manager_map.clear();
@@ -1125,9 +1171,37 @@ public class GlobalManagerImpl
   }
   
   protected void stopAllDownloads(boolean for_close ) {
-    for (Iterator iter = managers_cow.iterator(); iter.hasNext();) {
-      DownloadManager manager = (DownloadManager) iter.next();
+	  
+	if ( for_close ){	
+		if (progress_listener != null){
+			  progress_listener.reportCurrentTask(MessageText.getString("splash.unloadingTorrents"));
+		}
+	}
+	
+	long	lastListenerUpdate = 0;
+	
+	List	managers = managers_cow;
+	
+	int nbDownloads = managers.size();
+	
+    for ( int i=0;i<nbDownloads;i++){
+    	
+      DownloadManager manager = (DownloadManager)managers.get(i);
       
+      long	now = SystemTime.getCurrentTime();
+      
+	  if(progress_listener != null &&  SystemTime.getCurrentTime() - lastListenerUpdate > 100) {
+		  lastListenerUpdate = SystemTime.getCurrentTime();
+
+		  int	currentDownload = i+1;
+		  
+		  progress_listener.reportPercent(100 * currentDownload / nbDownloads);
+		  progress_listener.reportCurrentTask(MessageText.getString("splash.unloadingTorrent") 
+				  + " " + currentDownload + " "
+				  + MessageText.getString("splash.of") + " " + nbDownloads
+				  + " : " + manager.getTorrentFileName());
+	  }
+	  
       int state = manager.getState();
       
       if( state != DownloadManager.STATE_STOPPED &&
@@ -1202,6 +1276,13 @@ public class GlobalManagerImpl
   
   public void 
   pauseDownloads() 
+  {
+	  pauseDownloads( false );
+  }
+  
+  protected void 
+  pauseDownloads(
+	boolean	tag_only )
   {
     for( Iterator i = managers_cow.iterator(); i.hasNext(); ) {
       DownloadManager manager = (DownloadManager)i.next();
@@ -1232,7 +1313,10 @@ public class GlobalManagerImpl
 	    		paused_list_mon.exit();  
 	    	}
 	    	
-        	manager.stopIt( DownloadManager.STATE_STOPPED, false, false );
+	    	if ( !tag_only ){
+	    		
+	    		manager.stopIt( DownloadManager.STATE_STOPPED, false, false );
+	    	}
   
         }catch( TOTorrentException e ) {
         	Debug.printStackTrace( e );  
@@ -1408,197 +1492,211 @@ public class GlobalManagerImpl
   
   
   
-  private void loadDownloads(GlobalMangerProgressListener listener) 
+  private void loadDownloads() 
   {
-  	int triggerOnCount = 2;
-    ArrayList downloadsAdded = new ArrayList();
-    long lastListenerUpdate = 0;
-  	try{
-  		if (listener != null)
-  			listener.reportCurrentTask(MessageText.getString("splash.loadingTorrents"));
-
-      Map map = FileUtil.readResilientConfigFile("downloads.config");
-      
-      boolean debug = Boolean.getBoolean("debug");
- 
-      Iterator iter = null;
-      //v2.0.3.0+ vs older mode
-      List downloads = (List) map.get("downloads");
-      int nbDownloads;
-      if (downloads == null) {
-        //No downloads entry, then use the old way
-        iter = map.values().iterator();
-        nbDownloads = map.size();
-      }
-      else {
-        //New way, downloads stored in a list
-        iter = downloads.iterator();
-        nbDownloads = downloads.size();
-      }
-      int currentDownload = 0;
-      while (iter.hasNext()) {
-        currentDownload++;        
-        Map mDownload = (Map) iter.next();
-        try {
-          byte[]	torrent_hash = (byte[])mDownload.get( "torrent_hash" );
-          
-          Long	lPersistent = (Long)mDownload.get( "persistent" );
-          
-          boolean	persistent = lPersistent==null || lPersistent.longValue()==1;
-          
-          
-          String fileName = new String((byte[]) mDownload.get("torrent"), Constants.DEFAULT_ENCODING);
-          
-          if(listener != null &&  SystemTime.getCurrentTime() - lastListenerUpdate > 100) {
-          	lastListenerUpdate = SystemTime.getCurrentTime();
-          	
-						listener.reportPercent(100 * currentDownload / nbDownloads);
-            listener.reportCurrentTask(MessageText.getString("splash.loadingTorrent") 
-                + " " + currentDownload + " "
-                + MessageText.getString("splash.of") + " " + nbDownloads
-                + " : " + fileName );
-          }
-          
-          //migration from using a single savePath to a separate dir and file entry
-          String	torrent_save_dir;
-          String	torrent_save_file;
-          
-          byte[] torrent_save_dir_bytes   = (byte[]) mDownload.get("save_dir");
-          
-          if ( torrent_save_dir_bytes != null ){
-          	
-          	byte[] torrent_save_file_bytes 	= (byte[]) mDownload.get("save_file");
-          	       
-          	torrent_save_dir	= new String(torrent_save_dir_bytes, Constants.DEFAULT_ENCODING);
-          	  
-          	if ( torrent_save_file_bytes != null ){
-          		
-          		torrent_save_file	= new String(torrent_save_file_bytes, Constants.DEFAULT_ENCODING);       		
-          	}else{
-          		
-          		torrent_save_file	= null;
-          	}
-          }else{
-            
-            byte[] savePathBytes = (byte[]) mDownload.get("path");
-          	torrent_save_dir 	= new String(savePathBytes, Constants.DEFAULT_ENCODING);
-          	torrent_save_file	= null;
-          }
-          
-          
-          
-          int state = DownloadManager.STATE_WAITING;
-          if (debug){
-          	
-            state = DownloadManager.STATE_STOPPED;
-            
-          }else {
-          	
-            if (mDownload.containsKey("state")) {
-              state = ((Long) mDownload.get("state")).intValue();
-              if (state != DownloadManager.STATE_STOPPED &&
-                  state != DownloadManager.STATE_QUEUED &&
-                  state != DownloadManager.STATE_WAITING)
-              	
-                state = DownloadManager.STATE_QUEUED;
-              
-            }else{
-            	
-              int stopped = ((Long) mDownload.get("stopped")).intValue();
-              
-              if (stopped == 1){
-              	
-                state = DownloadManager.STATE_STOPPED;
-              }
-            } 
-          }        
-
-	      Long seconds_downloading = (Long)mDownload.get("secondsDownloading");
-
-		  boolean	has_ever_been_started = seconds_downloading != null && seconds_downloading.longValue() > 0;
+	  try{
+		  DownloadManagerStateFactory.loadGlobalStateCache();
 		  
-					if (torrent_hash != null) {
-						saved_download_manager_state.put(new HashWrapper(torrent_hash),
-								mDownload);
-					}
-          
-          	// for non-persistent downloads the state will be picked up if the download is re-added
-          	// it won't get saved unless it is picked up, hence dead data is dropped as required
-          
-          if ( persistent ){
-          		        
-	        List file_priorities = (List) mDownload.get("file_priorities");
-	        
-          	final DownloadManager dm = 
-          		DownloadManagerFactory.create(
-          				this, torrent_hash, fileName, torrent_save_dir, torrent_save_file, 
-          				state, true, true, has_ever_been_started, file_priorities );
-
-            if (addDownloadManager(dm, false, false) == dm) {
-            	downloadsAdded.add(dm);
-
-            	if (downloadsAdded.size() >= triggerOnCount) {
-            		triggerOnCount *= 2;
-              	triggerAddListener(downloadsAdded);
-              	downloadsAdded.clear();
-            	}
-            }
-          }
-        }
-        catch (UnsupportedEncodingException e1) {
-          //Do nothing and process next.
-        }
-        catch (Throwable e) {
-        	Logger.log(new LogEvent(LOGID,
-									"Error while loading downloads.  " +
-									"One download may not have been added to the list.", e));
-        }
-      }
-      
-      //load pause/resume state
-      ArrayList pause_data = (ArrayList)map.get( "pause_data" );
-      if( pause_data != null ) {
-        try {  paused_list_mon.enter();
-          for( int i=0; i < pause_data.size(); i++ ) {
-          	Object	pd = pause_data.get(i);
-          	
-          	byte[]		key;
-          	boolean		force;
-          	
-          	if ( pd instanceof byte[]){
-          			// old style, migration purposes
-          		key 	= (byte[])pause_data.get( i );
-          		force	= false;
-          	}else{
-          		Map	m = (Map)pd;
-          		
-          		key 	= (byte[])m.get("hash");
-          		force 	= ((Long)m.get("force")).intValue() == 1;
-          	}
-            paused_list.add( new Object[]{ new HashWrapper( key ), new Boolean( force )} );
-          }
-        }
-        finally {  paused_list_mon.exit();  }
-      }
-      
-
-      // Someone could have mucked with the config file and set weird positions,
-      // so fix them up.
-      fixUpDownloadManagerPositions();
-      Logger.log(new LogEvent(LOGID, "Loaded " + managers_cow.size()
-					+ " torrents"));
-
-  	}catch( Throwable e ){
-  			// there's been problems with corrupted download files stopping AZ from starting
-  			// added this to try and prevent such foolishness
-  		
-  		Debug.printStackTrace( e );
-  	} finally {
-			loadingComplete = true;
-			triggerAddListener(downloadsAdded);
-
-			loadingSem.releaseForever();
-		}
+		  int triggerOnCount = 2;
+		  ArrayList downloadsAdded = new ArrayList();
+		  long lastListenerUpdate = 0;
+		  try{
+			  if (progress_listener != null){
+				  progress_listener.reportCurrentTask(MessageText.getString("splash.loadingTorrents"));
+			  }
+			  
+			  Map map = FileUtil.readResilientConfigFile("downloads.config");
+	
+			  boolean debug = Boolean.getBoolean("debug");
+	
+			  Iterator iter = null;
+			  //v2.0.3.0+ vs older mode
+			  List downloads = (List) map.get("downloads");
+			  int nbDownloads;
+			  if (downloads == null) {
+				  //No downloads entry, then use the old way
+				  iter = map.values().iterator();
+				  nbDownloads = map.size();
+			  }
+			  else {
+				  //New way, downloads stored in a list
+				  iter = downloads.iterator();
+				  nbDownloads = downloads.size();
+			  }
+			  int currentDownload = 0;
+			  while (iter.hasNext()) {
+				  currentDownload++;        
+				  Map mDownload = (Map) iter.next();
+				  try {
+					  byte[]	torrent_hash = (byte[])mDownload.get( "torrent_hash" );
+	
+					  Long	lPersistent = (Long)mDownload.get( "persistent" );
+	
+					  boolean	persistent = lPersistent==null || lPersistent.longValue()==1;
+	
+	
+					  String fileName = new String((byte[]) mDownload.get("torrent"), Constants.DEFAULT_ENCODING);
+	
+					  if(progress_listener != null &&  SystemTime.getCurrentTime() - lastListenerUpdate > 100) {
+						  lastListenerUpdate = SystemTime.getCurrentTime();
+	
+						  progress_listener.reportPercent(100 * currentDownload / nbDownloads);
+						  progress_listener.reportCurrentTask(MessageText.getString("splash.loadingTorrent") 
+								  + " " + currentDownload + " "
+								  + MessageText.getString("splash.of") + " " + nbDownloads
+								  + " : " + fileName );
+					  }
+	
+					  //migration from using a single savePath to a separate dir and file entry
+					  String	torrent_save_dir;
+					  String	torrent_save_file;
+	
+					  byte[] torrent_save_dir_bytes   = (byte[]) mDownload.get("save_dir");
+	
+					  if ( torrent_save_dir_bytes != null ){
+	
+						  byte[] torrent_save_file_bytes 	= (byte[]) mDownload.get("save_file");
+	
+						  torrent_save_dir	= new String(torrent_save_dir_bytes, Constants.DEFAULT_ENCODING);
+	
+						  if ( torrent_save_file_bytes != null ){
+	
+							  torrent_save_file	= new String(torrent_save_file_bytes, Constants.DEFAULT_ENCODING);       		
+						  }else{
+	
+							  torrent_save_file	= null;
+						  }
+					  }else{
+	
+						  byte[] savePathBytes = (byte[]) mDownload.get("path");
+						  torrent_save_dir 	= new String(savePathBytes, Constants.DEFAULT_ENCODING);
+						  torrent_save_file	= null;
+					  }
+	
+	
+	
+					  int state = DownloadManager.STATE_WAITING;
+					  if (debug){
+	
+						  state = DownloadManager.STATE_STOPPED;
+	
+					  }else {
+	
+						  if (mDownload.containsKey("state")) {
+							  state = ((Long) mDownload.get("state")).intValue();
+							  if (state != DownloadManager.STATE_STOPPED &&
+									  state != DownloadManager.STATE_QUEUED &&
+									  state != DownloadManager.STATE_WAITING)
+	
+								  state = DownloadManager.STATE_QUEUED;
+	
+						  }else{
+	
+							  int stopped = ((Long) mDownload.get("stopped")).intValue();
+	
+							  if (stopped == 1){
+	
+								  state = DownloadManager.STATE_STOPPED;
+							  }
+						  } 
+					  }        
+	
+					  Long seconds_downloading = (Long)mDownload.get("secondsDownloading");
+	
+					  boolean	has_ever_been_started = seconds_downloading != null && seconds_downloading.longValue() > 0;
+	
+					  if (torrent_hash != null) {
+						  saved_download_manager_state.put(new HashWrapper(torrent_hash),
+								  mDownload);
+					  }
+	
+					  // for non-persistent downloads the state will be picked up if the download is re-added
+					  // it won't get saved unless it is picked up, hence dead data is dropped as required
+	
+					  if ( persistent ){
+	
+						  List file_priorities = (List) mDownload.get("file_priorities");
+	
+						  final DownloadManager dm = 
+							  DownloadManagerFactory.create(
+									  this, torrent_hash, fileName, torrent_save_dir, torrent_save_file, 
+									  state, true, true, has_ever_been_started, file_priorities );
+	
+						  if (addDownloadManager(dm, false, false) == dm) {
+							  downloadsAdded.add(dm);
+	
+							  if (downloadsAdded.size() >= triggerOnCount) {
+								  triggerOnCount *= 2;
+								  triggerAddListener(downloadsAdded);
+								  downloadsAdded.clear();
+							  }
+						  }
+					  }
+				  }
+				  catch (UnsupportedEncodingException e1) {
+					  //Do nothing and process next.
+				  }
+				  catch (Throwable e) {
+					  Logger.log(new LogEvent(LOGID,
+							  "Error while loading downloads.  " +
+							  "One download may not have been added to the list.", e));
+				  }
+			  }
+	
+			  // This is set to true by default, but once the downloads have been loaded, we have no reason to ever
+			  // to do this check again - we only want to do it once to upgrade the state of existing downloads
+			  // created before this code was around.
+			  COConfigurationManager.setParameter("Set Completion Flag For Completed Downloads On Start", false);
+	
+			  //load pause/resume state
+			  ArrayList pause_data = (ArrayList)map.get( "pause_data" );
+			  if( pause_data != null ) {
+				  try {  paused_list_mon.enter();
+				  for( int i=0; i < pause_data.size(); i++ ) {
+					  Object	pd = pause_data.get(i);
+	
+					  byte[]		key;
+					  boolean		force;
+	
+					  if ( pd instanceof byte[]){
+						  // old style, migration purposes
+						  key 	= (byte[])pause_data.get( i );
+						  force	= false;
+					  }else{
+						  Map	m = (Map)pd;
+	
+						  key 	= (byte[])m.get("hash");
+						  force 	= ((Long)m.get("force")).intValue() == 1;
+					  }
+					  paused_list.add( new Object[]{ new HashWrapper( key ), new Boolean( force )} );
+				  }
+				  }
+				  finally {  paused_list_mon.exit();  }
+			  }
+	
+	
+			  // Someone could have mucked with the config file and set weird positions,
+			  // so fix them up.
+			  fixUpDownloadManagerPositions();
+			  Logger.log(new LogEvent(LOGID, "Loaded " + managers_cow.size()
+					  + " torrents"));
+	
+		  }catch( Throwable e ){
+			  // there's been problems with corrupted download files stopping AZ from starting
+			  // added this to try and prevent such foolishness
+	
+			  Debug.printStackTrace( e );
+		  } finally {
+			  loadingComplete = true;
+			  triggerAddListener(downloadsAdded);
+	
+			  loadingSem.releaseForever();
+		  }
+			  
+	  }finally{
+		  
+		  DownloadManagerStateFactory.discardGlobalStateCache();
+	  }
   }
   
   private void triggerAddListener(List downloadsToAdd) {
@@ -1620,7 +1718,7 @@ public class GlobalManagerImpl
   }
 
 
-  public void 
+  protected void 
   saveDownloads(
   	boolean	immediate ) 
   {
@@ -2321,6 +2419,63 @@ public class GlobalManagerImpl
 		}
 	}
 	
+	protected void
+	seedPieceRecheck()
+	{
+		List	managers = managers_cow;
+		
+		if ( next_seed_piece_recheck_index >= managers.size()){
+			
+			next_seed_piece_recheck_index	= 0;
+		}
+		
+		for (int i=next_seed_piece_recheck_index;i<managers.size();i++){
+			
+			DownloadManager manager = (DownloadManager)managers.get(i);
+			
+			if ( seedPieceRecheck( manager )){
+				
+				next_seed_piece_recheck_index = i+1;
+				
+				if ( next_seed_piece_recheck_index >= managers.size()){
+					
+					next_seed_piece_recheck_index	= 0;
+				}
+				
+				return;
+			}
+		}
+		
+		for (int i=0;i<next_seed_piece_recheck_index;i++){
+			
+			DownloadManager manager = (DownloadManager)managers.get(i);
+			
+			if ( seedPieceRecheck( manager )){
+				
+				next_seed_piece_recheck_index = i+1;
+				
+				if ( next_seed_piece_recheck_index >= managers.size()){
+					
+					next_seed_piece_recheck_index	= 0;
+				}
+				
+				return;
+			}
+		}
+	}
+
+	protected boolean
+	seedPieceRecheck(
+		DownloadManager	manager )
+	{
+		if ( manager.getState() != DownloadManager.STATE_SEEDING ){
+			
+			return( false );
+		}
+		
+		return( manager.seedPieceRecheck());
+	}
+	
 	protected DownloadManagerInitialisationAdapter
 	getDMAdapter(
 		DownloadManagerInitialisationAdapter	adapter )
