@@ -43,7 +43,12 @@ PEPeerTransport
   public static final int CONNECTION_WAITING_FOR_HANDSHAKE  = 2;
   public static final int CONNECTION_FULLY_ESTABLISHED      = 4;
   
-
+  		/**
+  		 * Start message processing for the peer
+  		 */
+  
+  	public void
+  	start();
   	
 	public void
 	sendChoke();
@@ -78,14 +83,22 @@ PEPeerTransport
    * @param pieceNumber
    * @param pieceOffset
    * @param pieceLength
-   * @return true if the piece is really requested
+   * @return request if actually requested, null otherwise
    */
-	public boolean 
+	public DiskManagerReadRequest 
 	request(
 		int pieceNumber, 
 		int pieceOffset, 
 		int pieceLength );
 
+	/**
+	 * Returns the index of this request in the peer's queue or -1 if not found
+	 * @return
+	 */
+	
+	public int
+	getRequestIndex(
+		DiskManagerReadRequest request );
   
   /**
    * Close the peer connection
@@ -180,6 +193,12 @@ PEPeerTransport
    */
   public PeerItem getPeerItemIdentity();
   
+  /**
+   * is peer waiting for a disk read with no network writes queued
+   * @return
+   */
+  
+  public boolean isStalledPendingLoad();
   
   /**
    * Is the connection within the local LAN network.
