@@ -39,16 +39,22 @@ TOTorrentDeserialiseImpl
 		File		file )
 		
 		throws TOTorrentException
-	{		
+	{	
+		if ( !file.exists()){
+			throw( new TOTorrentException( 	"Torrent file '" + file.toString() + "' does not exist",
+					TOTorrentException.RT_FILE_NOT_FOUND ));
+
+		}
+		
 		if(!file.isFile()) {
 			
-			throw( new TOTorrentException( 	"TOTorrentDeserialise: Torrent must be a file ('" + file.getName() + "')",
+			throw( new TOTorrentException( 	"Torrent must be a file ('" + file.toString() + "')",
 											TOTorrentException.RT_FILE_NOT_FOUND ));
 		}
 
 		if ( file.length() == 0 ){
 		
-			throw( new TOTorrentException( 	"TOTorrentDeserialise: Torrent is zero length ('" + file.getName() + "')",
+			throw( new TOTorrentException( 	"Torrent is zero length ('" + file.toString() + "')",
 											TOTorrentException.RT_ZERO_LENGTH ));
 			
 		}
@@ -64,9 +70,9 @@ TOTorrentDeserialiseImpl
 	
 			construct( fis );
 	
-		}catch( IOException e ){
+		}catch( Throwable e ){
 			
-			throw( new TOTorrentException( "TOTorrentDeserialise: IO exception reading torrent '" + e.toString()+ "'",
+			throw( new TOTorrentException( "Error reading torrent file '" + file.toString() + " - " + Debug.getNestedExceptionMessage(e),
 											TOTorrentException.RT_READ_FAILS ));
 			
 		}finally{
@@ -185,9 +191,9 @@ TOTorrentDeserialiseImpl
 			
 				metaInfo.write(buf, 0, nbRead);
 			}
-		}catch( IOException e ){
+		}catch( Throwable e ){
 			
-			throw( new TOTorrentException( "TOTorrentDeserialise: IO exception reading torrent '" + e.toString()+ "'",
+			throw( new TOTorrentException( "Error reading torrent: " + Debug.getNestedExceptionMessage(e),
 											TOTorrentException.RT_READ_FAILS ));
 		}
 		
@@ -209,8 +215,8 @@ TOTorrentDeserialiseImpl
 			
 		}catch( IOException e ){
 			
-			throw( new TOTorrentException( 	"IO Error: " + e.getMessage(),
-											TOTorrentException.RT_DECODE_FAILS ));
+			throw( new TOTorrentException( 	"Error reading torrent: " + Debug.getNestedExceptionMessage(e),
+											TOTorrentException.RT_DECODE_FAILS, e ));
 		}
 	}
 	
@@ -386,7 +392,7 @@ TOTorrentDeserialiseImpl
 						}
 					}catch( Exception e ){
 						
-						System.out.println( "TOTorrentDeserialise: creation_date extraction fails, ignoring");
+						System.out.println( "creation_date extraction fails, ignoring");
 					}
 									
 				}else if ( key.equalsIgnoreCase( TK_INFO )){
@@ -453,16 +459,30 @@ TOTorrentDeserialiseImpl
 			}
 		
 			setName((byte[])info.get( TK_NAME ));
+				
+			long	piece_length = ((Long)info.get( TK_PIECE_LENGTH )).longValue();
+			
+			if ( piece_length <= 0 ){
+				
+				throw( new TOTorrentException( "Decode fails, piece-length is invalid",
+						TOTorrentException.RT_DECODE_FAILS ));
+			}
+			
+			setPieceLength( piece_length );
 						
 			setHashFromInfo( info );
 			
 			Long simple_file_length = (Long)info.get( TK_LENGTH );
 			
+			long	total_length = 0;
+			
 			if ( simple_file_length != null ){
 			
 				setSimpleTorrent( true );
 				
-				setFiles( new TOTorrentFileImpl[]{ new TOTorrentFileImpl( this, simple_file_length.longValue(), new byte[][]{getName()})});
+				total_length = simple_file_length.longValue();
+				
+				setFiles( new TOTorrentFileImpl[]{ new TOTorrentFileImpl( this, 0, total_length, new byte[][]{getName()})});
 				
 			}else{
 				
@@ -487,7 +507,9 @@ TOTorrentDeserialiseImpl
 						path_comps[j] = (byte[])paths.get(j);
 					}
 					
-					TOTorrentFileImpl file = files[i] = new TOTorrentFileImpl( this, len, path_comps );
+					TOTorrentFileImpl file = files[i] = new TOTorrentFileImpl( this, total_length, len, path_comps );
+					
+					total_length += len;
 					
 						// preserve any non-standard attributes
 						
@@ -510,12 +532,27 @@ TOTorrentDeserialiseImpl
 				
 				setFiles( files );
 			}
-											 
-			setPieceLength( ((Long)info.get( TK_PIECE_LENGTH )).longValue());
-						
+							
 			byte[]	flat_pieces = (byte[])info.get( TK_PIECES );
 			
-			byte[][]pieces = new byte[flat_pieces.length/20][20];
+				// work out how many pieces we require for the torrent
+			
+			int	pieces_required = (int)((total_length + (piece_length-1)) / piece_length);
+			
+			int		pieces_supplied = flat_pieces.length/20;
+			
+			if ( pieces_supplied < pieces_required ){
+				
+				throw( new TOTorrentException( "Decode fails, insufficient pieces supplied",
+						TOTorrentException.RT_DECODE_FAILS ));
+			}
+			
+			if ( pieces_supplied > pieces_required ){
+				
+				Debug.out( "Torrent '" + new String( getName()) + "' has too many pieces (required=" + pieces_required + ",supplied=" + pieces_supplied + ") - ignoring excess" );
+			}
+
+			byte[][]pieces = new byte[pieces_supplied][20];
 			
 			for (int i=0;i<pieces.length;i++){
 				
