@@ -26,11 +26,15 @@ import java.io.*;
 import java.net.*;
 import java.util.*;
 
+import org.gudy.azureus2.core3.logging.LogRelation;
 import org.gudy.azureus2.core3.torrent.*;
 import org.gudy.azureus2.core3.util.*;
 
+import com.aelitis.azureus.core.AzureusCoreFactory;
+
 public class 
 TOTorrentImpl
+	extends LogRelation
 	implements TOTorrent
 {
 	protected static final String TK_ANNOUNCE			= "announce";
@@ -47,23 +51,25 @@ TOTorrentImpl
 	protected static final String TK_PIECE_LENGTH		= "piece length";
 	protected static final String TK_PIECES				= "pieces";
 		
+	protected static final String TK_PRIVATE			= "private";
+
 	protected static final String TK_NAME_UTF8			= "name.utf-8";
 	protected static final String TK_PATH_UTF8			= "path.utf-8";
 	protected static final String TK_COMMENT_UTF8		= "comment.utf-8";
 	
 	protected static final List	TK_ADDITIONAL_OK_ATTRS = 
-		Arrays.asList(new String[]{TK_COMMENT_UTF8});
+		Arrays.asList(new String[]{TK_COMMENT_UTF8, AZUREUS_PROPERTIES });
 	
 	private byte[]							torrent_name;
 	private byte[]							torrent_name_utf8;
 	
 	private byte[]							comment;
 	private URL								announce_url;
-	private TOTorrentAnnounceURLGroupImpl	announce_group = new TOTorrentAnnounceURLGroupImpl();
+	private TOTorrentAnnounceURLGroupImpl	announce_group = new TOTorrentAnnounceURLGroupImpl(this);
 	
 	private long		piece_length;
-	private int			number_of_pieces;
 	private byte[][]	pieces;
+	private int			number_of_pieces;
 	
 	private byte[]		torrent_hash;
 	private HashWrapper	torrent_hash_wrapper;
@@ -77,6 +83,9 @@ TOTorrentImpl
 	private Map					additional_properties 		= new HashMap();
 	private Map					additional_info_properties	= new HashMap();
 	
+	private boolean				created;
+	private boolean				serialising;
+	
 	protected AEMonitor this_mon 	= new AEMonitor( "TOTorrent" );
 
 	/** 
@@ -100,18 +109,21 @@ TOTorrentImpl
 		
 		throws TOTorrentException
 	{
+		created	= true;
+		
 		try{
 		
 			torrent_name		= _torrent_name.getBytes( Constants.DEFAULT_ENCODING );
 			
 			torrent_name_utf8	= torrent_name;
 			
-			announce_url		= _announce_url;
+			setAnnounceURL( _announce_url );
+			
 			simple_torrent		= _simple_torrent;
 			
 		}catch( UnsupportedEncodingException e ){
 			
-			throw( new TOTorrentException( 	"TOTorrent: unsupported encoding for '" + _torrent_name + "'",
+			throw( new TOTorrentException( 	"Unsupported encoding for '" + _torrent_name + "'",
 											TOTorrentException.RT_UNSUPPORTED_ENCODING));
 		}
 	}
@@ -121,32 +133,115 @@ TOTorrentImpl
 		final File		output_file )
 	
 		throws TOTorrentException
-	{		
+	{	
+			// we have to defer marking as created until some kind of persisting occurs as we don't know that the info-hash is "permanent" until#
+			// this point (external code can set info-hash internal properties between create + complete )
+		
+		if ( created ){
+			
+			TorrentUtils.addCreatedTorrent( this );
+		}
+		
 		byte[]	res = serialiseToByteArray();
 						
         BufferedOutputStream bos = null;
 						
 		try{
-              File temp = new File( output_file.getParentFile(), output_file.getName() + ".saving");
-              
-              bos = new BufferedOutputStream( new FileOutputStream( temp, false ), 8192 );
-							bos.write( res );
-							bos.flush();
-              bos.close();
-              bos = null;
+			File parent = output_file.getParentFile();
+			if (parent == null) {
+				throw new TOTorrentException( "Path '" + output_file + "' is invalid", TOTorrentException.RT_WRITE_FAILS);
+			}
+			
+			// We would expect this to be normally true most of the time.
+			if (!parent.isDirectory()) {
+				
+				// Try to create a directory.
+				boolean dir_created = FileUtil.mkdirs(parent);
+				
+				// Something strange going on...
+				if (!dir_created) {
+					
+					// Does it exist already?
+					if (parent.exists()) {
+						
+						// And it really isn't a directory?
+						if (!parent.isDirectory()) {
+							
+							// How strange.
+							throw new TOTorrentException( "Path '" + output_file + "' is invalid", TOTorrentException.RT_WRITE_FAILS);
+							
+						}
+						
+						// It is a directory which does exist. But we tested for that earlier. Perhaps it has been created in the
+						// meantime.
+						else {
+							/* do nothing */
+						}
+					}
+					
+					// It doesn't exist, and we couldn't create it.
+					else {
+						throw new TOTorrentException( "Failed to create directory '" + parent + "'", TOTorrentException.RT_WRITE_FAILS );
+					}
+				} // end if (!dir_created)
+				
+			} // end if (!parent.isDirectory)
+			
+			
+			File temp = new File( parent, output_file.getName() + ".saving");
+            
+			if ( temp.exists()){
+				
+				if ( !temp.delete()){
+					
+					throw( new TOTorrentException( "Insufficient permissions to delete '" + temp + "'", TOTorrentException.RT_WRITE_FAILS ));
+				}
+			}else{
+				
+				boolean	ok = false;
+				
+				try{
+					ok = temp.createNewFile();
+					
+				}catch( Throwable e ){
+				}
+				
+				if ( !ok ){
+					
+					throw( new TOTorrentException( "Insufficient permissions to write '" + temp + "'", TOTorrentException.RT_WRITE_FAILS ));
+
+				}
+			}
+			
+            FileOutputStream fos = new FileOutputStream( temp, false );
+			
+            bos = new BufferedOutputStream( fos, 8192 );
+			
+            bos.write( res );
+			
+            bos.flush();
+			
+            fos.getFD().sync();
+            
+            bos.close();
+            
+            bos = null;
               
               //only use newly saved file if it got this far, i.e. it was written successfully
-              if ( temp.length() > 1L ) {
-                if ( output_file.exists() ) {
-                	output_file.delete();
-                }
+            
+            if ( temp.length() > 1L ) {
+            	output_file.delete(); // Will fail silently if it doesn't exist.
                 temp.renameTo( output_file );
-              }
-             							
+            }
+             	
+		}catch( TOTorrentException e ){
+			
+			throw( e );
+			
 		}catch( Throwable e){
 							
-			throw( new TOTorrentException( 	"TOTorrent::serialise: fails '" + e.toString() + "'",
-															TOTorrentException.RT_WRITE_FAILS ));
+			throw( new TOTorrentException( 	"Failed to serialise torrent: " + Debug.getNestedExceptionMessage(e),
+											TOTorrentException.RT_WRITE_FAILS ));
 							
 		}finally{
 							
@@ -168,6 +263,11 @@ TOTorrentImpl
 	
 		throws TOTorrentException
 	{
+		if ( created ){
+			
+			TorrentUtils.addCreatedTorrent( this );
+		}
+		
 		Map	root = serialiseToMap();
 			
 		try{
@@ -176,7 +276,7 @@ TOTorrentImpl
 		}catch( IOException e ){
 
 			throw( 	new TOTorrentException( 	
-							"TOTorrent::serialiseToByteArray: fails '" + e.toString() + "'",
+							"Failed to serialise torrent: " + Debug.getNestedExceptionMessage(e),
 							TOTorrentException.RT_WRITE_FAILS ));
 			
 		}
@@ -187,6 +287,21 @@ TOTorrentImpl
 	
 		throws TOTorrentException
 	{		
+			// protect against recursion when getting the hash 
+		
+		if ( created && !serialising ){
+			
+			try{
+				serialising	= true;	// not thread safe but we can live without the hassle of using TLS or whatever
+					
+				TorrentUtils.addCreatedTorrent( this );
+				
+			}finally{
+				
+				serialising = false;
+			}
+		}
+		
 		Map	root = new HashMap();
 		
 		writeStringToMetaData( root, TK_ANNOUNCE, announce_url.toString());
@@ -356,6 +471,11 @@ TOTorrentImpl
 		  
 	  throws TOTorrentException
 	{
+		if ( created ){
+			
+			TorrentUtils.addCreatedTorrent( this );
+		}
+		
 		TOTorrentXMLSerialiser	serialiser = new TOTorrentXMLSerialiser( this );
 		
 		serialiser.serialiseToFile( file );
@@ -420,11 +540,18 @@ TOTorrentImpl
 		return( announce_url );
 	}
 	
-	public void
+	public boolean
 	setAnnounceURL(
 		URL		url )
 	{
-		announce_url	= url;
+		URL newURL = anonymityTransform( url );
+		String s0 = (newURL == null) ? "" : newURL.toString();
+		String s1 = (announce_url == null) ? "" : announce_url.toString();
+		if (s0.equals(s1))
+			return false;
+		
+		announce_url	= newURL;
+		return true;
 	}
 
 	public long
@@ -469,6 +596,12 @@ TOTorrentImpl
 		return( created_by );
 	}
 	
+	public boolean
+	isCreated()
+	{
+		return( created );
+	}
+	
 	public byte[]
 	getHash()
 	
@@ -530,11 +663,39 @@ TOTorrentImpl
 			
 		}catch( Throwable e ){
 				
-			throw( new TOTorrentException( 	"TOTorrent::setHashFromInfo: fails '" + e.toString() + "'",
+			throw( new TOTorrentException( 	"Failed to calculate hash: " + Debug.getNestedExceptionMessage(e),
 											TOTorrentException.RT_HASH_FAILS ));
 		}
 	}
+	
+	public void
+	setPrivate(
+		boolean	_private_torrent )
+	
+		throws TOTorrentException
+	{
+		additional_info_properties.put( TK_PRIVATE, new Long(_private_torrent?1:0));
 		
+			// update torrent hash
+		
+		torrent_hash	= null;
+		
+		getHash();
+	}
+	
+	public boolean
+	getPrivate()
+	{
+		Object o = additional_info_properties.get( TK_PRIVATE );
+		
+		if ( o instanceof Long ){
+			
+			return(((Long)o).intValue() != 0 );
+		}
+		
+		return( false );
+	}
+	
 	public TOTorrentAnnounceURLGroup
 	getAnnounceURLGroup()
 	{
@@ -545,7 +706,7 @@ TOTorrentImpl
 	addTorrentAnnounceURLSet(
 		URL[]		urls )
 	{
-		announce_group.addSet( new TOTorrentAnnounceURLSetImpl( urls ));
+		announce_group.addSet( new TOTorrentAnnounceURLSetImpl( this, urls ));
 	}
 	
 	public long
@@ -577,6 +738,16 @@ TOTorrentImpl
 	public int
 	getNumberOfPieces()
 	{
+			// to support buggy torrents with extraneous pieces (they seem to exist) we calculate
+			// the required number of pieces rather than the using the actual. Note that we 
+			// can't adjust the pieces array itself as this results in incorrect torrent hashes
+			// being derived later after a save + restore
+		
+		if ( number_of_pieces == 0 ){
+		
+			number_of_pieces = (int)((getSize() + (piece_length-1)) / piece_length );
+		}
+		
 		return( number_of_pieces );
 	}
 	
@@ -591,11 +762,6 @@ TOTorrentImpl
 		byte[][]	_pieces )
 	{
 		pieces = _pieces;
-		
-		if ( pieces != null ){
-			
-			number_of_pieces = pieces.length;
-		}
 	}
 	
 	public TOTorrentFile[]
@@ -692,7 +858,14 @@ TOTorrentImpl
 	getAdditionalByteArrayProperty(
 		String		name )
 	{
-		return((byte[])additional_properties.get( name ));
+		Object	obj = additional_properties.get( name );
+		
+		if ( obj instanceof byte[] ){
+			
+			return((byte[])obj);
+		}
+		
+		return( null );
 	}
 	
 	public void
@@ -707,7 +880,14 @@ TOTorrentImpl
 	getAdditionalLongProperty(
 		String		name )
 	{
-		return((Long)additional_properties.get( name ));
+		Object	obj = additional_properties.get( name );
+		
+		if ( obj instanceof Long ){
+			
+			return((Long)obj);
+		}
+		
+		return( null );
 	}
 	
 	public void
@@ -722,7 +902,14 @@ TOTorrentImpl
 	getAdditionalListProperty(
 		String		name )
 	{
-		return((List)additional_properties.get( name ));
+		Object	obj = additional_properties.get( name );
+		
+		if ( obj instanceof List ){
+			
+			return((List)obj);
+		}
+		
+		return( null );	
 	}
 	
 	public void
@@ -737,7 +924,14 @@ TOTorrentImpl
 	getAdditionalMapProperty(
 		String		name )
 	{
-		return((Map)additional_properties.get( name ));
+		Object	obj = additional_properties.get( name );
+		
+		if ( obj instanceof Map ){
+			
+			return((Map)obj);
+		}
+		
+		return( null );	
 	}
 	
 	public Object
@@ -803,7 +997,14 @@ TOTorrentImpl
 		
 		throws TOTorrentException
 	{
-		return(readStringFromMetaData((byte[])meta_data.get(name)));			
+		Object	obj = meta_data.get(name);
+		
+		if ( obj instanceof byte[]){
+			
+			return(readStringFromMetaData((byte[])obj));
+		}
+		
+		return( null );
 	}
 	
 	protected String
@@ -822,7 +1023,7 @@ TOTorrentImpl
 			
 		}catch( UnsupportedEncodingException e ){
 			
-			throw( new TOTorrentException( 	"TOTorrentDeserialise: unsupported encoding for '" + value + "'",
+			throw( new TOTorrentException( 	"Unsupported encoding for '" + value + "'",
 											TOTorrentException.RT_UNSUPPORTED_ENCODING));
 		}
 	}
@@ -850,11 +1051,47 @@ TOTorrentImpl
 			
 		}catch( UnsupportedEncodingException e ){
 			
-			throw( new TOTorrentException( 	"TOTorrent::writeStringToMetaData: unsupported encoding for '" + value + "'",
+			throw( new TOTorrentException( 	"Unsupported encoding for '" + value + "'",
 											TOTorrentException.RT_UNSUPPORTED_ENCODING));
 		}
 	}
 	
+	protected URL
+	anonymityTransform(
+		URL		url )
+	{
+		/*
+		 * 	hmm, doing this is harder than it looks as we have issues hosting
+		 *  (both starting tracker instances and also short-cut loopback for seeding
+		 *  leave as is for the moment
+		if ( HostNameToIPResolver.isNonDNSName( url.getHost())){
+			
+			// remove the port as it is uninteresting and could leak information about the
+			// tracker
+			
+			String	url_string = url.toString();
+			
+			String	port_string = ":" + (url.getPort()==-1?url.getDefaultPort():url.getPort());
+			
+			int	port_pos = url_string.indexOf( ":" + url.getPort());
+			
+			if ( port_pos != -1 ){
+				
+				try{
+					
+					return( new URL( url_string.substring(0,port_pos) + url_string.substring(port_pos+port_string.length())));
+					
+				}catch( MalformedURLException e){
+					
+					Debug.printStackTrace(e);
+				}
+			}
+		}
+		*/
+		
+		return( url );
+	}
+	
 	public void
 	print()
 	{
@@ -947,4 +1184,25 @@ TOTorrentImpl
 	{
 		return( this_mon );
 	}
+
+	/* (non-Javadoc)
+	 * @see org.gudy.azureus2.core3.logging.LogRelation#getLogRelationText()
+	 */
+	public String getRelationText() {
+		return "Torrent: '" + new String(torrent_name) + "'";  
+	}
+
+	/* (non-Javadoc)
+	 * @see org.gudy.azureus2.core3.logging.LogRelation#queryForClass(java.lang.Class)
+	 */
+	public Object[] getQueryableInterfaces() {
+		// yuck
+		try {
+			return new Object[] { AzureusCoreFactory.getSingleton()
+					.getGlobalManager().getDownloadManager(this) };
+		} catch (Exception e) {
+		}
+
+		return null;
+	}
 }
\ No newline at end of file
