@@ -112,6 +112,12 @@ TRHostExternalTorrent
 		return( null );
 	}
 	
+	public boolean
+	isCreated()
+	{
+		return( false );
+	}
+	
 	public URL
 	getAnnounceURL()
 	{
@@ -169,7 +175,6 @@ TRHostExternalTorrent
 	addTorrentAnnounceURLSet( 
 		URL[] urls )
 	{
-		if ( urls == null ){}
 	}
   
 	
