@@ -34,32 +34,36 @@ import org.gudy.azureus2.core3.config.*;
 import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.core3.tracker.host.*;
 import org.gudy.azureus2.core3.tracker.server.*;
+import org.gudy.azureus2.core3.tracker.util.TRTrackerUtils;
 import org.gudy.azureus2.core3.tracker.client.*;
 import org.gudy.azureus2.core3.torrent.*;
 
 public class 
 TRHostImpl
-	implements 	TRHost, TRTrackerClientFactoryListener, 
+	implements 	TRHost, TRTrackerAnnouncerFactoryListener, 
 				TRTrackerServerListener, TRTrackerServerFactoryListener,
 				TRTrackerServerRequestListener, TRTrackerServerAuthenticationListener
 {
-	protected static final int URL_DEFAULT_PORT		= 80;	// port to use if none in announce URL
-	protected static final int URL_DEFAULT_PORT_SSL	= 443;	// port to use if none in announce URL
+	private static final LogIDs LOGID = LogIDs.TRACKER;
+	private static final int URL_DEFAULT_PORT		= 80;	// port to use if none in announce URL
+	private static final int URL_DEFAULT_PORT_SSL	= 443;	// port to use if none in announce URL
 	
-	protected static final int STATS_PERIOD_SECS	= 60;
+	public static final int STATS_PERIOD_SECS		= 60;
+	private static final int TICK_PERIOD_SECS			= 10;
+	private static final int TICKS_PER_STATS_PERIOD	= STATS_PERIOD_SECS/TICK_PERIOD_SECS;
 		
-	protected static TRHostImpl	singleton;
-	protected static AEMonitor 	class_mon 	= new AEMonitor( "TRHost:class" );
+	private static TRHostImpl	singleton;
+	private static AEMonitor 	class_mon 	= new AEMonitor( "TRHost:class" );
 
-	protected TRHostConfigImpl		config;
+	private TRHostConfigImpl		config;
 		
-	protected Hashtable				server_map 	= new Hashtable();
+	private Hashtable				server_map 	= new Hashtable();
 	
-	protected List	host_torrents			= new ArrayList();
-	protected Map	host_torrent_hash_map	= new HashMap();
+	private List	host_torrents			= new ArrayList();
+	private Map	host_torrent_hash_map	= new HashMap();
 	
-	protected Map	host_torrent_map		= new HashMap();
-	protected Map	tracker_client_map		= new HashMap();
+	private Map	host_torrent_map		= new HashMap();
+	private Map	tracker_client_map		= new HashMap();
 	
 	private static final int LDT_TORRENT_ADDED			= 1;
 	private static final int LDT_TORRENT_REMOVED		= 2;
@@ -92,12 +96,30 @@ TRHostImpl
 			}
 		});	
 	
-	protected List	auth_listeners		= new ArrayList();
+	private static boolean host_add_announce_urls;
 	
-	protected boolean	server_factory_listener_added;
+	static{
+		COConfigurationManager.addAndFireParameterListener(
+				"Tracker Host Add Our Announce URLs",
+				new ParameterListener()
+				{
+					public void 
+					parameterChanged(
+						String name )
+					{
+						host_add_announce_urls = COConfigurationManager.getBooleanParameter( name );
+					}
+				});
+	}
+	
+	private List	auth_listeners		= new ArrayList();
+	
+	private boolean	server_factory_listener_added;
 	
 	protected AEMonitor this_mon 	= new AEMonitor( "TRHost" );
 
+	private volatile boolean	closed;
+	
 	public static TRHost
 	create()
 	{
@@ -132,89 +154,120 @@ TRHostImpl
 					
 			config = new TRHostConfigImpl(this);	
 			
-			TRTrackerClientFactory.addListener( this );
+			TRTrackerAnnouncerFactory.addListener( this );
 			
 			Thread t = new AEThread("TRHost::stats.loop")
 						{
+							private int	tick_count = 0;
+							
+							private Set	failed_ports = new HashSet();
+							
 							public void
 							runSupport()
 							{
 								while(true){
 									
-									try{											
-										if ( COConfigurationManager.getBooleanParameter( "Tracker Port Enable", false )){
+									try{	
 										
-											try{
-													
-												int port = COConfigurationManager.getIntParameter("Tracker Port", TRHost.DEFAULT_PORT );
-												
-												startServer( TRTrackerServerFactory.PR_TCP, port, false );
-													
-											}catch( Throwable e ){
-												
-												Debug.printStackTrace( e );
-											}
-										}
+										URL[][]	url_sets = TRTrackerUtils.getAnnounceURLs();
 										
-										if ( COConfigurationManager.getBooleanParameter( "Tracker Port UDP Enable", false )){
+										for (int i=0;i<url_sets.length;i++){
 											
-											try{
-														
-												int port = COConfigurationManager.getIntParameter("Tracker Port", TRHost.DEFAULT_PORT );
-													
-												startServer( TRTrackerServerFactory.PR_UDP, port, false );
-														
-											}catch( Throwable e ){
-													
-												Debug.printStackTrace( e );
-											}
-										}
+											URL[]	urls = url_sets[i];
 											
-										if ( COConfigurationManager.getBooleanParameter( "Tracker Port SSL Enable", false )){
-										
-											try{
-													
-												int port = COConfigurationManager.getIntParameter("Tracker Port SSL", TRHost.DEFAULT_PORT_SSL );
+											for (int j=0;j<urls.length;j++){
 												
-												startServer( TRTrackerServerFactory.PR_TCP, port, true );
+												URL	url = urls[j];
 														
-											}catch( Throwable e ){
+												int port = url.getPort();
 												
-												Debug.printStackTrace( e );
-											}
-										}						
-										
-										Thread.sleep( STATS_PERIOD_SECS*1000 );
-										
-										try{
-											this_mon.enter();
+												if ( port == -1 ){
+													
+													port = url.getDefaultPort();
+												}
+												
+												String	protocol = url.getProtocol().toLowerCase();
+												
+												try{
+													if ( protocol.equals( "http" )){
+												
+														startServer( TRTrackerServerFactory.PR_TCP, port, false );
+													
+													}else if ( protocol.equals( "udp" )){
+														
+														startServer( TRTrackerServerFactory.PR_UDP, port, false );
+
+													}else if ( protocol.equals( "https" )){
+														
+														startServer( TRTrackerServerFactory.PR_TCP, port, true );
+
+													}else{
+														
+														Debug.out( "Unknown protocol '" + protocol + "'" );
+													}
+													
+												}catch( Throwable e ){
 											
-											for (int i=0;i<host_torrents.size();i++){
-				
-												TRHostTorrent	ht = (TRHostTorrent)host_torrents.get(i);
-												
-												if ( ht instanceof TRHostTorrentHostImpl ){
-																					
-													((TRHostTorrentHostImpl)ht).updateStats();
-													
-												}else{
-													
-													((TRHostTorrentPublishImpl)ht).updateStats();
+													Integer port_i = new Integer(port);
 													
+													if ( !failed_ports.contains(port_i)){
+														
+														failed_ports.add( port_i );
+														
+														Logger.log(
+																new LogEvent(LOGID,
+																"Tracker Host: failed to start server", e));
+													}
 												}
 											}
-										}finally{
-											
-											this_mon.exit();
 										}
 										
-										config.saveConfig();
+										Thread.sleep( TICK_PERIOD_SECS*1000 );
+										
+										if ( closed ){
+											
+											break;
+										}
+										
+										if ( tick_count % TICKS_PER_STATS_PERIOD == 0 ){
+											
+											try{
+												this_mon.enter();
+												
+												for (int i=0;i<host_torrents.size();i++){
+					
+													TRHostTorrent	ht = (TRHostTorrent)host_torrents.get(i);
+													
+													if ( ht instanceof TRHostTorrentHostImpl ){
+																						
+														((TRHostTorrentHostImpl)ht).updateStats();
+														
+													}else{
+														
+														((TRHostTorrentPublishImpl)ht).updateStats();
+														
+													}
+												}
+											}finally{
+												
+												this_mon.exit();
+											}
+											
+											config.saveConfig( true );
+											
+										}else{
+											
+											config.saveConfig( false );
+										}
 										
 									}catch( InterruptedException e ){
 										
 										Debug.printStackTrace( e );
 										
 										break;
+									}finally{
+										
+										tick_count++;
 									}
 								}
 							}
@@ -247,24 +300,16 @@ TRHostImpl
 	{
 		return( TRTrackerServer.DEFAULT_NAME );
 	}
-	
-	public TRHostTorrent
-	hostTorrent(
-		TOTorrent		torrent )
-		
-		throws TRHostException
-	{
-		return( hostTorrent( torrent, true ));
-	}
 
 	public TRHostTorrent
 	hostTorrent(
 		TOTorrent		torrent,
-		boolean			persistent )
+		boolean			persistent,
+		boolean			passive )
 	
 		throws TRHostException
 	{
-		return( addTorrent( torrent, TRHostTorrent.TS_STARTED, persistent ));
+		return( addTorrent( torrent, TRHostTorrent.TS_STARTED, persistent, passive, SystemTime.getCurrentTime() ));
 	}
 	
 	public TRHostTorrent
@@ -273,20 +318,33 @@ TRHostImpl
 		
 		throws TRHostException
 	{
-		return( addTorrent( torrent, TRHostTorrent.TS_PUBLISHED, true ));
+		return( addTorrent( torrent, TRHostTorrent.TS_PUBLISHED, true, false, SystemTime.getCurrentTime()));
 	}
 	
 	protected TRHostTorrent
 	addTorrent(
 		TOTorrent		torrent,
 		int				state,
-		boolean			persistent )
+		boolean			persistent,
+		boolean			passive,
+		long			date_added )
 		
 		throws TRHostException
 	{
 		try{
 			this_mon.enter();
 		
+				// non-persistent additions should know what they're doing regarding
+				// announce URL
+			
+			if ( persistent && state != TRHostTorrent.TS_PUBLISHED ){
+
+				if ( host_add_announce_urls ){
+					
+					addTrackerAnnounce( torrent );
+				}
+			}
+			
 			TRHostTorrent	ht = lookupHostTorrent( torrent );
 			
 			if ( ht != null ){
@@ -311,6 +369,11 @@ TRHostImpl
 								hti.setPersistent( true );
 							}
 							
+							if ( passive && !hti.isPassive()){
+								
+								hti.setPassive( true );
+							}
+							
 							if ( state != TRHostTorrent.TS_PUBLISHED ){
 					
 								startHosting( hti );
@@ -352,6 +415,10 @@ TRHostImpl
 				if ( protocol_str.equalsIgnoreCase("udp")){
 					
 					protocol = TRTrackerServerFactory.PR_UDP;
+					
+				}else if ( TorrentUtils.isDecentralised( torrent )){
+					
+					protocol = TRTrackerServerFactory.PR_DHT;
 				}
 				
 				boolean force_external = COConfigurationManager.getBooleanParameter("Tracker Port Force External", false );
@@ -389,15 +456,22 @@ TRHostImpl
 		
 			if ( state == TRHostTorrent.TS_PUBLISHED ){
 	
-				host_torrent = new TRHostTorrentPublishImpl( this, torrent );
+				TRHostTorrentPublishImpl new_torrent = new TRHostTorrentPublishImpl( this, torrent, date_added );
 	
+				new_torrent.setPersistent( persistent );
+				
+				host_torrent	= new_torrent;
 			}else{
 			
-				host_torrent = new TRHostTorrentHostImpl( this, server, torrent, port );
+				TRHostTorrentHostImpl	new_torrent = new TRHostTorrentHostImpl( this, server, torrent, port, date_added );
+				
+				new_torrent.setPersistent( persistent );
+				
+				new_torrent.setPassive( passive );
+
+				host_torrent	= new_torrent;
 			}
-			
-			host_torrent.setPersistent( persistent );
-			
+						
 			host_torrents.add( host_torrent );
 			
 			try{
@@ -429,7 +503,7 @@ TRHostImpl
 	
 			listeners.dispatch( LDT_TORRENT_ADDED, host_torrent );
 			
-			config.saveConfig();
+			config.saveRequired();
 			
 			return( host_torrent );
 			
@@ -460,11 +534,11 @@ TRHostImpl
 					
 					if ( ssl ){
 						
-						server = TRTrackerServerFactory.createSSL( protocol, port, true );
+						server = TRTrackerServerFactory.createSSL( "tracker", protocol, port, true, true );
 					
 					}else{
 					
-						server = TRTrackerServerFactory.create( protocol, port, true );
+						server = TRTrackerServerFactory.create( "tracker", protocol, port, true, true );
 					}
 						
 					server_map.put( key, server );
@@ -477,10 +551,8 @@ TRHostImpl
 					server.addListener( this );
 							
 				}catch( TRTrackerServerException e ){
-						
-					LGLogger.log(0, 0, LGLogger.ERROR, "Tracker Host: failed to start server: " + e.toString());
-		
-					throw( new TRHostException( e.getMessage()));
+								
+					throw( new TRHostException( "startServer failed", e ));
 				}
 			}
 			
@@ -516,7 +588,7 @@ TRHostImpl
 	{
 		TOTorrent	torrent = host_torrent.getTorrent();
 		
-		TRTrackerClient tc = (TRTrackerClient)tracker_client_map.get( torrent );
+		TRTrackerAnnouncer tc = (TRTrackerAnnouncer)tracker_client_map.get( torrent );
 		
 		if ( tc != null ){
 			
@@ -526,7 +598,7 @@ TRHostImpl
 	
 	protected void
 	startHosting(
-		TRTrackerClient	tracker_client )
+		TRTrackerAnnouncer	tracker_client )
 	{
 		TRHostTorrent	host_torrent = (TRHostTorrent)host_torrent_map.get( tracker_client.getTorrent());
 			
@@ -539,24 +611,39 @@ TRHostImpl
 	protected void
 	startHosting(
 		TRHostTorrentHostImpl	host_torrent,
-		final TRTrackerClient 	tracker_client )
+		final TRTrackerAnnouncer 	tracker_client )
 	{
 		final TOTorrent	torrent = host_torrent.getTorrent();	
 
 			// set the ip override so that we announce ourselves to other peers via the 
 			// real external address, not the local one used to connect to the tracker 
 			
-		tracker_client.setIPOverride( torrent.getAnnounceURL().getHost());
+		URL	announce = torrent.getAnnounceURL();
+		
+		if ( host_add_announce_urls ){
+		
+			tracker_client.setIPOverride( announce.getHost());
+			
+		}else{
+			
+				// prolly a backup tracker, we only want to override the IP if we're hosting it
+			
+			if ( TRTrackerUtils.isHosting( announce )){
+				
+				tracker_client.setIPOverride( announce.getHost());
+
+			}
+		}
 		
 			// hook into the client so that when the announce succeeds after the refresh below
 			// we can force a rescrape to pick up the new status 
 		
-		TRTrackerClientListener	listener = 
-			new TRTrackerClientListener()
+		TRTrackerAnnouncerListener	listener = 
+			new TRTrackerAnnouncerListener()
 			{
 				public void
 				receivedTrackerResponse(
-					TRTrackerResponse	response	)
+					TRTrackerAnnouncerResponse	response	)
 				{	
 					try{
 						TRTrackerScraperFactory.getSingleton().scrape( torrent, true );
@@ -569,8 +656,10 @@ TRHostImpl
 
 				public void
 				urlChanged(
-					String		url,
-					boolean		explicit )
+					TRTrackerAnnouncer	announcer,
+					URL					old_url,
+					URL					new_url,
+					boolean				explicit )
 				{	
 				}
 					
@@ -618,7 +707,10 @@ TRHostImpl
 			
 			listeners.dispatch( LDT_TORRENT_REMOVED, host_torrent );
 			
-			config.saveConfig();	
+				// this'll get saved sometime soon anyway - performance problems
+				// here when removing multiple torrents from a large set (e.g. 1000)
+			
+			// config.saveConfig();	
 			
 		}finally{
 			
@@ -632,7 +724,7 @@ TRHostImpl
 	{
 		TOTorrent	torrent = host_torrent.getTorrent();
 		
-		TRTrackerClient tc = (TRTrackerClient)tracker_client_map.get( torrent );
+		TRTrackerAnnouncer tc = (TRTrackerAnnouncer)tracker_client_map.get( torrent );
 		
 		if ( tc != null ){
 			
@@ -642,20 +734,28 @@ TRHostImpl
 	
 	protected void
 	stopHosting(
-		TRTrackerClient	tracker_client )
+		TRTrackerAnnouncer	tracker_client )
 	{
 		TRHostTorrent	host_torrent = (TRHostTorrent)host_torrent_map.get( tracker_client.getTorrent());
 			
 		if ( host_torrent instanceof TRHostTorrentHostImpl ){
 			
-			stopHosting( (TRHostTorrentHostImpl)host_torrent, tracker_client );
+				// we only "connect" the announcer and the hosted torrent if it isn't passive. This allows
+				// us to make a torrent passive without losing the tracker stats by
+				// 1) making it passive
+				// 2) removing the Download
+			
+			//if ( !host_torrent.isPassive()){
+			
+				stopHosting( (TRHostTorrentHostImpl)host_torrent, tracker_client );
+			//}
 		}
 	}
 	
 	protected void
 	stopHosting(
 		final TRHostTorrentHostImpl	host_torrent,
-		final TRTrackerClient 		tracker_client )
+		final TRTrackerAnnouncer 		tracker_client )
 	{				
 			// unfortunately a lot of the "stop" operations that occur when a tracker client
 			// connection is closed happen async. In particular the "stopped" message to the
@@ -704,14 +804,14 @@ TRHostImpl
 		thread.start();
 	}
 	
-	protected TRTrackerClient
+	protected TRTrackerAnnouncer
 	getTrackerClient(
 		TRHostTorrent host_torrent )
 	{
 		try{
 			this_mon.enter();
 		
-			return((TRTrackerClient)tracker_client_map.get( host_torrent.getTorrent()));
+			return((TRTrackerAnnouncer)tracker_client_map.get( host_torrent.getTorrent()));
 			
 		}finally{
 			
@@ -728,14 +828,16 @@ TRHostImpl
 		
 			TOTorrent	torrent = host_torrent.getTorrent();
 			
-			TRTrackerClient tc = (TRTrackerClient)tracker_client_map.get( torrent );
+			TRTrackerAnnouncer tc = (TRTrackerAnnouncer)tracker_client_map.get( torrent );
 			
 			if ( tc != null ){
 				
 				tc.refreshListeners();
 			}
 			
-			config.saveConfig();
+			// config will get saved soon anyway (periodic or on closedown) - perf issues
+			// here with multiple torrent removal if we save each time
+			// config.saveConfig();
 			
 		}finally{
 			
@@ -763,7 +865,7 @@ TRHostImpl
 	
 	public void
 	clientCreated(
-		TRTrackerClient		client )
+		TRTrackerAnnouncer		client )
 	{
 		try{
 			this_mon.enter();
@@ -780,7 +882,7 @@ TRHostImpl
 	
 	public void
 	clientDestroyed(
-		TRTrackerClient		client )
+		TRTrackerAnnouncer		client )
 	{
 		try{
 			this_mon.enter();
@@ -830,7 +932,7 @@ TRHostImpl
 				return( true );
 			}
 			
-			addExternalTorrent( hash, TRHostTorrent.TS_STARTED );
+			addExternalTorrent( hash, TRHostTorrent.TS_STARTED, SystemTime.getCurrentTime());
 			
 			return( true );
 			
@@ -843,7 +945,8 @@ TRHostImpl
 	protected void
 	addExternalTorrent(
 		byte[]		hash,
-		int			state )
+		int			state,
+		long		date_added )
 	{
 		try{
 			this_mon.enter();
@@ -862,7 +965,7 @@ TRHostImpl
 			try{
 				TOTorrent	external_torrent = new TRHostExternalTorrent(hash, new URL( "http://" + tracker_ip + ":" + port + "/announce"));
 			
-				addTorrent( external_torrent, state, true );	
+				addTorrent( external_torrent, state, true, false, date_added );	
 				
 			}catch( Throwable e ){
 				
@@ -885,11 +988,13 @@ TRHostImpl
 	
 	public boolean
 	handleExternalRequest(
-		String			client_address,
-		String			url,
-		String			header,
-		InputStream		is,
-		OutputStream	os )
+		InetSocketAddress	client_address,
+		String				user,
+		String				url,
+		URL					absolute_url,
+		String				header,
+		InputStream			is,
+		OutputStream		os )
 		
 		throws IOException
 	{
@@ -899,7 +1004,7 @@ TRHostImpl
 
 			TRHostListener	listener = (TRHostListener)listeners_copy.get(i);
 			
-			if ( listener.handleExternalRequest( client_address, url, header, is, os )){
+			if ( listener.handleExternalRequest( client_address, user, url, absolute_url, header, is, os )){
 				
 				return( true );
 			}
@@ -967,7 +1072,7 @@ TRHostImpl
 	}
 	
 	public void
-	postProcess(
+	preProcess(
 		TRTrackerServerRequest	request )
 	
 		throws TRTrackerServerException
@@ -988,14 +1093,58 @@ TRHostImpl
 				try{
 					if ( h_torrent instanceof TRHostTorrentHostImpl ){
 					
-						((TRHostTorrentHostImpl)h_torrent).postProcess( req );
+						((TRHostTorrentHostImpl)h_torrent).preProcess( req );
 					}else{
 						
-						((TRHostTorrentPublishImpl)h_torrent).postProcess( req );	
+						((TRHostTorrentPublishImpl)h_torrent).preProcess( req );	
 					}
 				}catch( TRHostException e ){
 					
-					throw( new TRTrackerServerException( "Post process fails", e ));
+					throw( new TRTrackerServerException( e.getMessage(), e ));
+					
+				}catch( Throwable e ){
+					
+					throw( new TRTrackerServerException( "Pre-process fails", e ));
+				}
+			}
+		}	
+	}
+	
+	public void
+	postProcess(
+		TRTrackerServerRequest	request )
+	
+		throws TRTrackerServerException
+	{
+		if ( 	request.getType() 	== TRTrackerServerRequest.RT_ANNOUNCE  ||
+				request.getType() 	== TRTrackerServerRequest.RT_SCRAPE ){
+			
+			TRTrackerServerTorrent ts_torrent = request.getTorrent();
+		
+				// can be null for multi-hash scrapes... should fix this sometime I guess
+			
+			if ( ts_torrent != null ){
+				
+				HashWrapper	hash_wrapper = ts_torrent.getHash();
+				
+				TRHostTorrent h_torrent = lookupHostTorrentViaHash( hash_wrapper.getHash());
+				
+				if ( h_torrent != null ){
+					
+					TRHostTorrentRequest	req = new TRHostTorrentRequestImpl( h_torrent, new TRHostPeerHostImpl(request.getPeer()), request );
+				
+					try{
+						if ( h_torrent instanceof TRHostTorrentHostImpl ){
+						
+							((TRHostTorrentHostImpl)h_torrent).postProcess( req );
+						}else{
+							
+							((TRHostTorrentPublishImpl)h_torrent).postProcess( req );	
+						}
+					}catch( TRHostException e ){
+						
+						throw( new TRTrackerServerException( "Post process fails", e ));
+					}
 				}
 			}
 		}
@@ -1004,7 +1153,9 @@ TRHostImpl
 	public void
 	close()
 	{
-		config.saveConfig();
+		closed	= true;
+		
+		config.saveConfig( true );
 	}
 	
 	public boolean
@@ -1134,4 +1285,45 @@ TRHostImpl
 			this_mon.exit();
 		}
 	}
+	
+	protected void
+	addTrackerAnnounce(
+		TOTorrent	torrent )
+	{
+		if ( TorrentUtils.isDecentralised( torrent )){
+	
+			return;
+		}
+		
+			// ensure that the tracker's announce details are in the torrent
+		
+		URL[][]	url_sets = TRTrackerUtils.getAnnounceURLs();
+
+		if ( url_sets.length == 0 ){
+			
+				// fall back to decentralised, no tracker defined
+			
+			TorrentUtils.setDecentralised( torrent );
+			
+		}else{			
+		
+			URL[]	primary_urls = url_sets[0];
+			
+				// backwards so that they end up in right order
+			
+			for (int i=primary_urls.length-1;i>=0;i--){
+			
+				String	url_str = primary_urls[i].toString();
+				
+				if ( TorrentUtils.announceGroupsContainsURL( torrent, url_str )){
+			
+					TorrentUtils.announceGroupsSetFirst( torrent, url_str );
+				
+				}else{
+				
+					TorrentUtils.announceGroupsInsertFirst( torrent, url_str );
+				}
+			}
+		}
+	}
 }
