@@ -27,17 +27,21 @@ package org.gudy.azureus2.core3.tracker.server;
  */
 
 import java.io.*;
+import java.net.InetSocketAddress;
+import java.net.URL;
 
 public interface 
 TRTrackerServerListener 
 {
 	public boolean
 	handleExternalRequest(
-		String			client_address,
-		String			url,
-		String			header,
-		InputStream		is,
-		OutputStream	os )
+		InetSocketAddress	client_address,
+		String				user,
+		String				url,
+		URL					absolute_url,
+		String				header,
+		InputStream			is,
+		OutputStream		os )
 		
 		throws IOException;
 		
