@@ -26,6 +26,8 @@ package org.gudy.azureus2.core3.tracker.server;
  *
  */
 
+import java.net.URL;
+
 import org.gudy.azureus2.core3.util.*;
 
 public interface 
@@ -43,6 +45,20 @@ TRTrackerServerTorrent
 	public void
 	disableCaching();
 	
+	public void
+	setEnabled(
+		boolean	enabled );
+	
+	public boolean
+	isEnabled();
+	
+	public void
+	setRedirects(
+		URL[]		urls );
+	
+	public URL[]
+	getRedirects();
+	
 	public void
 	addListener(
 		TRTrackerServerTorrentListener	l );
@@ -50,4 +66,15 @@ TRTrackerServerTorrent
 	public void
 	removeListener(
 		TRTrackerServerTorrentListener	l );
+	
+	public void
+	addPeerListener(
+		TRTrackerServerTorrentPeerListener	l );
+	
+	public void
+	removePeerListener(
+		TRTrackerServerTorrentPeerListener	l );
+	
+	public String
+	getString();
 }
