@@ -48,6 +48,12 @@ TRTrackerServerTorrentStats
 	public long
 	getDownloaded();
 	
+	public long
+	getBiasedUploaded();
+	
+	public long
+	getBiasedDownloaded();
+
 	public long
 	getAmountLeft();
 	
