@@ -58,6 +58,10 @@ TRTrackerServerImpl
 	public static boolean	all_networks_permitted		= true;
 	public static String[]	permitted_networks			= {};
 	
+	public static boolean	support_experimental_extensions;
+	
+	public static String	redirect_on_not_found		= "";
+	
 		// torrent map is static across all protocol servers
 	
 	private static Map		torrent_map = new HashMap(); 
@@ -83,7 +87,7 @@ TRTrackerServerImpl
 	protected static void
 	readConfig()
 	{
-		send_peer_ids = COConfigurationManager.getBooleanParameter( "Tracker Send Peer IDs", true );
+		send_peer_ids = COConfigurationManager.getBooleanParameter( "Tracker Send Peer IDs" );
 				
 		max_peers_to_send = COConfigurationManager.getIntParameter( "Tracker Max Peers Returned" );
 				
@@ -123,6 +127,9 @@ TRTrackerServerImpl
 		
 		full_scrape_enable = COConfigurationManager.getBooleanParameter( "Tracker Server Full Scrape Enable" );
 
+		redirect_on_not_found = COConfigurationManager.getStringParameter( "Tracker Server Not Found Redirect" ).trim();
+		
+		support_experimental_extensions = COConfigurationManager.getBooleanParameter( "Tracker Server Support Experimental Extensions" );
 	}
 	
 	protected static boolean
@@ -185,6 +192,12 @@ TRTrackerServerImpl
 		return( permitted_networks );
 	}
 	
+	public static boolean
+	supportsExtensions()
+	{
+		return( support_experimental_extensions );
+	}
+	
 	protected IpFilter	ip_filter	= IpFilterManagerFactory.getSingleton().getIPFilter();
 	
 	private long		current_announce_retry_interval;
@@ -193,7 +206,7 @@ TRTrackerServerImpl
 	
 	private int		current_min_poll_interval;
 	
-	private TRTrackerServerStatsImpl	stats = new TRTrackerServerStatsImpl();
+	private TRTrackerServerStatsImpl	stats = new TRTrackerServerStatsImpl( this );
 		
 	private String	name;
 	private boolean	web_password_enabled;
@@ -216,12 +229,19 @@ TRTrackerServerImpl
 	private COConfigurationListener		config_listener;
 	private boolean						destroyed;
 	
+	private Set							biased_peers;
+	
+	private boolean						is_ready;
+	
 	public
 	TRTrackerServerImpl(
-		String		_name )
+		String		_name,
+		boolean		_start_up_ready )
 	{
 		name		= _name==null?DEFAULT_NAME:_name;
-
+		is_ready	= _start_up_ready;
+		
+		
 		config_listener = 
 			new COConfigurationListener()
 			{
@@ -283,7 +303,64 @@ TRTrackerServerImpl
 		key_enabled = COConfigurationManager.getBooleanParameter("Tracker Key Enable Server");
 	}
 
-
+	public void
+	setReady()
+	{
+		is_ready	= true;
+	}
+	
+	public final boolean
+	isReady()
+	{
+		return( is_ready );
+	}
+	
+	public void
+	setBiasedPeers(
+		Set		peers )
+	{
+		if ( biased_peers != null && peers.equals( biased_peers )){
+			
+			return;
+		}
+		
+		String	str = "";
+		
+		Iterator	it = peers.iterator();
+		
+		while( it.hasNext()){
+			
+			str += " " + it.next();
+		}
+		
+		System.out.println( "biased peers: " + str );
+		
+		try{
+			class_mon.enter();
+		
+			biased_peers = new HashSet( peers );
+			
+			Iterator	tit = torrent_map.values().iterator();
+			
+			while(tit.hasNext()){
+							
+				TRTrackerServerTorrentImpl	this_torrent = (TRTrackerServerTorrentImpl)tit.next();
+				
+				this_torrent.updateBiasedPeers( biased_peers );
+			}
+			
+		}finally{
+			
+			class_mon.exit();
+		}
+	}
+	
+	protected Set
+	getBiasedPeers()
+	{
+		return( biased_peers );
+	}
+	
 	public boolean
 	isWebPasswordEnabled()
 	{
@@ -658,7 +735,7 @@ TRTrackerServerImpl
 				
 				if ( entry == null ){
 				
-					entry = new TRTrackerServerTorrentImpl( hash, _enabled );
+					entry = new TRTrackerServerTorrentImpl( this, hash, _enabled );
 				
 					torrent_map.put( hash, entry );
 				}
@@ -740,6 +817,12 @@ TRTrackerServerImpl
 		}
 	}
 	
+	public int
+	getTorrentCount()
+	{
+		return( torrent_map.size());
+	}
+	
 	public TRTrackerServerTorrentStats
 	getStats(
 		byte[]		hash )
