@@ -34,6 +34,8 @@ import org.gudy.azureus2.core3.tracker.util.TRTrackerUtils;
 import org.gudy.azureus2.core3.config.*;
 import org.gudy.azureus2.core3.util.*;
 
+import com.aelitis.azureus.core.dht.netcoords.DHTNetworkPosition;
+
 public abstract class 
 TRTrackerServerProcessor 
 	extends ThreadPoolTask
@@ -46,7 +48,7 @@ TRTrackerServerProcessor
 		}
 	}
 	
-	protected TRTrackerServerImpl		server;
+	private TRTrackerServerImpl		server;
 	
 	protected TRTrackerServerTorrentImpl
 	processTrackerRequest(
@@ -65,18 +67,27 @@ TRTrackerServerProcessor
 		boolean						stop_to_queue,
 		int							port,
 		int							udp_port,
+		int							http_port,
 		String						real_ip_address,
 		String						client_ip_address,
 		long						downloaded,
 		long						uploaded,
 		long						left,
 		int							num_want,
-		byte						crypto_level )
+		byte						crypto_level,
+		byte						az_ver,
+		int							up_speed,
+		DHTNetworkPosition			network_position )
 	
 		throws TRTrackerServerException
 	{
 		server	= _server;
 			
+		if ( !server.isReady()){
+			
+			throw( new TRTrackerServerException( "Tracker initialising, please wait" ));
+		}
+		
 		boolean	ip_override = real_ip_address != client_ip_address;
 		
 		boolean	loopback	= TRTrackerUtils.isLoopback( real_ip_address );
@@ -172,14 +183,15 @@ TRTrackerServerProcessor
 					torrent.peerContact( 	
 						request,
 						event, 
-						peer_id, port, udp_port, crypto_level, 
+						peer_id, port, udp_port, http_port, crypto_level, az_ver,
 						client_ip_address, ip_override, loopback, key,
 						uploaded, downloaded, left,
-						interval );
+						interval,
+						up_speed, network_position );
 				
 				if ( stop_to_queue && ( QUEUE_TEST || !( loopback || ip_override ))){
 					
-					torrent.peerQueued( client_ip_address, port, udp_port, crypto_level, (int)server.getScrapeRetryInterval( torrent ));
+					torrent.peerQueued( client_ip_address, port, udp_port, http_port, crypto_level, az_ver, (int)server.getScrapeRetryInterval( torrent ), left==0);
 				}
 				
 				HashMap	pre_map = new HashMap();
@@ -199,7 +211,7 @@ TRTrackerServerProcessor
 				
 				boolean	stopped 	= event != null && event.equalsIgnoreCase("stopped");
 				
-				root_out[0] = torrent.exportAnnounceToMap( pre_map, peer, left > 0, stopped?0:num_want, interval, server.getMinAnnounceRetryInterval(), no_peer_id, compact_mode, crypto_level );
+				root_out[0] = torrent.exportAnnounceToMap( pre_map, peer, left > 0, stopped?0:num_want, interval, server.getMinAnnounceRetryInterval(), no_peer_id, compact_mode, crypto_level, network_position );
 				
 				peer_out[0]	= peer;	
 				
@@ -274,9 +286,11 @@ TRTrackerServerProcessor
 
 					if ( scrape_chars != null && ( QUEUE_TEST || !( loopback || ip_override ))){ 
 						
+							// note, 'Q' is complete+queued so we set seed true below
+						
 						if ( scrape_chars[i] == 'Q' ){
 							
-							torrent.peerQueued(  client_ip_address, port, udp_port, crypto_level, (int)interval );
+							torrent.peerQueued(  client_ip_address, port, udp_port, http_port, crypto_level, az_ver, (int)interval, true );
 						}
 					}
 					
@@ -459,5 +473,37 @@ TRTrackerServerProcessor
 		{
 			return( peer_id );
 		}
+		
+		public boolean
+		isBiased()
+		{
+			return( false );
+		}
+		
+		public byte
+		getBias()
+		{
+			return( 0 );
+		}
+		
+		public void
+		setBias(
+			byte	bias )
+		{	
+		}
+		
+		public void
+		setUserData(
+			Object		key,
+			Object		data )
+		{
+		}
+		
+		public Object
+		getUserData(
+			Object		key )
+		{
+			return( null );
+		}
 	}
 }
