@@ -27,14 +27,16 @@ import java.net.*;
 import java.util.*;
 import java.util.zip.GZIPOutputStream;
 
-import sun.misc.BASE64Decoder;
-
 import org.gudy.azureus2.core3.tracker.server.*;
 import org.gudy.azureus2.core3.tracker.server.impl.*;
-import org.gudy.azureus2.core3.logging.*;
 import org.gudy.azureus2.core3.util.*;
 
-public class 
+import org.bouncycastle.util.encoders.Base64;
+
+import com.aelitis.azureus.core.dht.netcoords.DHTNetworkPosition;
+import com.aelitis.azureus.core.dht.netcoords.DHTNetworkPositionManager;
+
+public abstract class 
 TRTrackerServerProcessorTCP
 	extends 	TRTrackerServerProcessor
 {
@@ -44,6 +46,7 @@ TRTrackerServerProcessorTCP
 	protected static final char		FF			= '\012';
 	protected static final String	NL			= "\015\012";
 
+	private static final String	lc_azureus_name = Constants.AZUREUS_NAME.toLowerCase();
 
 	protected static final byte[]	HTTP_RESPONSE_START = (
 		"HTTP/1.1 200 OK" + NL + 
@@ -55,349 +58,54 @@ TRTrackerServerProcessorTCP
 	protected static final byte[]	HTTP_RESPONSE_END_GZIP 		= (NL + "Content-Encoding: gzip" + NL + NL).getBytes();
 	protected static final byte[]	HTTP_RESPONSE_END_NOGZIP 	= (NL + NL).getBytes();
 	
-	protected TRTrackerServerTCP	server;
-	protected Socket				socket;
+	private TRTrackerServerTCP	server;
+	private String				server_url;
 	
+	private boolean			disable_timeouts 	= false;
 
-	protected int					timeout_ticks		= 1;
-	protected boolean				disable_timeouts 	= false;
-	protected String				current_request;
 	
 	protected
 	TRTrackerServerProcessorTCP(
-		TRTrackerServerTCP		_server,
-		Socket					_socket )
+		TRTrackerServerTCP		_server )
 	{
 		server	= _server;
-		socket	= _socket;
-	}
-	
-	public void
-	runSupport()
-	{
-		try{
-	
-			setTaskState( "entry" );
-			
-			try{
-												
-				socket.setSoTimeout( SOCKET_TIMEOUT );
-										
-			}catch ( SocketException e ){
-													
-				// e.printStackTrace();
-			}
-										
-			String	header_plus = "";
-			
-			setTaskState( "reading header" );
-
-			try{
-										
-				InputStream	is = socket.getInputStream();
-				
-				byte[]	buffer = new byte[1024];
-				
-				while( header_plus.length()< 4096 ){
-						
-					int	len = is.read(buffer);
-						
-					if ( len == -1 ){
-					
-						break;
-					}
-									
-					header_plus += new String( buffer, 0, len, Constants.BYTE_ENCODING );
-									
-					if ( 	header_plus.endsWith(NL+NL) ||
-							header_plus.indexOf( NL+NL ) != -1 ){
-						
-						break;
-					}
-				}
 		
-				if ( LGLogger.isLoggingOn()){
-					
-					String	log_str = header_plus;
-					
-					int	pos = log_str.indexOf( NL );
-					
-					if ( pos != -1 ){
-						
-						log_str = log_str.substring(0,pos);
-					}
-					
-					LGLogger.log(0, 0, LGLogger.INFORMATION, "Tracker Server: received header '" + log_str + "'" );
-				}				
-					
-				// System.out.println( "got header:" + header_plus );
-				
-				InputStream	post_is 	= null;
-				File		post_file	= null;
-				
-				String	actual_header;
-				String	lowercase_header;
-				
-				boolean	head	= false;
-				
-				if ( header_plus.startsWith( "GET " )){
-				
-					timeout_ticks		= 1;
-					
-					actual_header		= header_plus;
-					lowercase_header	= actual_header.toLowerCase();
-	
-				}else if ( header_plus.startsWith( "HEAD " )){
-					
-					timeout_ticks		= 1;
-					
-					actual_header		= header_plus;
-					lowercase_header	= actual_header.toLowerCase();			
+		server_url = (server.isSSL()?"https":"http") + "://" + server.getHost() + ":" + server.getPort();
+	}	
 
-					head	= true;
-					
-				}else if ( header_plus.startsWith( "POST ")){
-					
-					timeout_ticks	= TRTrackerServerTCP.PROCESSING_POST_MULTIPLIER;
-					
-					if ( timeout_ticks == 0 ){
-						
-						disable_timeouts	= true;
-					}
-					
-					setTaskState( "reading content" );
-
-					int	header_end = header_plus.indexOf(NL+NL);
-					
-					if ( header_end == -1 ){
-					
-						throw( new TRTrackerServerException( "header truncated" ));
-					}
-					
-					actual_header 		= header_plus.substring(0,header_end+4);
-					lowercase_header	= actual_header.toLowerCase();
-					
-					int	cl_start = lowercase_header.indexOf("content-length:");
-					
-					if ( cl_start == -1 ){
-						
-						throw( new TRTrackerServerException( "header Content-Length start missing" ));
-					}
-					
-					int	cl_end = actual_header.indexOf( NL, cl_start );
-					
-					if ( cl_end == -1 ){
-						
-						throw( new TRTrackerServerException( "header Content-Length end missing" ));
-					}
-					
-					int	content_length = Integer.parseInt( actual_header.substring(cl_start+15,cl_end ).trim());
-				
-					ByteArrayOutputStream	baos		= null;
-					FileOutputStream		fos			= null;
-				
-					OutputStream	data_os;
-					
-					if ( content_length <= 256*1024 ){
-						
-						baos = new ByteArrayOutputStream();
-					
-						data_os	= baos;
-						
-					}else{
-						
-						post_file	= AETemporaryFileHandler.createTempFile( "AZU", null );
-						
-						post_file.deleteOnExit();
-						
-						fos	= new FileOutputStream( post_file );
-						
-						data_os	= fos;
-					}
-					
-						// if we have X<NL><NL>Y get Y
-					
-					int	rem = header_plus.length() - (header_end+4);
-					
-					if ( rem > 0 ){
-						
-						content_length	-= rem;
-						
-						data_os.write( header_plus.substring(header_plus.length()-rem).getBytes( Constants.BYTE_ENCODING ));
-					}
-					
-					while( content_length > 0 ){
-						
-						int	len = is.read( buffer );
-						
-						if ( len < 0 ){
-							
-							throw( new TRTrackerServerException( "premature end of input stream" ));
-						}
-						
-						data_os.write( buffer, 0, len );
-						
-						content_length -= len;
-					}
-					
-					if ( baos != null ){
-						
-						post_is = new ByteArrayInputStream(baos.toByteArray());
-						
-					}else{
-						
-						fos.close();
-						
-						post_is = new BufferedInputStream( new FileInputStream( post_file ), 256*1024 );
-					}
-					
-					// System.out.println( "TRTrackerServerProcessorTCP: request data = " + baos.size());
-				}else{
-					
-					throw( new TRTrackerServerException( "header doesn't start with GET or POST ('" + (header_plus.length()>256?header_plus.substring(0,256):header_plus)+"')" ));
-				}
-				
-				setTaskState( "processing request" );
-
-				current_request	= actual_header;
-				
-				try{
-					if ( post_is == null ){
-						
-							// set up a default input stream 
-						
-						post_is = new ByteArrayInputStream(new byte[0]);
-					}
-					
-					String	url = actual_header.substring(4).trim();
-					
-					int	pos = url.indexOf( " " );
-					
-					if ( pos == -1 ){
-						
-						throw( new TRTrackerServerException( "header doesn't have space in right place" ));
-					}
-									
-					url = url.substring(0,pos);
-					
-					if ( head ){
-						
-						ByteArrayOutputStream	head_response = new ByteArrayOutputStream(4096);
-						
-						processRequest( actual_header,
-								lowercase_header,
-								url, 
-								socket.getInetAddress().getHostAddress(),
-								post_is,
-								head_response );
-						
-						byte[]	head_data = head_response.toByteArray();
-						
-						int	header_length = head_data.length;
-						
-						for (int i=3;i<head_data.length;i++){
-							
-							if ( 	head_data[i-3] 	== CR &&
-									head_data[i-2] 	== FF &&
-									head_data[i-1] 	== CR &&
-									head_data[i]	== FF ){
-								
-								header_length = i+1;
-						
-								break;
-							}
-						}
-												
-						setTaskState( "writing head response" );
-
-						socket.getOutputStream().write( head_data, 0, header_length );
-						
-						socket.getOutputStream().flush();
-						
-					}else{
-					
-						processRequest( actual_header,
-										lowercase_header,
-										url, 
-										socket.getInetAddress().getHostAddress(),
-										post_is,
-										socket.getOutputStream() );
-					}
-				}finally{
-					
-					if ( post_is != null ){
-						
-						post_is.close();
-					}
-					
-					if ( post_file != null ){
-						
-						post_file.delete();
-					}
-				}
-			}catch( SocketTimeoutException e ){
-				
-				// System.out.println( "TRTrackerServerProcessor: timeout reading header, got '" + header + "'");
-				// ignore it
-							
-			}catch( Throwable e ){
-				
-				 // e.printStackTrace();
-			}
-	
-		}finally{
-					
-			setTaskState( "final socket close" );
-
-			try{
-				socket.close();
-																							
-			}catch( Throwable e ){
-													
-				// e.printStackTrace();
-			}
-		}
-	}
-	
-	public void
-	interruptTask()
+	protected boolean
+	areTimeoutsDisabled()
 	{
-		try{
-			if ( disable_timeouts ){
-				
-				// Debug.out( "External tracker request timeout ignored: state = " + getTaskState() + ", req = " + current_request  );
-				
-			}else{
-				
-				timeout_ticks--;
-					
-				if ( timeout_ticks <= 0 ){
-					
-					Debug.out( "Tracker task interrupted in state '" + getTaskState() + "' : processing time limit exceeded" );
-					
-					socket.close();
-				}
-			}
-																						
-		}catch( Throwable e ){
-												
-			// e.printStackTrace();
-		}
+		return( disable_timeouts );
 	}
-
+	
+	protected void
+	setTimeoutsDisabled(
+		boolean	d )
+	{
+		disable_timeouts	= d;
+	}
+	
+	protected TRTrackerServerTCP
+	getServer()
+	{
+		return( server );
+	}
+	
 	protected void
 	processRequest(
-		String			input_header,
-		String			lowercase_input_header,
-		String			url_path,
-		String			client_ip_address,
-		InputStream		is,
-		OutputStream	os )
+		String				input_header,
+		String				lowercase_input_header,
+		String				url_path,
+		InetSocketAddress	client_address,
+		boolean				announce_and_scrape_only,
+		InputStream			is,
+		OutputStream		os )
 		
 		throws IOException
 	{
 		String	str = url_path;
-		
+				
 		try{
 			Map	root = null;
 				
@@ -428,24 +136,44 @@ TRTrackerServerProcessorTCP
 				
 				}else{
 					
-					setTaskState( "external request" );
-
-					disable_timeouts	= true;
+					String	redirect = TRTrackerServerImpl.redirect_on_not_found;
 					
-						// check non-tracker authentication
+					if ( announce_and_scrape_only ){
 						
-					if ( !doAuthentication( url_path, input_header, os, false )){
-					
-						return;
+						if ( redirect.length() == 0 ){
+							
+							throw( new Exception( "Tracker only supports announce and scrape functions" ));
+						}
+					}else{
+						
+						setTaskState( "external request" );
+	
+						disable_timeouts	= true;
+						
+							// check non-tracker authentication
+							
+						String user = doAuthentication( url_path, input_header, os, false );
+						
+						if ( user == null ){
+							
+							return;
+						}
+						
+						if ( handleExternalRequest( client_address, user, str, input_header, is, os )){
+						
+							return;
+						}
 					}
 					
-					if ( handleExternalRequest( client_ip_address, str, input_header, is, os )){
-					
-						return;
+					if ( redirect.length() > 0 ){
+						
+						os.write( ("HTTP/1.1 301 Moved Permanently" + NL + "Location: " + redirect + NL + NL).getBytes() );
+						
+					}else{
+						
+						os.write( ("HTTP/1.1 404 Not Found" + NL + NL ).getBytes() );
 					}
 					
-					os.write( ("HTTP/1.1 404 Not Found\r\n\r\n").getBytes() );
-					
 					os.flush();
 
 					return; // throw( new Exception( "Unsupported Request Type"));
@@ -455,7 +183,7 @@ TRTrackerServerProcessorTCP
 				
 					// check tracker authentication
 					
-				if ( !doAuthentication( url_path, input_header, os, true )){
+				if ( doAuthentication( url_path, input_header, os, true ) == null ){
 					
 					return;
 				}
@@ -469,17 +197,78 @@ TRTrackerServerProcessorTCP
 					
 					if ( e_pos != -1 ){
 						
-						gzip_reply = input_header.substring(enc_pos+16,e_pos).toLowerCase().indexOf("gzip") != -1;
+							// check we've not found X-Accept-Encoding (for example)
+						
+						if ( enc_pos > 0 ){
+							
+							char	c = lowercase_input_header.charAt(enc_pos-1);
+							
+							if ( c != FF && c != ' ' ){
+								
+								enc_pos	= -1;
+							}
+						}
+						
+						if ( enc_pos != -1 ){
+							
+							String	accept_encoding = lowercase_input_header.substring(enc_pos+16,e_pos);
+														
+							int gzip_index = accept_encoding.indexOf("gzip");
+							
+							if ( gzip_index != -1 ){
+								
+								gzip_reply	= true;
+								
+								if ( accept_encoding.length() - gzip_index >= 8 ){
+								
+										// gzip;q=0
+										// look to see if there's a q=0 (or 0.0) disabling gzip
+	
+									char[]	chars = accept_encoding.toCharArray();
+									
+									boolean	q_value = false;
+																	
+									for (int i=gzip_index+4;i<chars.length;i++){
+										
+										char	c = chars[i];
+										
+										if ( c == ',' ){
+											
+											break;
+											
+										}else if ( c == '=' ){
+											
+											q_value		= true;
+											gzip_reply	= false;
+											
+										}else{
+											
+											if ( q_value ){
+												
+												if ( c != ' ' && c != '0' && c != '.' ){
+													
+													gzip_reply	= true;
+													
+													break;
+												}
+											}
+										}
+									}
+								}
+							}
+						}
 					}
 				}
-				
+								
 				setTaskState( "decoding announce/scrape" );
 
 				int	pos = 0;
 					
-				String		hash_str	= null;
+				byte[]		hash		= null;
+				List		hash_list	= null;
+				
 				HashWrapper	peer_id		= null;
-				int			port		= 0;
+				int			tcp_port	= 0;
 				String		event		= null;
 					
 				long		uploaded		= 0;
@@ -487,8 +276,21 @@ TRTrackerServerProcessorTCP
 				long		left			= 0;
 				int			num_want		= -1;
 				boolean		no_peer_id		= false;
-				boolean		compact			= false;
+				byte		compact_mode	= TRTrackerServerTorrentImpl.COMPACT_MODE_NONE;
 				String		key				= null;
+				byte		crypto_level 	= TRTrackerServerPeer.CRYPTO_NONE;
+				int			crypto_port		= 0;
+				int			udp_port		= 0;
+				int			http_port		= 0;
+				int			az_ver			= 0;
+				boolean		stop_to_queue	= false;
+				String		scrape_flags	= null;
+				int			up_speed		= 0;
+				
+				DHTNetworkPosition	network_position = null;
+				
+				String		real_ip_address		= client_address.getAddress().getHostAddress();
+				String		client_ip_address	= real_ip_address;
 				
 				while(pos < str.length()){
 						
@@ -521,7 +323,23 @@ TRTrackerServerProcessorTCP
 						
 					if ( lhs.equals( "info_hash" )){
 							
-						hash_str	= rhs;
+						byte[] b = rhs.getBytes(Constants.BYTE_ENCODING);
+						
+						if ( hash == null ){
+							
+							hash = b;
+							
+						}else{
+							
+							if ( hash_list == null ){
+								
+								hash_list = new ArrayList();
+								
+								hash_list.add( hash );
+							}
+							
+							hash_list.add( b );
+						}
 							
 					}else if ( lhs.equals( "peer_id" )){
 						
@@ -535,7 +353,10 @@ TRTrackerServerProcessorTCP
 						
 						if ( server.isCompactEnabled()){
 							
-							compact = rhs.equals("1");
+							if ( rhs.equals("1") && compact_mode == TRTrackerServerTorrentImpl.COMPACT_MODE_NONE ){
+								
+								compact_mode = TRTrackerServerTorrentImpl.COMPACT_MODE_NORMAL;
+							}
 						}
 					}else if ( lhs.equals( "key" )){
 						
@@ -546,7 +367,7 @@ TRTrackerServerProcessorTCP
 						
 					}else if ( lhs.equals( "port" )){
 							
-						port = Integer.parseInt( rhs );
+						tcp_port = Integer.parseInt( rhs );
 						
 					}else if ( lhs.equals( "event" )){
 							
@@ -554,6 +375,21 @@ TRTrackerServerProcessorTCP
 							
 					}else if ( lhs.equals( "ip" )){
 							
+						if ( AENetworkClassifier.categoriseAddress( rhs ) == AENetworkClassifier.AT_PUBLIC ){
+	
+								// only accept public resolved addresses
+							
+							for (int i=0;i<rhs.length();i++){
+								
+								char	c = rhs.charAt(i);
+								
+								if ( c != '.' && !Character.isDigit( c )){
+									
+									throw( new Exception( "IP override address must be resolved by the client" ));
+								}
+							}
+						}
+						
 						client_ip_address = rhs;
 						
 					}else if ( lhs.equals( "uploaded" )){
@@ -571,19 +407,95 @@ TRTrackerServerProcessorTCP
 					}else if ( lhs.equals( "numwant" )){
 						
 						num_want = Integer.parseInt( rhs );
-					}
 						
+					}else if ( lhs.equals( "azudp" )){
+						
+						udp_port 	= Integer.parseInt( rhs );
+						
+							// implicit compact mode for 2500 indicated by presence of udp port
+						
+						compact_mode = TRTrackerServerTorrentImpl.COMPACT_MODE_AZ;
+						
+					}else if ( lhs.equals( "azhttp" )){
+						
+						http_port 	= Integer.parseInt( rhs );
+
+					}else if ( lhs.equals( "azver" )){
+						
+						az_ver 	= Integer.parseInt( rhs );
+												
+					}else if ( lhs.equals( "supportcrypto" )){
+						
+						if ( crypto_level == TRTrackerServerPeer.CRYPTO_NONE ){
+						
+							crypto_level	= TRTrackerServerPeer.CRYPTO_SUPPORTED;
+						}
+						
+					}else if ( lhs.equals( "requirecrypto" )){
+												
+						crypto_level	= TRTrackerServerPeer.CRYPTO_REQUIRED;
+					
+					}else if ( lhs.equals( "cryptoport" )){
+
+						crypto_port = Integer.parseInt( rhs );
+						
+					}else if ( lhs.equals( "azq" )){
+					
+						stop_to_queue	= true;
+												
+					}else if ( lhs.equals( "azsf" )){
+					
+						scrape_flags = rhs;
+						
+					}else if ( TRTrackerServerImpl.supportsExtensions()){
+						
+						if ( lhs.equals( "aznp" )){
+
+							try{
+								network_position = DHTNetworkPositionManager.deserialisePosition( Base32.decode( rhs ));
+																
+							}catch( Throwable e ){
+								
+							}
+						}else if ( lhs.equals( "azup" )){
+	
+							up_speed = Integer.parseInt( rhs );
+						}
+					}
+					
 					if ( p1 == -1 ){
 							
 						break;
 					}
 				}
 				
-				byte[]	hash_bytes = null;
-				
-				if ( hash_str != null ){
+				if ( crypto_level == TRTrackerServerPeer.CRYPTO_REQUIRED ){
 					
-					hash_bytes = hash_str.getBytes(Constants.BYTE_ENCODING);
+					if ( crypto_port != 0 ){
+						
+						tcp_port = crypto_port;
+					}
+				}
+				
+				byte[][]	hashes = null;
+				
+				if ( hash_list != null ){
+						
+					hashes = new byte[hash_list.size()][];
+						
+					hash_list.toArray( hashes );
+						
+				}else if ( hash != null ){
+					
+					hashes = new byte[][]{ hash };
+				}
+				
+					// >= so that if this tracker is "old" and sees a version 3+ it replies with the
+					// best it can - version 2
+				
+				if ( az_ver >= 2 ){
+					
+					compact_mode = TRTrackerServerTorrentImpl.COMPACT_MODE_AZ_2;
 				}
 				
 				Map[]						root_out = new Map[1];
@@ -591,27 +503,88 @@ TRTrackerServerProcessorTCP
 				
 				specific_torrent = 
 						processTrackerRequest( 
-							server, root_out, peer_out,
+							server, str,
+							root_out, peer_out,
 							request_type,
-							hash_bytes,
-							peer_id, no_peer_id, compact, key, 
-							event,
-							port,
+							hashes, scrape_flags,
+							peer_id, no_peer_id, compact_mode, key, 
+							event, stop_to_queue,
+							tcp_port&0xffff, udp_port&0xffff, http_port&0xffff,
+							real_ip_address,
 							client_ip_address,
 							downloaded, uploaded, left,
-							num_want );
+							num_want,
+							crypto_level,
+							(byte)az_ver,
+							up_speed,
+							network_position );
 				
 				root	= root_out[0];
+
+				if ( request_type == TRTrackerServerRequest.RT_SCRAPE ){
+					
+						// add in tracker type for az clients so they know this is an AZ tracker
+					
+					if ( lowercase_input_header.indexOf( lc_azureus_name ) != -1 ){
 				
+						root.put( "aztracker", new Long(1));
+					}
+				}
+
 					// only post-process if this isn't a cached entry
 				
 				if ( root.get( "_data" ) == null ){
 	
-					server.postProcess( peer_out[0], specific_torrent, request_type, root );
+					TRTrackerServerPeer	post_process_peer = peer_out[0];
+					
+					if ( post_process_peer == null ){
+						
+						post_process_peer = new lightweightPeer( client_ip_address, tcp_port, peer_id );
+					}
+					
+					server.postProcess( post_process_peer, specific_torrent, request_type, str, root );
 				}
 				
 			}catch( Exception e ){
 				
+				
+				if ( e instanceof TRTrackerServerException ){
+					
+					TRTrackerServerException	tr_excep = (TRTrackerServerException)e;
+					
+					int	reason = tr_excep.getResponseCode();
+					
+					if ( reason != -1 ){
+						
+						String	resp = "HTTP/1.1 " + reason + " " + tr_excep.getResponseText() + NL;
+						
+						Map	headers = tr_excep.getResponseHeaders();
+						
+						Iterator	it = headers.entrySet().iterator();
+						
+						while( it.hasNext()){
+							
+							Map.Entry	entry = (Map.Entry)it.next();
+							
+							String	key 	= (String)entry.getKey();
+							String	value 	= (String)entry.getValue();
+							
+							resp += key + ": " + value + NL;
+						}
+
+						resp += NL;
+
+						os.write( resp.getBytes());
+						
+						os.flush();
+
+						return;
+					}
+				}else if ( e instanceof NullPointerException ){
+					
+					e.printStackTrace();
+				}
+				
 				String	message = e.getMessage();
 				
 				// e.printStackTrace();
@@ -703,7 +676,7 @@ TRTrackerServerProcessorTCP
 		}
 	}
 	
-	protected boolean
+	protected String
 	doAuthentication(
 		String			url_path,
 		String			header,
@@ -725,7 +698,7 @@ TRTrackerServerProcessorTCP
 			
 			os.flush();
 				
-			return( false );
+			return( null );
 
 		}else if (	apply_torrent_password ||
 					apply_web_password ){
@@ -748,7 +721,7 @@ TRTrackerServerProcessorTCP
 					
 						if ( server.performExternalAuthorisation( resource, "", "" )){
 							
-							return( true );
+							return( "" );
 						}
 					}catch( MalformedURLException e ){
 						
@@ -764,7 +737,7 @@ TRTrackerServerProcessorTCP
 				
 				String	body = header.substring( p2, header.indexOf( '\r', p2 )).trim();
 				
-				String decoded=new String(new BASE64Decoder().decodeBuffer(body));
+				String decoded=new String( Base64.decode(body));
 
 					// username:password
 									
@@ -786,7 +759,7 @@ TRTrackerServerProcessorTCP
 					
 						if ( server.performExternalAuthorisation( resource, user, pw )){
 							
-							return( true );
+							return( user );
 						}
 					}catch( MalformedURLException e ){
 						
@@ -817,16 +790,16 @@ TRTrackerServerProcessorTCP
 						
 						if ( user.equals( "<internal>")){
 							
-							byte[] internal_pw = new BASE64Decoder().decodeBuffer(pw);
+							byte[] internal_pw = Base64.decode(pw);
 	
 							if ( Arrays.equals( internal_pw, server.getPassword())){
 								
-								return( true );
+								return( user );
 							}
 						}else if ( 	user.equalsIgnoreCase(server.getUsername()) &&
 									Arrays.equals(encoded, server.getPassword())){
 							 	
-							 return( true );			 	
+							 return( user );			 	
 						}
 					}catch( Exception e ){
 						
@@ -839,24 +812,27 @@ TRTrackerServerProcessorTCP
 			
 			os.flush();
 				
-			return( false );
+			return( null );
 
 		}else{
 		
-			return( true );
+			return( "" );
 		}
 	}
 		
 	protected boolean
 	handleExternalRequest(
-		String			client_address,
-		String			url,
-		String			header,
-		InputStream		is,
-		OutputStream	os )
+		InetSocketAddress	client_address,
+		String				user,
+		String				url,
+		String				header,
+		InputStream			is,
+		OutputStream		os )
 		
 		throws IOException
 	{
-		return( server.handleExternalRequest(client_address,url,header, is, os));
+		URL	absolute_url = new URL( server_url + (url.startsWith("/")?url:("/"+url)));
+			
+		return( server.handleExternalRequest(client_address,user,url,absolute_url,header, is, os));
 	}
 }
