@@ -25,7 +25,6 @@ package org.gudy.azureus2.core3.util;
 import java.util.*;
 import java.io.*;
 import java.nio.*;
-import java.nio.charset.*;
 
 /**
  * A set of utility methods to decode a bencoded array of byte into a Map.
@@ -450,7 +449,7 @@ public class BDecoder {
   main(
 	 String[]	args )
   {	  
-	  print( 	new File( "C:\\Temp\\8565658FA6C187A602A5360A69F11933624DD9B5.dat.bak" ),
+	  print( 	new File( "C:\\Temp\\test2.torrent" ),
 			  	new File( "C:\\Temp\\bdecoder.log" ));
   }
 }
