@@ -25,6 +25,8 @@ package org.gudy.azureus2.core3.util;
 import java.io.*;
 import java.net.URI;
 import java.util.*;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.logging.*;
@@ -50,6 +52,22 @@ public class FileUtil {
   private static List		reserved_file_handles 	= new ArrayList();
   private static AEMonitor	class_mon				= new AEMonitor( "FileUtil:class" );
 
+  public static boolean isAncestorOf(File parent, File child) {
+	  parent = canonise(parent);
+	  child = canonise(child);
+	  if (parent.equals(child)) {return true;}
+	  String parent_s = parent.getPath();
+	  String child_s = child.getPath();
+	  if (parent_s.charAt(parent_s.length()-1) != File.separatorChar) {
+		  parent_s += File.separatorChar;
+	  }
+	  return child_s.startsWith(parent_s);
+  }
+  
+  public static File canonise(File file) {
+	  try {return file.getCanonicalFile();}
+	  catch (IOException ioe) {return file;}
+  }
   
   public static String getCanonicalFileName(String filename) {
     // Sometimes Windows use filename in 8.3 form and cannot
@@ -86,7 +104,7 @@ public class FileUtil {
    * Deletes the given dir and all files/dirs underneath
    */
   public static void recursiveDelete(File f) {
-    String defSaveDir = COConfigurationManager.getStringParameter("Default save path", "");
+    String defSaveDir = COConfigurationManager.getStringParameter("Default save path");
     String moveToDir = COConfigurationManager.getStringParameter("Completed Files Directory", "");
     
     try{
@@ -155,7 +173,7 @@ public class FileUtil {
 	boolean	log_warnings ) 
   {
      try {
-      String defSaveDir 	= COConfigurationManager.getStringParameter("Default save path", "");
+      String defSaveDir 	= COConfigurationManager.getStringParameter("Default save path");
       String moveToDir 		= COConfigurationManager.getStringParameter("Completed Files Directory", "");
         
       if ( defSaveDir.trim().length() > 0 ){
@@ -380,9 +398,11 @@ public class FileUtil {
 		    
 		    try{
 		    	byte[] encoded_data = BEncoder.encode(data);
-		    	baos = new BufferedOutputStream( new FileOutputStream( temp, false ), 8192 );
+		    	FileOutputStream tempOS = new FileOutputStream( temp, false );
+		    	baos = new BufferedOutputStream( tempOS, 8192 );
 		    	baos.write( encoded_data );
 		    	baos.flush();
+		    	tempOS.getFD().sync();
 	        baos.close();
 	        baos = null;
 	           
@@ -595,9 +615,12 @@ public class FileUtil {
 	  					Logger.log(new LogAlert(LogAlert.UNREPEATABLE, LogAlert.AT_ERROR,
 								"Load of '" + file_name + "' fails, no usable file or backup"));
 	  				}else{
-	  					if (Logger.isEnabled())
-								Logger.log(new LogEvent(LOGID, LogEvent.LT_ERROR, "Load of '"
-										+ file_name + "' fails, file not found"));
+	  					// drop this log, it doesn't really help to inform about the failure to 
+	  					// find a .saving file
+	  					
+	  					//if (Logger.isEnabled())
+						//		Logger.log(new LogEvent(LOGID, LogEvent.LT_ERROR, "Load of '"
+						//				+ file_name + "' fails, file not found"));
 					
 	  				}
   				}
@@ -995,7 +1018,7 @@ public class FileUtil {
     		
     		File	new_parent = new File( to_parent_dir, from_file_or_dir.getName());
     		
-    		new_parent.mkdirs();
+    		FileUtil.mkdirs(new_parent);
     		
     		for (int i=0;i<files.length;i++){
     			
@@ -1127,7 +1150,7 @@ public class FileUtil {
             return( false );
         }
     	File to_file_parent = to_file.getParentFile();
-    	if (!to_file_parent.exists()) {to_file_parent.mkdirs();}
+    	if (!to_file_parent.exists()) {FileUtil.mkdirs(to_file_parent);}
     	
     	if ( from_file.isDirectory()){
     		
@@ -1381,4 +1404,131 @@ public class FileUtil {
 		
 		core.createOperation( AzureusCoreOperation.OP_FILE_MOVE, task );
 	}
+
+	/**
+	 * Makes Directories as long as the directory isn't directly in Volumes (OSX)
+	 * @param f
+	 * @return
+	 */
+	public static boolean mkdirs(File f) {
+		if (Constants.isOSX) {
+			Pattern pat = Pattern.compile("^(/Volumes/[^/]+)");
+			Matcher matcher = pat.matcher(f.getParent());
+			if (matcher.find()) {
+				String sVolume = matcher.group();
+				File fVolume = new File(sVolume);
+				if (!fVolume.isDirectory()) {
+					Logger.log(new LogEvent(LOGID, LogEvent.LT_WARNING, sVolume
+							+ " is not mounted or not available."));
+					return false;
+				}
+			}
+		}
+		return f.mkdirs();
+	}
+	
+	public static String getExtension(String fName) {
+		final int fileSepIndex = fName.lastIndexOf(File.separator);
+		final int fileDotIndex = fName.lastIndexOf('.');
+		if (fileSepIndex == fName.length() - 1 || fileDotIndex == -1
+				|| fileSepIndex > fileDotIndex) {
+			return "";
+		}
+		
+		return fName.substring(fileDotIndex);
+	}
+	
+	public static String
+	readFileAsString(
+		File	file,
+		int		size_limit )
+	
+		throws IOException
+	{
+		FileInputStream fis = new FileInputStream(file);
+		try {
+			return readInputStreamAsString(fis, size_limit);
+		} finally {
+
+			fis.close();
+		}
+	}
+		
+	public static String
+	readInputStreamAsString(
+		InputStream is,
+		int		size_limit )
+	
+		throws IOException
+	{
+		StringBuffer result = new StringBuffer(1024);
+
+		byte[] buffer = new byte[1024];
+
+		while (true) {
+
+			int len = is.read(buffer);
+
+			if (len <= 0) {
+
+				break;
+			}
+
+			result.append(new String(buffer, 0, len, "ISO-8859-1"));
+
+			if (size_limit >= 0 && result.length() > size_limit) {
+
+				result.setLength(size_limit);
+
+				break;
+			}
+		}
+
+		return (result.toString());
+	}
+
+	public static String
+	readFileEndAsString(
+		File	file,
+		int		size_limit )
+	
+		throws IOException
+	{
+		FileInputStream	fis = new FileInputStream( file );
+		
+		try{
+			if (file.length() > size_limit) {
+				fis.skip(file.length() - size_limit);
+			}
+			
+			StringBuffer	result = new StringBuffer(1024);
+			
+			byte[]	buffer = new byte[1024];
+			
+			while( true ){
+			
+				int	len = fis.read( buffer );
+			
+				if ( len <= 0 ){
+					
+					break;
+				}
+			
+				result.append( new String( buffer, 0, len, "ISO-8859-1" ));
+				
+				if ( result.length() > size_limit ){
+					
+					result.setLength( size_limit );
+					
+					break;
+				}
+			}
+			
+			return( result.toString());
+			
+		}finally{
+			
+			fis.close();
+		}
+	}
 }
