@@ -2,7 +2,22 @@
  * Created on Apr 12, 2004
  * Created by Olivier Chalouhi
  * Modified Apr 13, 2004 by Alon Rohter
- * Copyright (C) 2004 Aelitis, All Rights Reserved.
+ * Copyright (C) 2004, 2005, 2006 Aelitis, All Rights Reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * AELITIS, SAS au capital de 46,603.30 euros
+ * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
  * 
  */
  
@@ -30,11 +45,11 @@ public final class SHA1 {
    * Create a new SHA-1 message digest hasher.
    */
   public SHA1() {
-    finalBuffer = ByteBuffer.allocateDirect(64);
+    finalBuffer = ByteBuffer.allocate(64);
     finalBuffer.position(0);
     finalBuffer.limit(64);
     
-    saveBuffer = ByteBuffer.allocateDirect(64);
+    saveBuffer = ByteBuffer.allocate(64);
     saveBuffer.position(0);
     saveBuffer.limit(64);
     
