@@ -1,7 +1,7 @@
 /*
  * Created on 11-Jul-2004
  * Created by Paul Gardner
- * Copyright (C) 2004 Aelitis, All Rights Reserved.
+ * Copyright (C) 2004, 2005, 2006 Aelitis, All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  * 
- * AELITIS, SARL au capital de 30,000 euros
+ * AELITIS, SAS au capital de 46,603.30 euros
  * 8 Allee Lenotre, La Grille Royale, 78600 Le Mesnil le Roi, France.
  *
  */
@@ -43,9 +43,25 @@ ThreadPoolTask
 	public String
 	getTaskState()
 	{
-		return( worker.getState());
+		return( worker == null ? "" : worker.getState());
+	}
+	
+	public String
+	getName()
+	{
+		return( null );
 	}
 	
 	public abstract void
 	interruptTask();
+	
+	public void
+	taskStarted()
+	{
+	}
+	
+	public void
+	taskCompleted()
+	{
+	}
 }
