@@ -36,13 +36,16 @@ import org.gudy.azureus2.core3.util.SystemProperties;
 import org.gudy.azureus2.platform.PlatformManager;
 import org.gudy.azureus2.platform.PlatformManagerCapabilities;
 import org.gudy.azureus2.platform.PlatformManagerListener;
+import org.gudy.azureus2.platform.PlatformManagerPingCallback;
 import org.gudy.azureus2.platform.win32.access.AEWin32Access;
 import org.gudy.azureus2.platform.win32.access.AEWin32AccessListener;
 import org.gudy.azureus2.platform.win32.access.AEWin32Manager;
+
 import org.gudy.azureus2.plugins.platform.PlatformManagerException;
 
 import java.io.File;
 import java.io.IOException;
+import java.net.InetAddress;
 import java.util.*;
 
 
@@ -163,18 +166,24 @@ PlatformManagerImpl
 	        capabilitySet.add(PlatformManagerCapabilities.SetTCPTOSEnabled);
 	        
 	        
-	        if ( 	Constants.compareVersions( access.getVersion(), "1.11" ) >= 0 &&
+	        if ( Constants.compareVersions( access.getVersion(), "1.11" ) >= 0 &&
 	        		!Constants.isWindows9598ME ){
 	        	
 	            capabilitySet.add(PlatformManagerCapabilities.CopyFilePermissions);
 	            
 	        }
 	        
-	        if ( 	Constants.compareVersions( access.getVersion(), "1.12" ) >= 0 ){
+	        if ( Constants.compareVersions( access.getVersion(), "1.12" ) >= 0 ){
 	        	
 	            capabilitySet.add(PlatformManagerCapabilities.TestNativeAvailability);
 	        }
 	        
+	        if ( Constants.compareVersions( access.getVersion(), "1.2" ) >= 0 ){
+	        	
+	            capabilitySet.add(PlatformManagerCapabilities.TraceRouteAvailability);
+	            capabilitySet.add(PlatformManagerCapabilities.PingAvailability);
+	        }
+
     	}else{
     		
     			// disabled -> only available capability is that to get the version
@@ -360,7 +369,8 @@ PlatformManagerImpl
 		throws PlatformManagerException
 	{
 		try{
-			return( access.getUserAppData());
+			return access.getUserAppData() + SystemProperties.SEP
+					+ SystemProperties.APPLICATION_NAME + SystemProperties.SEP;
 			
 		}catch( Throwable e ){
 			
@@ -381,11 +391,26 @@ PlatformManagerImpl
 	    }else if ( location_id == LOC_MUSIC ){
 	    	
 	    	try{
-		    	return( new File(
-		    		access.readStringValue(
-		    			AEWin32Access.HKEY_CURRENT_USER,
-		    			"software\\microsoft\\windows\\currentversion\\explorer\\shell folders",
-		    			"My Music" )));
+	    		
+		    	return(new File(access.getUserMusicDir()));
+		    	
+	    	}catch( Throwable e ){
+	    		
+				throw( new PlatformManagerException( "Failed to read registry details", e ));
+	    	}
+	    } else if (location_id == LOC_DOCUMENTS) {
+	    	try{
+	    		
+		    	return(new File(access.getUserDocumentsDir()));
+		    	
+	    	}catch( Throwable e ){
+	    		
+				throw( new PlatformManagerException( "Failed to read registry details", e ));
+	    	}
+	    } else if (location_id == LOC_VIDEO) {
+	    	try{
+	    		
+		    	return(new File(access.getUserVideoDir()));
 		    	
 	    	}catch( Throwable e ){
 	    		
@@ -434,7 +459,7 @@ PlatformManagerImpl
 			
 			unregisterAdditionalFileType( OLD_MAIN_ASS0C, ".torrent" );
 			
-			registerAdditionalFileType( NEW_MAIN_ASSOC, "Azureus Torrent", ".torrent", "application/x-bittorrent" );
+			registerAdditionalFileType( NEW_MAIN_ASSOC, "Azureus Download", ".torrent", "application/x-bittorrent" );
 		}
 		
 		boolean	reg = isAdditionalFileTypeRegistered( NEW_MAIN_ASSOC, ".torrent" );
@@ -443,7 +468,7 @@ PlatformManagerImpl
 		
 		if ( !reg && !COConfigurationManager.getBooleanParameter( "platform.win32.autoregdone", false )){
 			
-			registerAdditionalFileType( NEW_MAIN_ASSOC, "Azureus Torrent", ".torrent", "application/x-bittorrent" );
+			registerAdditionalFileType( NEW_MAIN_ASSOC, "Azureus Download", ".torrent", "application/x-bittorrent" );
 
 			COConfigurationManager.setParameter( "platform.win32.autoregdone", true );
 			
@@ -569,7 +594,7 @@ PlatformManagerImpl
 	{
 		registerMagnet();
 		
-		registerAdditionalFileType( NEW_MAIN_ASSOC, "Azureus Torrent", ".torrent", "application/x-bittorrent" );
+		registerAdditionalFileType( NEW_MAIN_ASSOC, "Azureus Download", ".torrent", "application/x-bittorrent" );
 	}
 	
 	protected void
@@ -832,6 +857,50 @@ PlatformManagerImpl
 		}
 	}
 	
+	public void
+	traceRoute(
+		InetAddress							interface_address,
+		InetAddress							target,
+		PlatformManagerPingCallback	callback )
+	
+		throws PlatformManagerException
+	{
+		if ( !hasCapability( PlatformManagerCapabilities.TraceRouteAvailability )){
+			
+			throw new PlatformManagerException("Unsupported capability called on platform manager");
+		}
+		
+		try{
+			access.traceRoute( interface_address, target, callback );
+			
+		}catch( Throwable e ){
+			
+			throw( new PlatformManagerException( "Failed to trace route", e ));
+		}	
+	}
+	
+	public void
+	ping(
+		InetAddress							interface_address,
+		InetAddress							target,
+		PlatformManagerPingCallback			callback )
+	
+		throws PlatformManagerException
+	{
+		if ( !hasCapability( PlatformManagerCapabilities.PingAvailability )){
+			
+			throw new PlatformManagerException("Unsupported capability called on platform manager");
+		}
+		
+		try{
+			access.ping( interface_address, target, callback );
+			
+		}catch( Throwable e ){
+			
+			throw( new PlatformManagerException( "Failed to trace route", e ));
+		}	
+	}
+	
     /**
      * {@inheritDoc}
      */
