@@ -109,7 +109,24 @@ Download
 	
 	public static final long FLAG_ONLY_EVER_SEEDED			= 0x00000001;
 	public static final long FLAG_SCAN_INCOMPLETE_PIECES	= 0x00000002;
+	
+	/**
+	 * Flag value - if set, it prevents any of the "move on completion" or
+	 * "move on removal" rules taking place.
+	 * 
+	 * @since 2.5.0.1
+	 */
+	public static final long FLAG_DISABLE_AUTO_FILE_MOVE   = 0x00000004;
 
+    /**
+     * Flag value - if set, then it means this download has been considered
+     * for "move on completion", and it should not be considered again for
+     * it. This value is more for internal use rather than plugin use.
+     * 
+     * @since 2.5.0.1
+     */
+    public static final long FLAG_MOVE_ON_COMPLETION_DONE  = 0x00000008;
+	
 	/** get state from above ST_ set
    * @return ST_ constant
    *
@@ -142,9 +159,17 @@ Download
 		 * @return
 		 */
 	
-	public boolean
-	getFlag(
-		long		flag );
+	public boolean getFlag(long	flag);
+	
+	/**
+	 * Set the flag value.
+	 * 
+	 * @since 2.5.0.1
+	 * @param flag FLAG value from above
+	 * @param set <code>true</code> to enable the flag, <code>false</code> to disable it.
+	 */
+	public void setFlag(long flag, boolean set);
+
 	
 	/**
 	 * Index of download. {@link #getPosition()}
@@ -301,6 +326,22 @@ Download
 	public boolean
 	isPaused();
 	
+	/**
+	 * Pause the download
+	 * @since 2501
+	 */
+	
+	public void
+	pause();
+	
+	/**
+	 * Resume the download if paused
+	 * @since 2501
+	 */
+	
+	public void
+	resume();
+	
 	/** Returns the name of the torrent.  Similar to Torrent.getName() and is usefull
    * if getTorrent() returns null and you still need the name.
    * @return name of the torrent
@@ -344,6 +385,14 @@ Download
   getListAttribute(
 	TorrentAttribute		attribute );
   
+  /**
+   * 
+   * @param attribute
+   * @param value
+   * @since 2.5.0.1
+   */
+  public void setListAttribute(TorrentAttribute attribute, String[] value);
+  
   /**
    * 
    * @param attribute
@@ -359,6 +408,45 @@ Download
   getMapAttribute(
 	TorrentAttribute		attribute );
   
+  /**
+   * Gets the value of the given attribute from the download. If no value is
+   * set, then <code>0</code> will be returned.  
+   */
+  public int getIntAttribute(TorrentAttribute attribute);
+  
+  /**
+   * Sets an integer attribute on this download.
+   */
+  public void setIntAttribute(TorrentAttribute attribute, int value);
+
+  /**
+   * Gets the value of the given attribute from the download. If no value is
+   * set, then <code>0</code> will be returned.  
+   */
+  public long getLongAttribute(TorrentAttribute attribute);
+  
+  /**
+   * Sets a long attribute on this download.
+   */
+  public void setLongAttribute(TorrentAttribute attribute, long value);
+
+  /**
+   * Gets the value of the given attribute from the download. If no value is
+   * set, then <code>false</code> will be returned.  
+   */
+  public boolean getBooleanAttribute(TorrentAttribute attribute);
+  
+  /**
+   * Sets a boolean attribute on this download.
+   */
+  public void setBooleanAttribute(TorrentAttribute attribute, boolean value);
+  
+  /**
+   * Returns <code>true</code> if the download has an explicit value stored for
+   * the given attribute.
+   */
+  public boolean hasAttribute(TorrentAttribute attribute);
+  
   /** Returns the name of the Category
    *
    * @return name of the category
@@ -591,6 +679,14 @@ Download
 	public boolean
  	isChecking();
 	
+	/**
+	 * This returns the full save path for the download. If the download is a simple torrent,
+	 * this will be the full path of the file being downloaded. If the download is a multiple
+	 * file torrent, this will be the path to the directory containing all the files in the
+	 * torrent.
+	 * 
+	 * @return Full save path for this download.
+	 */
   	public String
 	getSavePath();
   	
@@ -624,6 +720,8 @@ Download
   	 * download is being saved to. The download must be in a state to move the data files to a new location
   	 * (see {@link #moveDataFiles(File)}).
   	 * 
+  	 * <p>
+  	 * 
   	 * This will not rename the displayed name for the torrent - if you wish to do that, you must do it via
   	 * the {@link org.gudy.azureus2.plugins.torrent.TorrentAttribute TorrentAttribute} class.
   	 * 
@@ -848,5 +946,60 @@ Download
    * @param auth handler
    */
   public void setSessionAuthenticator( SessionAuthenticator auth );
+  
+  /**
+   * Returns an array of size 2 indicating the appropriate locations for this
+   * download's data files and torrent file, based on Azureus's rules regarding
+   * default save paths, and move on completion rules.
+   * 
+   * <p>
+   * 
+   * This method takes one argument - <i>for_moving</i>. This essentially
+   * indicates whether you are getting this information for purposes of just
+   * finding where Azureus would store these files by default, or whether you
+   * are considering moving the files from its current location.
+   * 
+   * <p>
+   * 
+   * If <i>for_moving</i> is <tt>false</tt>, this method will determine locations
+   * for the download and the torrent file where Azureus would store them by
+   * default (it may return the current paths used by this download). 
+   * 
+   * <p>
+   * 
+   * If <i>for_moving</i> is <tt>true</tt>, then this method will consider the
+   * download's current location, and whether it is allowed to move it - you
+   * may not be allowed to move this download (based on Azureus's current rules)
+   * if the download doesn't exist within a default save directory already. If
+   * a download is complete, we consider whether we are allowed to move downloads
+   * on completion, and whether that includes downloads outside the default save
+   * directory.
+   * 
+   * <p>
+   * 
+   * In this case, the array may contain <tt>null</tt> indicating that the Azureus
+   * doesn't believe that the download should be moved (based on the current rules
+   * the user has set out). However, you are not prevented from changing the
+   * location of the torrent file or download.
+   * 
+   * @since 2.5.0.2
+   * @param for_moving Indicates whether you want this information for the purposes
+   *     of moving the download or not.
+   * @author amc1
+   * @return An array of type <tt>File</tt> of size 2, first element containing the
+   *     calculated location for the download's data files, and the second element
+   *     containing the location for the download's torrent file.
+   */
+  public File[] calculateDefaultPaths(boolean for_moving);
+  
+  /**
+   * Returns <tt>true</tt> if the download is being saved to one of the default
+   * save directories.
+   * 
+   * @since 2.5.0.2
+   * @author amc1
+   */
+  public boolean isInDefaultSaveDir();
+
 
 }
