@@ -54,6 +54,11 @@ public interface MenuItem
 	
 	public static final int STYLE_SEPARATOR			= 4;
 	
+	   /**
+	    * menu containing submenu items
+	    */
+	public static final int STYLE_MENU              = 5;
+	
   /** Retrieve the resource key ("name") of this menu item
    *
    * @return resource key for this menu
@@ -61,14 +66,6 @@ public interface MenuItem
 	public String
 	getResourceKey();
 	
-	/** Retrieve the parent menu's resource key.
-	 *
-	 * @return parent menu's resource key, or null if no parent
-	 *
-	public Menu
-	getParent();
-   */
-	
 		/**
 		 * Get the type of the menu item
 		 */
@@ -144,20 +141,52 @@ public interface MenuItem
 	removeFillListener(
 		MenuItemFillListener	listener );
 
-	/** Add a Selection Listener for this menu item
-	 *
-	 * @param l listener to be notified when user has selected menu item.
+	/**
+	 * Adds a selection listener for this menu item.
+	 * @param l listener to be notified when user has selected the menu item.
 	 */
-	public void
-	addListener(
-		MenuItemListener	l );
+	public void	addListener(MenuItemListener l);
+	
+   /**
+    * Removes a selection listener from this menu item.
+    * @param l listener to remove
+    */
+	public void	removeListener(MenuItemListener	l);
+	
+	/** 
+	 * Retrieve the parent MenuItem.
+	 *
+	 * @return parent menu object, or null if no parent
+	 */
+	public MenuItem
+	getParent();
+
 	
 	/**
-   * Remove a Selection Listener from this menu item
-   *
-   * @param l listener to remove
-   */
-	public void
-	removeListener(
-		MenuItemListener	l );
+	 * Get all child items currently associated with this MenuItem.
+	 * 
+	 * @return An array of items (if this object has the menu style
+	 * associated) or null otherwise.
+	 */
+	public MenuItem[] getItems();
+	
+	/**
+	 * Get the child item with the given resource key.
+	 * 
+	 * @return The child MenuItem object which has the resource key
+	 * specified, or null otherwise.
+	 */
+	public MenuItem getItem(String key_id);
+	
+	/**
+	 * Gets the text to display for this menu item.
+	 */
+	public String getText();
+	
+	/**
+	 * Sets the text to display for this menu item. You can also
+	 * pass null to revert back to the default behaviour.
+	 */
+	public void setText(String text);
+
 }
