@@ -135,7 +135,14 @@ Utilities
 	createTimer(
 		String		name,
 		boolean		lightweight );
-	
+
+	/**
+	 * @param name
+	 * @param priority Thread.XXX_PRIORITY
+	 * @return
+	 */
+	public UTTimer createTimer(String name, int priority);
+
 		/**
 		 * create and run a thread for the target. This will be a daemon thread so that
 		 * its existence doesn't interfere with Azureus closedown
@@ -266,6 +273,20 @@ Utilities
 	 * @since 2.3.0.7
 	 */
 	public int compareVersions(String v1, String v2);
+	
+	/**
+	 * Converts a file name so that all characters in the file name are
+	 * compatible with the underlying filesystem. This includes quote
+	 * characters, back and forwarded slashes, newline characters and so on.
+	 * 
+	 * <p>
+	 * 
+	 * Note - this is only intended for file names, rather than file paths.
+	 * 
+	 * @param f_name File name to convert.
+	 * @return Converted file name.
+	 */
+	public String normaliseFileName(String f_name);
 }
 
 
