@@ -261,6 +261,10 @@ DownloadImpl
 		return( download_manager.getDownloadState().getFlag( flag ));
 	}
 	
+	public void setFlag(long flag, boolean set) {
+		download_manager.getDownloadState().setFlag(flag, set);
+	}
+	
 	public int
 	getIndex()
 	{
@@ -398,6 +402,18 @@ DownloadImpl
 		return( download_manager.isPaused());
 	}
 	
+	public void
+	pause()
+	{
+		download_manager.pause();
+	}
+	
+	public void
+	resume()
+	{
+		download_manager.resume();
+	}
+	
 	public int
 	getPosition()
 	{
@@ -484,6 +500,13 @@ DownloadImpl
 	  	return( null );
   }
   
+  public void setListAttribute(TorrentAttribute attribute, String[] value) {
+	  String name = convertAttribute(attribute);
+	  if (name != null) {
+		  download_manager.getDownloadState().setListAttribute(name, value);
+	  }
+  }
+  
   public void
   setMapAttribute(
 	TorrentAttribute		attribute,
@@ -524,6 +547,51 @@ DownloadImpl
   	}
   }
   
+  public boolean hasAttribute(TorrentAttribute attribute) {
+	  String name = convertAttribute(attribute);
+	  if (name == null) {return false;}
+	  return download_manager.getDownloadState().hasAttribute(name);
+  }
+  
+  public boolean getBooleanAttribute(TorrentAttribute attribute) {
+	  String name = convertAttribute(attribute);
+	  if (name == null) {return false;} // Default value
+	  return download_manager.getDownloadState().getBooleanAttribute(name);
+  }
+  
+  public void setBooleanAttribute(TorrentAttribute attribute, boolean value) {
+	  String name = convertAttribute(attribute);
+	  if (name != null) {
+		  download_manager.getDownloadState().setBooleanAttribute(name, value);
+	  }
+  }
+
+  public int getIntAttribute(TorrentAttribute attribute) {
+	  String name = convertAttribute(attribute);
+	  if (name == null) {return 0;} // Default value
+	  return download_manager.getDownloadState().getIntAttribute(name);
+  }
+  
+  public void setIntAttribute(TorrentAttribute attribute, int value) {
+	  String name = convertAttribute(attribute);
+	  if (name != null) {
+		  download_manager.getDownloadState().setIntAttribute(name, value);
+	  }
+  }
+  
+  public long getLongAttribute(TorrentAttribute attribute) {
+	  String name = convertAttribute(attribute);
+	  if (name == null) {return 0L;} // Default value
+	  return download_manager.getDownloadState().getLongAttribute(name);
+  }
+  
+  public void setLongAttribute(TorrentAttribute attribute, long value) {
+	  String name = convertAttribute(attribute);
+	  if (name != null) {
+		  download_manager.getDownloadState().setLongAttribute(name, value);
+	  }
+  }
+  
   protected String
   convertAttribute(
   	TorrentAttribute		attribute )
@@ -548,6 +616,14 @@ DownloadImpl
 		
 		return( DownloadManagerState.AT_DISPLAY_NAME );
 		
+	}else if ( attribute.getName() == TorrentAttribute.TA_USER_COMMENT ){
+		
+		return( DownloadManagerState.AT_USER_COMMENT );	
+
+	}else if ( attribute.getName() == TorrentAttribute.TA_RELATIVE_SAVE_PATH ){
+		
+		return( DownloadManagerState.AT_RELATIVE_SAVE_PATH );	
+		
 	}else if ( attribute.getName() == TorrentAttribute.TA_CONTENT_MAP ){
 		
 		return( DownloadManagerState.AT_CONTENT_MAP );
@@ -594,6 +670,14 @@ DownloadImpl
 		
 		return ( TorrentManagerImpl.getSingleton().getAttribute( TorrentAttribute.TA_DISPLAY_NAME ));
 		
+	}else if ( name.equals ( DownloadManagerState.AT_USER_COMMENT)){
+		
+		return ( TorrentManagerImpl.getSingleton().getAttribute( TorrentAttribute.TA_USER_COMMENT ));
+
+	}else if ( name.equals ( DownloadManagerState.AT_RELATIVE_SAVE_PATH)){
+		
+		return ( TorrentManagerImpl.getSingleton().getAttribute( TorrentAttribute.TA_RELATIVE_SAVE_PATH ));
+		
 	}else if ( name.equals ( DownloadManagerState.AT_CONTENT_MAP)){
 		
 		return ( TorrentManagerImpl.getSingleton().getAttribute( TorrentAttribute.TA_CONTENT_MAP ));
@@ -701,9 +785,7 @@ DownloadImpl
  	public boolean
 	isComplete()
  	{
- 		int	state = getState();
- 		
- 		return state == ST_SEEDING || download_manager.isDownloadComplete(false);
+ 		return download_manager.isDownloadComplete(false);
  	}
 
  	public boolean isComplete(boolean bIncludeDND) {
@@ -713,16 +795,9 @@ DownloadImpl
  	public boolean
  	isChecking()
  	{
- 		org.gudy.azureus2.core3.disk.DiskManager	dm = download_manager.getDiskManager();
- 		
- 		if ( dm != null ){
- 			
- 			return( dm.getCompleteRecheckStatus() != -1 );
- 		}
- 		
- 		return( false );
+ 		return( download_stats.getCheckingDoneInThousandNotation() != -1 );
  	}
- 	
+
 	protected void
 	isRemovable()
 		throws DownloadRemovalVetoException
@@ -1441,6 +1516,14 @@ DownloadImpl
  		}  	
  	}
   	
+  	public File[] calculateDefaultPaths(boolean for_moving) {
+  		return download_manager.calculateDefaultPaths(for_moving);
+  	}
+  	
+  	public boolean isInDefaultSaveDir() {
+  		return download_manager.isInDefaultSaveDir();
+  	}
+  	
  	public void
 	requestTrackerAnnounce()
  	{
