@@ -38,6 +38,8 @@ import org.gudy.azureus2.plugins.network.Connection;
 import org.gudy.azureus2.plugins.peers.*;
 import org.gudy.azureus2.pluginsimpl.local.messaging.MessageAdapter;
 
+import com.aelitis.azureus.core.networkmanager.NetworkConnectionBase;
+import com.aelitis.azureus.core.networkmanager.NetworkManager;
 import com.aelitis.azureus.core.peermanager.messaging.Message;
 import com.aelitis.azureus.core.peermanager.peerdb.*;
 import com.aelitis.azureus.core.peermanager.piecepicker.util.BitFlags;
@@ -53,6 +55,8 @@ PeerForeignDelegate
 	private PeerManagerImpl		manager;
 	private Peer				foreign;
 	
+	private NetworkConnectionBase	network_connection;
+	
 	private long	create_time		= SystemTime.getCurrentTime();
 	private long	last_data_received_time =-1;
 	private long	last_data_message_received_time =-1;
@@ -74,8 +78,28 @@ PeerForeignDelegate
 	{
 		manager		= _manager;
 		foreign		= _foreign;
+		
+		PEPeerManager pm = manager.getDelegate();
+		
+		network_connection = new PeerForeignNetworkConnection( foreign );
+				
+		NetworkManager.getSingleton().startTransferProcessing( network_connection, pm.getUploadLimitedRateGroup(), pm.getDownloadLimitedRateGroup() );
+		
+		NetworkManager.getSingleton().upgradeTransferProcessing( network_connection );
 	}
 	
+	public void
+	start()
+	{
+		// should never be called
+		Debug.out( "eh?" );
+	}
+	
+	protected void
+	closed()
+	{
+		NetworkManager.getSingleton().stopTransferProcessing( network_connection );
+	}
 	
     /**
      * Should never be called
@@ -119,15 +143,31 @@ PeerForeignDelegate
    * @param pieceLength
    * @return true is the piece is really requested
    */
-	public boolean 
+	public DiskManagerReadRequest 
 	request(
 		int pieceNumber, 
 		int pieceOffset, 
 		int pieceLength )
 	{
-		return( foreign.addRequest( manager.getDelegate().getDiskManager().createReadRequest( pieceNumber, pieceOffset, pieceLength )));
-	}
+		DiskManagerReadRequest	request = manager.getDelegate().getDiskManager().createReadRequest( pieceNumber, pieceOffset, pieceLength );
 		
+		if ( foreign.addRequest( request )){
+			
+			return( request );
+			
+		}else{
+			
+			return( null );
+		}
+	}
+	
+	public int
+	getRequestIndex(
+		DiskManagerReadRequest request )
+	{
+		return( foreign.getRequests().indexOf( request ));
+	}
+	
 	protected  void
 	dataReceived()
 	{
@@ -138,7 +178,13 @@ PeerForeignDelegate
 	closeConnection( 
 		String reason ) 
 	{
-		foreign.close( reason, false, false ); 
+		try{
+			foreign.close( reason, false, false );
+			
+		}finally{
+			
+			closed();
+		}
 	}
 		
 	public List
@@ -198,7 +244,9 @@ PeerForeignDelegate
 				foreign.getTCPListenPort(), 
 				PeerItemFactory.PEER_SOURCE_PLUGIN, 
 				PeerItemFactory.HANDSHAKE_TYPE_PLAIN,
-				foreign.getUDPListenPort());
+				foreign.getUDPListenPort(),
+				PeerItemFactory.CRYPTO_LEVEL_1,
+				0 );
 	}
   
   
@@ -491,6 +539,7 @@ PeerForeignDelegate
 
 	public void setUploadHint(int timeToSpread) {}  
 	
+	public boolean isStalledPendingLoad(){return( false );}
 
 	public void addListener(final PEPeerListener l )
 	{
