@@ -32,6 +32,7 @@ import org.gudy.azureus2.core3.disk.DiskManagerPiece;
 import org.gudy.azureus2.core3.disk.DiskManagerReadRequest;
 import org.gudy.azureus2.core3.peer.*;
 import org.gudy.azureus2.core3.util.AEMonitor;
+import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.plugins.disk.DiskManager;
 import org.gudy.azureus2.plugins.download.*;
 import org.gudy.azureus2.plugins.peers.*;
@@ -44,6 +45,8 @@ public class
 PeerManagerImpl
 	implements PeerManager
 {
+	private static final String	PEPEER_DATA_KEY	= PeerManagerImpl.class.getName();
+	
 	protected PEPeerManager	manager;
 	
 	protected static AEMonitor	pm_map_mon	= new AEMonitor( "PeerManager:Map" );
@@ -79,7 +82,9 @@ PeerManagerImpl
 
 	private final DiskManagerPiece[]	dm_pieces;
 	private final PEPiece[]				pe_pieces;
-	private pieceFacade[]	piece_facades;
+	private pieceFacade[]				piece_facades;
+	
+	private boolean	destroyed;
 	
 	protected
 	PeerManagerImpl(
@@ -89,6 +94,53 @@ PeerManagerImpl
 		
 		dm_pieces	= _manager.getDiskManager().getPieces();
 		pe_pieces	= _manager.getPieces();
+		
+		manager.addListener(
+			new PEPeerManagerListener()
+			{
+				 public void
+				 peerAdded(
+					PEPeerManager	manager, 
+					PEPeer 			peer )
+				 {
+					 
+				 }
+				 
+				 public void 
+				 peerRemoved( 
+					PEPeerManager 	manager, 
+					PEPeer 			peer )
+				 {
+					 PeerImpl	dele = getPeerForPEPeer( peer );
+					 
+					 if ( dele != null ){
+						 
+						 dele.closed();
+					 }
+				 }
+				 
+				public void
+				destroyed()
+				{	
+					synchronized( foreign_map ){
+						
+						destroyed	= true;
+						
+						Iterator it = foreign_map.values().iterator();
+						
+						while( it.hasNext()){
+							
+							try{
+								((PeerForeignDelegate)it.next()).closed();
+								
+							}catch( Throwable e ){
+								
+								Debug.printStackTrace( e );
+							}
+						}
+					}
+				}
+			});
 	}
 
 	public PEPeerManager
@@ -152,7 +204,9 @@ PeerManagerImpl
 	createPeerStats(
 		Peer	peer )
 	{
-		return( new PeerStatsImpl( this, peer, manager.createPeerStats()));
+		PEPeer	delegate = mapForeignPeer( peer );
+		
+		return( new PeerStatsImpl( this, peer, manager.createPeerStats( delegate )));
 	}
 	
 	
@@ -243,12 +297,40 @@ PeerManagerImpl
 		
 		for (int i=0;i<res.length;i++){
 			
-			res[i] = new PeerImpl((PEPeer)l.get(i));
+			res[i] = getPeerForPEPeer((PEPeer)l.get(i));
 		}
 		
 		return( res );
 	}
 	
+	public Peer[]
+	getPeers(
+		String		address )
+	{
+		List	l = manager.getPeers( address );
+		
+		Peer[]	res= new Peer[l.size()];
+		
+			// this is all a bit shagged as we should maintain the PEPeer -> Peer link rather
+			// than continually creating new PeerImpls...
+		
+		for (int i=0;i<res.length;i++){
+			
+			res[i] = getPeerForPEPeer((PEPeer)l.get(i));
+		}
+		
+		return( res );
+	}
+	
+
+	
+	public PeerDescriptor[]
+	getPendingPeers(
+		String		address )
+	{
+		return( manager.getPendingPeers( address ));
+	}
+	
 	public long
 	getTimeSinceConnectionEstablished(
 		Peer		peer )
@@ -278,18 +360,28 @@ PeerManagerImpl
 			return(((PeerImpl)_foreign).getDelegate());
 		}
 		
-		PEPeer	local = (PEPeer)foreign_map.get( _foreign );
-		
-		if( local == null ){
+		synchronized( foreign_map ){
 			
-			local 	= new PeerForeignDelegate( this, _foreign );
+			PEPeer	local = (PEPeer)foreign_map.get( _foreign );
 			
-			_foreign.setUserData( PeerManagerImpl.class, local );
+			if( local == null ){
+				
+				if ( destroyed ){
+					
+					Debug.out( "Peer added to destroyed peer manager" );
+					
+					return( null );
+				}
+				
+				local 	= new PeerForeignDelegate( this, _foreign );
+				
+				_foreign.setUserData( PeerManagerImpl.class, local );
+				
+				foreign_map.put( _foreign, local );
+			}
 			
-			foreign_map.put( _foreign, local );
+			return( local );
 		}
-		
-		return( local );
 	}
 	
 	protected PeerForeignDelegate
@@ -320,6 +412,21 @@ PeerManagerImpl
 		return( res );
 	}
 	
+	public static PeerImpl
+	getPeerForPEPeer(
+		PEPeer	pe_peer )
+	{
+		PeerImpl	peer = (PeerImpl)pe_peer.getData( PEPEER_DATA_KEY );
+		
+		if ( peer == null ){
+			
+			peer = new PeerImpl( pe_peer );
+			
+			pe_peer.setData( PEPEER_DATA_KEY, peer );
+		}
+		
+		return( peer );
+	}
 	
 	public void
 	addListener(
@@ -332,7 +439,7 @@ PeerManagerImpl
 			
       PEPeerManagerListener core_listener = new PEPeerManagerListener() {
         public void peerAdded( PEPeerManager manager, PEPeer peer ) {
-          PeerImpl pi = new PeerImpl( peer );
+          PeerImpl pi = getPeerForPEPeer( peer );
           peer_map.put( peer, pi );
           l.peerAdded( PeerManagerImpl.this, pi );
         }
@@ -349,6 +456,10 @@ PeerManagerImpl
             l.peerRemoved( PeerManagerImpl.this, pi );
           }
         }
+        public void
+        destroyed()
+        {
+        }
       };
       
 			listener_map.put( l, core_listener );
