@@ -37,12 +37,13 @@ import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.core3.config.*;
 import org.gudy.azureus2.core3.logging.*;
 import org.gudy.azureus2.core3.torrent.*;
-import org.gudy.azureus2.core3.tracker.host.*;
+import org.gudy.azureus2.core3.tracker.util.TRTrackerUtils;
 
 public class 
 ShareManagerImpl
 	implements ShareManager, TOTorrentProgressListener, ParameterListener
 {
+	private static final LogIDs LOGID = LogIDs.PLUGIN;
 	public static final String		TORRENT_STORE 		= "shares";
 	public static final String		TORRENT_SUBSTORE	= "cache";
 	
@@ -78,17 +79,19 @@ ShareManagerImpl
 	}
 	
 	
-	protected boolean			initialised;
-	protected File				share_dir;
+	private volatile boolean	initialised;
+	private volatile boolean	initialising;
 	
-	protected URL				announce_url;
-	protected ShareConfigImpl	config;
+	private File				share_dir;
 	
-	protected Map				shares 		= new HashMap();
+	private URL[]				announce_urls;
+	private ShareConfigImpl		config;
 	
-	protected shareScanner		current_scanner;
+	private Map					shares 		= new HashMap();
 	
-	protected List				listeners	= new ArrayList();
+	private shareScanner		current_scanner;
+	
+	private List				listeners	= new ArrayList();
 	
 	protected
 	ShareManagerImpl()
@@ -101,7 +104,7 @@ ShareManagerImpl
 				public void
 				configurationSaved()
 				{
-					announce_url	= null;
+					announce_urls	= null;
 				}
 			});
 	}
@@ -115,49 +118,57 @@ ShareManagerImpl
 		
 			if ( !initialised ){
 			
-				initialised	= true;
-				
-				share_dir = FileUtil.getUserFile( TORRENT_STORE );
-				
-				share_dir.mkdirs();
-								
-				config = new ShareConfigImpl();
-				
 				try{
-					config.suspendSaving();
+					initialising	= true;
+					
+					initialised		= true;
+					
+					share_dir = FileUtil.getUserFile( TORRENT_STORE );
+					
+					FileUtil.mkdirs(share_dir);
+									
+					config = new ShareConfigImpl();
+					
+					try{
+						config.suspendSaving();
+					
+						config.loadConfig(this);
 				
-					config.loadConfig(this);
-			
-					checkConsistency();
-					
-				}finally{
-				
-					Iterator it = shares.values().iterator();
-					
-					while(it.hasNext()){
-					
-						ShareResourceImpl	resource = (ShareResourceImpl)it.next();
+						checkConsistency();
 						
-						if ( resource.getType() == ShareResource.ST_DIR_CONTENTS ){
-				
-							for (int i=0;i<listeners.size();i++){
-								
-								try{
+					}finally{
+					
+						Iterator it = shares.values().iterator();
+						
+						while(it.hasNext()){
+						
+							ShareResourceImpl	resource = (ShareResourceImpl)it.next();
+							
+							if ( resource.getType() == ShareResource.ST_DIR_CONTENTS ){
+					
+								for (int i=0;i<listeners.size();i++){
 									
-									((ShareManagerListener)listeners.get(i)).resourceAdded( resource );
-									
-								}catch( Throwable e ){
-									
-									Debug.printStackTrace( e );
+									try{
+										
+										((ShareManagerListener)listeners.get(i)).resourceAdded( resource );
+										
+									}catch( Throwable e ){
+										
+										Debug.printStackTrace( e );
+									}
 								}
 							}
 						}
+						
+						config.resumeSaving();
 					}
 					
-					config.resumeSaving();
+					readAZConfig();
+					
+				}finally{
+					
+					initialising	= false;
 				}
-				
-				readAZConfig();
 			}
 		}finally{
 			
@@ -165,6 +176,12 @@ ShareManagerImpl
 		}
 	}
 	
+	public boolean
+	isInitialising()
+	{
+		return( initialising );
+	}
+	
 	protected void
 	readAZConfig()
 	{
@@ -222,7 +239,13 @@ ShareManagerImpl
 			
 			ShareResourceImpl	resource = (ShareResourceImpl)it.next();
 			
-			resource.checkConsistency();
+			try{
+				resource.checkConsistency();
+				
+			}catch( ShareException e ){
+				
+				Debug.printStackTrace(e);
+			}
 		}
 	}
 	
@@ -295,7 +318,7 @@ ShareManagerImpl
 			
 			if ( !cache_dir.exists()){
 				
-				cache_dir.mkdirs();
+				FileUtil.mkdirs(cache_dir);
 			}
 			
 			if ( cache_dir.listFiles().length < MAX_FILES_PER_DIR ){
@@ -316,7 +339,7 @@ ShareManagerImpl
 			}
 		}
 		
-		throw( new ShareException( "ShareManager: failed to allocate cache file"));
+		throw( new ShareException( "ShareManager: Failed to allocate cache file"));
 	}
 	
 	protected void
@@ -330,7 +353,7 @@ ShareManagerImpl
 			
 		}catch( TorrentException e ){
 			
-			throw( new ShareException( "ShareManager: torrent write fails", e ));
+			throw( new ShareException( "ShareManager: Torrent write fails", e ));
 		}
 	}
 	
@@ -347,7 +370,7 @@ ShareManagerImpl
 			
 		}catch( TOTorrentException e ){
 			
-			throw( new ShareException( "ShareManager: torrent read fails", e ));
+			throw( new ShareException( "ShareManager: Torrent read fails", e ));
 		}
 	}
 	
@@ -356,9 +379,7 @@ ShareManagerImpl
 		ShareItemImpl		item )
 	{
 		File	torrent_file = getTorrentFile(item);
-		
-		System.out.println( "deleting torrent '".concat(torrent_file.toString()).concat("'"));
-		
+				
 		torrent_file.delete();
 	}
 	
@@ -376,44 +397,48 @@ ShareManagerImpl
 		return( new File(share_dir+File.separator+item.getTorrentLocation()));
 	}
 	
-	protected URL
-	getAnnounceURL()
+	protected URL[]
+	getAnnounceURLs()
 	
 		throws ShareException
 	{
-		if ( announce_url == null ){
+		if ( announce_urls == null ){
+						
+			String	protocol = COConfigurationManager.getStringParameter( "Sharing Protocol" );
 			
-			String 	tracker_ip 		= COConfigurationManager.getStringParameter("Tracker IP", "");
-			
-			if ( tracker_ip.length() == 0 ){
+			if ( protocol.equalsIgnoreCase( "DHT" )){
 				
-				throw( new ShareException( "ShareManager: tracker must be configured"));
-			}
-			
-			boolean	use_ssl = COConfigurationManager.getBooleanParameter( "Sharing Use SSL", false );
-			
-			int	port;
-			
-			if ( use_ssl ){
-				
-				port = COConfigurationManager.getIntParameter("Tracker Port SSL", TRHost.DEFAULT_PORT_SSL );
+				announce_urls	= new URL[]{ TorrentUtils.getDecentralisedEmptyURL()};
 				
 			}else{
-				
-				port = COConfigurationManager.getIntParameter("Tracker Port", TRHost.DEFAULT_PORT );
-			}
 			
-			try{
-				announce_url = new URL( (use_ssl?"https":"http").concat("://").concat(tracker_ip).concat(":").concat(String.valueOf(port)).concat("/announce") );
+				URL[][]	tracker_url_sets = TRTrackerUtils.getAnnounceURLs();
 				
-			}catch( MalformedURLException e ){
-				
-				throw( new ShareException( "Announce URL invalid", e ));
-			}
+				if ( tracker_url_sets.length == 0 ){
+					
+					throw( new ShareException( "ShareManager: Tracker must be configured"));
+				}
 			
+				for (int i=0;i<tracker_url_sets.length;i++){
+				
+					URL[]	tracker_urls = tracker_url_sets[i];
+			
+					if ( tracker_urls[0].getProtocol().equalsIgnoreCase( protocol )){
+					
+						announce_urls = tracker_urls;
+						
+						break;
+					}
+				}
+				
+				if ( announce_urls == null ){
+					
+					throw( new ShareException( "ShareManager: Tracker must be configured for protocol '" + protocol + "'" ));
+				}
+			}
 		}
 		
-		return( announce_url );
+		return( announce_urls );
 	}
 	
 	protected boolean
@@ -435,15 +460,29 @@ ShareManagerImpl
 	protected ShareResourceImpl
 	getResource(
 		File		file )
+	
+		throws ShareException
 	{
-		return((ShareResourceImpl)shares.get(file.toString()));
+		try{
+			return((ShareResourceImpl)shares.get(file.getCanonicalFile().toString()));
+			
+		}catch( IOException e ){
+			
+			throw( new ShareException( "getCanonicalFile fails", e ));
+		}
 	}
 	
 	public ShareResource
 	getShare(
 		File	file_or_dir )
 	{
-		return( getResource( file_or_dir ));
+		try{
+			return( getResource( file_or_dir ));
+			
+		}catch( ShareException e ){
+						
+			return( null );
+		}
 	}
 	
 	public ShareResourceFile
@@ -452,10 +491,22 @@ ShareManagerImpl
 	
 		throws ShareException, ShareResourceDeletionVetoException
 	{
-		LGLogger.log( "ShareManager: addFile '" + file.toString()+ "'" );
+		return( addFile( null, file ));
+	}
+	
+	protected ShareResourceFile
+	addFile(
+		ShareResourceDirContentsImpl	parent,
+		File							file )
+
+		throws ShareException, ShareResourceDeletionVetoException
+	{
+		if (Logger.isEnabled())
+			Logger.log(new LogEvent(LOGID, "ShareManager: addFile '"
+					+ file.toString() + "'"));
 
 		try{
-			return( (ShareResourceFile)addFileOrDir( file, ShareResource.ST_FILE, false ));
+			return( (ShareResourceFile)addFileOrDir( parent, file, ShareResource.ST_FILE, false ));
 			
 		}catch( ShareException e ){
 			
@@ -480,12 +531,24 @@ ShareManagerImpl
 	
 		throws ShareException, ShareResourceDeletionVetoException
 	{
-		LGLogger.log( "ShareManager: addDir '" + dir.toString()+ "'" );
+		return( addDir( null, dir ));
+	}
+	
+	public ShareResourceDir
+	addDir(
+		ShareResourceDirContentsImpl	parent,
+		File							dir )
+	
+		throws ShareException, ShareResourceDeletionVetoException
+	{
+		if (Logger.isEnabled())
+			Logger.log(new LogEvent(LOGID, "ShareManager: addDir '" + dir.toString()
+					+ "'"));
 
 		try{
 			this_mon.enter();
 			
-			return( (ShareResourceDir)addFileOrDir( dir, ShareResource.ST_DIR, false ));
+			return( (ShareResourceDir)addFileOrDir( parent, dir, ShareResource.ST_DIR, false ));
 			
 		}catch( ShareException e ){
 			
@@ -510,16 +573,17 @@ ShareManagerImpl
 	
 	protected ShareResource
 	addFileOrDir(
-		File		file,
-		int			type,
-		boolean		modified )
+		ShareResourceDirContentsImpl	parent,
+		File							file,
+		int								type,
+		boolean							modified )
 	
 		throws ShareException, ShareResourceDeletionVetoException
 	{
 		try{
 			this_mon.enter();
 		
-			String	name = file.toString();
+			String	name = file.getCanonicalFile().toString();
 			
 			ShareResource	old_resource = (ShareResource)shares.get(name);
 			
@@ -534,13 +598,13 @@ ShareManagerImpl
 		
 				reportCurrentTask( "Adding file '" + name + "'");
 				
-				new_resource = new ShareResourceFileImpl( this, file );
+				new_resource = new ShareResourceFileImpl( this, parent, file );
 				
 			}else{
 				
 				reportCurrentTask( "Adding dir '" + name + "'");
 				
-				new_resource = new ShareResourceDirImpl( this, file );
+				new_resource = new ShareResourceDirImpl( this, parent, file );
 			}
 			
 			shares.put(name, new_resource );
@@ -567,6 +631,10 @@ ShareManagerImpl
 			
 			return( new_resource );
 			
+		}catch( IOException e ){
+			
+			throw( new ShareException( "getCanoncialFile fails", e ));
+			
 		}finally{
 			
 			this_mon.exit();
@@ -582,12 +650,14 @@ ShareManagerImpl
 	
 		throws ShareException, ShareResourceDeletionVetoException
 	{
-		LGLogger.log( "ShareManager: addDirContents '" + dir.toString()+ "'" );
+		if (Logger.isEnabled())
+			Logger.log(new LogEvent(LOGID, "ShareManager: addDirContents '"
+					+ dir.toString() + "'"));
 
 		try{
 			this_mon.enter();
 			
-			String	name = dir.toString();
+			String	name = dir.getCanonicalFile().toString();
 			
 			reportCurrentTask( "Adding dir contents '" + name + "', recursive = " + recursive );
 	
@@ -618,6 +688,12 @@ ShareManagerImpl
 			
 			return( new_resource );
 			
+		}catch( IOException e ){
+			
+			reportError(e);
+			
+			throw( new ShareException( "getCanoncialFile fails", e ));
+			
 		}catch( ShareException e ){
 			
 			reportError(e);
@@ -628,7 +704,6 @@ ShareManagerImpl
 			
 			this_mon.exit();
 		}
-		
 	}	
 	
 	protected void
@@ -637,7 +712,9 @@ ShareManagerImpl
 	
 		throws ShareException
 	{
-		LGLogger.log( "ShareManager: resource '" + resource.getName() + "' deleted" );
+		if (Logger.isEnabled())
+			Logger.log(new LogEvent(LOGID, "ShareManager: resource '"
+					+ resource.getName() + "' deleted"));
 		
 		try{
 			this_mon.enter();
@@ -670,7 +747,9 @@ ShareManagerImpl
 	
 		throws ShareException
 	{
-		LGLogger.log( "ShareManager: scanning resources for changes" );
+		if (Logger.isEnabled())
+			Logger.log(new LogEvent(LOGID,
+					"ShareManager: scanning resources for changes"));
 
 		checkConsistency();
 	}
@@ -738,7 +817,7 @@ ShareManagerImpl
 		
 		if ( message != null ){
 			
-			reportCurrentTask( message );
+			reportCurrentTask( Debug.getNestedExceptionMessage(e));
 			
 		}else{
 			
