@@ -379,6 +379,14 @@ RPPluginInterface
 		return( null );
 	}
 	
+	public boolean
+	isShared()
+	{
+		notSupported();
+		
+		return( false );
+	}
+	
     public String getPluginName()
 	{
 		notSupported();
@@ -587,4 +595,9 @@ RPPluginInterface
 	{
 		notSupported();
 	}
+
+	public ConfigSection[] getConfigSections() {
+		// TODO Auto-generated method stub
+		return null;
+	}
 }
