@@ -38,7 +38,6 @@ import org.gudy.azureus2.plugins.torrent.*;
 
 import org.gudy.azureus2.pluginsimpl.remote.*;
 import org.gudy.azureus2.pluginsimpl.remote.disk.RPDiskManagerFileInfo;
-import org.gudy.azureus2.pluginsimpl.remote.ipfilter.RPIPRange;
 import org.gudy.azureus2.pluginsimpl.remote.torrent.*;
 
 
@@ -448,6 +447,18 @@ RPDownload
 		return( false );
 	}
 	
+	public void
+	pause()
+	{
+		notSupported();
+	}
+	
+	public void
+	resume()
+	{
+		notSupported();
+	}
+	
 	public int
 	getPriority()
 	{
@@ -942,4 +953,21 @@ RPDownload
 		public void renameDownload(String new_name) {
 			notSupported();
 		}
+		
+		public File[] calculateDefaultPaths(boolean for_moving) {
+			notSupported();
+			return null;
+		}
+		
+		public boolean isInDefaultSaveDir() {notSupported(); return false;}
+		
+	public boolean getBooleanAttribute(TorrentAttribute ta) {notSupported(); return false;}
+	public int getIntAttribute(TorrentAttribute ta) {notSupported(); return 0;}
+	public long getLongAttribute(TorrentAttribute ta) {notSupported(); return 0L;}
+	public boolean hasAttribute(TorrentAttribute ta) {notSupported(); return false;}
+	public void setBooleanAttribute(TorrentAttribute ta, boolean value) {notSupported();}
+	public void setIntAttribute(TorrentAttribute ta, int value) {notSupported();}
+	public void setListAttribute(TorrentAttribute ta, String[] value) {notSupported();}
+	public void setLongAttribute(TorrentAttribute ta, long value) {notSupported();}
+	public void setFlag(long flag, boolean set) {notSupported();}
 }
\ No newline at end of file
