@@ -11,6 +11,7 @@
 
 package org.gudy.azureus2.ui.console;
 
+import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
@@ -31,6 +32,11 @@ import java.util.Map;
 import java.util.Properties;
 import java.util.Vector;
 
+import org.apache.log4j.Appender;
+import org.apache.log4j.ConsoleAppender;
+import org.apache.log4j.Logger;
+import org.apache.log4j.PatternLayout;
+import org.apache.log4j.varia.DenyAllFilter;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.download.DownloadManager;
 import org.gudy.azureus2.core3.download.DownloadManagerState;
@@ -40,6 +46,7 @@ import org.gudy.azureus2.core3.torrentdownloader.TorrentDownloader;
 import org.gudy.azureus2.core3.torrentdownloader.TorrentDownloaderCallBackInterface;
 import org.gudy.azureus2.core3.torrentdownloader.TorrentDownloaderFactory;
 import org.gudy.azureus2.core3.torrentdownloader.impl.TorrentDownloaderManager;
+import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.plugins.PluginException;
 import org.gudy.azureus2.plugins.PluginInterface;
 import org.gudy.azureus2.plugins.PluginManager;
@@ -50,25 +57,7 @@ import org.gudy.azureus2.plugins.update.UpdateCheckInstance;
 import org.gudy.azureus2.plugins.update.UpdateCheckInstanceListener;
 import org.gudy.azureus2.plugins.update.UpdateManager;
 import org.gudy.azureus2.ui.common.UIConst;
-import org.gudy.azureus2.ui.console.commands.AddFind;
-import org.gudy.azureus2.ui.console.commands.Priority;
-import org.gudy.azureus2.ui.console.commands.Alias;
-import org.gudy.azureus2.ui.console.commands.Hack;
-import org.gudy.azureus2.ui.console.commands.IConsoleCommand;
-import org.gudy.azureus2.ui.console.commands.Log;
-import org.gudy.azureus2.ui.console.commands.Move;
-import org.gudy.azureus2.ui.console.commands.Set;
-import org.gudy.azureus2.ui.console.commands.Share;
-import org.gudy.azureus2.ui.console.commands.Show;
-import org.gudy.azureus2.ui.console.commands.TorrentCheck;
-import org.gudy.azureus2.ui.console.commands.TorrentForceStart;
-import org.gudy.azureus2.ui.console.commands.TorrentHost;
-import org.gudy.azureus2.ui.console.commands.TorrentPublish;
-import org.gudy.azureus2.ui.console.commands.TorrentQueue;
-import org.gudy.azureus2.ui.console.commands.TorrentRemove;
-import org.gudy.azureus2.ui.console.commands.TorrentStart;
-import org.gudy.azureus2.ui.console.commands.TorrentStop;
-import org.gudy.azureus2.ui.console.commands.XML;
+import org.gudy.azureus2.ui.console.commands.*;
 import org.gudy.azureus2.update.CorePatchChecker;
 import org.gudy.azureus2.update.UpdaterUpdateChecker;
 
@@ -81,7 +70,6 @@ public class ConsoleInput extends Thread {
 
 	private static final String ALIASES_CONFIG_FILE = "console.aliases.properties";
 	public final AzureusCore azureus_core;
-	public final GlobalManager gm;
 	public final PrintStream out;
 	public final List torrents = new ArrayList();
 	public File[] adds = null;
@@ -132,8 +120,7 @@ public class ConsoleInput extends Thread {
 		super("Console Input: " + con);
 		this.out = _out;
 		this.azureus_core	= _azureus_core;
-		this.userProfile = profile;
-		this.gm  			= _azureus_core.getGlobalManager();
+		this.userProfile 	= profile;
 		this.controlling = _controlling.booleanValue();
 		this.br = new CommandReader(_in);
 		
@@ -146,6 +133,33 @@ public class ConsoleInput extends Thread {
 		start();
 		System.out.println( "ConsoleInput: started OK" );
 	}
+	
+		/**
+		 * Simple constructor to allow other components to use the console commands such as "set"
+		 * @param con
+		 * @param _azureus_core
+		 * @param _out
+		 */
+	
+	public ConsoleInput(AzureusCore _azureus_core, PrintStream _out )
+	{
+		super( "" );
+		this.out = _out;
+		this.azureus_core	= _azureus_core;
+		this.userProfile 	= UserProfile.DEFAULT_USER_PROFILE;
+		this.controlling 	= false;
+		this.br 			= new CommandReader( new InputStreamReader( new ByteArrayInputStream(new byte[0])));
+				
+		if (Logger.getRootLogger().getAppender("ConsoleAppender")==null) {
+	      Appender app;
+	      app = new ConsoleAppender(new PatternLayout(PatternLayout.TTCC_CONVERSION_PATTERN));
+	      app.setName("ConsoleAppender");
+	      app.addFilter( new DenyAllFilter() );  //'log off' by default
+	      Logger.getRootLogger().addAppender(app);
+	    }
+		
+		initialise();
+	}
 
 	protected void initialise() {
 		registerAlertHandler();
@@ -170,7 +184,7 @@ public class ConsoleInput extends Thread {
 	 */
 	public void downloadTorrent( String filename, String outputDir )
 	{
-		DownloadManager manager = gm.addDownloadManager(filename, outputDir);
+		DownloadManager manager = azureus_core.getGlobalManager().addDownloadManager(filename, outputDir);
 		manager.getDownloadState().setAttribute(DownloadManagerState.AT_USER, getUserProfile().getUsername());
 	}
 	
@@ -236,7 +250,7 @@ public class ConsoleInput extends Thread {
 	protected void
 	registerAlertHandler()
 	{
-		Logger.addListener(new ILogAlertListener() {
+		org.gudy.azureus2.core3.logging.Logger.addListener(new ILogAlertListener() {
 			private java.util.Set	history = Collections.synchronizedSet( new HashSet());
 			
 			public void alertRaised(LogAlert alert) {
@@ -270,6 +284,7 @@ public class ConsoleInput extends Thread {
 		registerCommand(new TorrentHost());
 		registerCommand(new TorrentPublish());
 		registerCommand(new TorrentForceStart());
+		registerCommand(new TorrentLog());
 		registerCommand(new Log());
 		registerCommand(new Move());
 		registerCommand(new Share());
@@ -449,8 +464,15 @@ public class ConsoleInput extends Thread {
 			try {
 				if ( !ci.controlling ){
 					
-					ci.out.close();
+						// we never want to close System.out - could be remote command exec
+					
+					if ( ci.out != System.out ){
 						
+						ci.out.println( "Logged out" );
+						
+						ci.out.close();
+					}
+					
 					ci.br.close();
 				}
 				
@@ -500,7 +522,7 @@ public class ConsoleInput extends Thread {
 				return true;
 			} catch (Exception e)
 			{
-				out.println("> Invoking Command '"+command+"' failed. Exception: "+e.getMessage());
+				out.println("> Invoking Command '"+command+"' failed. Exception: "+ Debug.getNestedExceptionMessage(e));
 				return false;
 			}
 		} else
@@ -709,4 +731,16 @@ public class ConsoleInput extends Thread {
 		checker.start();
 		
 	}
+	
+	public AzureusCore
+	getCore()
+	{
+		return( azureus_core );
+	}
+	
+	public GlobalManager
+	getGlobalManager()
+	{
+		return( azureus_core.getGlobalManager());
+	}
 }
