@@ -24,9 +24,7 @@
 
 package org.gudy.azureus2.ui.swt;
 
-import java.io.File;
-import java.io.FileFilter;
-import java.io.IOException;
+import java.io.*;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Iterator;
@@ -42,6 +40,7 @@ import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.layout.RowLayout;
 import org.eclipse.swt.widgets.*;
+
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.StringIterator;
 import org.gudy.azureus2.core3.config.StringList;
@@ -61,6 +60,7 @@ import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.ui.swt.components.shell.ShellFactory;
 import org.gudy.azureus2.ui.swt.mainwindow.Colors;
 import org.gudy.azureus2.ui.swt.mainwindow.TorrentOpener;
+import org.gudy.azureus2.ui.swt.shells.MessageBoxShell;
 import org.gudy.azureus2.ui.swt.shells.MessageSlideShell;
 
 import com.aelitis.azureus.core.AzureusCore;
@@ -76,10 +76,21 @@ import com.aelitis.azureus.core.AzureusCoreFactory;
 public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 {
 
-	/** Don't allow disabling of downloading for files smaller than this */
-	private final static int MIN_NODOWNLOAD_SIZE = 1024 * 1024;
+	/**
+	 * We prevent users from unselecting small files to prevent them missing
+	 * out "signature" files from trackers (.nfo file, readme file etc)
+	 * 
+	 * We define two constants to control this - one defines what a small file
+	 * is, and the other defines whether we believe a torrent has signature
+	 * files or not - we do this by seeing how many small files the torrent has.
+	 * 
+	 * If it has several small files, then it would be silly for us to assume
+	 * that the torrent consists of multiple signature files.
+	 */
+	private final static int MIN_NODOWNLOAD_SIZE = 64 * 1024;
+	private final static int MAX_NODOWNLOAD_COUNT = 3;
 
-	private final static int MIN_BUTTON_HEIGHT = Utils.isGTK ? -1 : 24;
+	private final static int MIN_BUTTON_HEIGHT = Constants.isWindows ? 24 : -1;
 
 	private final static String PARAM_DEFSAVEPATH = "Default save path";
 
@@ -98,12 +109,16 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 	private final static int QUEUELOCATION_BOTTOM = 1;
 
 	private final static String[] startModes = {
-			"queued",
-			"stopped",
-			"forceStarted",
-			"seeding" };
+		"queued",
+		"stopped",
+		"forceStarted",
+		"seeding"
+	};
 
-	private final static String[] queueLocations = { "first", "last" };
+	private final static String[] queueLocations = {
+		"first",
+		"last"
+	};
 
 	/** Only one window, since it can handle multiple torrents */
 	private static OpenTorrentWindow stOpenTorrentWindow = null;
@@ -159,7 +174,17 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 
 	private static final AzureusCore core;
 
+	private StringList dirList;
+
+	private Label dataFileTableLabel;
+
 	static {
+		if (!AzureusCoreFactory.isCoreAvailable()) {
+			// This should be only called in test mode
+			AzureusCore core2 = AzureusCoreFactory.create();
+			core2.start();
+		}
+
 		core = AzureusCoreFactory.getSingleton();
 	}
 
@@ -175,8 +200,7 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 	 */
 	public synchronized static final void invoke(Shell parent, GlobalManager gm,
 			String sPathOfFilesToOpen, String[] sFilesToOpen,
-			boolean bDefaultStopped, boolean bForSeeding, boolean bPopupOpenURL)
-	{
+			boolean bDefaultStopped, boolean bForSeeding, boolean bPopupOpenURL) {
 
 		String saveSilentlyDir = null;
 
@@ -216,6 +240,11 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 
 			if (saveSilentlyDir != null) {
 				openTorrentWindow.sDestDir = saveSilentlyDir;
+				for (int i = 0; i < openTorrentWindow.torrentList.size(); i++) {
+					final TorrentInfo info = (TorrentInfo) openTorrentWindow.torrentList.get(i);
+					info.renameDuplicates();
+				}
+
 				openTorrentWindow.openTorrents();
 				openTorrentWindow.close(true, false);
 			}
@@ -228,14 +257,12 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 	 * @param gm
 	 */
 	public synchronized static final void invoke(final Shell parent,
-			GlobalManager gm)
-	{
+			GlobalManager gm) {
 		invoke(parent, gm, null, null, false, false, false);
 	}
 
 	public synchronized static final void invokeURLPopup(final Shell parent,
-			GlobalManager gm)
-	{
+			GlobalManager gm) {
 		invoke(parent, gm, null, null, false, false, true);
 	}
 
@@ -249,7 +276,7 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 			boolean bOpenWindow) {
 		this.gm = gm;
 
-		sDestDir = COConfigurationManager.getStringParameter(PARAM_DEFSAVEPATH, "");
+		sDestDir = COConfigurationManager.getStringParameter(PARAM_DEFSAVEPATH);
 
 		if (bOpenWindow)
 			openWindow(parent);
@@ -308,13 +335,15 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 			public void handleEvent(Event event) {
 				FileDialog fDialog = new FileDialog(shell, SWT.OPEN | SWT.MULTI);
 				fDialog.setFilterExtensions(new String[] {
-						"*.torrent",
-						"*.tor",
-						Constants.FILE_WILDCARD });
+					"*.torrent",
+					"*.tor",
+					Constants.FILE_WILDCARD
+				});
 				fDialog.setFilterNames(new String[] {
-						"*.torrent",
-						"*.tor",
-						Constants.FILE_WILDCARD });
+					"*.torrent",
+					"*.tor",
+					Constants.FILE_WILDCARD
+				});
 				fDialog.setFilterPath(TorrentOpener.getFilterPathTorrent());
 				fDialog.setText(MessageText.getString("MainWindow.dialog.choose.file"));
 				String fileName = TorrentOpener.setFilterPathTorrent(fDialog.open());
@@ -470,7 +499,16 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 
 				tableTorrents.clearAll();
 
-				updateOKButton();
+				checkSeedingMode();
+
+				File file = new File(sDestDir);
+				if (!file.isDirectory()) {
+					cmbDataDir.setBackground(Colors.colorErrorBG);
+				} else {
+					cmbDataDir.setBackground(null);
+				}
+				cmbDataDir.redraw();
+				cmbDataDir.update();
 			}
 		});
 
@@ -478,7 +516,7 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 		if (sDestDir != null && sDestDir.length() > 0) {
 			cmbDataDir.add(sDestDir);
 		}
-		final StringList dirList = COConfigurationManager.getStringListParameter("saveTo_list");
+		dirList = COConfigurationManager.getStringListParameter("saveTo_list");
 		StringIterator iter = dirList.iterator();
 		while (iter.hasNext()) {
 			String s = iter.next();
@@ -499,7 +537,7 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 					File f = new File(sDefPath);
 
 					if (!f.exists()) {
-						f.mkdirs();
+						FileUtil.mkdirs(f);
 					}
 				}
 
@@ -511,12 +549,17 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 				if (sSavePath != null) {
 					cmbDataDir.setText(sSavePath);
 				}
+
+				File file = new File(sDestDir);
+				if (!file.isDirectory()) {
+					cmbDataDir.setBackground(Colors.colorErrorBG);
+				} else {
+					cmbDataDir.setBackground(null);
+				}
 			}
 		});
 
 		gridData = new GridData(GridData.FILL_HORIZONTAL);
-		if (Constants.isOSX)
-			gridData.heightHint = cSaveTo.computeSize(SWT.DEFAULT, SWT.DEFAULT).y - 9;
 		cSaveTo.setLayoutData(gridData);
 
 		// File List
@@ -547,73 +590,10 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 		shell.setDefaultButton(ok);
 		ok.addListener(SWT.Selection, new Listener() {
 			public void handleEvent(Event event) {
-				String sDefaultPath = COConfigurationManager.getStringParameter(PARAM_DEFSAVEPATH);
-				if (!sDestDir.equals(sDefaultPath)) {
-					// Move sDestDir to top of list
-
-					// First, check to see if sDestDir is already in the list
-					File fDestDir = new File(sDestDir);
-					int iDirPos = -1;
-					for (int i = 0; i < dirList.size(); i++) {
-						String sDirName = dirList.get(i);
-						File dir = new File(sDirName);
-						if (dir.equals(fDestDir)) {
-							iDirPos = i;
-							break;
-						}
-					}
-
-					// If already in list, remove it
-					if (iDirPos > 0 && iDirPos < dirList.size())
-						dirList.remove(iDirPos);
-
-					// and add it to the top
-					dirList.add(0, sDestDir);
-
-					// Limit
-					if (dirList.size() > 15)
-						dirList.remove(dirList.size() - 1);
-
-					// Temporary list cleanup
-					try {
-						for (int j = 0; j < dirList.size(); j++) {
-							File dirJ = new File(dirList.get(j));
-							for (int i = 0; i < dirList.size(); i++) {
-								try {
-									if (i == j)
-										continue;
-
-									File dirI = new File(dirList.get(i));
-
-									if (dirI.equals(dirJ)) {
-										dirList.remove(i);
-										// dirList shifted up, fix indexes
-										if (j > i)
-											j--;
-										i--;
-									}
-								} catch (Exception e) {
-									// Ignore
-								}
-							}
-						}
-					} catch (Exception e) {
-						// Ignore
-					}
-
-					COConfigurationManager.setParameter("saveTo_list", dirList);
-					COConfigurationManager.save();
-				}
-
-				if (COConfigurationManager.getBooleanParameter("DefaultDir.AutoUpdate")
-						&& !COConfigurationManager.getBooleanParameter("Use default data dir"))
-					COConfigurationManager.setParameter(PARAM_DEFSAVEPATH, sDestDir);
-
-				openTorrents();
-				close(true, false);
+				okPressed();
 			}
 		});
-		updateOKButton();
+		checkSeedingMode();
 
 		Button cancel = new Button(cArea, SWT.PUSH);
 		Messages.setLanguageText(cancel, "Button.cancel");
@@ -677,7 +657,152 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 		resizeTables(3);
 		shell.open();
 
-		cSaveTo.setFocus();
+		if (cSaveTo != null && !cSaveTo.isDisposed()) {
+			cSaveTo.setFocus();
+		}
+	}
+
+	protected void okPressed() {
+		if (bClosed) {
+			return;
+		}
+
+		if ((torrentList.size() == 0 && downloaders.size() == 0)) {
+			close(true, false);
+			return;
+		}
+
+		File file = new File(cmbDataDir.getText());
+		
+		File fileDefSavePath = new File(
+				COConfigurationManager.getStringParameter(PARAM_DEFSAVEPATH));
+
+		if (file.equals(fileDefSavePath) && !fileDefSavePath.isDirectory()) {
+			FileUtil.mkdirs(fileDefSavePath);
+		}
+
+		if (cmbDataDir.getText().length() == 0 || !file.isDirectory()) {
+			Utils.openMessageBox(shellForChildren, SWT.OK | SWT.ICON_ERROR,
+					"OpenTorrentWindow.mb.noGlobalDestDir",
+					new String[] { file.toString()
+					});
+			cmbDataDir.setFocus();
+			return;
+		}
+
+
+		String sExistingFiles = "";
+		int iNumExistingFiles = 0;
+		for (int i = 0; i < torrentList.size(); i++) {
+			TorrentInfo info = (TorrentInfo) torrentList.get(i);
+
+			file = new File(info.sDestDir);
+			if (!file.isDirectory() && !FileUtil.mkdirs(file)) {
+				Utils.openMessageBox(shellForChildren, SWT.OK | SWT.ICON_ERROR,
+						"OpenTorrentWindow.mb.noDestDir", new String[] {
+							file.toString(),
+							info.getTorrentName()
+						});
+				return;
+			}
+
+			if (!info.isValid) {
+				Utils.openMessageBox(shellForChildren, SWT.OK | SWT.ICON_ERROR,
+						"OpenTorrentWindow.mb.notValid",
+						new String[] { info.getTorrentName()
+						});
+				return;
+			}
+			
+			TorrentFileInfo[] files = info.getFiles();
+			for (int j = 0; j < files.length; j++) {
+				TorrentFileInfo fileInfo = files[j];
+				if (fileInfo.getDestFile().exists()) {
+					sExistingFiles += fileInfo.sFileName + " - " + info.getTorrentName()
+							+ "\n";
+					iNumExistingFiles++;
+					if (iNumExistingFiles > 5) {
+						// this has the potential effect of adding 5 files from the first 
+						// torrent and then 1 file from each of the remaining torrents
+						break;
+					}
+				}
+			}
+		}
+		
+		if (sExistingFiles.length() > 0) {
+			if (Utils.openMessageBox(shellForChildren, SWT.OK | SWT.CANCEL
+					| SWT.ICON_WARNING, "OpenTorrentWindow.mb.existingFiles",
+					new String[] { sExistingFiles
+					}) != SWT.OK) {
+				return;
+			}
+		}
+
+		String sDefaultPath = COConfigurationManager.getStringParameter(PARAM_DEFSAVEPATH);
+		if (!sDestDir.equals(sDefaultPath)) {
+			// Move sDestDir to top of list
+
+			// First, check to see if sDestDir is already in the list
+			File fDestDir = new File(sDestDir);
+			int iDirPos = -1;
+			for (int i = 0; i < dirList.size(); i++) {
+				String sDirName = dirList.get(i);
+				File dir = new File(sDirName);
+				if (dir.equals(fDestDir)) {
+					iDirPos = i;
+					break;
+				}
+			}
+
+			// If already in list, remove it
+			if (iDirPos > 0 && iDirPos < dirList.size())
+				dirList.remove(iDirPos);
+
+			// and add it to the top
+			dirList.add(0, sDestDir);
+
+			// Limit
+			if (dirList.size() > 15)
+				dirList.remove(dirList.size() - 1);
+
+			// Temporary list cleanup
+			try {
+				for (int j = 0; j < dirList.size(); j++) {
+					File dirJ = new File(dirList.get(j));
+					for (int i = 0; i < dirList.size(); i++) {
+						try {
+							if (i == j)
+								continue;
+
+							File dirI = new File(dirList.get(i));
+
+							if (dirI.equals(dirJ)) {
+								dirList.remove(i);
+								// dirList shifted up, fix indexes
+								if (j > i)
+									j--;
+								i--;
+							}
+						} catch (Exception e) {
+							// Ignore
+						}
+					}
+				}
+			} catch (Exception e) {
+				// Ignore
+			}
+
+			COConfigurationManager.setParameter("saveTo_list", dirList);
+			COConfigurationManager.save();
+		}
+
+		if (COConfigurationManager.getBooleanParameter("DefaultDir.AutoUpdate")
+				&& !COConfigurationManager.getBooleanParameter("Use default data dir"))
+			COConfigurationManager.setParameter(PARAM_DEFSAVEPATH, sDestDir);
+
+		openTorrents();
+		close(true, false);
 	}
 
 	/**
@@ -739,7 +864,7 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 		if (cmbStartMode == null)
 			return;
 
-		int[] indexes = tableTorrents.getSelectionIndices();
+		int[] indexes = tableTorrents.getSelectionIndices(); 
 		String[] sItemsText = new String[startModes.length];
 		int iMaxMatches = 0;
 		int iIndexToSelect = getDefaultStartMode();
@@ -761,8 +886,9 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 			if (iMatches > 0)
 				sText += " "
 						+ MessageText.getString("OpenTorrentWindow.xOfTotal", new String[] {
-								Integer.toString(iMatches),
-								Integer.toString(indexes.length) });
+							Integer.toString(iMatches),
+							Integer.toString(indexes.length)
+						});
 			sItemsText[i] = sText;
 		}
 		cmbStartMode.setItems(sItemsText);
@@ -796,8 +922,9 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 			if (iMatches > 0)
 				sText += " "
 						+ MessageText.getString("OpenTorrentWindow.xOfTotal", new String[] {
-								Integer.toString(iMatches),
-								Integer.toString(indexes.length) });
+							Integer.toString(iMatches),
+							Integer.toString(indexes.length)
+						});
 			sItemsText[i] = sText;
 		}
 		cmbQueueLocation.setItems(sItemsText);
@@ -809,8 +936,10 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 	 * @param keyListener
 	 */
 	private void setPasteKeyListener(Control c, KeyListener keyListener) {
-		if (!(c instanceof Text || c instanceof Combo))
+		if (!(c instanceof Text) && !(c instanceof Combo)
+				&& !(c instanceof Composite) || (c instanceof Table)) {
 			c.addKeyListener(keyListener);
+		}
 		if (c instanceof Composite) {
 			Control[] controls = ((Composite) c).getChildren();
 			for (int i = 0; i < controls.length; i++) {
@@ -915,12 +1044,13 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 				}
 
 				item.setText(new String[] {
-						info.getTorrentName(),
-						sRealDestDir,
-						MessageText.getString("OpenTorrentWindow.startMode."
-								+ startModes[info.iStartID]),
-						MessageText.getString("OpenTorrentWindow.addPosition."
-								+ queueLocations[info.iQueueLocation]) });
+					info.getTorrentName(),
+					sRealDestDir,
+					MessageText.getString("OpenTorrentWindow.startMode."
+							+ startModes[info.iStartID]),
+					MessageText.getString("OpenTorrentWindow.addPosition."
+							+ queueLocations[info.iQueueLocation])
+				});
 				if (!info.isValid) {
 					item.setForeground(Colors.red);
 					Font font = item.getFont();
@@ -952,6 +1082,7 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 
 				dataFileTable.setItemCount(dataFiles.size());
 				dataFileTable.clearAll();
+				updateSize();
 				resizeTables(2);
 			}
 		});
@@ -991,7 +1122,7 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 					Long l = (Long) e.widget.getData("Value");
 					if (l != null) {
 						setSelectedStartMode(l.intValue());
-						updateOKButton();
+						checkSeedingMode();
 					}
 				}
 			});
@@ -1042,7 +1173,8 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 
 					TorrentFileInfo[] files = info.getFiles();
 					if (files.length == 1) {
-						chageFileDestination(new int[] { 0 });
+						chageFileDestination(new int[] { 0
+						});
 					} else {
 						DirectoryDialog dDialog = new DirectoryDialog(shellForChildren,
 								SWT.SYSTEM_MODAL);
@@ -1068,7 +1200,7 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 
 				} // for i
 
-				updateOKButton();
+				checkSeedingMode();
 			}
 		});
 
@@ -1173,10 +1305,59 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 			TorrentInfo info = (TorrentInfo) torrentList.get(indices[i]);
 			info.iStartID = iStartID;
 		}
+
+		checkSeedingMode();
 		updateStartModeCombo();
 		tableTorrents.clear(indices);
 	}
 
+	private void checkSeedingMode() {
+		// Check for seeding
+		for (int i = 0; i < torrentList.size(); i++) {
+			boolean bTorrentValid = true;
+			TorrentInfo info = (TorrentInfo) torrentList.get(i);
+
+			if (info.iStartID == STARTMODE_SEEDING) {
+				// check if all selected files exist
+				TorrentFileInfo[] files = info.getFiles();
+				for (int j = 0; j < files.length; j++) {
+					TorrentFileInfo fileInfo = files[j];
+					if (!fileInfo.bDownload)
+						continue;
+
+					String sFullPath;
+					if (fileInfo.sDestFileName == null) {
+						File f = new File(info.sDestDir, fileInfo.sFullFileName);
+						sFullPath = f.getAbsolutePath();
+					} else {
+						sFullPath = fileInfo.sDestFileName;
+					}
+
+					File file = new File(sFullPath);
+					if (!file.exists()) {
+						fileInfo.isValid = false;
+						bTorrentValid = false;
+					} else if (!fileInfo.isValid) {
+						fileInfo.isValid = true;
+					}
+				}
+			}
+
+			info.isValid = bTorrentValid;
+		}
+
+		Utils.execSWTThread(new AERunnable() {
+			public void runSupport() {
+				if (tableTorrents != null && !tableTorrents.isDisposed()) {
+					tableTorrents.clearAll();
+				}
+				if (dataFileTable != null && !dataFileTable.isDisposed()) {
+					dataFileTable.clearAll();
+				}
+			}
+		});
+	}
+	
 	private void deleteSelected(Table table, ArrayList list) {
 		int[] indexes = table.getSelectionIndices();
 		Arrays.sort(indexes);
@@ -1231,9 +1412,10 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 				final TorrentFileInfo file = (TorrentFileInfo) dataFiles.get(index);
 
 				item.setText(new String[] {
-						file.sFileName,
-						file.sDestFileName,
-						DisplayFormatters.formatByteCountToKiBEtc(file.lSize) });
+					file.sFileName,
+					file.sDestFileName,
+					DisplayFormatters.formatByteCountToKiBEtc(file.lSize)
+				});
 				if (!file.isValid) {
 					item.setForeground(Colors.red);
 					Font font = item.getFont();
@@ -1265,6 +1447,8 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 						item.setChecked(true);
 					else
 						file.bDownload = item.getChecked();
+
+					updateSize();
 				}
 			}
 
@@ -1287,8 +1471,19 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 			}
 		});
 
-		Composite cButtons = new Composite(cArea, SWT.NONE);
+		Composite cBottomArea = new Composite(cArea, SWT.NONE);
+		GridLayout gLayout = new GridLayout();
+		gLayout.marginHeight = 0;
+		gLayout.marginWidth = 0;
+		gLayout.numColumns = 2;
+		cBottomArea.setLayout(gLayout);
+
+		gridData = new GridData(GridData.FILL_HORIZONTAL);
+		cBottomArea.setLayoutData(gridData);
+
+		Composite cButtons = new Composite(cBottomArea, SWT.NONE);
 		RowLayout rLayout = new RowLayout(SWT.HORIZONTAL);
+		rLayout.wrap = false;
 		rLayout.marginBottom = 0;
 		rLayout.marginLeft = 0;
 		rLayout.marginRight = 0;
@@ -1303,9 +1498,6 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 			}
 		});
 
-		Utils.setGridData(cButtons, GridData.FILL_HORIZONTAL, btnSelectAll,
-				MIN_BUTTON_HEIGHT);
-
 		Button btnMarkSelected = new Button(cButtons, SWT.PUSH);
 		Messages.setLanguageText(btnMarkSelected, "Button.markSelected");
 		btnMarkSelected.addListener(SWT.Selection, new Listener() {
@@ -1316,6 +1508,7 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 					file.bDownload = true;
 				}
 				dataFileTable.clearAll();
+				updateSize();
 			}
 		});
 
@@ -1330,8 +1523,14 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 						file.bDownload = false;
 				}
 				dataFileTable.clearAll();
+				updateSize();
 			}
 		});
+
+		dataFileTableLabel = new Label(cBottomArea, SWT.WRAP);
+		dataFileTableLabel.setAlignment(SWT.RIGHT);
+		gridData = new GridData(GridData.FILL_BOTH);
+		dataFileTableLabel.setLayoutData(gridData);
 	}
 
 	/**
@@ -1380,15 +1579,16 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 				else {
 					Utils.openMessageBox(shellForChildren, SWT.OK,
 							"OpenTorrentWindow.mb.badSize", new String[] {
-									file.getName(),
-									fileInfo.sFullFileName });
+								file.getName(),
+								fileInfo.sFullFileName
+							});
 				}
 			} else {
 				fileInfo.sDestFileName = sNewName;
 			}
 		} // for i
 
-		updateOKButton();
+		checkSeedingMode();
 	}
 
 	/**
@@ -1409,7 +1609,12 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 		// no use checking the whole clipboard (which may be megabytes)
 		final int MAX_CONSECUTIVE_NONTORRENT_LINES = 100;
 
-		final String[] splitters = { "\r\n", "\n", "\r", "\t" };
+		final String[] splitters = {
+			"\r\n",
+			"\n",
+			"\r",
+			"\t"
+		};
 
 		for (int i = 0; i < splitters.length; i++)
 			if (sClipText.indexOf(splitters[i]) >= 0) {
@@ -1418,13 +1623,18 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 			}
 
 		if (lines == null)
-			lines = new String[] { sClipText };
+			lines = new String[] { sClipText
+			};
 
 		// Check if URL, 20 byte hash, Dir, or file
 		for (int i = 0; i < lines.length; i++) {
 			String line = lines[i].trim();
 			if (line.startsWith("\"") && line.endsWith("\"")) {
-				line = line.substring(1, line.length() - 2);
+				if (line.length() < 3) {
+					line = "";
+				} else {
+					line = line.substring(1, line.length() - 2);
+				}
 			}
 
 			boolean ok;
@@ -1510,21 +1720,24 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 			if (sTorrentFilenames[i] == null || sTorrentFilenames[i] == "")
 				continue;
 
-			// Process URL
-			String sURL = UrlUtils.parseTextForURL(sTorrentFilenames[i], true);
-			if (sURL != null) {
-				if (COConfigurationManager.getBooleanParameter("Add URL Silently"))
-					new FileDownloadWindow(core, shellForChildren, sURL, null, this);
-				else
-					new OpenUrlWindow(core, shellForChildren, sURL, null, this);
-				numAdded++;
-				continue;
-			}
-
 			// Process File
 			String sFileName = ((sTorrentFilePath == null) ? "" : sTorrentFilePath)
 					+ sTorrentFilenames[i];
 
+			if (!new File(sFileName).exists()) {
+				// Process URL
+				String sURL = UrlUtils.parseTextForURL(sTorrentFilenames[i], true);
+				if (sURL != null) {
+					if (COConfigurationManager.getBooleanParameter("Add URL Silently")) {
+						new FileDownloadWindow(core, shellForChildren, sURL, null, this);
+					} else {
+						new OpenUrlWindow(core, shellForChildren, sURL, null, this);
+					}
+					numAdded++;
+					continue;
+				}
+			}
+
 			if (addTorrent(sFileName, sFileName) != null)
 				numAdded++;
 		}
@@ -1540,14 +1753,13 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 			tableTorrents.notifyListeners(SWT.Selection, new Event());
 
 			resizeTables(1);
-			updateOKButton();
+			checkSeedingMode();
 		}
 		return numAdded;
 	}
 
 	private TorrentInfo addTorrent(String sFileName,
-			final String sOriginatingLocation)
-	{
+			final String sOriginatingLocation) {
 		TorrentInfo info = null;
 		TOTorrent torrent = null;
 		File torrentFile;
@@ -1564,13 +1776,15 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 						if (shell == null)
 							new MessageSlideShell(Display.getCurrent(), SWT.ICON_ERROR,
 									"OpenTorrentWindow.mb.openError", "", new String[] {
-											sOriginatingLocation,
-											"Not a File" });
+										sOriginatingLocation,
+										"Not a File"
+									});
 						else
 							Utils.openMessageBox(shell, SWT.OK,
 									"OpenTorrentWindow.mb.openError", new String[] {
-											sOriginatingLocation,
-											"Not a File" });
+										sOriginatingLocation,
+										"Not a File"
+									});
 					}
 				});
 				return null;
@@ -1597,12 +1811,16 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 					if (shell == null)
 						new MessageSlideShell(Display.getCurrent(), SWT.ICON_ERROR,
 								"OpenTorrentWindow.mb.openError", Debug.getStackTrace(e),
-								new String[] { sOriginatingLocation, e.getMessage() });
+								new String[] {
+									sOriginatingLocation,
+									e.getMessage()
+								});
 					else
 						Utils.openMessageBox(shell, SWT.OK,
 								"OpenTorrentWindow.mb.openError", new String[] {
-										sOriginatingLocation,
-										e.getMessage() });
+									sOriginatingLocation,
+									e.getMessage()
+								});
 				}
 			});
 
@@ -1620,8 +1838,13 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 					try {
 						TorrentInfo existing = (TorrentInfo) torrentList.get(i);
 						if (existing.torrent.getHashWrapper().equals(hash)) {
-							sExistingName = existing.sOriginatingLocation;
-							break;
+							//sExistingName = existing.sOriginatingLocation;
+
+							// Exit without warning when it already exists in list
+							if (bDeleteFileOnCancel)
+								torrentFile.delete();
+
+							return null;
 						}
 					} catch (Exception e) {
 					}
@@ -1653,13 +1876,15 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 					if (shell == null)
 						new MessageSlideShell(Display.getCurrent(), SWT.ICON_ERROR,
 								"OpenTorrentWindow.mb.alreadyExists", null, new String[] {
-										sOriginatingLocation,
-										sfExistingName });
+									sOriginatingLocation,
+									sfExistingName
+								});
 					else
 						Utils.openMessageBox(shell, SWT.OK,
 								"OpenTorrentWindow.mb.alreadyExists", new String[] {
-										sOriginatingLocation,
-										sfExistingName });
+									sOriginatingLocation,
+									sfExistingName
+								});
 				}
 			});
 
@@ -1670,66 +1895,6 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 		return info;
 	}
 
-	private void updateOKButton() {
-		if (ok == null || bClosed)
-			return;
-
-		boolean bEnable = torrentList != null && torrentList.size() > 0
-				&& downloaders.size() == 0;
-
-		File file = new File(cmbDataDir.getText());
-		if (!file.isDirectory()) {
-			cmbDataDir.setBackground(Colors.colorErrorBG);
-		} else {
-			cmbDataDir.setBackground(null);
-		}
-
-		// Check for seeding
-		for (int i = 0; i < torrentList.size(); i++) {
-			boolean bTorrentValid = true;
-			TorrentInfo info = (TorrentInfo) torrentList.get(i);
-
-			file = new File(info.sDestDir);
-			if (!file.isDirectory()) {
-				bEnable = false;
-			}
-
-			if (info.iStartID == STARTMODE_SEEDING) {
-				// check if all selected files exist
-				TorrentFileInfo[] files = info.getFiles();
-				for (int j = 0; j < files.length; j++) {
-					TorrentFileInfo fileInfo = files[j];
-					if (!fileInfo.bDownload)
-						continue;
-
-					String sFullPath;
-					if (fileInfo.sDestFileName == null) {
-						File f = new File(info.sDestDir, fileInfo.sFullFileName);
-						sFullPath = f.getAbsolutePath();
-					} else {
-						sFullPath = fileInfo.sDestFileName;
-					}
-
-					file = new File(sFullPath);
-					if (!file.exists()) {
-
-						bEnable = false;
-						fileInfo.isValid = false;
-						bTorrentValid = false;
-					} else if (!fileInfo.isValid) {
-						fileInfo.isValid = true;
-					}
-				}
-			}
-
-			info.isValid = bTorrentValid;
-		}
-
-		ok.setEnabled(bEnable);
-		tableTorrents.clearAll();
-		dataFileTable.clearAll();
-	}
-
 	/**
 	 * Resize the columns of the tables to fit without horizontal scrollbar
 	 * 
@@ -1740,7 +1905,8 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 	private void resizeTables(int which) {
 		try {
 			TableColumn[] tcs;
-			if ((which & 1) > 0) {
+			if ((which & 1) > 0 && tableTorrents != null
+					&& !tableTorrents.isDisposed()) {
 				tcs = tableTorrents.getColumns();
 				int newSize = tableTorrents.getClientArea().width - 20;
 				int iLength = tcs.length;
@@ -1760,7 +1926,8 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 			}
 
 			// Adjust only first column
-			if ((which & 2) > 0) {
+			if ((which & 2) > 0 && dataFileTable != null
+					&& !dataFileTable.isDisposed()) {
 				tcs = dataFileTable.getColumns();
 				int newSize = dataFileTable.getClientArea().width - 20;
 				int iLength = tcs.length;
@@ -1823,7 +1990,10 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 										File fDest;
 										if (files[iIndex].sDestFileName != null) {
 											fDest = new File(files[iIndex].sDestFileName);
-											fileInfo.setLink(fDest);
+											// Can't use fileInfo.setLink(fDest) as it renames
+											// the existing file if there is one
+											dm.getDownloadState().setFileLink(
+													fileInfo.getFile(false), fDest);
 										} else {
 											fDest = new File(info.sDestDir,
 													files[iIndex].sFullFileName);
@@ -1857,10 +2027,16 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 				if (shell == null)
 					new MessageSlideShell(Display.getCurrent(), SWT.ICON_ERROR,
 							"OpenTorrentWindow.mb.openError", Debug.getStackTrace(e),
-							new String[] { info.sOriginatingLocation, e.getMessage() });
+							new String[] {
+								info.sOriginatingLocation,
+								e.getMessage()
+							});
 				else
 					Utils.openMessageBox(shell, SWT.OK, "OpenTorrentWindow.mb.openError",
-							new String[] { info.sOriginatingLocation, e.getMessage() });
+							new String[] {
+								info.sOriginatingLocation,
+								e.getMessage()
+							});
 			}
 		}
 
@@ -1882,6 +2058,38 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 	// TorrentDownloaderCallBackInterface
 	public void TorrentDownloaderEvent(int state, final TorrentDownloader inf) {
 		// This method is run even if the window is closed.
+		if (!inf.getDeleteFileOnCancel()
+				&& (state == TorrentDownloader.STATE_CANCELLED
+						|| state == TorrentDownloader.STATE_ERROR
+						|| state == TorrentDownloader.STATE_DUPLICATE || state == TorrentDownloader.STATE_FINISHED)) {
+			if (!downloaders.contains(inf))
+				return;
+			downloaders.remove(inf);
+
+			File file = inf.getFile();
+			String html = null;
+			if (file.exists()) {
+				try {
+					html = FileUtil.readFileAsString(file, 16384);
+				} catch (IOException e) {
+					// TODO Auto-generated catch block
+					e.printStackTrace();
+				}
+				file.delete();
+			}
+			MessageBoxShell boxShell = new MessageBoxShell(shellForChildren,
+					MessageText.getString("OpenTorrentWindow.mb.notTorrent.title"),
+					MessageText.getString("OpenTorrentWindow.mb.notTorrent.text",
+							new String[] {
+								inf.getURL(),
+								""
+							}), new String[] { MessageText.getString("Button.ok")
+					}, 0);
+			boxShell.setHtml(html);
+			boxShell.open();
+
+			return;
+		}
 
 		if (state == TorrentDownloader.STATE_INIT) {
 			downloaders.add(inf);
@@ -1919,6 +2127,11 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 					String saveSilentlyDir = getSaveSilentlyDir();
 					if (saveSilentlyDir != null) {
 						sDestDir = saveSilentlyDir;
+						for (int i = 0; i < torrentList.size(); i++) {
+							final TorrentInfo info = (TorrentInfo) torrentList.get(i);
+							info.renameDuplicates();
+						}
+
 						openTorrents();
 					}
 				}
@@ -1928,15 +2141,23 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 				|| state == TorrentDownloader.STATE_ERROR
 				|| state == TorrentDownloader.STATE_DUPLICATE) {
 			downloaders.remove(inf);
-		} else
-			return;
+		} else if (state == TorrentDownloader.STATE_DOWNLOADING) {
+			int count = inf.getLastReadCount();
+			int numRead = inf.getTotalRead();
 
-		// definitely on a different thread..
-		Utils.execSWTThread(new AERunnable() {
-			public void runSupport() {
-				updateOKButton();
+			if (!inf.getDeleteFileOnCancel() && numRead >= 16384) {
+				inf.cancel();
+			} else if (numRead == count && count > 0) {
+				final byte[] bytes = inf.getLastReadBytes();
+				if (bytes[0] != 'd') {
+					inf.setDeleteFileOnCancel(false);
+				}
 			}
-		});
+		} else {
+			return;
+		}
+
+		checkSeedingMode();
 	}
 
 	/**
@@ -2005,8 +2226,9 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 				int totalSegmentsLengths = 0;
 
 				String[][] segments = {
-						name.split("[^a-zA-Z]+"),
-						sFileName.split("[^a-zA-Z]+") };
+					name.split("[^a-zA-Z]+"),
+					sFileName.split("[^a-zA-Z]+")
+				};
 				List downloadManagers = gm.getDownloadManagers();
 
 				for (int x = 0; x < segments.length; x++) {
@@ -2059,9 +2281,11 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 					int iMatchLevel = (maxMatches * 100 / totalSegmentsLengths);
 					if (iMatchLevel >= 30) {
 						File f = match.getSaveLocation();
-						if (!f.isDirectory()) {
+						if (!f.isDirectory() || match.getDiskManagerFileInfo().length > 1) {
+							// don't place data within another torrent's data dir
 							f = f.getParentFile();
 						}
+
 						if (f != null && f.isDirectory()) {
 							sSmartDir = f.getAbsolutePath();
 						}
@@ -2112,6 +2336,106 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 			}
 			return true;
 		}
+
+		public boolean allFilesExist() {
+			// check if all selected files exist
+			TorrentFileInfo[] files = getFiles();
+			for (int i = 0; i < files.length; i++) {
+				TorrentFileInfo fileInfo = files[i];
+				if (!fileInfo.bDownload)
+					continue;
+
+				String sFullPath;
+				if (fileInfo.sDestFileName == null) {
+					File f = new File(sDestDir, fileInfo.sFullFileName);
+					sFullPath = f.getAbsolutePath();
+				} else {
+					sFullPath = fileInfo.sDestFileName;
+				}
+
+				File file = new File(sFullPath);
+				if (!file.exists() || file.length() != fileInfo.lSize) {
+					return false;
+				}
+			}
+			return true;
+		}
+
+		public void renameDuplicates() {
+			if (iStartID == STARTMODE_SEEDING
+					|| !COConfigurationManager.getBooleanParameter("DefaultDir.AutoSave.AutoRename")
+					|| !allFilesExist()) {
+				return;
+			}
+
+			if (!torrent.isSimpleTorrent()) {
+				if (new File(sDestDir, getTorrentName()).isDirectory()) {
+					File f;
+					int idx = 0;
+					do {
+						idx++;
+						f = new File(sDestDir, getTorrentName() + "-" + idx);
+					} while (f.isDirectory());
+
+					String sNewDir = f.getAbsolutePath();
+
+					TorrentFileInfo[] fileInfos = getFiles();
+					for (int i = 0; i < fileInfos.length; i++) {
+						TorrentFileInfo info = fileInfos[i];
+						File file = new File(sNewDir, info.sFileName);
+						info.sDestFileName = file.getAbsolutePath();
+					}
+				}
+			} else {
+				TorrentFileInfo[] fileInfos = getFiles();
+				for (int i = 0; i < fileInfos.length; i++) {
+					TorrentFileInfo info = fileInfos[i];
+
+					File file = new File(sDestDir, info.sFileName);
+					int idx = 0;
+					while (file.exists()) {
+						idx++;
+						file = new File(sDestDir, idx + "-" + info.sFileName);
+					}
+
+					info.sDestFileName = file.getAbsolutePath();
+				}
+			}
+		}
+		
+		private Boolean has_multiple_small_files = null; 
+		private boolean hasMultipleSmallFiles() {
+			TorrentFileInfo[] tfi_files = getFiles();
+			if (tfi_files.length <= MAX_NODOWNLOAD_COUNT) return false;
+			
+			int small_files_counted = 0;
+			for (int i=0; i<tfi_files.length; i++) {
+				if (tfi_files[i].lSize < MIN_NODOWNLOAD_SIZE) {
+					small_files_counted++;
+					if (small_files_counted > MAX_NODOWNLOAD_COUNT) {
+						return true;
+					}
+				}
+			}
+			
+			return false;
+		}
+			
+		// Indicates whether all files in this torrent can be deselected
+		// (if not, then it occurs on a per-file basis).
+		public boolean okToDisableAll() {
+			if (iStartID == STARTMODE_SEEDING) return true;
+			
+			// Do we have multiple small files? We'll allow all of them to
+			// be disabled if we do.
+			if (has_multiple_small_files == null) {
+				has_multiple_small_files = new Boolean(hasMultipleSmallFiles());
+			}
+			
+			// You can disable all files if there are lots of small files.
+			return has_multiple_small_files.booleanValue();
+		}
+		
 	}
 
 	/**
@@ -2170,10 +2494,17 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 
 			return new File(parent.sDestDir, sFullFileName).getParent();
 		}
+		
+		public File getDestFile() {
+			if (sDestFileName != null) {
+				return new File(sDestFileName);
+			} else {
+				return new File(sDestDir, sFullFileName);
+			}
+		}
 
 		public boolean okToDisable() {
-			return lSize >= MIN_NODOWNLOAD_SIZE
-					|| parent.iStartID == STARTMODE_SEEDING;
+			return lSize >= MIN_NODOWNLOAD_SIZE	|| parent.okToDisableAll();
 		}
 	}
 
@@ -2202,15 +2533,45 @@ public class OpenTorrentWindow implements TorrentDownloaderCallBackInterface
 		return (sDefDir == "") ? null : sDefDir;
 	}
 
+	private void updateSize() {
+		long totalSize = 0;
+		long checkedSize = 0;
+
+		for (int i = 0; i < dataFiles.size(); i++) {
+			TorrentFileInfo file = (TorrentFileInfo) dataFiles.get(i);
+
+			totalSize += file.lSize;
+
+			if (file.bDownload) {
+				checkedSize += file.lSize;
+			}
+		}
+
+		if (totalSize == 0) {
+			dataFileTableLabel.setText("");
+		} else {
+			dataFileTableLabel.setText(MessageText.getString(
+					"OpenTorrentWindow.filesInfo", new String[] {
+						DisplayFormatters.formatByteCountToKiBEtc(checkedSize),
+						DisplayFormatters.formatByteCountToKiBEtc(totalSize)
+					}));
+		}
+		dataFileTableLabel.update();
+		dataFileTableLabel.getParent().getParent().layout(true, true);
+	}
+
 	public static void main(String[] args) {
 		Display display = Display.getDefault();
 
 		ImageRepository.loadImages(display);
 
-		OpenTorrentWindow window = new OpenTorrentWindow(null, null, true);
-		while (!window.bClosed) {
+		invoke(null, core.getGlobalManager());
+		//OpenTorrentWindow window = new OpenTorrentWindow(null, null, true);
+		while (stOpenTorrentWindow != null && !stOpenTorrentWindow.bClosed) {
 			if (!display.readAndDispatch())
 				display.sleep();
 		}
+
+		core.stop();
 	}
 }
