@@ -78,10 +78,10 @@ public class TrayWindow implements GlobalManagerListener {
     label.setSize(bounds.width, bounds.height);
     minimized.setSize(bounds.width + 2, bounds.height + 2);
     screen = display.getClientArea();
-    //System.out.println(screen);
  //NICO handle macosx and multiple monitors
     if (!Constants.isOSX) {
-    	minimized.setLocation(screen.width - bounds.width - 2, screen.height - bounds.height - 2);
+    	minimized.setLocation(screen.x + screen.width - bounds.width - 2,
+					screen.y + screen.height - bounds.height - 2);
     } else {
     	minimized.setLocation(20, 20);
     }
@@ -174,6 +174,7 @@ public class TrayWindow implements GlobalManagerListener {
       }
     });
 
+    Utils.createTorrentDropTarget(minimized, false);
     globalManager = main.getGlobalManager();
     globalManager.addListener(this);
   }
@@ -200,6 +201,7 @@ public class TrayWindow implements GlobalManagerListener {
   public void refresh() {
     if (minimized.isDisposed() || !minimized.isVisible())
       return;
+
     StringBuffer toolTip = new StringBuffer();
     String separator = ""; //$NON-NLS-1$
     try{
@@ -215,9 +217,11 @@ public class TrayWindow implements GlobalManagerListener {
         toolTip.append(" -- C: ");
         toolTip.append(completed);
         toolTip.append(", D : ");
-        toolTip.append(DisplayFormatters.formatByteCountToKiBEtcPerSec(stats.getDataReceiveRate()));
-        toolTip.append(", U : ");
-        toolTip.append(DisplayFormatters.formatByteCountToKiBEtcPerSec(stats.getDataSendRate()));
+				toolTip.append(DisplayFormatters.formatDataProtByteCountToKiBEtcPerSec(
+						stats.getDataReceiveRate(), stats.getProtocolReceiveRate()));
+				toolTip.append(", U : ");
+				toolTip.append(DisplayFormatters.formatDataProtByteCountToKiBEtcPerSec(
+						stats.getDataSendRate(), stats.getProtocolSendRate()));
         separator = "\n"; //$NON-NLS-1$
       }
     }finally{
