@@ -276,8 +276,8 @@ public class URLTransfer extends ByteArrayTransfer {
 			return false;
 		}
 
-		if (UrlUtils.isURL(url.linkURL)) {
-			if (DEBUG) System.out.println("Yes, type #" + transferData.type);
+		if (UrlUtils.isURL(url.linkURL, false)) {
+			if (DEBUG) System.out.println("Yes, " + url.linkURL + " of type #" + transferData.type);
 			return true;
 		}
 
