@@ -23,7 +23,6 @@ import org.eclipse.swt.graphics.Color;
 import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.graphics.Rectangle;
-import org.eclipse.swt.widgets.Table;
 
 /**
  * @author TuxPaper
@@ -37,7 +36,9 @@ public interface BufferedTableItem
 
 	public abstract boolean setText(String text);
 
-	public abstract void setImage(Image img);
+	public abstract Image getIcon();
+
+	public abstract void setIcon(Image img);
 
 	public abstract void setRowForeground(Color color);
 
@@ -53,6 +54,11 @@ public interface BufferedTableItem
 
 	public abstract void dispose();
 
+	/**
+	 * Is the cell currently shown on the screen?
+	 * 
+	 * @return
+	 */
 	public abstract boolean isShown();
 
 	public abstract boolean needsPainting();
