@@ -22,14 +22,12 @@
 package org.gudy.azureus2.ui.swt.components;
 
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.graphics.Color;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.graphics.Point;
 import org.eclipse.swt.graphics.Rectangle;
 import org.eclipse.swt.widgets.Table;
 import org.eclipse.swt.widgets.TableItem;
-import org.gudy.azureus2.core3.util.Constants;
 import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.mainwindow.Colors;
@@ -207,6 +205,13 @@ BufferedTableRow
 		
 		item.setImage( index, new_image );	
 	}
+	
+	public Image getImage(int index) {
+		if (!checkWidget(REQUIRE_TABLEITEM_INITIALIZED))
+			return null;
+		
+		return item.getImage(index);
+	}
 
 	/**
 	 * Checks if the widget is valid
@@ -501,6 +506,11 @@ BufferedTableRow
   		e.printStackTrace();
   		return false;
   	}
+  	
+  	if (newRow.isDisposed()) {
+  		Debug.out("newRow disposed from " + Debug.getCompressedStackTrace());
+  		return false;
+  	}
 
   	if (newRow == item) {
   		if (newRow == null || newRow.getData("TableRow") == this) {
@@ -524,6 +534,7 @@ BufferedTableRow
 	  		int numColumns = table.getColumnCount();
 	  		for (int i = 0; i < numColumns; i++) {
 	        try {
+        		newRow.setImage(i, null);
         		newRow.setForeground(i, null);
 	        } catch (NoSuchMethodError e) {
 	          /* Ignore for Pre 3.0 SWT.. */
@@ -531,6 +542,7 @@ BufferedTableRow
 	  		}
 	 		} else {
 	 			newRow.setData("SD", "1");
+	 			setIconSize(ptIconSize);
 	 		}
 
    		setAlternatingBGColor(false);
