@@ -41,6 +41,8 @@ import org.gudy.azureus2.ui.swt.Messages;
 import org.gudy.azureus2.ui.swt.wizard.AbstractWizardPanel;
 import org.gudy.azureus2.ui.swt.wizard.IWizardPanel;
 
+import com.aelitis.azureus.core.networkmanager.admin.NetworkAdmin;
+
 /**
  * @author Olivier
  * 
@@ -75,17 +77,18 @@ public class NatPanel extends AbstractWizardPanel {
       //if (lowPort <= highPort && (highPort-lowPort < 10)) {
         //for (int port = lowPort; port <= highPort && bContinue; port++) {
           printMessage(MessageText.getString("configureWizard.nat.testing") + " " + TCPListenPort + " ... ");
-          int portResult = NatChecker.test(wizard.getAzureusCore(), TCPListenPort);
-          switch (portResult) {
+          NatChecker checker = new NatChecker(wizard.getAzureusCore(), NetworkAdmin.getSingleton().getDefaultBindAddress(), TCPListenPort, false );
+          switch (checker.getResult()) {
             case NatChecker.NAT_OK :
-              printMessage(MessageText.getString("configureWizard.nat.ok") + "\n");
+              String	additional_info = checker.getAdditionalInfo();
+              printMessage(MessageText.getString("configureWizard.nat.ok") + "\n" + additional_info );
               break;
             case NatChecker.NAT_KO :
-              printMessage(MessageText.getString("configureWizard.nat.ko") + "\n");
+              printMessage( "\n" + MessageText.getString("configureWizard.nat.ko") + " - " + checker.getAdditionalInfo()+".\n");
               bContinue = false;
               break;
             default :
-              printMessage(MessageText.getString("configureWizard.nat.unable") + "\n");
+              printMessage( "\n" + MessageText.getString("configureWizard.nat.unable") + ". \n(" + checker.getAdditionalInfo()+").\n");
               break;
           }
         //}
@@ -156,8 +159,12 @@ public class NatPanel extends AbstractWizardPanel {
     });
     textServerTCPListen.addListener(SWT.Modify, new Listener() {
       public void handleEvent(Event e) {
-        final int TCPListenPort = Integer.parseInt(textServerTCPListen.getText());
-        ((ConfigureWizard) wizard).serverTCPListenPort = TCPListenPort;
+      	try {
+	        final int TCPListenPort = Integer.parseInt(textServerTCPListen.getText());
+	        ((ConfigureWizard) wizard).serverTCPListenPort = TCPListenPort;
+      	} catch (NumberFormatException ex) {
+      		// ignore
+      	}
       }
     });
 
@@ -223,7 +230,7 @@ public class NatPanel extends AbstractWizardPanel {
     bCancel.setLayoutData(gridData);
     bCancel.setEnabled(false);
 
-    textResults = new StyledText(panel, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
+    textResults = new StyledText(panel, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.WRAP );
     gridData = new GridData(GridData.FILL_BOTH);
     gridData.heightHint = 70;
     gridData.horizontalSpan = 4;
@@ -268,12 +275,20 @@ public class NatPanel extends AbstractWizardPanel {
     });
   }
 
-  public void enableNext() {
+  private void enableNext() {
     Display display = wizard.getDisplay();
     if (display == null || display.isDisposed())
       return;
     display.asyncExec(new AERunnable(){
       public void runSupport() {
+      	if (bTest == null || bTest.isDisposed()) {
+      		return;
+      	}
+      	
+      	if (wizard.getCurrentPanel().equals(this)) {
+      		return;
+      	}
+
         wizard.setNextEnabled(true);
         bTest.setEnabled(true);
         bCancel.setEnabled(false);
