@@ -26,8 +26,8 @@ import java.util.Iterator;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.*;
-import org.eclipse.swt.program.Program;
 import org.eclipse.swt.widgets.*;
+
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.internat.MessageText;
@@ -44,7 +44,6 @@ import org.gudy.azureus2.ui.swt.*;
 import org.gudy.azureus2.ui.swt.components.shell.ShellManager;
 import org.gudy.azureus2.ui.swt.config.wizard.ConfigureWizard;
 import org.gudy.azureus2.ui.swt.debug.UIDebugGenerator;
-import org.gudy.azureus2.ui.swt.donations.DonationWindow2;
 import org.gudy.azureus2.ui.swt.exporttorrent.wizard.ExportTorrentWizard;
 import org.gudy.azureus2.ui.swt.help.AboutWindow;
 import org.gudy.azureus2.ui.swt.help.HealthHelpWindow;
@@ -57,10 +56,7 @@ import org.gudy.azureus2.ui.swt.pluginsimpl.BasicPluginViewImpl;
 import org.gudy.azureus2.ui.swt.pluginsimpl.UISWTViewImpl;
 import org.gudy.azureus2.ui.swt.pluginsinstaller.InstallPluginWizard;
 import org.gudy.azureus2.ui.swt.pluginsuninstaller.UnInstallPluginWizard;
-import org.gudy.azureus2.ui.swt.predicate.shell.ShellCanMaximizePredicate;
-import org.gudy.azureus2.ui.swt.predicate.shell.ShellCanMinimizePredicate;
-import org.gudy.azureus2.ui.swt.predicate.shell.ShellIsMinimizedPredicate;
-import org.gudy.azureus2.ui.swt.predicate.shell.ShellIsModalPredicate;
+import org.gudy.azureus2.ui.swt.predicate.shell.*;
 import org.gudy.azureus2.ui.swt.predicate.shellmanager.AllManagedShellsAreMinimizedPredicate;
 import org.gudy.azureus2.ui.swt.predicate.shellmanager.ShellManagerIsEmptyPredicate;
 import org.gudy.azureus2.ui.swt.sharing.ShareUtils;
@@ -76,6 +72,9 @@ import com.aelitis.azureus.ui.UIFunctionsManager;
 import com.aelitis.azureus.ui.swt.UIFunctionsManagerSWT;
 import com.aelitis.azureus.ui.swt.UIFunctionsSWT;
 
+import org.gudy.azureus2.plugins.update.UpdateCheckInstance;
+import org.gudy.azureus2.plugins.update.UpdateCheckInstanceListener;
+
 /**
  * @author Olivier Chalouhi
  * @author James Yeh Accessibility: Changes to allow better validation and unified menu bar state
@@ -83,6 +82,7 @@ import com.aelitis.azureus.ui.swt.UIFunctionsSWT;
 public class MainMenu {
 	public static int MENU_BAR = 0;
 	public static int MENU_TRANSFER = 1;
+	public static int MENU_VIEW = 2;
 	
 	private static final LogIDs LOGID = LogIDs.GUI;
 
@@ -96,6 +96,7 @@ public class MainMenu {
   private Menu pluginMenu;
   private Menu pluginLogsMenu;
   private Menu transferMenu;
+  private Menu viewMenu;
   
   private AzureusCore core;
 
@@ -117,7 +118,7 @@ public class MainMenu {
           this.display = SWTThread.getInstance().getDisplay();
           attachedShell = shell;
 
-          buildMenu(shell);
+          buildMenu(shell, true);
       }
   }
 
@@ -126,19 +127,23 @@ public class MainMenu {
     this.display = SWTThread.getInstance().getDisplay();
     attachedShell = mainWindow.getShell();
 
-    buildMenu(mainWindow.getShell());
+    buildMenu(mainWindow.getShell(), true);
   }
   
   public void setMainWindow(MainWindow mainWindow) {
     this.mainWindow = mainWindow;
   }
-  
+
+	public void linkMenuBar(Shell parent) {
+		parent.setMenuBar(menuBar);
+	}
+
   public void createMenu(AzureusCore core, Shell shell) {
     this.display = SWTThread.getInstance().getDisplay();
     attachedShell = shell;
 
     this.core = core;
-    buildMenu(shell);
+    buildMenu(shell, false);
   }
 
   /**
@@ -146,7 +151,7 @@ public class MainMenu {
    * @param locales
    * @param parent
    */
-  private void buildMenu(final Shell parent) {
+  private void buildMenu(final Shell parent, boolean linkToParent) {
     try {
     	if (core == null) {
     		core = AzureusCoreFactory.getSingleton();
@@ -417,6 +422,10 @@ public class MainMenu {
           appendWindowMenuItems(windowMenu);
 
       }
+      
+      if (Constants.isCVSVersion()) {
+      	addDebugMenu(menuBar);
+      }
 
       //The Help Menu
       MenuItem helpItem = new MenuItem(menuBar, SWT.CASCADE);
@@ -448,7 +457,7 @@ public class MainMenu {
       Messages.setLanguageText(help_whatsnew, "MainWindow.menu.help.releasenotes");
       help_whatsnew.addListener(SWT.Selection, new Listener() {
         public void handleEvent(Event e) {
-          new WelcomeWindow();
+          new WelcomeWindow(parent);
         }
       });
 
@@ -456,7 +465,7 @@ public class MainMenu {
       Messages.setLanguageText(help_new, "MainWindow.menu.help.whatsnew"); //$NON-NLS-1$
       help_new.addListener(SWT.Selection, new Listener() {
         public void handleEvent(Event e) {
-          Program.launch("http://azureus.sourceforge.net/changelog.php?version=" + Constants.AZUREUS_VERSION);
+        	Utils.launch("http://azureus.sourceforge.net/changelog.php?version=" + Constants.AZUREUS_VERSION);
         }
       });
 
@@ -464,8 +473,8 @@ public class MainMenu {
       Messages.setLanguageText(help_faq, "MainWindow.menu.help.faq"); //$NON-NLS-1$
       help_faq.addListener(SWT.Selection, new Listener() {
         public void handleEvent(Event e) {
-          String faqString = Constants.AELITIS_WEB_SITE + "wiki/";
-          Program.launch(faqString);
+          String faqString = Constants.AZUREUS_WIKI;
+          Utils.launch(faqString);
         }
       });
 
@@ -475,7 +484,7 @@ public class MainMenu {
       help_plugin.addListener(SWT.Selection, new Listener() {
           public void handleEvent(Event e) {
             String pluginString = "http://azureus.sourceforge.net/plugin_list.php";
-            Program.launch(pluginString);
+            Utils.launch(pluginString);
           }
         });
       
@@ -491,21 +500,26 @@ public class MainMenu {
         		if (uiFunctions != null) {
         			uiFunctions.bringToFront();
         		}
-        		UpdateMonitor.getSingleton(core).performCheck(true);
+        		UpdateMonitor.getSingleton(core).performCheck(true, false, new UpdateCheckInstanceListener() {
+        			public void cancelled(UpdateCheckInstance instance) {
+        			}
+        			
+        			public void complete(UpdateCheckInstance instance) {
+        				if (instance.getUpdates().length == 0) {
+        					Utils.execSWTThread(new AERunnable() {
+        						public void runSupport() {
+            					Utils.openMessageBox(parent,
+													SWT.ICON_INFORMATION | SWT.OK,
+													"window.update.noupdates", (String[]) null);
+        						}
+        					});
+        				}
+        			}
+        		});
         	}
         });
       }
 
-      MenuItem help_donate = new MenuItem(helpMenu, SWT.NULL);
-      Messages.setLanguageText(help_donate, "MainWindow.menu.help.donate"); //$NON-NLS-1$
-      help_donate.addListener(SWT.Selection, new Listener() {
-        public void handleEvent(Event e) {
-          new DonationWindow2(display).show();
-          //String donationString = "https://www.paypal.com/xclick/business=olivier%40gudy.org&item_name=Azureus&no_note=1&tax=0&currency_code=EUR";
-          //Program.launch(donationString);
-        }
-      });
-      
       new MenuItem(helpMenu,SWT.SEPARATOR);
       MenuItem help_debug = new MenuItem(helpMenu, SWT.NULL);
       Messages.setLanguageText(help_debug, "MainWindow.menu.help.debug");
@@ -542,9 +556,63 @@ public class MainMenu {
     } catch (Exception e) {
     	Logger.log(new LogEvent(LOGID, "Error while creating menu items", e));
     }
-    parent.setMenuBar(menuBar);
+    
+    if (linkToParent) {
+    	parent.setMenuBar(menuBar);
+    }
   }
 
+	private void addDebugMenu(Menu menu) {
+		MenuItem item;
+
+		item = new MenuItem(menu, SWT.CASCADE);
+		item.setText("Debug");
+		Menu menuDebug = new Menu(menu.getParent(), SWT.DROP_DOWN);
+		item.setMenu(menuDebug);
+
+		item = new MenuItem(menuDebug, SWT.CASCADE);
+		item.setText("ScreenSize");
+		Menu menuSS = new Menu(menu.getParent(), SWT.DROP_DOWN);
+		item.setMenu(menuSS);
+		
+		item = new MenuItem(menuSS, SWT.NONE);
+		item.setText("640x400");
+		item.addSelectionListener(new SelectionAdapter() {
+			public void widgetSelected(SelectionEvent e) {
+				mainWindow.getShell().setSize(640, 400);
+			}
+		});
+
+	
+		item = new MenuItem(menuSS, SWT.NONE);
+		item.setText("800x560");
+		item.addSelectionListener(new SelectionAdapter() {
+			public void widgetSelected(SelectionEvent e) {
+				mainWindow.getShell().setSize(850, 560);
+			}
+		});
+
+	
+		item = new MenuItem(menuSS, SWT.NONE);
+		item.setText("1024x700");
+		item.addSelectionListener(new SelectionAdapter() {
+			public void widgetSelected(SelectionEvent e) {
+				mainWindow.getShell().setSize(1024, 700);
+			}
+		});
+
+	
+		item = new MenuItem(menuSS, SWT.NONE);
+		item.setText("1280x980");
+		item.addSelectionListener(new SelectionAdapter() {
+			public void widgetSelected(SelectionEvent e) {
+				mainWindow.getShell().setSize(1280, 980);
+			}
+		});
+
+	
+	}
+
 	private void addTransferMenu(final Shell parent, boolean modal, boolean notMainWindow)
   {
       // ******** The Download Menu
@@ -625,7 +693,7 @@ public class MainMenu {
       // ******** The View Menu
       MenuItem viewItem = new MenuItem(menuBar, SWT.CASCADE);
       Messages.setLanguageText(viewItem, "MainWindow.menu.view"); //$NON-NLS-1$
-      Menu viewMenu = new Menu(parent, SWT.DROP_DOWN);
+      viewMenu = new Menu(parent, SWT.DROP_DOWN);
       viewItem.setMenu(viewMenu);
       if(notMainWindow) {performOneTimeDisable(viewItem, true);}
 
@@ -734,7 +802,6 @@ public class MainMenu {
   	final AbstractIView 	view,
   	final String			name )
   {
-
   	Utils.execSWTThread(new AERunnable() {
       public void runSupport()
       {
@@ -1227,6 +1294,10 @@ public class MainMenu {
 		if (id == MENU_TRANSFER) {
 			return transferMenu;
 		}
+		if (id == MENU_VIEW) {
+			return viewMenu;
+		}
+
 		return null;
 	}
 }
