@@ -9,24 +9,27 @@
  *******************************************************************************/
 package org.gudy.azureus2.ui.swt.osx;
 
+import java.io.IOException;
+
+import org.eclipse.swt.SWT;
 import org.eclipse.swt.internal.Callback;
 import org.eclipse.swt.internal.carbon.*;
-import org.eclipse.swt.widgets.Display;
+import org.eclipse.swt.widgets.*;
+
 import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.util.AERunnable;
 import org.gudy.azureus2.core3.util.Debug;
 import org.gudy.azureus2.platform.macosx.access.jnilib.OSXAccess;
+import org.gudy.azureus2.ui.swt.Utils;
+import org.gudy.azureus2.ui.swt.config.wizard.ConfigureWizard;
 import org.gudy.azureus2.ui.swt.help.AboutWindow;
 import org.gudy.azureus2.ui.swt.mainwindow.TorrentOpener;
 import org.gudy.azureus2.ui.swt.nat.NatTestWindow;
-import org.gudy.azureus2.ui.swt.config.wizard.ConfigureWizard;
 
 import com.aelitis.azureus.core.AzureusCoreFactory;
 import com.aelitis.azureus.ui.UIFunctions;
 import com.aelitis.azureus.ui.UIFunctionsManager;
 
-import java.io.IOException;
-
 //import com.apple.eawt.*; //Application and ApplicationAdapter
 
 public class CarbonUIEnhancer {
@@ -40,6 +43,9 @@ public class CarbonUIEnhancer {
    private static final int typeAEList = ('l'<<24) + ('i'<<16) + ('s'<<8) + 't';
    private static final int kCoreEventClass = ('a'<<24) + ('e'<<16) + ('v'<<8) + 't';
    private static final int kAEOpenDocuments = ('o'<<24) + ('d'<<16) + ('o'<<8) + 'c';
+   private static final int kURLEventClass = ('G'<<24) + ('U'<<16) + ('R'<<8) + 'L';
+
+   private static final int typeText = ('T'<<24) + ('E'<<16) + ('X'<<8) + 'T';
 
    private static final String RESOURCE_BUNDLE= "org.eclipse.ui.carbon.Messages"; //$NON-NLS-1$
    private static String fgAboutActionName;
@@ -49,7 +55,7 @@ public class CarbonUIEnhancer {
 
    public CarbonUIEnhancer() {
       if (fgAboutActionName == null) {
-         fgAboutActionName = MessageText.getString("MainWindow.menu.help.about");
+         fgAboutActionName = MessageText.getString("MainWindow.menu.help.about").replaceAll("&", "");
       }
       if(fgWizardActionName == null) {
           fgWizardActionName = MessageText.getString("MainWindow.menu.file.configure").replaceAll("&", "");
@@ -64,13 +70,40 @@ public class CarbonUIEnhancer {
       registerTorrentFile();
    }
    
-   
+   public static void registerToolbarToggle(Shell shell) {
+		final Callback toolbarToggleCB = new Callback(target, "toolbarToggle", 3);
+		int toolbarToggle = toolbarToggleCB.getAddress();
+		if (toolbarToggle == 0) {
+			Debug.out("OSX: Could not find callback 'toolbarToggle'");
+			toolbarToggleCB.dispose();
+			return;
+		}
+
+		shell.getDisplay().disposeExec(new Runnable() {
+			public void run() {
+				toolbarToggleCB.dispose();
+			}
+		});
+
+		//	 add the button to the window trim
+		int windowHandle = OS.GetControlOwner(shell.handle);
+		OS.ChangeWindowAttributes(windowHandle, OS.kWindowToolbarButtonAttribute, 0);
+
+		int[] mask = new int[] {
+			OS.kEventClassWindow,
+			OS.kEventWindowToolbarSwitchMode
+		};
+		// register the handler with the OS
+		OS.InstallEventHandler(OS.GetApplicationEventTarget(), toolbarToggle,
+				mask.length / 2, mask, 0, null);
+	}
 
    private void registerTorrentFile() {
 		Callback openDocCallback = new Callback(target, "openDocProc", 3);
 		int openDocProc = openDocCallback.getAddress();
 		if (openDocProc == 0) {
 			Debug.out("OSX: Could not find Callback 'openDocProc'");
+			openDocCallback.dispose();
 			return;
 		}
 
@@ -83,10 +116,18 @@ public class CarbonUIEnhancer {
 			return;
 		}
 
+		result = OS.AEInstallEventHandler(kURLEventClass, kURLEventClass,
+				openDocProc, 0, false);
+
+		if (result != OS.noErr) {
+			Debug.out("OSX: Could Install OpenDocs Event Handler. Error: " + result);
+			return;
+		}
+
 		int appTarget = OS.GetApplicationEventTarget();
 		Callback appleEventCallback = new Callback(this, "appleEventProc", 3);
 		int appleEventProc = appleEventCallback.getAddress();
-		int[] mask3 = new int[] { OS.kEventClassAppleEvent, OS.kEventAppleEvent, };
+		int[] mask3 = new int[] { OS.kEventClassAppleEvent, OS.kEventAppleEvent, kURLEventClass, };
 		result = OS.InstallEventHandler(appTarget, appleEventProc,
 				mask3.length / 2, mask3, 0, null);
 		if (result != OS.noErr) {
@@ -243,7 +284,7 @@ public class CarbonUIEnhancer {
 //		int eventKind = OS.GetEventKind(theEvent);
 
 //		System.out.println("appleEventProc " + OSXtoString(eventClass) + ";"
-//				+ OSXtoString(eventKind) + ";" + OSXtoString(theEvent) + ";"
+//				+ OS.GetEventKind(theEvent) + ";" + OSXtoString(theEvent) + ";"
 //				+ OSXtoString(userData));
 
 		// Process teh odoc event
@@ -253,10 +294,11 @@ public class CarbonUIEnhancer {
 					null, 4, null, aeEventID) != OS.noErr) {
 				return OS.eventNotHandledErr;
 			}
-			//System.out.println("EventID = " + OSXtoString(aeEventID[0]));
-			if (aeEventID[0] != kAEOpenDocuments) {
+//			System.out.println("EventID = " + OSXtoString(aeEventID[0]));
+			if (aeEventID[0] != kAEOpenDocuments && aeEventID[0] != kURLEventClass) {
 				return OS.eventNotHandledErr;
 			}
+
 			EventRecord eventRecord = new EventRecord();
 			OS.ConvertEventRefToEventRecord(theEvent, eventRecord);
 			OS.AEProcessAppleEvent(eventRecord);
@@ -293,7 +335,7 @@ public class CarbonUIEnhancer {
 				Debug.out("OSX: AEGetParamDesc not available.  Can't open sent file");
 				return OS.noErr;
 			}
-			
+
 			int[] count = new int[1];
 			OS.AECountItems(aeDesc, count);
 			//System.out.println("COUNT: " + count[0]);
@@ -307,18 +349,26 @@ public class CarbonUIEnhancer {
 				for (int i = 0; i < count[0]; i++) {
 					if (OS.AEGetNthPtr(aeDesc, i + 1, OS.typeFSRef, aeKeyword, typeCode,
 							dataPtr, maximumSize, actualSize) == OS.noErr) {
-						byte [] fsRef = new byte [actualSize [0]];
-						OS.memcpy (fsRef, dataPtr, actualSize [0]);
-						int dirUrl = OS.CFURLCreateFromFSRef (OS.kCFAllocatorDefault, fsRef);
-						int dirString = OS.CFURLCopyFileSystemPath(dirUrl, OS.kCFURLPOSIXPathStyle);
-						OS.CFRelease (dirUrl);						
-						int length = OS.CFStringGetLength (dirString);
-						char[] buffer= new char [length];
-						CFRange range = new CFRange ();
+						byte[] fsRef = new byte[actualSize[0]];
+						OS.memcpy(fsRef, dataPtr, actualSize[0]);
+						int dirUrl = OS.CFURLCreateFromFSRef(OS.kCFAllocatorDefault, fsRef);
+						int dirString = OS.CFURLCopyFileSystemPath(dirUrl,
+								OS.kCFURLPOSIXPathStyle);
+						OS.CFRelease(dirUrl);
+						int length = OS.CFStringGetLength(dirString);
+						char[] buffer = new char[length];
+						CFRange range = new CFRange();
 						range.length = length;
-						OS.CFStringGetCharacters (dirString, range, buffer);
-						OS.CFRelease (dirString);
-						fileNames[i] = new String (buffer);
+						OS.CFStringGetCharacters(dirString, range, buffer);
+						OS.CFRelease(dirString);
+						fileNames[i] = new String(buffer);
+					}
+
+					if (OS.AEGetNthPtr(aeDesc, i + 1, typeText, aeKeyword, typeCode,
+							dataPtr, maximumSize, actualSize) == OS.noErr) {
+						byte[] urlRef = new byte[actualSize[0]];
+						OS.memcpy(urlRef, dataPtr, actualSize[0]);
+						fileNames[i] = new String(urlRef);
 					}
 
 					//System.out.println(fileNames[i]);
@@ -326,9 +376,52 @@ public class CarbonUIEnhancer {
 
 				TorrentOpener.openTorrents(fileNames);
 			}
-			
+
+			return OS.noErr;
+		}
+
+		int toolbarToggle(int nextHandler, int theEvent, int userData) {
+			int eventKind = OS.GetEventKind(theEvent);
+			if (eventKind != OS.kEventWindowToolbarSwitchMode) {
+				return OS.eventNotHandledErr;
+			}
+
+			int[] theWindow = new int[1];
+			OS.GetEventParameter(theEvent, OS.kEventParamDirectObject,
+					OS.typeWindowRef, null, 4, null, theWindow);
+
+			int[] theRoot = new int[1];
+			OS.GetRootControl(theWindow[0], theRoot);
+			final Widget widget = Display.getCurrent().findWidget(theRoot[0]);
+
+			if (!(widget instanceof Shell)) {
+				return OS.eventNotHandledErr;
+			}
+			final Shell shellAffected = (Shell) widget;
+
+			Utils.execSWTThread(new AERunnable() {
+				public void runSupport() {
+					int type;
+					Long l = (Long) shellAffected.getData("OSX.ToolBarToggle");
+					if (l == null || l.longValue() == 0) {
+						type = SWT.Collapse;
+					} else {
+						type = SWT.Expand;
+					}
+
+					Event event = new Event();
+					event.type = type;
+					event.display = widget.getDisplay();
+					event.widget = widget;
+					shellAffected.notifyListeners(type, event);
+
+					shellAffected.setData("OSX.ToolBarToggle", new Long(
+							type == SWT.Collapse ? 1 : 0));
+				}
+			});
+
 			return OS.noErr;
 		}
 	};
    
-}
+}
\ No newline at end of file
