@@ -38,7 +38,6 @@ import org.eclipse.swt.graphics.Point;
 import org.eclipse.swt.graphics.Rectangle;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
-import org.eclipse.swt.program.Program;
 import org.eclipse.swt.widgets.*;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
@@ -118,6 +117,7 @@ public class GeneralView extends AbstractIView implements ParameterListener,
   BufferedLabel pieceSize;
   Control lblComment;
   BufferedLabel creation_date;
+  BufferedLabel user_comment;
   BufferedLabel hashFails;
   BufferedLabel shareRatio;
   Button		updateButton;
@@ -507,7 +507,7 @@ public class GeneralView extends AbstractIView implements ParameterListener,
 						if ((pos = url.indexOf("/announce")) != -1) {
 							url = url.substring(0, pos + 1);
 						}
-						Program.launch(url);
+						Utils.launch(url);
 					}
 				}
 			}
@@ -554,7 +554,14 @@ public class GeneralView extends AbstractIView implements ParameterListener,
     label = new Label(gInfo, SWT.LEFT);
     
     	// row
-    
+
+    label = new Label(gInfo, SWT.LEFT);
+    Messages.setLanguageText(label, "GeneralView.label.user_comment"); //$NON-NLS-1$
+    user_comment = new BufferedLabel(gInfo, SWT.LEFT);
+    gridData = new GridData(GridData.FILL_HORIZONTAL);
+    gridData.horizontalSpan = 3;
+    user_comment.setLayoutData(gridData);
+
     label = new Label(gInfo, SWT.LEFT);
     gridData = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
     label.setLayoutData(gridData);
@@ -564,7 +571,7 @@ public class GeneralView extends AbstractIView implements ParameterListener,
     	lblComment = new Link(gInfo, SWT.LEFT | SWT.WRAP);
     	((Link)lblComment).addSelectionListener(new SelectionAdapter() {
 				public void widgetSelected(SelectionEvent e) {
-					Program.launch(e.text);
+					Utils.launch(e.text);
 				}
 			});
     } catch (Throwable e) {
@@ -726,7 +733,8 @@ public class GeneralView extends AbstractIView implements ParameterListener,
       manager.getNbPieces(),
       manager.getPieceLength(),
       manager.getTorrentComment(),
-      DisplayFormatters.formatDate(manager.getTorrentCreationDate()*1000));
+      DisplayFormatters.formatDate(manager.getTorrentCreationDate()*1000),
+      manager.getDownloadState().getUserComment());
     
     
     //A special layout, for OS X and Linux, on which for some unknown reason
@@ -1154,7 +1162,8 @@ public class GeneralView extends AbstractIView implements ParameterListener,
     final int _pieceNumber,
     final String _pieceLength,
     final String _comment,
-	final String _creation_date) {
+	final String _creation_date,
+	final String _user_comment) {
 		if (display == null || display.isDisposed())
 			return;
 		Utils.execSWTThread(new AERunnable() {
@@ -1192,6 +1201,7 @@ public class GeneralView extends AbstractIView implements ParameterListener,
 				}
 
 				creation_date.setText(_creation_date);
+				user_comment.setText(_user_comment);
 			}
 		});
 	}
