@@ -23,29 +23,19 @@
 package org.gudy.azureus2.ui.swt.views;
 
 import java.io.File;
-import java.util.Arrays;
-import java.util.Comparator;
-import java.util.HashSet;
-import java.util.Iterator;
+import java.util.*;
 import java.util.List;
-import java.util.Map;
-import java.util.Set;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.dnd.*;
 import org.eclipse.swt.events.*;
-import org.eclipse.swt.graphics.Font;
-import org.eclipse.swt.graphics.FontData;
-import org.eclipse.swt.graphics.Image;
-import org.eclipse.swt.graphics.Point;
+import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.layout.*;
 import org.eclipse.swt.widgets.*;
-import org.gudy.azureus2.core3.category.Category;
-import org.gudy.azureus2.core3.category.CategoryListener;
-import org.gudy.azureus2.core3.category.CategoryManager;
-import org.gudy.azureus2.core3.category.CategoryManagerListener;
+
+import org.gudy.azureus2.core3.category.*;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.disk.DiskManagerFileInfo;
@@ -53,7 +43,6 @@ import org.gudy.azureus2.core3.download.DownloadManager;
 import org.gudy.azureus2.core3.download.DownloadManagerListener;
 import org.gudy.azureus2.core3.download.DownloadManagerState;
 import org.gudy.azureus2.core3.global.GlobalManager;
-import org.gudy.azureus2.core3.global.GlobalManagerDownloadRemovalVetoException;
 import org.gudy.azureus2.core3.global.GlobalManagerListener;
 import org.gudy.azureus2.core3.internat.MessageText;
 import org.gudy.azureus2.core3.logging.*;
@@ -63,10 +52,8 @@ import org.gudy.azureus2.core3.torrent.TOTorrentFactory;
 import org.gudy.azureus2.core3.tracker.client.TRTrackerAnnouncer;
 import org.gudy.azureus2.core3.tracker.util.TRTrackerUtils;
 import org.gudy.azureus2.core3.util.*;
-import org.gudy.azureus2.plugins.ui.tables.TableManager;
 import org.gudy.azureus2.ui.swt.*;
 import org.gudy.azureus2.ui.swt.URLTransfer;
-import org.gudy.azureus2.ui.swt.SimpleTextEntryWindow.TextEntrySubmissionListener;
 import org.gudy.azureus2.ui.swt.exporttorrent.wizard.ExportTorrentWizard;
 import org.gudy.azureus2.ui.swt.help.HealthHelpWindow;
 import org.gudy.azureus2.ui.swt.mainwindow.Colors;
@@ -79,12 +66,13 @@ import org.gudy.azureus2.ui.swt.views.table.TableColumnCore;
 import org.gudy.azureus2.ui.swt.views.table.TableRowCore;
 import org.gudy.azureus2.ui.swt.views.table.impl.TableCellImpl;
 import org.gudy.azureus2.ui.swt.views.utils.ManagerUtils;
-import org.gudy.azureus2.ui.swt.wizards.sendtorrent.SendTorrentWizard;
 
 import com.aelitis.azureus.core.AzureusCore;
 import com.aelitis.azureus.ui.UIFunctions;
 import com.aelitis.azureus.ui.UIFunctionsManager;
 
+import org.gudy.azureus2.plugins.ui.tables.TableManager;
+
 /** Displays a list of torrents in a table view.
  *
  * @author Olivier
@@ -106,7 +94,8 @@ public class MyTorrentsView
 	private static final LogIDs LOGID = LogIDs.GUI;
 	private static final int ASYOUTYPE_MODE_FIND = 0;
 	private static final int ASYOUTYPE_MODE_FILTER = 1;
-	private static final int ASYOUTYPE_MODE = ASYOUTYPE_MODE_FILTER; 
+	private static final int ASYOUTYPE_MODE = ASYOUTYPE_MODE_FILTER;
+	private static final int ASYOUTYPE_UPDATEDELAY = 300;
 	
 	private AzureusCore		azureus_core;
 
@@ -134,11 +123,11 @@ public class MyTorrentsView
   // table item index, where the drag has started
   private int drag_drop_line_start = -1;
 
-  private boolean confirmDataDelete = COConfigurationManager.getBooleanParameter("Confirm Data Delete", true);
-  
+	private TimerEvent searchUpdateEvent;
   private String sLastSearch = "";
   private long lLastSearchTime;
   private boolean bRegexSearch = false;
+	private boolean bDNDalwaysIncomplete;
 
   /**
    * Initialize
@@ -179,8 +168,10 @@ public class MyTorrentsView
 
     createDragDrop();
 
-    COConfigurationManager.addParameterListener("Confirm Data Delete", this);
-    COConfigurationManager.addAndFireParameterListener("User Mode", this);
+    COConfigurationManager.addAndFireParameterListeners(new String[] {
+				"DND Always In Incomplete",
+				"Confirm Data Delete",
+				"User Mode" }, this);
 
     if (currentCategory != null) {
     	currentCategory.addCategoryListener(this);
@@ -551,7 +542,7 @@ public class MyTorrentsView
   		return false;
   	}
 
-		boolean bCompleted = dm.isDownloadComplete(false);
+		boolean bCompleted =  dm.isDownloadComplete(bDNDalwaysIncomplete);
 		boolean bOurs = (bCompleted && isSeedingView)
 				|| (!bCompleted && !isSeedingView);
 		
@@ -562,9 +553,23 @@ public class MyTorrentsView
 
 		if (bOurs && sLastSearch.length() > 0) {
 			try {
-				String name = dm.getDisplayName();
-				String s = bRegexSearch ? sLastSearch : "\\Q"
-						+ sLastSearch.replaceAll("[|;]", "\\\\E|\\\\Q") + "\\E";
+				String[][] names = {	{"", 		dm.getDisplayName()},
+												{"t:", 	dm.getTorrent().getAnnounceURL().getHost()},
+												{"st:", 	"" + dm.getState()}
+											};
+				
+				String name = names[0][1];
+				String tmpSearch = sLastSearch;
+				
+				for(int i = 0; i < names.length; i++){
+					if (tmpSearch.startsWith(names[i][0])) {
+						tmpSearch = tmpSearch.substring(names[i][0].length());
+						name = names[i][1];
+					}
+				}
+				
+				String s = bRegexSearch ? tmpSearch : "\\Q"
+						+ tmpSearch.replaceAll("[|;]", "\\\\E|\\\\Q") + "\\E";
 				Pattern pattern = Pattern.compile(s, Pattern.CASE_INSENSITIVE);
 
 				if (!pattern.matcher(name).find())
@@ -620,13 +625,12 @@ public class MyTorrentsView
 		// Enable/Disable Logic
 
 
-		boolean moveUp, moveDown, bChangeDir;
-		moveUp = moveDown = bChangeDir = hasSelection;
+		boolean bChangeDir = hasSelection;
 
 		boolean start, stop, changeUrl, barsOpened, forceStart;
-		boolean forceStartEnabled, recheck, manualUpdate, changeSpeed, fileMove, fileRescan;
+		boolean forceStartEnabled, recheck, manualUpdate, fileMove, fileRescan;
 
-		changeUrl = barsOpened = manualUpdate = changeSpeed = fileMove = fileRescan = true;
+		changeUrl = barsOpened = manualUpdate = fileMove = fileRescan = true;
 		forceStart = forceStartEnabled = recheck = start = stop = false;
 
 		boolean upSpeedDisabled = false;
@@ -645,8 +649,6 @@ public class MyTorrentsView
 		boolean	allStopped			= true;
 		
 		if (hasSelection) {
-			bChangeDir = true;
-
 			for (int i = 0; i < dms.length; i++) {
 				DownloadManager dm = (DownloadManager) dms[i];
 
@@ -679,8 +681,6 @@ public class MyTorrentsView
 					}
 					totalDownSpeed += maxdl;
 
-				} catch (NullPointerException ex) {
-					changeSpeed = false;
 				} catch (Exception ex) {
 					Debug.printStackTrace(ex);
 				}
@@ -710,14 +710,6 @@ public class MyTorrentsView
 					
 				fileMove = fileMove && stopped && dm.isPersistent();
 
-				if (!dm.getGlobalManager().isMoveableDown(dm)) {
-					moveDown = false;
-				}
-
-				if (!dm.getGlobalManager().isMoveableUp(dm)) {
-					moveUp = false;
-				}
-
 				if (userMode > 1) {
 					TRTrackerAnnouncer trackerClient = dm.getTrackerClient();
 
@@ -831,16 +823,6 @@ public class MyTorrentsView
 		});
 		itemExplore.setEnabled(hasSelection);
 
-		// Send
-		final MenuItem itemSend = new MenuItem(menu, SWT.PUSH);
-		Messages.setLanguageText(itemSend, "MyTorrentsView.menu.sendTorrent");
-		itemSend.addListener(SWT.Selection, new Listener() {
-			public void handleEvent(Event event) {
-				openSendTorrentWizForSelected();
-			}
-		});
-		itemSend.setEnabled(hasSelection);
-		
 		// === advanced menu ===
 
 		final MenuItem itemAdvanced = new MenuItem(menu, SWT.CASCADE);
@@ -1661,7 +1643,7 @@ public class MyTorrentsView
         	itemRenameBoth.setData("save_name", Boolean.valueOf(true));
         	itemRenameBoth.setData("msg_key", "displayed_and_save_path");
         }
-
+        
         Listener rename_listener = new Listener() {
         	public void handleEvent(Event event) {
         		MenuItem mi = (MenuItem)event.widget;
@@ -1669,9 +1651,13 @@ public class MyTorrentsView
         		final boolean change_displayed_name = ((Boolean)mi.getData("display_name")).booleanValue();
         		final boolean change_save_name = ((Boolean)mi.getData("save_name")).booleanValue();
         		String msg_key_prefix = "MyTorrentsView.menu.rename." + (String)mi.getData("msg_key") + ".enter.";
-        		SimpleTextEntryWindow text_entry = new SimpleTextEntryWindow(getComposite().getDisplay(), msg_key_prefix + "title", msg_key_prefix + "message", suggested, null);
-        		if (text_entry.wasDataSubmitted()) {
-        			String value = text_entry.getStoredString();
+        		SimpleTextEntryWindow text_entry = new SimpleTextEntryWindow(getComposite().getDisplay());
+        		text_entry.setTitle(msg_key_prefix + "title");
+        		text_entry.setMessages(new String[]{msg_key_prefix + "message", msg_key_prefix + "message.2"});
+        		text_entry.setPreenteredText(suggested, false);
+        		text_entry.prompt();
+        		if (text_entry.hasSubmittedInput()) {
+        			String value = text_entry.getSubmittedInput();
         			final String value_to_set = (value.length() == 0) ? null : value;
         			MyTorrentsView.this.runForSelectedRows(new GroupTableRowRunner() {
                         public void run(TableRowCore row) {
@@ -1696,6 +1682,38 @@ public class MyTorrentsView
         itemRenameSavePath.addListener(SWT.Selection, rename_listener);
         itemRenameBoth.addListener(SWT.Selection, rename_listener);
 
+        // Edit Comment
+        final MenuItem itemEditComment = new MenuItem(menu, SWT.CASCADE);
+        Messages.setLanguageText(itemEditComment, "MyTorrentsView.menu.edit_comment");
+        itemEditComment.setEnabled(dms.length > 0);
+        if (itemEditComment.isEnabled()) {
+        	itemEditComment.setData("suggested_text", first_selected.getDownloadState().getUserComment());
+        }
+        
+        Listener edit_comment_listener = new Listener() {
+        	public void handleEvent(Event event) {
+        		MenuItem mi = (MenuItem)event.widget;
+        		String suggested = (String)mi.getData("suggested_text");
+        		String msg_key_prefix = "MyTorrentsView.menu.edit_comment.enter.";
+        		SimpleTextEntryWindow text_entry = new SimpleTextEntryWindow(getComposite().getDisplay());
+        		text_entry.setTitle(msg_key_prefix + "title");
+        		text_entry.setMessage(msg_key_prefix + "message");
+        		text_entry.setPreenteredText(suggested, false);
+        		text_entry.prompt();
+        		if (text_entry.hasSubmittedInput()) {
+        			String value = text_entry.getSubmittedInput();
+        			final String value_to_set = (value.length() == 0) ? null : value;
+        			MyTorrentsView.this.runForSelectedRows(new GroupTableRowRunner() {
+                        public void run(TableRowCore row) {
+                        	((DownloadManager)row.getDataSource(true)).getDownloadState().setUserComment(value_to_set);
+                        }
+        			});
+        		}
+        	}
+        };  
+        
+        itemEditComment.addListener(SWT.Selection, edit_comment_listener);
+        
 		// ---
 		new MenuItem(menu, SWT.SEPARATOR);
 
@@ -2042,7 +2060,7 @@ public class MyTorrentsView
       int iOldPos = dm.getPosition();
       
       globalManager.moveTo(dm, iNewPos);
-      if (rowSorter.isAscending()) {
+      if (sortColumn.isSortAscending()) {
         if (iOldPos > iNewPos)
           iNewPos++;
       } else {
@@ -2051,7 +2069,7 @@ public class MyTorrentsView
       }
     }
 
-    boolean bForceSort = rowSorter.getColumnName().equals("#");
+    boolean bForceSort = sortColumn.getName().equals("#");
     columnInvalidate("#");
     refresh(bForceSort);
   }
@@ -2315,7 +2333,17 @@ public class MyTorrentsView
 			lblX.setImage(img);
 		}
 
-		activateCategory(currentCategory);
+		if (searchUpdateEvent != null) {
+			searchUpdateEvent.cancel();
+		}
+		searchUpdateEvent = SimpleTimer.addEvent("SearchUpdate",
+				SystemTime.getOffsetTime(ASYOUTYPE_UPDATEDELAY),
+				new TimerEventPerformer() {
+					public void perform(TimerEvent event) {
+						searchUpdateEvent = null;
+						activateCategory(currentCategory);
+					}
+				});
 	}
 
 	public void keyReleased(KeyEvent e) {
@@ -2328,14 +2356,14 @@ public class MyTorrentsView
       return;
 
     String sDefPath = COConfigurationManager.getBooleanParameter("Use default data dir") ?
-                      COConfigurationManager.getStringParameter("Default save path", "") :
+                      COConfigurationManager.getStringParameter("Default save path") :
                       "";
     
     if ( sDefPath.length() > 0 ){
 	    File	f = new File(sDefPath);
 	    
 	    if ( !f.exists()){
-	    	f.mkdirs();
+	    	FileUtil.mkdirs(f);
 	    }
     }
     
@@ -2363,63 +2391,7 @@ public class MyTorrentsView
   }
 
   private void removeTorrent(final DownloadManager dm, final boolean bDeleteTorrent, final boolean bDeleteData) {
-    
-    if( COConfigurationManager.getBooleanParameter( "confirm_torrent_removal" ) ) {
-    	
-      MessageBox mb = new MessageBox(cTablePanel.getShell(), SWT.ICON_WARNING | SWT.YES | SWT.NO);
-      
-      mb.setText(MessageText.getString("deletedata.title"));
-      
-      mb.setMessage(MessageText.getString("deletetorrent.message1")
-            + dm.getDisplayName() + " :\n"
-            + dm.getTorrentFileName()
-            + MessageText.getString("deletetorrent.message2"));
-      
-      if( mb.open() == SWT.NO ) {
-        return;
-      }
-    }
-    
-    int choice;
-    if (confirmDataDelete && bDeleteData) {
-      String path = dm.getSaveLocation().toString();
-      
-      MessageBox mb = new MessageBox(cTablePanel.getShell(), SWT.ICON_WARNING | SWT.YES | SWT.NO);
-      
-      mb.setText(MessageText.getString("deletedata.title"));
-      
-      mb.setMessage(MessageText.getString("deletedata.message1")
-          + dm.getDisplayName() + " :\n"
-          + path
-          + MessageText.getString("deletedata.message2"));
-
-      choice = mb.open();
-    } else {
-      choice = SWT.YES;
-    }
-
-    if (choice == SWT.YES) {
-      	new AEThread( "asyncStop", true )
-		{
-    		public void
-			runSupport()
-    		{
-
-		      try {
-		        dm.stopIt( DownloadManager.STATE_STOPPED, bDeleteTorrent, bDeleteData );
-		        dm.getGlobalManager().removeDownloadManager( dm );
-		      }
-		      catch (GlobalManagerDownloadRemovalVetoException f) {
-		    	  if(!f.isSilent()) {
-		    		  Alerts.showErrorMessageBoxUsingResourceString("globalmanager.download.remove.veto", f);
-		    	  }
-		      }
-		      catch (Exception ex) {
-		        Debug.printStackTrace( ex );
-		      }
-    		}
-		}.start();
-    }
+  	ManagerUtils.remove(dm, cTablePanel.getShell(), bDeleteTorrent, bDeleteData);
   }
 
   private void removeSelectedTorrents() {
@@ -2513,7 +2485,7 @@ public class MyTorrentsView
       }
     }
 
-    boolean bForceSort = rowSorter.getColumnName().equals("#");
+    boolean bForceSort = sortColumn.getName().equals("#");
     columnInvalidate("#");
     refresh(bForceSort);
   }
@@ -2533,7 +2505,7 @@ public class MyTorrentsView
       }
     }
 
-    boolean bForceSort = rowSorter.getColumnName().equals("#");
+    boolean bForceSort = sortColumn.getName().equals("#");
     columnInvalidate("#");
     refresh(bForceSort);
   }
@@ -2579,7 +2551,7 @@ public class MyTorrentsView
 			globalManager.moveTo(dm, newPositions[i]);
 		}
 
-		boolean bForceSort = rowSorter.getColumnName().equals("#");
+		boolean bForceSort = sortColumn.getName().equals("#");
 		columnInvalidate("#");
 		refresh(bForceSort);
 	}
@@ -2600,7 +2572,7 @@ public class MyTorrentsView
       globalManager.moveTop(downloadManagers);
     else
       globalManager.moveEnd(downloadManagers);
-    if (rowSorter.getColumnName().equals("#")) {
+    if (sortColumn.getName().equals("#")) {
       columnInvalidate("#");
       refresh(true);
     }
@@ -2611,10 +2583,17 @@ public class MyTorrentsView
    * @see org.gudy.azureus2.core3.config.ParameterListener#parameterChanged(java.lang.String)
    */
   public void parameterChanged(String parameterName) {
-    super.parameterChanged(parameterName);
-    confirmDataDelete = COConfigurationManager.getBooleanParameter("Confirm Data Delete", true);
-    userMode = COConfigurationManager.getIntParameter("User Mode");
-  }
+		super.parameterChanged(parameterName);
+
+		if (parameterName == null || parameterName.equals("User Mode")) {
+			userMode = COConfigurationManager.getIntParameter("User Mode");
+		}
+
+		if (parameterName == null
+				|| parameterName.equals("DND Always In Incomplete")) {
+			bDNDalwaysIncomplete = COConfigurationManager.getBooleanParameter("DND Always In Incomplete");
+		}
+	}
 
   private boolean top,bottom,up,down,run,host,publish,start,stop,remove;
 
@@ -2661,8 +2640,6 @@ public class MyTorrentsView
       return up;
     if(itemKey.equals("down"))
       return down;
-    if(itemKey.equals("send"))
-      return run;
     return false;
   }
 
@@ -2707,25 +2684,8 @@ public class MyTorrentsView
       removeSelectedTorrents();
       return;
     }
-
-		if (itemKey.equals("send")) {
-			openSendTorrentWizForSelected();
-			return;
-		}
   }
   
-  public void openSendTorrentWizForSelected() {
-		Object[] dms = getSelectedDataSources();
-		TOTorrent[] torrents = new TOTorrent[dms.length];
-		for (int i = 0; i < torrents.length; i++) {
-			if (dms[i] instanceof DownloadManager) {
-				torrents[i] = ((DownloadManager) dms[i]).getTorrent();
-			}
-		}
-
-		new SendTorrentWizard(azureus_core, getTable().getDisplay(), torrents);
-	}
-
   private Category addCategory() {
     CategoryAdderWindow adderWindow = new CategoryAdderWindow(getComposite().getDisplay());
     Category newCategory = adderWindow.getNewCategory();
