@@ -25,14 +25,16 @@
 package org.gudy.azureus2.ui.swt.views.configsections;
 
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.widgets.Composite;
-import org.eclipse.swt.widgets.Label;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
+import org.eclipse.swt.widgets.Composite;
+import org.eclipse.swt.widgets.Label;
+
+import org.gudy.azureus2.core3.config.COConfigurationManager;
+import org.gudy.azureus2.ui.swt.config.BooleanParameter;
+import org.gudy.azureus2.ui.swt.plugins.UISWTConfigSection;
 
 import org.gudy.azureus2.plugins.ui.config.ConfigSection;
-import org.gudy.azureus2.ui.swt.config.*;
-import org.gudy.azureus2.ui.swt.plugins.UISWTConfigSection;
 
 public class ConfigSectionInterfaceStart implements UISWTConfigSection {
   public String configSectionGetParentSection() {
@@ -63,15 +65,21 @@ public class ConfigSectionInterfaceStart implements UISWTConfigSection {
 
     new BooleanParameter(cStart, "Show Splash", true, "ConfigView.label.showsplash");
     //new BooleanParameter(cStart, "Auto Update", true, "ConfigView.label.autoupdate");
-    new BooleanParameter(cStart, "update.start", true, "ConfigView.label.checkonstart");
-    new BooleanParameter(cStart, "update.periodic", true, "ConfigView.label.periodiccheck");
-    new BooleanParameter(cStart, "update.opendialog", true, "ConfigView.label.opendialog");
+    new BooleanParameter(cStart, "update.start", "ConfigView.label.checkonstart");
+    new BooleanParameter(cStart, "update.periodic", "ConfigView.label.periodiccheck");
+    new BooleanParameter(cStart, "update.opendialog", "ConfigView.label.opendialog");
     new Label(cStart,SWT.NULL);
+    new BooleanParameter(cStart, "Open MyTorrents", "ConfigView.label.openmytorrents");
     new BooleanParameter(cStart, "Open Console", false, "ConfigView.label.openconsole");
     new BooleanParameter(cStart, "Open Stats On Start", false, "ConfigView.label.openstatsonstart");
     new BooleanParameter(cStart, "Open Config", false, "ConfigView.label.openconfig");
     new BooleanParameter(cStart, "Start Minimized", false, "ConfigView.label.startminimized");
     
+    if (COConfigurationManager.getStringParameter("ui").equals("az3")) {
+			new BooleanParameter(cStart, "v3.Start Advanced",
+					"ConfigView.interface.start.advanced");
+		}
+    
     return cStart;
   }
 }
