@@ -26,6 +26,9 @@ package org.gudy.azureus2.ui.swt.views.configsections;
 
 import java.io.PrintWriter;
 import java.io.StringWriter;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.Map;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.SelectionAdapter;
@@ -41,9 +44,12 @@ import org.gudy.azureus2.core3.logging.LogIDs;
 import org.gudy.azureus2.core3.logging.Logger;
 import org.gudy.azureus2.core3.logging.impl.FileLogging;
 import org.gudy.azureus2.core3.util.AEDiagnostics;
+import org.gudy.azureus2.core3.util.AEThread;
+import org.gudy.azureus2.core3.util.IndentWriter;
 import org.gudy.azureus2.plugins.ui.config.ConfigSection;
 import org.gudy.azureus2.ui.swt.ImageRepository;
 import org.gudy.azureus2.ui.swt.Messages;
+import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.config.BooleanParameter;
 import org.gudy.azureus2.ui.swt.config.ChangeSelectionActionPerformer;
 import org.gudy.azureus2.ui.swt.config.IAdditionalActionPerformer;
@@ -52,6 +58,9 @@ import org.gudy.azureus2.ui.swt.config.StringParameter;
 import org.gudy.azureus2.ui.swt.mainwindow.ClipboardCopy;
 import org.gudy.azureus2.ui.swt.plugins.UISWTConfigSection;
 
+import com.aelitis.azureus.core.networkmanager.admin.NetworkAdmin;
+import com.aelitis.azureus.core.stats.AzureusCoreStats;
+
 public class ConfigSectionLogging implements UISWTConfigSection {
   private static final LogIDs LOGID = LogIDs.GUI;
   private static final String CFG_PREFIX =  "ConfigView.section.logging.";
@@ -257,14 +266,110 @@ public class ConfigSectionLogging implements UISWTConfigSection {
         }
     );
 
-    
+    /*
     BooleanParameter udp_transport = new BooleanParameter(gLogging, "Logging Enable UDP Transport", CFG_PREFIX + "udptransport");
     gridData = new GridData();
     gridData.horizontalSpan = 2;
     udp_transport.setLayoutData(gridData);
+    */
     
-		// diagnostics
+		// network diagnostics
 	
+	Label generate_net_info = new Label(gLogging, SWT.NULL);
+
+	generate_net_info.setText( "Generate network info" );
+	
+	Button generate_net_button = new Button(gLogging, SWT.PUSH);
+
+	generate_net_button.setText( "Go!" );
+	
+	generate_net_button.addListener(
+			SWT.Selection, 
+			new Listener() 
+			{
+				public void 
+				handleEvent(Event event) 
+				{
+					new AEThread("GenerateNetDiag", true)
+					{
+						public void
+						runSupport()
+						{
+							StringWriter sw = new StringWriter();
+							
+							PrintWriter	pw = new PrintWriter( sw );
+									
+							IndentWriter iw = new IndentWriter( pw );
+							
+							NetworkAdmin admin = NetworkAdmin.getSingleton();
+							
+							admin.generateDiagnostics( iw );
+							
+							pw.close();
+							
+							final String	info = sw.toString();
+							
+							Logger.log( new LogEvent(LOGID, "Network Info:\n" + info));
+
+							Utils.execSWTThread(
+								new Runnable()
+								{
+									public void
+									run()
+									{
+										ClipboardCopy.copyToClipBoard( info );
+									}
+								});
+						}
+					}.start();
+				}
+			});
+    
+	// stats
+	
+	Label generate_stats_info = new Label(gLogging, SWT.NULL);
+
+	generate_stats_info.setText( "Generate stats info" );
+	
+	Button generate_stats_button = new Button(gLogging, SWT.PUSH);
+
+	generate_stats_button.setText( "Go!" );
+	
+	generate_stats_button.addListener(
+			SWT.Selection, 
+			new Listener() 
+			{
+				public void 
+				handleEvent(Event event) 
+				{
+					java.util.Set	types = new HashSet();
+					
+					types.add( AzureusCoreStats.ST_ALL );
+					
+					Map	reply = AzureusCoreStats.getStats( types );
+					
+					Iterator	it = reply.entrySet().iterator();
+					
+					StringBuffer buffer = new StringBuffer(16000);
+					
+					while( it.hasNext()){
+						
+						Map.Entry	entry = (Map.Entry)it.next();
+						
+						buffer.append( entry.getKey() + " -> " + entry.getValue() + "\r\n" );
+					}
+					
+					String	str = buffer.toString();
+					
+					ClipboardCopy.copyToClipBoard( str );
+
+					Logger.log( new LogEvent(LOGID, "Stats Info:\n" + str));
+				}
+			});
+	
+		// diagnostics
+
+    
 	Label generate_info = new Label(gLogging, SWT.NULL);
 
 	Messages.setLanguageText(generate_info, CFG_PREFIX + "generatediagnostics.info");
