@@ -33,7 +33,6 @@ import org.eclipse.swt.events.SelectionAdapter;
 import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
-import org.eclipse.swt.program.Program;
 import org.eclipse.swt.widgets.*;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.internat.MessageText;
@@ -42,7 +41,6 @@ import org.gudy.azureus2.plugins.PluginException;
 import org.gudy.azureus2.plugins.PluginInterface;
 import org.gudy.azureus2.plugins.ui.config.ConfigSection;
 import org.gudy.azureus2.plugins.ui.config.Parameter;
-import org.gudy.azureus2.pluginsimpl.local.PluginInitializer;
 import org.gudy.azureus2.pluginsimpl.local.PluginInterfaceImpl;
 import org.gudy.azureus2.pluginsimpl.local.ui.config.BooleanParameterImpl;
 import org.gudy.azureus2.pluginsimpl.local.ui.config.ParameterRepository;
@@ -338,12 +336,12 @@ public class ConfigSectionPlugins implements UISWTConfigSection {
 						|| _sUserPluginDir.endsWith("\\plugins\\")) {
 					File f = new File(_sUserPluginDir);
 					if (f.exists() && f.isDirectory()) {
-						Program.launch(_sUserPluginDir);
+						Utils.launch(_sUserPluginDir);
 					} else {
 						String azureusDir = _sUserPluginDir.substring(0, _sUserPluginDir
 								.length() - 9);
 						System.out.println(azureusDir);
-						Program.launch(azureusDir);
+						Utils.launch(azureusDir);
 					}
 				}
 			}
@@ -370,12 +368,12 @@ public class ConfigSectionPlugins implements UISWTConfigSection {
 						|| _sAppPluginDir.endsWith("\\plugins\\")) {
 					File f = new File(_sAppPluginDir);
 					if (f.exists() && f.isDirectory()) {
-						Program.launch(_sAppPluginDir);
+						Utils.launch(_sAppPluginDir);
 					} else {
 						String azureusDir = _sAppPluginDir.substring(0, _sAppPluginDir
 								.length() - 9);
 						System.out.println(azureusDir);
-						Program.launch(azureusDir);
+						Utils.launch(azureusDir);
 					}
 				}
 			}
