@@ -64,14 +64,21 @@ public interface TableCellCore
    * 
    * @param bDoGraphics Whether to update graphic cells 
    */
-  public void refresh(boolean bDoGraphics);
+  public boolean refresh(boolean bDoGraphics);
 
   /** 
    * Refresh the cell, including graphic types 
    */
-  public void refresh();
+  public boolean refresh();
   
-  /**
+	/**
+	 * @param bDoGraphics
+	 * @param bRowVisible
+	 * @param bCellVisible
+	 */
+	public boolean refresh(boolean bDoGraphics, boolean bRowVisible, boolean bCellVisible);
+
+	/**
    * Refresh the cell.  This method overide takes a bRowVisible paramater in
    * order to reduce the number of calls to TableRow.isVisible() in cases where
    * multiple cells on the same row are being refreshed.
@@ -79,16 +86,21 @@ public interface TableCellCore
    * @param bDoGraphics Whether to update graphic cells
    * @param bRowVisible Visibility state of row
    */
-  public void refresh(boolean bDoGraphics, boolean bRowVisible);
+  public boolean refresh(boolean bDoGraphics, boolean bRowVisible);
   
   /** dispose of the cell */
   public void dispose();
   
-  /** Set the cell's image
+	/**
+	 * @return
+	 */
+	Image getIcon();
+
+  /** Set the cell's icon
    *
-   * @param img Cell's new image
+   * @param img Cell's new icon
    */
-  public void setImage(Image img);
+  public void setIcon(Image img);
 
   /** Retrieve whether the cell need any paint calls (graphic)
    *
@@ -119,6 +131,7 @@ public interface TableCellCore
 
   public void invokeToolTipListeners(int type);
   public void invokeMouseListeners(TableCellMouseEvent event);
+	public void invokeVisibilityListeners(int visibility);
   
   public void setUpToDate(boolean upToDate);
 
@@ -135,7 +148,7 @@ public interface TableCellCore
 	/**
 	 * @return
 	 */
-	Color getForeground();
+	Color getForegroundSWT();
 
 	/**
 	 * @return
@@ -156,4 +169,9 @@ public interface TableCellCore
 	 * @return
 	 */
 	boolean isUpToDate();
+
+	/**
+	 * @return
+	 */
+	boolean getVisuallyChangedSinceRefresh();
 }
