@@ -26,10 +26,7 @@
 package org.gudy.azureus2.ui.swt.views.table.impl;
 
 import java.text.Collator;
-import java.util.ArrayList;
-import java.util.Comparator;
-import java.util.Iterator;
-import java.util.Locale;
+import java.util.*;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.graphics.*;
@@ -37,16 +34,10 @@ import org.eclipse.swt.graphics.*;
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.internat.MessageText;
-import org.gudy.azureus2.core3.logging.*;
-import org.gudy.azureus2.core3.util.AEMonitor;
-import org.gudy.azureus2.core3.util.AERunnable;
-import org.gudy.azureus2.core3.util.Debug;
-import org.gudy.azureus2.core3.util.SystemTime;
-
-import org.gudy.azureus2.plugins.ui.Graphic;
-import org.gudy.azureus2.plugins.ui.UIRuntimeException;
-import org.gudy.azureus2.plugins.ui.SWT.GraphicSWT;
-import org.gudy.azureus2.plugins.ui.tables.*;
+import org.gudy.azureus2.core3.logging.LogEvent;
+import org.gudy.azureus2.core3.logging.LogIDs;
+import org.gudy.azureus2.core3.logging.Logger;
+import org.gudy.azureus2.core3.util.*;
 import org.gudy.azureus2.ui.swt.Utils;
 import org.gudy.azureus2.ui.swt.components.*;
 import org.gudy.azureus2.ui.swt.debug.ObfusticateCellText;
@@ -56,6 +47,11 @@ import org.gudy.azureus2.ui.swt.views.table.TableCellCore;
 import org.gudy.azureus2.ui.swt.views.table.TableColumnCore;
 import org.gudy.azureus2.ui.swt.views.table.TableRowCore;
 
+import org.gudy.azureus2.plugins.ui.Graphic;
+import org.gudy.azureus2.plugins.ui.UIRuntimeException;
+import org.gudy.azureus2.plugins.ui.SWT.GraphicSWT;
+import org.gudy.azureus2.plugins.ui.tables.*;
+
 
 /** TableCellImpl represents one cell in the table.  
  * Table access is provided by BufferedTableItem.  
@@ -78,6 +74,7 @@ public class TableCellImpl
   private ArrayList disposeListeners;
   private ArrayList tooltipListeners;
 	private ArrayList cellMouseListeners;
+	private ArrayList cellVisibilityListeners;
   private TableColumnCore tableColumn;
   private boolean valid;
   private int refreshErrLoopCount;
@@ -85,6 +82,7 @@ public class TableCellImpl
   private int loopFactor;
   private Object oToolTip;
 	private int iCursorID = -1;
+	private Graphic graphic = null;
   
   /**
    * For refreshing, this flag manages whether the row is actually up to date.
@@ -107,6 +105,7 @@ public class TableCellImpl
 	public boolean bDebug = false;
   
   private AEMonitor 	this_mon 	= new AEMonitor( "TableCell" );
+	private boolean bCellVisuallyChangedSinceRefresh;
 
   private static final String CFG_PAINT = "GUI_SWT_bAlternateTablePainting";
   private static boolean bAlternateTablePainting;
@@ -220,7 +219,16 @@ public class TableCellImpl
   ////////////////
   
   public Object getDataSource() {
-    return tableRow.getDataSource(tableColumn.getUseCoreDataSource());
+		// if we've been disposed then row/col are null
+	  
+	TableRowCore	row = tableRow;
+	TableColumnCore	col	= tableColumn;
+	
+	if ( row == null || col == null){
+		return( null );
+	}
+	
+    return row.getDataSource(col.getUseCoreDataSource());
   }
   
   public TableColumn getTableColumn() {
@@ -239,12 +247,24 @@ public class TableCellImpl
     return valid;
   }
   
-  public Color getForeground() {
+  public Color getForegroundSWT() {
   	checkCellForSetting();
 
     return bufferedTableItem.getForeground();
   }
   
+  // @see org.gudy.azureus2.plugins.ui.tables.TableCell#getForeground()
+  public int[] getForeground() {
+		Color color = bufferedTableItem.getForeground();
+
+		if (color == null) {
+			return new int[3];
+		}
+
+		return new int[] { color.getRed(), color.getGreen(), color.getBlue()
+		};
+	}
+  
   public boolean setForeground(Color color) {
   	checkCellForSetting();
 
@@ -252,6 +272,7 @@ public class TableCellImpl
   	if (isInvisibleAndCanRefresh())
   		return false;
 
+  	bCellVisuallyChangedSinceRefresh = true;
     return bufferedTableItem.setForeground(color);
   }
   
@@ -262,6 +283,7 @@ public class TableCellImpl
   	if (isInvisibleAndCanRefresh())
   		return false;
 
+  	bCellVisuallyChangedSinceRefresh = true;
     return bufferedTableItem.setForeground(red, green, blue);
   }
 
@@ -295,6 +317,9 @@ public class TableCellImpl
 			debug("setText (" + bChanged + ") : " + text);
 		}
 
+		if (bChanged) {
+			bCellVisuallyChangedSinceRefresh = true;
+		}
   	return bChanged;
   }
   
@@ -336,6 +361,11 @@ public class TableCellImpl
 	      // cell wasn't created at the time of setting)
 	      setText((String)sortValue);
     }
+    
+    if ((valueToSort instanceof String) && (sortValue instanceof String)
+				&& sortValue.equals(valueToSort)) {
+			return false;
+		}
 
   	if (bDebug)
   		debug("Setting SortValue to "
@@ -402,18 +432,32 @@ public class TableCellImpl
   }
 
   public int getWidth() {
-    if (!(bufferedTableItem instanceof BufferedGraphicTableItem))
-      return -1;
-    Point pt = ((BufferedGraphicTableItem)bufferedTableItem).getSize();
+  	Point pt = null;
+  	
+    if (bufferedTableItem instanceof BufferedGraphicTableItem) {
+    	pt = ((BufferedGraphicTableItem)bufferedTableItem).getSize();
+    } else {
+    	Rectangle bounds = bufferedTableItem.getBounds();
+    	if (bounds != null) {
+    		pt = new Point(bounds.width, bounds.height);
+    	}
+    }
     if (pt == null)
       return -1;
     return pt.x;
   }
 
   public int getHeight() {
-    if (!(bufferedTableItem instanceof BufferedGraphicTableItem))
-      return -1;
-    Point pt = ((BufferedGraphicTableItem)bufferedTableItem).getSize();
+  	Point pt = null;
+  	
+    if (bufferedTableItem instanceof BufferedGraphicTableItem) {
+    	pt = ((BufferedGraphicTableItem)bufferedTableItem).getSize();
+    } else {
+    	Rectangle bounds = bufferedTableItem.getBounds();
+    	if (bounds != null) {
+    		pt = new Point(bounds.width, bounds.height);
+    	}
+    }
     if (pt == null)
       return -1;
     return pt.y;
@@ -425,32 +469,55 @@ public class TableCellImpl
     if (!(bufferedTableItem instanceof BufferedGraphicTableItem))
       return false;
 
-    return ((BufferedGraphicTableItem)bufferedTableItem).setGraphic(img);
+    graphic = null;
+    boolean b = ((BufferedGraphicTableItem)bufferedTableItem).setGraphic(img);
+    if (b) {
+      bCellVisuallyChangedSinceRefresh = true;
+    }
+    return b;
   }
 
   public boolean setGraphic(Graphic img) {
-  	checkCellForSetting();
+  	if (img != null){
+  		checkCellForSetting();
+  	}
 
     if (!(bufferedTableItem instanceof BufferedGraphicTableItem))
       return false;
 
-    if (img == null)
-      return ((BufferedGraphicTableItem)bufferedTableItem).setGraphic(null);
+    graphic = img;
+
+    if (img == null) {
+      boolean b = ((BufferedGraphicTableItem)bufferedTableItem).setGraphic(null);
+      if (b) {
+        bCellVisuallyChangedSinceRefresh = true;
+      }
+    }
 
     if (img instanceof GraphicSWT){
     	Image imgSWT = ((GraphicSWT)img).getImage();
-    	return ((BufferedGraphicTableItem)bufferedTableItem).setGraphic(imgSWT);
+    	boolean b = ((BufferedGraphicTableItem)bufferedTableItem).setGraphic(imgSWT);
+      if (b) {
+        bCellVisuallyChangedSinceRefresh = true;
+      }
     }
     
     if (img instanceof UISWTGraphic){
     	Image imgSWT = ((UISWTGraphic)img).getImage();
-    	return ((BufferedGraphicTableItem)bufferedTableItem).setGraphic(imgSWT);
+    	boolean b = ((BufferedGraphicTableItem)bufferedTableItem).setGraphic(imgSWT);
+      if (b) {
+        bCellVisuallyChangedSinceRefresh = true;
+      }
     }
     
     return( false );
   }
 
   public Graphic getGraphic() {
+  	if (graphic != null) {
+  		return graphic;
+  	}
+
     if (!(bufferedTableItem instanceof BufferedGraphicTableItem))
       return null;
     Image img = ((BufferedGraphicTableItem)bufferedTableItem).getGraphic();
@@ -473,6 +540,7 @@ public class TableCellImpl
     	((BufferedGraphicTableItem)bufferedTableItem).setOrientation(SWT.FILL);
     else
     	setOrientationViaColumn();
+    bCellVisuallyChangedSinceRefresh = true;
   }
 
 	public void setMarginHeight(int height) {
@@ -481,6 +549,7 @@ public class TableCellImpl
     if (!(bufferedTableItem instanceof BufferedGraphicTableItem))
       return;
     ((BufferedGraphicTableItem)bufferedTableItem).setMargin(-1, height);
+    bCellVisuallyChangedSinceRefresh = true;
   }
 
   public void setMarginWidth(int width) {
@@ -489,6 +558,7 @@ public class TableCellImpl
     if (!(bufferedTableItem instanceof BufferedGraphicTableItem))
       return;
     ((BufferedGraphicTableItem)bufferedTableItem).setMargin(width, -1);
+    bCellVisuallyChangedSinceRefresh = true;
   }
 
   /* End TYPE_GRAPHIC Functions */
@@ -610,6 +680,35 @@ public class TableCellImpl
 		}
 	}
 
+
+	public void addVisibilityListener(TableCellVisibilityListener listener) {
+		try {
+			this_mon.enter();
+
+			if (cellVisibilityListeners == null)
+				cellVisibilityListeners = new ArrayList(1);
+
+			cellVisibilityListeners.add(listener);
+
+		} finally {
+			this_mon.exit();
+		}
+	}
+
+	public void removeVisibilityListener(TableCellVisibilityListener listener) {
+		try {
+			this_mon.enter();
+
+			if (cellVisibilityListeners == null)
+				return;
+
+			cellVisibilityListeners.remove(listener);
+
+		} finally {
+			this_mon.exit();
+		}
+	}
+
 	public void addListeners(Object listenerObject) {
 		if (listenerObject instanceof TableCellDisposeListener)
 			addDisposeListener((TableCellDisposeListener)listenerObject);
@@ -622,6 +721,9 @@ public class TableCellImpl
 
 		if (listenerObject instanceof TableCellMouseListener)
 			addMouseListener((TableCellMouseListener)listenerObject);
+
+		if (listenerObject instanceof TableCellVisibilityListener)
+			addVisibilityListener((TableCellVisibilityListener)listenerObject);
 	}
 
 	/**
@@ -647,47 +749,56 @@ public class TableCellImpl
   		this.bMustRefresh = true;
   }
 
-  public void refresh() {
-    refresh(true);
+  public boolean refresh() {
+    return refresh(true);
   }
   
-  public void refresh(boolean bDoGraphics) {
-  	refresh(bDoGraphics, isShown());
+  public boolean refresh(boolean bDoGraphics) {
+  	return refresh(bDoGraphics, isShown());
+  }
+
+  public boolean refresh(boolean bDoGraphics, boolean bRowVisible) {
+  	return refresh(bDoGraphics, bRowVisible, isShown());
   }
 
   private boolean bInRefresh = false;
-  public void refresh(boolean bDoGraphics, boolean bRowVisible) {
-    if (refreshErrLoopCount > 2)
-      return;
-    int iErrCount = tableColumn.getConsecutiveErrCount();
-    if (iErrCount > 10) {
-    	refreshErrLoopCount = 3;
-      return;
-    }
-    
-    boolean bVisible = bRowVisible ? isShown() : false;
-    
-    if (bInRefresh) {
-    	// Skip a Refresh call when being called from within refresh.
-    	// This could happen on virtual tables where SetData calls us again, or
-    	// if we ever introduce plugins to refresh.
-    	if (bDebug)
-    		debug("Calling Refresh from Refresh :) Skipping.");
-    	return;
-    }
-  	bInRefresh = true;
+  public boolean refresh(boolean bDoGraphics, boolean bRowVisible,
+			boolean bCellVisible)
+	{
+  	boolean ret = bCellVisuallyChangedSinceRefresh;
 
-    // See bIsUpToDate variable comments
-    if (bVisible && !bIsUpToDate) {
-    	if (bDebug)
-    		debug("Setting Invalid because visible & not up to date");
-    	valid = false;
-    	bIsUpToDate = true;
-    } else if (!bVisible && bIsUpToDate) {
-    	bIsUpToDate = false;
-    }
+  	int iErrCount = 0;
+  	try {
+      if (refreshErrLoopCount > 2) {
+        return bCellVisuallyChangedSinceRefresh;
+      }
+
+      iErrCount = tableColumn.getConsecutiveErrCount();
+      if (iErrCount > 10) {
+      	refreshErrLoopCount = 3;
+        return bCellVisuallyChangedSinceRefresh;
+      }
+      
+      if (bInRefresh) {
+      	// Skip a Refresh call when being called from within refresh.
+      	// This could happen on virtual tables where SetData calls us again, or
+      	// if we ever introduce plugins to refresh.
+      	if (bDebug)
+      		debug("Calling Refresh from Refresh :) Skipping.");
+      	return bCellVisuallyChangedSinceRefresh;
+      }
+    	bInRefresh = true;
+  
+      // See bIsUpToDate variable comments
+      if (bCellVisible && !bIsUpToDate) {
+      	if (bDebug)
+      		debug("Setting Invalid because visible & not up to date");
+      	valid = false;
+      	bIsUpToDate = true;
+      } else if (!bCellVisible && bIsUpToDate) {
+      	bIsUpToDate = false;
+      }
 
-    try {
     	if (bDebug) {
     		debug("Cell Valid?" + valid + "; Visible?" + tableRow.isVisible() + "/" + bufferedTableItem.isShown());
     	}
@@ -695,7 +806,7 @@ public class TableCellImpl
     	if (iInterval == TableColumnCore.INTERVAL_INVALID_ONLY && !valid
     			&& !bMustRefresh && bSortValueIsText && sortValue != null
 					&& tableColumn.getType() == TableColumnCore.TYPE_TEXT_ONLY) {
-    		if (bVisible) {
+    		if (bCellVisible) {
 	      	if (bDebug)
 	      		debug("fast refresh: setText");
 	    		setText((String)sortValue);
@@ -733,6 +844,8 @@ public class TableCellImpl
       refreshErrLoopCount = 0;
       if (iErrCount > 0)
         tableColumn.setConsecutiveErrCount(0);
+      
+    	ret = bCellVisuallyChangedSinceRefresh;
     } catch (Throwable e) {
       refreshErrLoopCount++;
       tableColumn.setConsecutiveErrCount(++iErrCount);
@@ -741,8 +854,14 @@ public class TableCellImpl
       	Logger.log(new LogEvent(LOGID, LogEvent.LT_ERROR,
 						"TableCell will not be refreshed anymore this session."));
     } finally {
+    	bCellVisuallyChangedSinceRefresh = false;
     	bInRefresh = false;
     }
+  	return ret;
+  }
+  
+  public boolean getVisuallyChangedSinceRefresh() {
+  	return bCellVisuallyChangedSinceRefresh;
   }
 
 
@@ -773,11 +892,17 @@ public class TableCellImpl
     sortValue = null;
   }
   
-  public void setImage(Image img) {
+  public void setIcon(Image img) {
   	if (isInvisibleAndCanRefresh())
   		return;
 
-    bufferedTableItem.setImage(img);
+    bufferedTableItem.setIcon(img);
+    graphic = null;
+    bCellVisuallyChangedSinceRefresh = true;
+  }
+  
+  public Image getIcon() {
+  	return bufferedTableItem.getIcon();
   }
 
   public boolean needsPainting() {
@@ -811,7 +936,12 @@ public class TableCellImpl
 	 * @see java.lang.Object#toString()
 	 */
 	public String toString() {
-		return getText();
+		return "TableCell {"
+				+ tableColumn.getName()
+				+ ","
+				+ (bufferedTableItem == null ? "null" : ""
+						+ bufferedTableItem.getPosition()) + "," + getText() + ","
+				+ getSortValue() + "}";
 	}
 
 	/* Comparable Implementation */
@@ -895,6 +1025,22 @@ public class TableCellImpl
 		}
 	}
 
+  public void invokeVisibilityListeners(int visibility) {
+		if (cellVisibilityListeners == null)
+			return;
+
+		for (int i = 0; i < cellVisibilityListeners.size(); i++) {
+			try {
+				TableCellVisibilityListener l = (TableCellVisibilityListener) (cellVisibilityListeners.get(i));
+
+				l.cellVisibilityChanged(this, visibility);
+
+			} catch (Throwable e) {
+				Debug.printStackTrace(e);
+			}
+		}
+	}
+
 
   public static final Comparator TEXT_COMPARATOR = new TextComparator();
   private static class TextComparator implements Comparator {
