@@ -24,26 +24,31 @@
 
 package org.gudy.azureus2.ui.swt.views.tableitems.mytorrents;
 
+import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.graphics.Image;
+import org.eclipse.swt.graphics.Rectangle;
+import org.eclipse.swt.widgets.Display;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
 import org.gudy.azureus2.core3.download.DownloadManager;
-import org.gudy.azureus2.plugins.ui.tables.TableCell;
-import org.gudy.azureus2.plugins.ui.tables.TableCellRefreshListener;
-import org.gudy.azureus2.plugins.ui.tables.TableColumn;
+import org.gudy.azureus2.core3.torrent.TOTorrent;
+import org.gudy.azureus2.core3.util.Constants;
 import org.gudy.azureus2.ui.swt.ImageRepository;
 import org.gudy.azureus2.ui.swt.debug.ObfusticateCellText;
 import org.gudy.azureus2.ui.swt.views.table.TableCellCore;
 import org.gudy.azureus2.ui.swt.views.table.utils.CoreTableColumn;
 
+import org.gudy.azureus2.plugins.ui.tables.*;
+
 /** Torrent name cell for My Torrents.
  *
  * @author Olivier
  * @author TuxPaper (2004/Apr/17: modified to TableCellAdapter)
  */
 public class NameItem extends CoreTableColumn implements
-		TableCellRefreshListener, ObfusticateCellText {
+		TableCellRefreshListener, ObfusticateCellText, TableCellDisposeListener
+{
 	private static boolean bShowIcon;
 
 	static {
@@ -54,7 +59,7 @@ public class NameItem extends CoreTableColumn implements
 					}
 				});
 	}
-	
+
 	/** Default Constructor */
 	public NameItem(String sTableID) {
 		super("name", POSITION_LAST, 250, sTableID);
@@ -74,12 +79,57 @@ public class NameItem extends CoreTableColumn implements
 		//setText returns true only if the text is updated
 		if (cell.setText(name) || !cell.isValid()) {
 			if (dm != null && bShowIcon) {
-				// Don't ever dispose of PathIcon, it's cached and may be used elsewhere
-				Image icon = ImageRepository.getPathIcon(dm.getSaveLocation()
-						.toString());
-				// cheat for core, since we really know it's a TabeCellImpl and want to
-				// use those special functions not available to Plugins
-				((TableCellCore) cell).setImage(icon);
+				String path = dm.getDownloadState().getPrimaryFile();
+				if (path != null) {
+					// Don't ever dispose of PathIcon, it's cached and may be used elsewhere
+					Image icon = ImageRepository.getPathIcon(path);
+
+					if (Constants.isWindows) {
+						disposeCellIcon(cell);
+
+						Rectangle iconBounds = icon.getBounds();
+						// recomposite to avoid artifacts - transparency mask does not work
+						
+						int cellHeight = cell.getHeight();
+						if (cellHeight < 20) {
+							cellHeight = 16;
+						}
+
+						final Image dstImage = new Image(Display.getCurrent(),
+								cellHeight, cellHeight);
+						GC gc = new GC(dstImage);
+						try {
+							// for drawing alpha
+							gc.setAdvanced(true);
+						} catch (Exception e) {
+						}
+						try {
+							gc.drawImage(icon, 0, 0, iconBounds.width, iconBounds.height, 0,
+									0, cellHeight, cellHeight);
+							TOTorrent torrent = dm.getTorrent();
+							if (torrent != null && !torrent.isSimpleTorrent()) {
+								Image imgFolder = ImageRepository.getImage("foldersmall");
+								Rectangle folderBounds = imgFolder.getBounds();
+								gc.drawImage(imgFolder, folderBounds.x, folderBounds.y,
+										folderBounds.width, folderBounds.height, cellHeight
+												- folderBounds.width, cellHeight
+												- folderBounds.height, folderBounds.width,
+										folderBounds.height);
+							}
+						} finally {
+							gc.dispose();
+						}
+						icon = dstImage;
+					}
+
+					// cheat for core, since we really know it's a TabeCellImpl and want to
+					// use those special functions not available to Plugins
+					((TableCellCore) cell).setIcon(icon);
+				} else {
+					if (Constants.isWindows) {
+						disposeCellIcon(cell);
+					}
+				}
 			}
 		}
 	}
@@ -94,9 +144,25 @@ public class NameItem extends CoreTableColumn implements
 				name = name.substring(i + 1);
 			}
 		}
-		
+
 		if (name == null)
 			name = "";
 		return name;
 	}
+
+	public void dispose(TableCell cell) {
+		if (bShowIcon && Constants.isWindows) {
+			disposeCellIcon(cell);
+		}
+	}
+
+	private void disposeCellIcon(TableCell cell) {
+		final Image img = ((TableCellCore) cell).getIcon();
+		if (img != null) {
+			((TableCellCore) cell).setIcon(null);
+			if (!img.isDisposed()) {
+				img.dispose();
+			}
+		}
+	}
 }
