@@ -20,8 +20,9 @@
 
 package org.gudy.azureus2.ui.swt.views.tableitems.mytracker;
 
+import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.graphics.Image;
-import org.eclipse.swt.program.Program;
+import org.eclipse.swt.widgets.Display;
 
 import org.gudy.azureus2.core3.config.COConfigurationManager;
 import org.gudy.azureus2.core3.config.ParameterListener;
@@ -29,16 +30,14 @@ import org.gudy.azureus2.core3.torrent.TOTorrent;
 import org.gudy.azureus2.core3.torrent.TOTorrentException;
 import org.gudy.azureus2.core3.tracker.host.TRHostTorrent;
 import org.gudy.azureus2.core3.util.ByteFormatter;
+import org.gudy.azureus2.core3.util.Constants;
 import org.gudy.azureus2.core3.util.TorrentUtils;
 import org.gudy.azureus2.ui.swt.ImageRepository;
 import org.gudy.azureus2.ui.swt.debug.ObfusticateCellText;
 import org.gudy.azureus2.ui.swt.views.table.TableCellCore;
 import org.gudy.azureus2.ui.swt.views.table.utils.CoreTableColumn;
 
-import org.gudy.azureus2.plugins.ui.tables.TableCell;
-import org.gudy.azureus2.plugins.ui.tables.TableCellRefreshListener;
-import org.gudy.azureus2.plugins.ui.tables.TableColumn;
-import org.gudy.azureus2.plugins.ui.tables.TableManager;
+import org.gudy.azureus2.plugins.ui.tables.*;
 
 /**
  *
@@ -46,7 +45,7 @@ import org.gudy.azureus2.plugins.ui.tables.TableManager;
  * @since 2.0.8.5
  */
 public class NameItem extends CoreTableColumn implements
-		TableCellRefreshListener, ObfusticateCellText
+		TableCellRefreshListener, ObfusticateCellText, TableCellDisposeListener
 {
 	private static boolean bShowIcon;
 
@@ -92,7 +91,7 @@ public class NameItem extends CoreTableColumn implements
 
 					Image icon = ImageRepository.getFolderImage();
 
-					((TableCellCore) cell).setImage(icon);
+					((TableCellCore) cell).setIcon(icon);
 
 				} else {
 
@@ -101,12 +100,22 @@ public class NameItem extends CoreTableColumn implements
 					if (sep < 0)
 						sep = 0;
 
-					name = name.substring(sep);
-					Program program = Program.findProgram(name);
-					Image icon = ImageRepository.getIconFromProgram(program);
+					String ext = name.substring(sep);
+					Image icon = ImageRepository.getIconFromExtension(ext);
+
+					if (Constants.isWindows) {
+						// recomposite to avoid artifacts - transparency mask does not work
+						final Image dstImage = new Image(Display.getCurrent(),
+								icon.getBounds().width, icon.getBounds().height);
+						GC gc = new GC(dstImage);
+						gc.drawImage(icon, 0, 0);
+						gc.dispose();
+						icon = dstImage;
+					}
+
 					// cheat for core, since we really know it's a TabeCellImpl and want to use
 					// those special functions not available to Plugins
-					((TableCellCore) cell).setImage(icon);
+					((TableCellCore) cell).setIcon(icon);
 				}
 			}
 		}
@@ -125,4 +134,17 @@ public class NameItem extends CoreTableColumn implements
 			name = "";
 		return name;
 	}
+
+	public void dispose(TableCell cell) {
+		if (bShowIcon && Constants.isWindows) {
+			final Image img = ((TableCellCore) cell).getIcon();
+			Image icon = ImageRepository.getFolderImage();
+			if (img != null && !img.equals(icon)) {
+				((TableCellCore) cell).setIcon(null);
+				if (!img.isDisposed()) {
+					img.dispose();
+				}
+			}
+		}
+	}
 }
