@@ -41,7 +41,6 @@ import org.gudy.azureus2.plugins.*;
 import org.gudy.azureus2.plugins.logging.*;
 import org.gudy.azureus2.plugins.update.*;
 import org.gudy.azureus2.plugins.utils.resourcedownloader.*;
-import org.gudy.azureus2.pluginsimpl.local.update.UpdateManagerImpl;
 
 import com.aelitis.azureus.core.versioncheck.*;
 
@@ -49,7 +48,7 @@ import com.aelitis.azureus.core.versioncheck.*;
 public class 
 CoreUpdateChecker
 	implements Plugin, UpdatableComponent
-{
+{		
 	public static final String	LATEST_VERSION_PROPERTY	= "latest_version";
 	public static final String	MESSAGE_PROPERTY		= "message";
 	
@@ -154,8 +153,8 @@ CoreUpdateChecker
       
 			displayUserMessage( decoded );
 			
-			String latest_version 			= null;
-			String latest_file_name			= null;
+			String latest_version;
+			String latest_file_name;
 			
 			byte[] b_version = (byte[])decoded.get("version");
 			
@@ -175,10 +174,44 @@ CoreUpdateChecker
 			if ( b_filename != null ){
 			
 				latest_file_name = new String( b_filename );
+				
+			}else{
+				
+				throw( new Exception( "No update file details in reply" ));
 			}
 			
+			//latest_version 		= "3.0.0.3";
+			//latest_file_name	= "http://torrents.aelitis.com:88/torrents/Azureus2.5.0.0.jar.torrent";
+			//latest_file_name	= "Azureus2.5.0.0.jar.torrent";
+			
 			String	msg = "Core: latest_version = '" + latest_version + "', file = '" + latest_file_name + "'";
 			
+			URL		full_download_url;
+			
+				// since 2501 we support a full download URL, falling back to SF mirrors if this
+				// fails. 
+					
+			if ( latest_file_name.startsWith( "http" )){
+				
+				try{
+					full_download_url	= new URL( latest_file_name );
+					
+				}catch( Throwable e ){
+					
+					full_download_url = null;
+					
+					log.log( e );
+				}
+				
+				int	pos = latest_file_name.lastIndexOf( '/' );
+				
+				latest_file_name = latest_file_name.substring( pos+1 );
+				
+			}else{
+				
+				full_download_url	= null;
+			}
+			
 			checker.reportProgress( msg );
 			
 			log.log( msg );
@@ -191,40 +224,85 @@ CoreUpdateChecker
 			final String	f_latest_version	= latest_version;
 			final String	f_latest_file_name	= latest_file_name;
 			
-
-			ResourceDownloader[]	primary_mirrors;
+			ResourceDownloader	top_downloader;
+			
+			if ( full_download_url == null ){
 				
-			primary_mirrors = getPrimaryDownloaders( latest_file_name );
-
-				// the download hierarchy is primary mirrors first (randomised alternate)
-				// then backup mirrors (randomised alternate)
-			
-				// we don't want to load the backup mirrors until the primary mirrors fail
-			
-			ResourceDownloader		random_primary_mirrors = rdf.getRandomDownloader( primary_mirrors );
-			
-			ResourceDownloader		backup_downloader =
-				rdf.create(
-					new ResourceDownloaderDelayedFactory()
-					{
-						public ResourceDownloader
-						create()
+				ResourceDownloader[]	primary_mirrors;
+					
+				primary_mirrors = getPrimaryDownloaders( latest_file_name );
+	
+					// the download hierarchy is primary mirrors first (randomised alternate)
+					// then backup mirrors (randomised alternate)
+				
+					// we don't want to load the backup mirrors until the primary mirrors fail
+				
+				ResourceDownloader		random_primary_mirrors = rdf.getRandomDownloader( primary_mirrors );
+				
+				ResourceDownloader		backup_downloader =
+					rdf.create(
+						new ResourceDownloaderDelayedFactory()
 						{
-							ResourceDownloader[]	backup_mirrors = getBackupDownloaders( f_latest_file_name );
-						
-							return( rdf.getRandomDownloader( backup_mirrors ));
-						}
-					});
-			
-			ResourceDownloader	top_downloader = 
-				rdf.getAlternateDownloader( 
-						new ResourceDownloader[]
+							public ResourceDownloader
+							create()
 							{
-								random_primary_mirrors,
-								backup_downloader,
-							});
-			
+								ResourceDownloader[]	backup_mirrors = getBackupDownloaders( f_latest_file_name );
+							
+								return( rdf.getRandomDownloader( backup_mirrors ));
+							}
+						});
+								
+				top_downloader = 
+					rdf.getAlternateDownloader( 
+							new ResourceDownloader[]
+								{
+									random_primary_mirrors,
+									backup_downloader,
+								});
 
+			}else{
+				
+				ResourceDownloader full_rd = rdf.create( full_download_url );
+				
+				full_rd = rdf.getSuffixBasedDownloader( full_rd );
+				
+				ResourceDownloader		primary_downloader =
+					rdf.create(
+						new ResourceDownloaderDelayedFactory()
+						{
+							public ResourceDownloader
+							create()
+							{
+								ResourceDownloader[]	primary_mirrors = getPrimaryDownloaders( f_latest_file_name );
+							
+								return( rdf.getRandomDownloader( primary_mirrors ));
+							}
+						});
+					
+				ResourceDownloader		backup_downloader =
+					rdf.create(
+						new ResourceDownloaderDelayedFactory()
+						{
+							public ResourceDownloader
+							create()
+							{
+								ResourceDownloader[]	backup_mirrors = getBackupDownloaders( f_latest_file_name );
+							
+								return( rdf.getRandomDownloader( backup_mirrors ));
+							}
+						});
+				
+				
+				top_downloader = 
+					rdf.getAlternateDownloader( 
+							new ResourceDownloader[]
+								{
+									full_rd,
+									primary_downloader,
+									backup_downloader,
+								});
+			}
+			
 			top_downloader.addListener( rd_logger );
 			
 				// get size so it is cached
